#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package ArgEdit;

$padx = 3;
$pady = 3;

require Tk::Frame;
@ISA = qw(Tk::Derived Tk::Frame);
Tk::Widget->Construct('ArgEdit');

=head1 NAME:

Widget: ArgEdit

=head1 SYNOPSIS:

use class::ArgEdit;

=head1 DIRECTORY/FILE:

The file ArgEdit.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

This is a widget derived from Tk::Frame that gets connected with one argument of a tag and allows to display/modify the value of the argument.

Later it will also check the arguments types and display different widgets for entries (Entry or List or Optionmenu or Text or Checkbox).
Possible argument types: TEXT, NUMBER, MULTILINE, OPTION([item1, item2, ...]), REFERENCE([$stylegroup1.$stlye1.tag_name, $stylegroup1.$stlye1.property_value(property_name)], ...), BOOL
It should also check possible ranges by number entries or the maximum length of text entries.

If the argument is a reference to another tag, this should be modifyd using a special List-derived widget that shows all (possible) tags that could be referenced.

This object is coded as an Perl/Tk widget, so it has no Constructor new, it has the standard Tk functions to initialise itself (Populate,ClassInit,...)
The function Populate() is used like an constructor here.

=head1 STATUS:

Coding: 80% done

Documentation: up-to-date

NOTE: Here all objects get correctly destroyed after closing the window --> !NO CIRCULAR REFERENCES!

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

$self->{'foreground'} #The foreground color definition

$self->{'background'} #The background color definition

$self->{'font'} #The font definition

$self->{'relief'} #The relief definition

$self->{'tagman'} #A reference to the associated TagMan object

$self->{'tag_name'} #The name of the tag the displayed argument belogs to (=this ArgEdit object is connected to)

$self->{'is_object'} #Flag, weather the ArgEdit 'belongs' to an EmbObj object

$self->{'ref_object'} #Reference to that EmbObj.

$self->{'is_doc_style'} #Flag, weather the ArgEdit 'belongs' to the document itself (Document arguments)

$self->{$arg_index}->{'_arg_name'} #the argument name of one entry (indexed by {$arg_index} ) 
This attribute has no Get/Set method, use it directly.

NOTE: The following attributes are used for the Tk widgets building the GUI and are accessed directly (no get/set methods):

$self->{$arg_index}->{'_frame'} #the base frame of one argument enty (indexed by {$arg_index} )

$self->{$arg_index}->{'_entry'} #the entry of one argument(indexed by {$arg_index} )

$self->{$arg_index}->{'_label'} #the label of one argument entry(indexed by {$arg_index} )

=head1 METHODS:

=head2 Class Methods:

=head3 ClassInit($class, $parentwindow)

! Not called directly !

This "constructor" initialises a new object of the class ComObj and calls ClassInit() of the Superclass. 

=cut

##########################
# Klassen-Initalisierung #
##########################
sub ClassInit
{
# Tk-spezifischer Code
my ($class,$mainwindow)=@_; # Klassenname und Mainwindow
$class->SUPER::ClassInit($mainwindow); # Aufruf des Vorfahren
return $class;
}


=head2 Constructors:

=head3 Populate

$object = ArgEdit(%options); 

This "constructor" sets the options of the new object of the class ArgEdit.

The binding of events and the creation of the associated Baloonhelp is done here too.

The keys/values in %options: 

(+) '-foreground' =>  The foreground color 

(+) '-background' =>  The background color 

(+) '-relief' =>  The relief style ('flat', 'raised', 'sunken', 'ridged')

(+) '-font' =>  The font definition

(+) '-tagman' =>  A reference to the associated TagMan object

(+) '-tag_name' =>  The name of the tag which arguments should be edited by this ArgEdit object

(+) '-is_object' =>  Flag is set to 1, if argument is stored in an object, else it is 0

(+) '-ref_object' =>  Reference to the associated object (if any) or undef

(+) '-is_doc_style' =>  Flag is set to 1, if argument belongs to the whole document

=cut

###########################################################################
# Uebergabe, ev. Filterung und Weiterleitung der "-option=>'value'"-Paare #
###########################################################################
sub Populate
{
my ($self,$args)=@_; #Auslesen der $self-Referenz und der option/value-paare
#Display-Einstellungswerte speichern
$self->foreground( delete $args->{'-foreground'} );
$self->background( delete $args->{'-background'} );
$self->relief( delete $args->{'-relief'} );
$self->font( delete $args->{'-font'} );
$self->tag_name( delete $args->{'-tag_name'} );
$self->tagman( delete $args->{'-tagman'} );
$self->is_object(0); #default value
$self->ref_object(undef); #default value
$self->is_object( delete $args->{'-is_object'} );
$self->is_doc_style( delete $args->{'-is_doc_style'} );
$self->ref_object( delete $args->{'-ref_object'} );

$self->SUPER::Populate($args); #Weiterleitung an Vorfahr (Basisklasse)

$self->{'_base'} = $self->Frame()->pack(-expand=>1, -fill=>'both', -ipadx=>5, -ipady=>5);

#Build up the GUI part#
my $arg_number;
my $arg_base_branch;

if ($self->is_object() == 1)
	{
	#Argument belongs to an EmbObj
	$arg_base_branch = $self->ref_object()->{'args'};
	$arg_number = $arg_base_branch->{'arg_number'};
	}
elsif ($self->is_doc_style() == 1)
	{
	$arg_base_branch = $self->tagman()->{'doc_args'};
	$arg_number = $arg_base_branch->{'arg_number'};
	}
else
	{
	#Argument belongs to a tag
	$arg_base_branch = $self->tagman()->{'args'}->{$self->tag_name()};
	$arg_number = $arg_base_branch->{'arg_number'};
	}

if ($arg_number > 0)
	{
	for (my $i=1 ; $i<=$arg_number; $i++)
		{
		#Getting all argument info from the TagMan object
		$self->{$i}->{'_arg_name'} = $arg_base_branch->{$i}->{'name'};
		$self->{$i}->{'_arg_type'} = $arg_base_branch->{$i}->{'type'};
		$self->{$i}->{'_arg_settings'} = $arg_base_branch->{$i}->{'settings'};
		$self->{$i}->{'_arg_hint'} = $arg_base_branch->{$i}->{'hint'};

		#Build  and configure the base widgets
		$self->{$i}->{'_label'} = $self->{'_base'}->Label(-text=>$self->{$i}->{'_arg_name'}.":", -justify=>'right', -anchor=>'e',  -borderwidth=>2)->grid(-column=>0, -row=>($i-1), -sticky=>'ne', -padx=>$padx, -pady=>$pady);
		if (defined $self->{'foreground'}) {$self->{$i}->{'_label'}->configure(-foreground=>$self->{'foreground'})};
		if (defined $self->{'background'}) {$self->{$i}->{'_label'}->configure(-background=>$self->{'background'})};
		if (defined $self->{'relief'}) {$self->{$i}->{'_label'}->configure(-relief=>$self->{'relief'})};
		if (defined $self->{'font'}) {$self->{$i}->{'_label'}->configure(-font=>$self->{'font'})};

		#If the type is 'LIST', show a dropdown-list
		if ((defined $self->{$i}->{'_arg_type'}) and
			($self->{$i}->{'_arg_type'} eq 'LIST') and 
			(defined $self->{$i}->{'_arg_settings'}) and
			(not ($self->{$i}->{'_arg_settings'} eq '')))
			{
			#Create the BrowseEntry that contains the list
			$self->_create_type_list($i, $arg_base_branch);
			}
		#If the type is "TAG_LIST", show a list with all entries of the given argument od all tags that have the appropriate style
		elsif ((defined $self->{$i}->{'_arg_type'}) and
			($self->{$i}->{'_arg_type'} eq 'TAG_LIST') and 
			(defined $self->{$i}->{'_arg_settings'}) and
			(not ($self->{$i}->{'_arg_settings'} eq '')))
			{
			#Create the BrowseEntry that contains the taglist
			$self->_create_type_tag_list($i, $arg_base_branch);
			}
		#If the type is set to 'MULTILINE'
		elsif ((defined $self->{$i}->{'_arg_type'}) and
			($self->{$i}->{'_arg_type'} eq 'MULTILINE'))
			{
			#Create a normal Entry widget
			$self->_create_type_multiline($i, $arg_base_branch);
			}
		#If the type is set to 'TEXT' or anything else (Errors)
		else
			{
			#Create a normal Entry widget
			$self->_create_type_text($i, $arg_base_branch);
			}
		#Bind the key-events for the content_changed()-flag
		$self->{$i}->{'_entry'}->bind('<Key>'=>sub{$self->_value_changed()});
		if (exists $self->{$i}->{'_arg_hint'} and defined $self->{$i}->{'_arg_hint'})
			{
			$self->{$i}->{'_balloon'} = $self->{'_base'}->Balloon(-state=>'balloon');
			$self->{$i}->{'_balloon'}->attach($self->{$i}->{'_entry'}, -msg=>$self->{$i}->{'_arg_hint'});
			}
		}
	}
#Bind the destroy event for cleanup-code
$self->bind('<Destroy>' => sub{ $self->do_destroy()}); #Aufraeumroutine binden

print "Object $self created.\n"; #debug

$self->{1}->{'_entry'}->focus() if exists $self->{1}->{'_entry'} and defined $self->{1}->{'_entry'};

return $self;
}

=head2 Destructors:

=head3 DESTROY; 

The standard automatically called PERL destructor for objects.

=cut

#############################
# Automatic Perl destructor #
#############################
sub DESTROY
{
my $self = shift;
print "Object $self destroyed.\n"; #debug
}

=head3 do_detstroy

$object->do_detsroy(); 

This "destructor" method is binded to the "<DESTROY>" event and should be performed whenever
the element is destroyed. It should not be called directly.

=cut

###################################################
# Destruktor zum sauberen aufraeumen des Objektes #
###################################################
sub do_destroy
{
my $sender = shift; 
}

=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 tagman

$ref_tagman = $object->tagman(); #gets a reference to the TagMan object 

$object->tagman($ref_tagman); #sets a reference to the Tagman object 

=cut

### tagman ###
sub tagman
{
my $self=shift;
@_ ? $self->{'tagman'} = shift : return $self->{'tagman'};
}

=head3 tag_name

$tag_name = $object->tag_name(); #gets the tags name which has the arguments to be edited

$object->tag_name($tag_name); #sets the tags name which has the arguments to be edited

=cut

### tag_name ###
sub tag_name
{
my $self=shift;
@_ ? $self->{'tag_name'} = shift : return $self->{'tag_name'};
}

=head3 foreground

$color_def = $object->foreground(); #gets the foreground color definition

$object->foreground($color_def); #sets the foreground color definition

=cut

### foreground ###
sub foreground
{
my $self=shift;
@_ ? $self->{'foreground'} = shift : return $self->{'foreground'};
}

=head3 background

$color_def = $object->background(); #gets the background color definition

$object->background($color_def); #sets the background color definition

=cut

### background ###
sub background
{
my $self=shift;
@_ ? $self->{'background'} = shift : return $self->{'background'};
}

=head3 font

$font_def = $object->font(); #gets the font definition

$object->font($font_def); #sets the font definition

=cut

### font ###
sub font
{
my $self=shift;
@_ ? $self->{'font'} = shift : return $self->{'font'};
}

=head3 relief

$relief_def = $object->relief(); #gets the relief definition

$object->relief($relief_def); #sets the relief definition

=cut

### relief ###
sub relief
{
my $self=shift;
@_ ? $self->{'relief'} = shift : return $self->{'relief'};
}

=head3 is_object

$is_object = $object->is_object(); #gets the is_object flag

$object->is_object($is_object); #sets the is_object flag

=cut

### is_object ###
sub is_object
{
my $self=shift;
@_ ? $self->{'is_object'} = shift : return $self->{'is_object'};
}

=head3 ref_object

$ref_object = $object->ref_object(); #gets the reference to the associated object

$object->ref_object($ref_object); #sets the reference to the associated object

=cut

### ref_object ###
sub ref_object
{
my $self=shift;
@_ ? $self->{'ref_object'} = shift : return $self->{'ref_object'};
}

=head3 is_doc_style

$is_doc_style = $object->is_doc_style(); #gets the is_doc_style flag

$object->is_doc_style($is_doc_style); #sets the is_doc_style flag

=cut

### is_doc_style ###
sub is_doc_style
{
my $self=shift;
@_ ? $self->{'is_doc_style'} = shift : return $self->{'is_doc_style'};
}


=head2 Other public methods:

no other public methods so far.

=head2 Private methods:

=head3 _value_changed

$object->_value_changed(); 

This is the callback binded to the "<Key>" event of the entry. It calls automatically the
method "set_content_changed()" of the DocForm object the managed TagMan object belongs to.
This means that when you change one argument entry, the whole document is set to CHANGED.

=cut

###############################################
# Wenn etwas ins Eingabefeld eingegeben wurde #
###############################################
sub _value_changed
{
my $sender =shift;
#Set the changed-flag in the document to true.
$sender->tagman()->docform()->set_content_changed();
}

=head3 _create_type_list

my $widget = $object->_create_type_list($arg_index); 

This function creates a BrowsEntry for displaying a 'LIST' argument.

The function returns a reference to the created widget.

Parameters:

(+) $arg_index: The index (number) of the argument that gets processed.

(+) The base of the hash-branch that holds all argument information (in the TagMan or in the EmbObj or in the Document itself)

=cut

#######################
# Argument type: LIST #
#######################
sub _create_type_list
{
my $self = shift;
my $arg_index = shift;
my $arg_base_branch = shift;

#Build up the list

my @list = sort( split(/\|\|/, $self->{$arg_index}->{'_arg_settings'}) );
#Build up the dropdown widget
my $entry = $self->{$arg_index}->{'_entry'} = $self->{'_base'}->BrowseEntry(-choices=>\@list, -textvariable=>\$arg_base_branch->{$arg_index}->{'value'},-width=>40)->grid(-column=>1, -row=>($arg_index-1), -sticky=>'nsew', -padx=>$padx, -pady=>$pady);
Utils::set_browseentry_color($entry, 'white');
Utils::beautify_browseentry($entry);
return $entry;
}

=head3 _create_type_tag_list

my $widget = $object->_create_type_tag_list($arg_index); 

This function creates a BrowsEntry for displaying a 'TAG_LIST' argument.

The function returns a reference to the created widget.

Parameters:

(+) $arg_index: The index (number) of the argument that gets processed.

(+) The base of the hash-branch that holds all argument information (in the TagMan or in the EmbObj or in the Document itself)

=cut

###########################
# Argument type: TAG_LIST #
###########################
sub _create_type_tag_list
{
my $self = shift;
my $arg_index = shift;
my $arg_base_branch = shift;

#Build up the taglist

#Split up the settings if its a '||' seperated list
my @styles_list =  split(/\|\|/, $self->{$arg_index}->{'_arg_settings'});
#The result list
my @list; 
foreach my $style_def (@styles_list)
	{
	#EXAMPLE of right settings: "ENV_STYLES::CodeBlock(1)" means: all values of the first argument of all tags belonging to stylegroup "ENV_STYLES", style "CodeBlock"
	if ($style_def =~ /(.+?)::(.+?)\((\d+)\)/o ) 
		{
		my $stylegroup = $1;
		my $style = $2;
		my $arg_index = $3;
		#Refresh TagMan object (maybe not neccessary, costs speed)
		$self->tagman()->read_from_widget($self->tagman->docform()->doctext());
		$mask = quotemeta("<$stylegroup><$style>");
		foreach my $key (keys %{$self->tagman()->{'args'}}) #Iterate over all tags that have arguments
			{
			if ($key =~ /$mask/)#Look if stylegroup/style fits
				{
				if (exists $self->tagman()->{'args'}->{$key}->{$arg_index}->{'value'} and defined $self->tagman()->{'args'}->{$key}->{$arg_index}->{'value'})
					{
					#If the argument with this argument-index exists->get it
					push (@list, $self->tagman()->{'args'}->{$key}->{$arg_index}->{'value'});
					}
				else
					{
					print "ERROR: No argumnet number $arg_index for stylegroup: $stylegroup $style: $style \n"; 
					}
				}
			}
		}
	}
#Sort the list
@list = sort (@list); 
#Build up the dropdown list
my $entry = $self->{$arg_index}->{'_entry'} = $self->{'_base'}->BrowseEntry(-choices=>\@list, -textvariable=>\$arg_base_branch->{$arg_index}->{'value'},-width=>40)->grid(-column=>1, -row=>($arg_index-1), -sticky=>'nsew', -padx=>$padx, -pady=>$pady);
Utils::set_browseentry_color($entry, 'white');
Utils::beautify_browseentry($entry);

return $entry;
}

=head3 _create_type_text

my $widget = $object->_create_type_text($arg_index); 

This function creates an Entry widget for displaying a 'TEXT' argument.

The function returns a reference to the created widget.

Parameters:

(+) $arg_index: The index (number) of the argument that gets processed.

(+) The base of the hash-branch that holds all argument information (in the TagMan or in the EmbObj or in the Document itself)

=cut

#######################
# Argument type: TEXT #
#######################
sub _create_type_text
{
my $self = shift;
my $arg_index = shift;
my $arg_base_branch = shift;

my $frame = $self->{$arg_index}->{'_entry_frame'} = $self->{'_base'}->LabFrame(-borderwidth=>1, -labelside=>'left', -relief=>'flat')->grid(-column=>1, -row=>($arg_index-1), -sticky=>'nsew', -padx=>$padx, -pady=>$pady);
my $entry = $self->{$arg_index}->{'_entry'} = $frame->Entry(-textvariable=>\$arg_base_branch->{$arg_index}->{'value'}, -width=>40, -background=>'white')->pack(-fill=>'both', -expand=>1, -side=>'left');
return $entry;
}

=head3 _create_type_multiline

my $widget = $object->_create_type_multiline($arg_index); 

This function creates a Text widget for displaying a 'MULTILINE' argument.

The function returns a reference to the created widget.

Parameters:

(+) $arg_index: The index (number) of the argument that gets processed.

(+) The base of the hash-branch that holds all argument information (in the TagMan or in the EmbObj or in the Document itself)

=cut

############################
# Argument type: MULTILINE #
############################
sub _create_type_multiline
{
my $self = shift;
my $arg_index = shift;
my $arg_base_branch = shift;

my $frame = $self->{$arg_index}->{'_entry_frame'} = $self->{'_base'}->LabFrame(-borderwidth=>1, -labelside=>'left', -relief=>'flat')->grid(-column=>1, -row=>($arg_index-1), -sticky=>'nsew', -padx=>$padx, -pady=>$pady);
my $entry = $self->{$arg_index}->{'_entry'} = $frame->Scrolled('Text', -scrollbars=>'oe', -wrap=>'word', -height=>6, -width=>40, -background=>'white')->pack(-fill=>'both', -expand=>1, -side=>'left');
$entry->insert('1.0', $arg_base_branch->{$arg_index}->{'value'});
$entry->bind('<KeyRelease>' => [\&multiline_arg_changed, $arg_base_branch, $arg_index] );

return $entry;
}

=head2 multiline_arg_changed

uiDocForm::multiline_arg_changed($arg_base_branch, $arg_index); 

Updates the value of the argument.

Parameters:

(+) $arg_base_branch: Base of the 'args'-branch

(+) $arg_index: The index number of the argument

=cut

###########################
# The text widget changed #
###########################
sub multiline_arg_changed
{
my $sender = shift;
my $arg_base_branch = shift; #Base of the 'args'-branch
my $arg_index = shift; #The index of the argument

my $text = $sender->get('1.0', 'end');
$text =~ s/(\n+)$//g ; #remove Newlines at the end
$arg_base_branch->{$arg_index}->{'value'} = $text;
}


1; #So muss ein Package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
