#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package ComObj;

require Tk::Frame;
@ISA = qw(Tk::Frame);
Tk::Widget->Construct('ComObj');

=head1 NAME:

Widget: ComObj

=head1 SYNOPSIS:

use class::ComObj;

=head1 DIRECTORY/FILE:

The file ComObj.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

This is a widget derived from Tk::Frame that gets embeded into the documents text widget in the DocForm object (=document window).

It is used for representing commands which are different from the styles you can apply to some parts of the text. A style is applied to some text
so it just encloses a part of the text and gives it a special meaning/formatting (f. ex. in LaTeX: "\chapter{This is my chapter}" or in HTML: "<H1>This is my chapter</H1>").
A command is not layed over a text part, it is inserted into the text and could have some arguments with it (f.ex. in LaTeX: "\footnote{some text}" here is "some text" not part of the
original document text, it is an ARGUMENT that belongs to the "\footnote"-command. In HTML: "<IMG SRC="img.jpg">".)
The difference between formatting a part of the documents text and arguments of a command is that the arguments belong to the command and are f. ex.
deleted together with the command, whereas you can apply and remove all styles to textparts without harming this textparts themselves.

This object has 2 different states.

(+) In the collapsed state, it looks like a button and has the name of the command it reprents as caption on itself.

(+) In the expanded state, it shows one ArgEdit object per defined argument. NOTE: What arguments and how many arguments
    a command has is defined in the configuration file "edoc.int". A command with no argument can not be expanded. There can
    also be dfined some arguments for "normal" styles, but this arguments are not set by an embeded argument editor, they are
    set by the "TagForm" object (a special argument setting window for styles) which uses again ArgEdit objects to handle the arguments.

This object is coded as an Perl/Tk widget, so it has no Constructor new, it has the standard Tk functions to initialise itself (Populate,ClassInit,...)
The function Populate() is used like an constructor here.

=head1 STATUS:

Coding: 80% done

Documentation: up-to-date

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

$self->{'foreground'} #The foreground color definition

$self->{'background'} #The background color definition

$self->{'font'} #The font definition

$self->{'relief'} #The relief definition

$self->{'docform'} #A reference to the associated DocForm object

$self->{'tagman'} #A reference to the associated TagMan object

$self->{'command'} #The command name the object representates

$self->{'window_tag'} #The name of the tag the ComObj is tagged with in the text widget

$self->{'argedit'} #The automatically created ArgEdit object when the ComObj gets expanded

$self->{'_expanded'} #Flag wether the object is in expanded state ore not

NOTE: The following attributes are used for the Tk widgets building the GUI and are accessed directly (no get/set methods):

$self->{'_label'} #The label that holds the caption of the ComObj

$self->{'_label_text'} #What text should be displayed in the label

$self->{'_base'} #The frame that builds the base of the ComObj

=head1 METHODS:

=head2 Class Methods:

=head3 ClassInit($class, $parentwindow)

! Not called directly !

This "constructor" initialises a new object of the class ComObj and calls ClassInit() of the Superclass. 

=cut

##########################
# Klassen-Initalisierung #
##########################
sub ClassInit
{
# Tk-spezifischer Code
my ($class,$mainwindow)=@_; # Klassenname und Mainwindow
$class->SUPER::ClassInit($mainwindow); # Aufruf des Vorfahren
print "Object $class created.\n"; #debug
return $class;
}

=head2 Constructors:

=head3 Populate

$object = ComObj(%options); 

This "constructor" sets the options of the new object of the class ComObj. 

The binding of events and the creation of the associated Baloonhelp is done here too.

The keys/values in %options: 

(+) '-foreground' =>  The foreground color 

(+) '-background' =>  The background color 

(+) '-relief' =>  The relief style ('flat', 'raised', 'sunken', 'ridged')

(+) '-font' =>  The font definition

(+) '-docform' =>  A reference to the "DocForm" object to which this "ComObj" widget belongs (!Circular references!)

(+) '-command' =>  The name of the command it representates

(+) '-window_tag' =>  The name of the tag that surrounds the embeded "ComObj" in the text widget

=cut

###########################################################################
# Uebergabe, ev. Filterung und Weiterleitung der "-option=>'value'"-Paare #
###########################################################################
sub Populate
{
require Tk::Label;
my ($self,$args)=@_; #Auslesen der $self-Referenz und der option/value-paare
$self->foreground(delete $args->{'-foreground'});
$self->background(delete $args->{'-background'});
$self->relief(delete $args->{'-relief'});
$self->font(delete $args->{'-font'});
$self->docform(delete $args->{'-docform'});
$self->tagman($self->docform()->tagman());
$self->command(delete $args->{'-command'});
$self->window_tag (delete $args->{'-window_tag'});
$self->{'_label_text'}=$self->command();
$self->SUPER::Populate($args); #Weiterleitung an Vorfahr (Basisklasse)
#Label erzeugen+packen
$self->{'_base'}= $self->Frame(-borderwidth=>0, -relief=>'flat')->pack();
$self->{'_label'}= $self->{'_base'}->Label(-textvariable=>\$self->{'_label_text'}, -borderwidth=>2, -relief=>'raised')->pack(-expand=>'1', -fill=>'both');
$self->Advertise('label'=>$self->{'_label'});
$self->ConfigSpecs(DEFAULT=>[$self->{'_label'}]);
$self->Delegates(DEFAULT => $self->{'_label'});
#Einstellungen an Front-Widgets uebergeben
if (defined $self->foreground()) 
	{
	$self->{'_label'}->configure(-foreground=>$self->foreground());
	}
if (defined $self->background()) 
	{
	$self->{'_label'}->configure(-background=>$self->background());
	}
if (defined $self->relief()) 
	{
	$self->{'_label'}->configure(-relief=>$self->relief());
	}
if (defined $self->font()) 
	{
	$self->{'_label'}->configure(-font=>$self->font());
	}
#Initialiserung beginnt #
$self->expanded(0); #Flag
$self->bind('<Destroy>'=>sub{ $self->do_destroy($self->window_tag())}); #Aufraeumroutine binden
$self->{'_label'}->bind('<Button-1>' => sub{ $self->_do_click()});
#Initialiserung endet #
return $self;
}

=head1 $object->on_create(); 

This method has to be called after the ComObj has been correctly created, embeded into the textwidget and tagged with his own tag.
Afterwards the 'window_tag' attribute should be set to this tagname by using the method '$obj->window_tag($tagname)'. Then the
arguments stored in the TagMan-object should also be initialised correctly.

This is VERY IMPORTANT because this 'on_create' method performs the binding of events that
are necessary to edit the arguments assigned to this ComObj.


=cut

######################################################################################
# Call this AFTER the window_tag was set around the embeded ComObj and the arguments #
# for this style are already initialised                                             #
######################################################################################
sub on_create
{
my $self= shift;
my $do_not_sort= shift;
if (defined $self->window_tag())
	{
	#Create the baloon help if there are arguments defined for the ComObj
	if ($self->tagman()->{'args'}->{$self->window_tag()}->{'arg_number'}>0)
		{
		$self->{'_label'}->configure(-cursor=>'based_arrow_down');
		$self->{'_balloon_label'} = $self->docform()->top()->Balloon(-state=>'balloon');
		$self->{'_balloon_label'}->attach($self->{'_label'}, -msg=>'Click here to expand or collapse the object');
		}
	print 'Command '.$self->window_tag()." was created. \n" ; #Debug
	}
else
	{
	print "ERROR: No tag was set around the \"ComObj\" object !\n";
	}
}


=head2 Destructors:

=head3 DESTROY; 

The standard automatically called PERL destructor for objects.

=cut

#############################
# Automatic Perl destructor #
#############################
sub DESTROY
{
my $self = shift;
print "Object $self destroyed.\n"; #debug
}

=head3 do_detstroy

$object->do_detsroy(); 

This "destructor" method is binded to the "<DESTROY>" event and should be performed whenever
the element is destroyed. It should not be called directly.

=cut

###################################################
# Destruktor zum sauberen aufraeumen des Objektes #
###################################################
sub do_destroy
{
my $sender =shift;
my $window_tag = shift;
if (exists $sender->tagman()->{'args'}->{$window_tag})
	{
	delete $sender->tagman()->{'args'}->{$window_tag};
	print "Arguments freed for command $window_tag \n"; #debug
	}
$sender = undef;
}

=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 docform

$ref_doc_form = $object->docform(); #gets a reference to the DocForm object the ComObj belongs to

$object->docform($ref_doc_form); #sets a reference to the DocForm object the ComObj belongs to

=cut

### docform ###
sub docform
{
my $self=shift;
@_ ? $self->{'docform'} = shift : return $self->{'docform'};
}

=head3 tagman

$ref_tagman = $object->tagman(); #gets a reference to the TagMan object of the DocForm object the ComObj belongs to

$object->tagman($ref_tagman); #sets a reference to the Tagman object of the DocForm object the ComObj belongs to

=cut

### tagman ###
sub tagman
{
my $self=shift;
@_ ? $self->{'tagman'} = shift : return $self->{'tagman'};
}

=head3 argedit

$ref_argedit = $object->argedit(); #gets a reference to the ArgEdit object that is created at expansion time

$object->argedit($ref_argedit); #sets a reference to the ArgEdit object that is created at expansion time

=cut

### argedit ###
sub argedit
{
my $self=shift;
@_ ? $self->{'argedit'} = shift : return $self->{'argedit'};
}

=head3 foreground

$color_def = $object->foreground(); #gets the foreground color definition

$object->foreground($color_def); #sets the foreground color definition

=cut

### foreground ###
sub foreground
{
my $self=shift;
@_ ? $self->{'foreground'} = shift : return $self->{'foreground'};
}

=head3 background

$color_def = $object->background(); #gets the background color definition

$object->background($color_def); #sets the background color definition

=cut

### background ###
sub background
{
my $self=shift;
@_ ? $self->{'background'} = shift : return $self->{'background'};
}

=head3 font

$font_def = $object->font(); #gets the font definition

$object->font($font_def); #sets the font definition

=cut

### font ###
sub font
{
my $self=shift;
@_ ? $self->{'font'} = shift : return $self->{'font'};
}

=head3 relief

$relief_def = $object->relief(); #gets the relief definition

$object->relief($relief_def); #sets the relief definition

=cut

### relief ###
sub relief
{
my $self=shift;
@_ ? $self->{'relief'} = shift : return $self->{'relief'};
}

=head3 command

$command_name = $object->command(); #gets the command name of the ComObj

$object->command($command_name); #sets the command name of the ComObj

=cut

### command ###
sub command
{
my $self=shift;
@_ ? $self->{'command'} = shift : return $self->{'command'};
}

=head3 window_tag

$tag_name = $object->window_tag(); #gets the tag name of the ComObj

$object->window_tag($tag_name); #sets the tag name of the ComObj

=cut

### window_tag ###
sub window_tag
{
my $self=shift;
@_ ? $self->{'window_tag'} = shift : return $self->{'window_tag'};
}

=head3 expanded

$extpanded_flag = $object->expanded(); #gets the state of the ComObj

$object->expanded($expanded_flag); #sets the state of the ComObj

=cut

### expanded ###
sub expanded
{
my $self=shift;
@_ ? $self->{'_expanded'} = shift : return $self->{'_expanded'};
}


=head2 Other public methods:

=head3 get_current_line

$line = $object->get_current_line(); 

This returns the line number where the ComObj widget is situated inside of the text widget it belongs to.

=cut

##############################################
# Liefert Zeilennummer im Text-Widget zurueck #
##############################################
sub get_current_line
{
my $self = shift;
my @range = $self->docform()->doctext()->tagRanges($self->window_tag()); # Numlabel ist mit Tag markiert um Zeile eruieren zu koennen
my $anfang = shift(@range); #Anfangsindex auslesen
$anfang =~ s/.\d*$//g; #Zeile herausholen
return $anfang; #Wert zurueckliefern
}

=head2 Private methods:

=head3 _do_click

$object->_do_click(); 

This is the callback function for the "<Button-1>" event (Click).

It causes the ComObj widget to collapse or expand itself.

=cut

###########################
# Callback fuer Mausklick #
###########################
sub _do_click
{
my $sender=shift;
if ($sender->tagman()->{'args'}->{$sender->window_tag()}->{'arg_number'}>0)
	{
	if ($sender->expanded()==1)
		{
		$sender->expanded(0);
		$sender->_collapse();
		$sender->{'_label'}->configure(-cursor=>'based_arrow_down');
		}
	else
		{
		$sender->expanded(1);
		$sender->_expand();
		$sender->{'_label'}->configure(-cursor=>'based_arrow_up');
		}
	}
}

=head3 _expand

$object->_expand(); 

Expands the object. (Creates and packs the ArgEdit widgets.)

=cut

#############################
# Show the Argument-Editor  #
#############################
sub _expand{
my $self = shift;
$self->{'_label_text'}="Arguments of ".$self->command();
$self->{'_base'}->configure(-borderwidth=>4);
$self->{'_base'}->configure(-relief=>'ridge');
$self->{'_label'}->configure(-borderwidth=>1);
$self->{'_label'}->configure(-relief=>'flat');
$self->argedit($self->{'_base'}->ArgEdit(-tagman=>$self->tagman(), -tag_name=>$self->window_tag(), -is_object => 0, -ref_object => undef, -is_doc_style=>0 )->pack(-side=>'bottom', -fill=>'x'));
}

=head3 _collapse

$object->_collapse(); 

Collapses the object. (Destroys the ArgEdit widgets.) ! a nice place for a memory leak -> test!

=cut

########################
# Hide Argument-Editor #
########################
sub _collapse
{
my $self = shift;
$self->{'_label_text'}=$self->command();
$self->{'_base'}->configure(-borderwidth=>0); 
$self->{'_base'}->configure(-relief=>'flat'); 
$self->{'_label'}->configure(-borderwidth=>2); 
$self->{'_label'}->configure(-relief=>'raised'); 
$self->argedit()->destroy();
$self->argedit( undef );
}

1; #So muss ein Package enden


=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
