#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package DocForm;

@ISA = qw(BaseForm);


use strict;

=head1 NAME:

Object: DocForm

=head1 SYNOPSIS:

use class::DocForm;

=head1 DIRECTORY/FILE:

The file DocForm.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

The object DocForm encapsulates the document window. It is now coded as a normal non-visual Perl class
that creates an user interface in the $self->{'ui'} branch. It might be more clever to (later) transform it
into a Toplevel-derived widget.

This form class is derived form the class 'BaseForm' that builds up the basic GUI elements
and does form-registrations and deregistartaions in the global @main::forms array.
For viewing this inherited functionality see the files '/class/BaseForm.pm' and '/class/ui/uiBaseForm.pm'.

Now the GUI buildup functions are stored in the package "./ui/uiDocForm.pm".

=head1 STATUS:

Coding: 80% done

Documentation: up-do-date

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

$self->{'ui'}->{'doctext'} #Reference to the text widget that contains the documents text

$self->{'ui'}->{'outline'} #Reference to the tree widget that contains the documents outline view

$self->{'ui'}->{'top'} #Reference to the toplevel of the document window

$self->{'tagman'} #Reference to the TagMan object this Docform is working with

$self->{'app_window'} #Reference to the main application window

$self->{'backend'} #Reference to the anonymous $backend hash. This is an anonymous configuration hash, that holds the export definition of the current backend. Each
DocForm object has his own one, this allows several DocForm objects to export the same document into different backends.

$self->{'document'}->{'current_doc_style'} # The current document style name(= stylegroup 'DOC_STYLES') of the document.

$self->{'document'}->{'current_par_style'} # The current paragraph style name (= stylegroup 'PAR_STYLES').

$self->{'document'}->{'current_char_style'} # The current character style name (= stylegroup 'CHAR_STYLES').

$self->{'document'}->{'current_env_style'} # The current environment style name (= stylegroup 'ENV_STYLES').

$self->{'document'}->{'current_command'} # The current command name (= stylegroup 'COMMANDS').

$self->{'document'}->{'current_backend_name'} # The current backends name (got fromt the *.out files)

$self->{'document'}->{'search_history'} #A reference to an array that holds the search history of this document

$self->{'document'}->{'replace_history'} #A reference to an array that holds the replace history of this document

$self->{'backend_names'}; #Reference to an array of all names of the available backends (*.out files)

$self->{'no_style_text'} ; #What text should appear if there is no style selected in the Optionmenue (Standard  is "-none-") 

$self->{'doc_styles'}; #Reference to an array of all available document styles (DOC_STYLES)

$self->{'par_styles'}; #Reference to an array of all available paragraph styles (PAR_STYLES)

$self->{'char_styles'}; #Reference to an array of all available character styles (CHAR_STYLES)

$self->{'env_styles'}; #Reference to an array of all available environment styles (ENV_STYLES)

$self->{'commands'};  #Reference to an array of all available commands (COMMANDS)

$self->{'search_dialog'};  #A reference to the search and replace dialog

$self->{'doc_parts'};  #A reference to an hash of arrays. Each hash-key represents the name of a document-part, each array contains the parts content in the clipboard format

$self->{'_tag_doctor_on'};  #The flag if the TAG-DOCTOR features is turned on or off (CAn have the values 'TAG-DOCTOR:ON' or 'TAG-DOCTOR:OFF', just for internal use, use Get/Set-method "&tag_doctot_on()" instead

$self->{'_full_screen_mode'};  #The flag if the FULL SCREEN MODE features is turned on or off (CAn have the values 'TAG-DOCTOR:ON' or 'TAG-DOCTOR:OFF', just for internal use, use Get/Set-method "&full_screen_mode()" instead

=head1 METHODS:

=head2 Class Methods:

=head2 Constructors:

=head3 new

$object = DocForm->new($prj_form); 

This constructor creates a new object of the class DocForm. 

Parameters: 

(+) $prj_form: a reference to the project form that created this document form

=cut

###############
# Konstructor #
###############
sub new
{
my $class = shift;
my $prj_form = shift; #Zeiger auf das Applikationsfenster

#Call the contructor of the base class
my $self = $class->SUPER::new('DOCUMENT_FORM');

my $ref_internal = $main::internal; #Zeiger auf $main::internal-Hash
my $ref_config = $main::config; #Zeiger auf $main::config-Hash

chdir($prj_form->path());
$self->prjform($prj_form);

$self->app_window($prj_form->app_window()); #Here is a reference to the applications mainform stored
my $ref_backend = {}; #build an hash for the content of the current *.out file
$self->{'backend'} = $ref_backend;# Store it in the right attribute

#Initialisierung
my @search_history = ();
my @replace_history = ();
$self->ref_search_history (\@search_history);
$self->ref_replace_history (\@replace_history);
$self->{'doc_parts'} = {};

$self->{'search_dialog'} = undef;
$self->{'no_style_text'} ='-none-'; #Was soll aufscheinen, wenn kein Stil selektiert wurde
$self->{'document'}->{'current_doc_style'} = 'Book'; # Current document style (= stylegroup 'DOC_STYLES')
$self->{'document'}->{'current_par_sytle'} = $self->no_style_text(); # Aktuelles Absatzformat bei Cursorposition
$self->{'document'}->{'current_env_style'} = $self->no_style_text(); # Aktuelles Umgebungsformat bei Cursorposition 
$self->{'document'}->{'current_char_style'} = $self->no_style_text(); # Aktuelles Zeichenformat bei Cursorposition
$self->{'document'}->{'current_command'} = $self->no_style_text(); # Aktuelles Commando an Cursorposition

#Listen fuer Namen der Dokument-, Absatz-, Umgebungs- und Zeichenstile
$self->{'doc_styles'}=[]; #Liste fuer verfuegbare Dokument-Stile
$self->{'par_styles'}=[]; #Liste fuer verfuegbare Absatz-Stile
$self->{'char_styles'}=[]; #Liste fuer verfuegbare Zeichen-Stile
$self->{'env_styles'}=[]; #Liste fuer verfuegbare Umgebungen
$self->{'commands'}=[];  #Liste fuer verfuegbare Kommandos

# The flag for the "tagdoctor" feature
$self->{'_tag_doctor_on'} = 'TAG-DOCTOR:OFF'; #The flag if the TAG-DOCTOR features is turned on or off

# The flag for the "full screen mode" feature
$self->{'_full_screen_mode'} = '0';

#Build up the main parts of the user interface
uiDocForm::build_ui_first_step($self);

#A puffer text widget for export
$self->{'puffer_text'}= $self->top()->Text();

#build up the list of available backends
$self->build_backend_list();

#check if there were any backend files found
if (not (@{$self->backend_names()})) 
	{ die ("ERROR: No \"*.out\" files in the applications directory !\n") };

#Take the first backend in list as the current one 
$self->current_backend_name($self->backend_names()->[0]); 
#Read in the content of the *.out file into the hash $backend
$self->backend_changed($self->current_backend_name());

#Build up the doc_styles()-attribute
$self->build_docstyle_list();

#Aktuelles Basis-Dokumentformat einstellen 
$self->current_doc_style($main::internal->{'GLOBAL'}->{'doc_style_base'});

### Tag-Manager aufbauen ###
$self->tagman(TagMan->new($self)); #This has to be done AFTER the UI is alreday created in &build_ui, because the TagMan objects fetches some references to widgets

#Build up the stylelist for the current doc-style and fill the Optionmenus
$self->doc_style_changed($self->current_doc_style());

# Build up the rest of the UI after the backend and the styles are initialised
uiDocForm::build_ui_second_step($self);


#Initialise the displaying of the combos
$self->{'document'}->{'current_par_sytle'} = $self->no_style_text(); # Aktuelles Absatzformat bei Cursorposition
$self->{'document'}->{'current_env_style'} = $self->no_style_text(); # Aktuelles Umgebungsformat bei Cursorposition 
$self->{'document'}->{'current_char_style'} = $self->no_style_text(); # Aktuelles Zeichenformat bei Cursorposition
$self->{'document'}->{'current_command'} = $self->no_style_text(); # Aktuelles Commando an Cursorposition
$self->_cursor_moved();

# Make the window visible at the very end of the process
uiDocForm::build_ui_third_step($self);

#Update the parts tree
$self->update_parts_all();

#initialise outline 
$self->refresh_outline();

#Store the form-referenec in the global @docforms array
push (@main::docforms, $self);

# Focus ins Textwidget setzten
$self->doctext()->focus();

return $self;
}

=head2 Destructors:

=head3 DESTROY; 

The standard automatically called PERL destructor for objects.
Unfortionatly it is called very late (when the whole application terminates.)
This is due to the circular cross references between DocForm, TagMan, NumList etc.
I will have to unlink them manually in the closing method of the document window.

=cut

#############################
# Automatic Perl destructor #
#############################
sub DESTROY
{
my $self = shift;

}

=head3 do_destroy

$object ->do_destroy(); 

This destructor is binded to the protocol-message WM_DELETE_WINDOW
and performs the 'unsaved changes'-check, destroys the TopLevel manually
and then destroys the DocForm object manually too.

=cut

##############
# Destruktor #
##############
sub do_destroy
{
my $self = shift;

#If the document has unsaved changes
if ($self->is_content_changed())
	{
	my $btn=$self->top()->Dialog(-title=>'Work not saved yet !', -text=>"There are unsaved changes in the current document.\nDo you want to save them to disk ?", -buttons=>['Yes','No'], -bitmap=>'question')->Show();
	if ($btn eq 'Yes') 
		{
		#save it
		$self->file_save($self->filename());
		} 
	}

#Destroy the search&replace dialog manually
if (defined $self->search_dialog())
	{
	$self->search_dialog()->do_destroy();
	$self->search_dialog(undef);
	}

#Remove this DocForm object from the global @main::docforms array
for (my $i = 0; $i <= $#main::docforms; $i++)
	{
	#Get the stored DocForm-reference there
	my $dlg = $main::docforms[$i];
	#If its this DocForm
	if ($dlg == $self)
		{
		#remove it from the array
		splice (@main::docforms,$i,1);
		last;
		}
	}

#CALL THE DESTRUCTOR OF THE SUPERCLASS  AT THE END
$self->SUPER::do_destroy();
}

=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 doctext

$ref_doctext = $object->doctext(); #gets a reference to the text widget that contains the document

$object ->doctext($ref_textwidget); #sets a reference to the text widget that contains the document

=cut

### doctext ###
sub doctext
{
my $self=shift;
@_ ? $self->{'ui'}->{'doctext'} = shift : return $self->{'ui'}->{'doctext'};
}

=head3 prjform

$ref_prj_form = $object->prjform(); #gets a reference to the project form that created this document form

$object ->prjform($ref_prj_form); #sets a reference to the project form that created this document form

=cut

### prjform ###
sub prjform
{
my $self=shift;
@_ ? $self->{'prjform'} = shift : return $self->{'prjform'};
}


=head3 app_window

$ref_app_window = $object->app_window(); #gets a reference to the main application window

$object ->doctext($ref_app_window); #sets a reference to the main application window

=cut

### app_window ###
sub app_window
{
my $self=shift;
@_ ? $self->{'app_window'} = shift : return $self->{'app_window'};
}

=head3 outline

$ref_treewidget = $object->outline(); #gets a reference to the tree widget that contains the documents outline view

$object->outline($ref_treewidget); #sets a reference to the tree widget that contains the documents outline view

=cut

### outline ###
sub outline
{
my $self=shift;
@_ ? $self->{'ui'}->{'outline'} = shift : return $self->{'ui'}->{'outline'};
}


=head3 tagman

$ref_tagman = $object->tagman(); #gets a reference to the TagMan object this Docform is working with

$object ->tagman($ref_tagman): #sets a reference to the TagMan object this Docform is working with

=cut

### tagman ###
sub tagman
{
my $self=shift;
@_ ? $self->{'tagman'} = shift : return $self->{'tagman'};
}

=head3 backend

$ref_backend = $object->backend(); #gets a reference to the anonymous $backend hash

$object ->backend($ref_backend); #sets a reference to the anonymous $backend hash

This is an anonymous configuration hash, that holds the export definition of the current backend. Each
DocForm object has his own one, this allows several DocForm objects to export the same document into different backends.

=cut

### backend ###
sub backend
{
my $self=shift;
@_ ? $self->{'backend'} = shift : return $self->{'backend'};
}

=head3 current_doc_style

$doc_style_name = $object->current_doc_style(); #gets the current document style name (= stylegroup 'DOC_STYLES')

$object->current_doc_style($doc_style_name); #sets the current document style name (= stylegroup 'DOC_STYLES')

=cut

### current_doc_style ###
sub current_doc_style
{
my $self=shift;
@_ ? $self->{'document'}->{'current_doc_style'} = shift : return $self->{'document'}->{'current_doc_style'};
}

=head3 current_par_style

$style_name = $object->current_par_style(); #gets the current paragraph style name (= stylegroup 'PAR_STYLES')

$object->current_par_style($style_name); #sets the paragraph style name (= stylegroup 'PAR_STYLES')

=cut

### current_par_style ###
sub current_par_style
{
my $self=shift;
@_ ? $self->{'document'}->{'current_par_style'} = shift : return $self->{'document'}->{'current_par_style'};
}

=head3 current_env_style

$style_name = $object->current_env_style(); #gets the current environment style name (= stylegroup 'ENV_STYLES')

$object->current_env_style($style_name); #sets the current environment style name (= stylegroup 'ENV_STYLES')

=cut

### current_env_style ###
sub current_env_style
{
my $self=shift;
@_ ? $self->{'document'}->{'current_env_style'} = shift : return $self->{'document'}->{'current_env_style'};
}

=head3 current_char_style

$style_name = $object->current_char_style(); #gets the current character style name (= stylegroup 'CHAR_STYLES')

$object->current_char_style($style_name); #sets the current character style name (= stylegroup 'CHAR_STYLES')

=cut

### current_char_style ###
sub current_char_style
{
my $self=shift;
@_ ? $self->{'document'}->{'current_char_style'} = shift : return $self->{'document'}->{'current_char_style'};
}

=head3 current_command

$command_name = $object->current_command(); #gets the current command name (= stylegroup 'COMMANDS')

$object->current_command($command_name); #sets the current command name (= stylegroup 'COMMANDS')

=cut

### current_command ###
sub current_command
{
my $self=shift;
@_ ? $self->{'document'}->{'current_command'} = shift : return $self->{'document'}->{'current_command'};
}

=head3 current_backend_name

$current_backend_name = $object->current_backend_name(); #gets the current backends name

$object->current_backend_name($current_backend_name); #sets the current backends name

=cut

### current_backend_name ###
sub current_backend_name
{
my $self=shift;
@_ ? $self->{'current_backend_name'} = shift : return $self->{'current_backend_name'};
}

=head3 ref_current_doc_style

$ref_doc_style_name = $object->ref_current_doc_style(); #returns a reference to the current document style name (= stylegroup 'DOC_STYLES') (for use with Tk widgets that need a reference, READONLY)

=cut

### ref_current_doc_style ###
sub ref_current_doc_style
{
my $self=shift;
return \$self->{'document'}->{'current_doc_style'};
}

=head3 ref_current_par_style

$ref_style_name = $object->ref_current_par_style(); #returns a reference to the current paragraph style name (= stylegroup 'PAR_STYLES') (for use with Tk widgets that need a reference, READONLY)

=cut

### ref_current_par_style ###
sub ref_current_par_style
{
my $self=shift;
return \$self->{'document'}->{'current_par_style'};
}

=head3 ref_current_env_style

$ref_style_name = $object->current_env_style(); #returns a reference to the current environment style name (= stylegroup 'ENV_STYLES') (for use with Tk widgets that need a reference, READONLY)

=cut

### ref_current_env_style ###
sub ref_current_env_style
{
my $self=shift;
return \$self->{'document'}->{'current_env_style'};
}

=head3 ref_current_char_style

$ref_style_name = $object->current_char_style(); #returns a reference to the current character style name (= stylegroup 'CHAR_STYLES') (for use with Tk widgets that need a reference, READONLY)

=cut

### ref_current_char_style ###
sub ref_current_char_style
{
my $self=shift;
return \$self->{'document'}->{'current_char_style'};
}

=head3 ref_current_command

$ref_command_name = $object->current_command(); #returns a reference to the current command name (= stylegroup 'COMMANDS') (for use with Tk widgets that need a reference, READONLY)

=cut

### ref_current_command ###
sub ref_current_command
{
my $self=shift;
return \$self->{'document'}->{'current_command'};
}

=head3 ref_current_backend_name

$ref_current_backend_name = $object->ref_current_backend_name(); #returns a reference to the current backends name. (for use with Tk widgets that need a reference, READONLY)

=cut

### ref_current_backend_name ###
sub ref_current_backend_name
{
my $self=shift;
return \$self->{'current_backend_name'};
}


=head3 no_style_text

$text = $object->no_style_text(); #gets a reference to the text that shoul appear in the optionmenu when no style is selected

$object ->no_style_text($text); #sets a reference to the text that shoul appear in the optionmenu when no style is selected

=cut

### no_style_text ###
sub no_style_text
{
my $self=shift;
@_ ? $self->{'no_style_text'} = shift : return $self->{'no_style_text'};
}

=head3 doc_styles

$ref_doc_styles_array = $object->doc_styles(); #gets a reference to the list (array) of all available document styles

$object->doc_styles($ref_doc_styles_array); #sets a reference to the list (array) of all available document styles

=cut

### doc_styles ###
sub doc_styles
{
my $self=shift;
@_ ? $self->{'doc_styles'} = shift : return $self->{'doc_styles'};
}

=head3 par_styles

$ref_par_styles_array = $object->par_styles(); #gets a reference to  the list (array) of all available paragraph styles

$object->par_styles($ref_par_styles_array); #sets a reference to the list (array) of all available paragraph styles

=cut

### par_styles ###
sub par_styles
{
my $self=shift;
@_ ? $self->{'par_styles'} = shift : return $self->{'par_styles'};
}

=head3 env_styles

$ref_env_styles_array = $object->env_styles(); #gets a reference to  the list (array) of all available environment styles

$object->env_styles($ref_env_styles_array); #sets a reference to  the list (array) of all available environment styles

=cut

### env_styles ###
sub env_styles
{
my $self=shift;
@_ ? $self->{'env_styles'} = shift : return $self->{'env_styles'};
}

=head3 char_styles

$ref_char_styles_array = $object->char_styles(); #gets a reference to the list (array) of all available character styles

$object->char_styles($ref_char_styles_array); #sets a reference to the list (array) of all available character styles

=cut

### char_styles ###
sub char_styles
{
my $self=shift;
@_ ? $self->{'char_styles'} = shift : return $self->{'char_styles'};
}

=head3 commands

$ref_commands_array = $object->commands(); #gets a reference to the list (array) of all available commands

$object->commands($ref_commands_array); #sets a reference to the list (array) of all available commands

=cut

### commands ###
sub commands
{
my $self=shift;
@_ ? $self->{'commands'} = shift : return $self->{'commands'};
}

=head3 backend_names

$ref_backend_names_array = $object->backend_names(); #gets a reference to the list (array) of all available document styles

$object->backend_names($ref_backend_names_array); #sets a reference to the list (array) of all available document styles

=cut

### backend_names ###
sub backend_names
{
my $self=shift;
@_ ? $self->{'backend_names'} = shift : return $self->{'backend_names'};
}

=head3 ref_doc_styles

$ref_doc_styles_array = $object->ref_doc_styles(); #gets a reference to the list (array) of all available document styles

=cut

### ref_doc_styles ###
sub ref_doc_styles
{
my $self=shift;
return \$self->{'doc_styles'};
}

=head3 ref_par_styles

$ref_par_styles_array = $object->ref_par_styles(); #gets a reference to the list (array) of all available paragraph styles

=cut

### ref_par_styles ###
sub ref_par_styles
{
my $self=shift;
return \$self->{'par_styles'};
}

=head3 ref_env_styles

$ref_env_styles_array = $object->ref_env_styles(); #gets a reference to the list (array) of all available environment styles

=cut

### ref_env_styles ###
sub ref_env_styles
{
my $self=shift;
return \$self->{'env_styles'};
}

=head3 ref_char_styles

$ref_char_styles_array = $object->ref_char_styles(); #gets a reference to the list (array) of all available character styles

=cut

### ref_char_styles ###
sub ref_char_styles
{
my $self=shift;
return \$self->{'char_styles'};
}

=head3 ref_commands

$ref_commands_array = $object->ref_commands(); #gets a reference to the list (array) of all available commands

=cut

### ref_commands ###
sub ref_commands
{
my $self=shift;
return \$self->{'commands'};
}

=head3 doc_parts_root

$ref_root = $object->doc_parts_root(); #gets a reference to the root of the hash of arrays that holds the parts

=cut

### doc_parts_root ###
sub doc_parts_root
{
my $self=shift;
return $self->{'doc_parts'};
}

=head3 doc_parts_names

@doc_parts_names = $object->doc_parts_names(); #gets an array containing the names of all parts in this document (READONLY)

=cut

### doc_parts_names ###
sub doc_parts_names
{
my $self=shift;
return keys %{$self->{'doc_parts'}};
}

=head3 ref_doc_part

$ref_content = $object->ref_doc_part( $name ); #gets a reference to an array containing the part in clipboard-format

$object->ref_doc_part( $name, $ref_content ); #stores a reference to an array containing the part in clipboard-format

Returns undef, if the given part doesn't exist.

Parameters:

(+) $name: The name of the part (has to be an unique string)

(+) $ref_content: A reference to an array holding the content of a part in clipboard format

=cut

### ref_doc_part ###
sub ref_doc_part
{
my $self=shift;
my $name = shift;
my $ref_content = shift;

#Default error-return value
my $return_value = undef;

#If a reference to a part-content was given (SET a value)
if (defined $ref_content and defined $name)
	{
	#Store it
	$self->{'doc_parts'}->{$name} = $ref_content;
	}
else #No reference (GET a value)
	{
	#If a part with this name exists
	if (exists $self->{'doc_parts'}->{$name})
		{
		#Return the reference to the content
		$return_value = $self->{'doc_parts'}->{$name};
		}
	}

return $return_value;
}

=head3 ref_backend_names

$ref_backend_names_array = $object->ref_backend_names(); #gets a reference to the list (array) of all available document styles

=cut

### ref_backend_names ###
sub ref_backend_names
{
my $self=shift;
return \$self->{'backend_names'};
}

=head3 tag_doctor_on()

$flag = $object->tag_doctor_on(); #Gets the State of the Tagdoctor

$object->tag_doctor_on($flag); #Sets the state of the Tagdoctor

Checks if the TagDoctor feature is turned on.

Returns 0 if no and 1 if yes.

=cut

#################
# tag_doctor_on #
#################
sub tag_doctor_on
{
my $self = shift;
my $set= shift;
if (not defined $set)
	{
	if ($self->{'_tag_doctor_on'} eq 'TAG-DOCTOR:ON')
		{
		return 1;
		}
	else
		{
		return 0;
		}
	}
else
	{
	if ($set == 1)
		{
		$self->{'_tag_doctor_on'} = 'TAG-DOCTOR:ON';
		}
	else
		{
		$self->{'_tag_doctor_on'} = 'TAG-DOCTOR:OFF'
		}
	}
}

=head3 full_screen_mode()

$flag = $object->full_screen_mode(); #Gets the State of the full screen mode

$object->full_screen_mode($flag); #Sets the state of the full screen mode

Checks if the Full_SCREEN_MODE feature is turned on.

Returns 0 if no and 1 if yes.

=cut

### full_screen_mode ###

sub full_screen_mode
{
my $self = shift;
my $value= shift;

if (not defined $value)
	{
	return $self->{'_full_screen_mode'};
	}
elsif ($value == 1)
	{
	$self->_show_full_screen_mode;
	$self->{'_full_screen_mode'} = 1;
	}
else
	{
	$self->_hide_full_screen_mode;
	$self->{'_full_screen_mode'} = 0;
	}
}

=head3 search_dialog

$ref_dialog = $object->search_dialog(); #gets an reference to the search dialog if it is already created

$object->search_dialog($ref_dialog); #sets an reference to the search dialog 

=cut

### search_dialog ###
sub search_dialog
{
my $self=shift;
@_ ? $self->{'search_dialog'} = shift : return $self->{'search_dialog'};
}

=head3 ref_search_history

$ref_array = $object->ref_search_history(); #gets a reference to an array that holds the search history 

$object->ref_search_history($ref_array); #sets a reference to an array that holds the search history 

=cut

### ref_search_history ###
sub ref_search_history
{
my $self=shift;
@_ ? $self->{'document'}->{'search_history'} = shift : return $self->{'document'}->{'search_history'};
}

=head3 ref_replace_history

$ref_array = $object->ref_replace_history(); #gets a reference to an array that holds the replace history 

$object->ref_replace_history($ref_array); #sets a reference to an array that holds the replace history 

=cut

### ref_replace_history ###
sub ref_replace_history
{
my $self=shift;
@_ ? $self->{'document'}->{'replace_history'} = shift : return $self->{'document'}->{'replace_history'};
}


=head2 Other public methods:

=head3 document_exit

$object->document_exit();

Exit the DocForm. 

=cut

############
### EXIT ###
############
sub document_exit
{
my $self=shift;
$self->do_destroy();
}

=head3 build_docstyle_list

$object->build_docstyle_list(); 

Builds an sorted array of all (visible = non-abstract) document styles (DOC_STYLES) and writes it into the "doc_styles()" attribute.

=cut

####################################
### Dokumentstilliste erstellen  ###
####################################
sub build_docstyle_list
{
my $self = shift;
my $internal = $main::internal;
my @doc_styles;
#Build the list of document styles
foreach my $key (keys %{$internal->{'DOC_STYLES'}}) 
	{
	if (($key =~/^_/)==0) #if it's not an abstract style
		{
		push(@doc_styles, $key); #Add the docstale to the list
		}
	}
#Sort it
@doc_styles = sort(@doc_styles);
#Write it into the attribute (as a reference)
$self->doc_styles(\@doc_styles);
}

=head3 build_backend_list

$object->build_backend_list(); 

Builds an sorted array of all available backends. This means all *.out files that exist.

=cut

###################################################
### Build a list (array) od available backends  ###
###################################################
sub build_backend_list
{
my $self = shift;
my $backend = $self->backend();
my @backend_file_list = InOut::build_filelist_by_extension($main::edoc_path."/config/",'out'); #get all *.out files in the '/config/' directory into an array of filenames
my @backend_name_list; #Array for all names of the backends
foreach my $item (@backend_file_list) #Iterate over the filename list
	{
	$item =~ s/\.out$// ; #remove the extension
	push (@backend_name_list, $item); #add it to the name-list
	}
#Sort the list alphabetically
@backend_name_list = sort(@backend_name_list);
#Write it into the attribute (as a reference)
$self->backend_names( \@backend_name_list );
}


=head3 file_new

$object->file_new(); 

Resets the DocForm object for starting a new document. This function destroys the whole Text widget and creates a new one,
which is significantly faster then deleting the old one. It does the same with the corresponding "TagMan" object.

=cut

#################
### Datei neu ###
#################
sub file_new
{
my $self = shift;
#Abfrage fuer den ersten start
if (($self->filename() eq '* scratch *')&& ($self->is_content_changed()==0)) {goto EXIT};
#Fortschrittsbalken initialisieren
$self->_progress_init('Preparing for a new document');
#Bei geaendertem Text ums Speichern nachfragen
if ($self->is_content_changed())
	{
	my $btn=$self->top()->Dialog(-title=>'Work not saved yet !', -text=>"There are unsaved changes in the current document.\nDo you want to save them to disk ?", -buttons=>['Yes','No'], -bitmap=>'question')->Show();
	if ($btn eq 'Yes') {$self->file_save($self->filename())};
	}
#Fortschrittsbalken auf 40%
$self->_progress_update('Deleting text info',40);
#Textinfo lschen
$self->doctext()->destroy; #The fastest way to delete the content is to delete the whole widget
uiDocForm::build_doctext($self,1,1); #And to rebuild & pack a new one afterwards 
#Fortschrittsbalken auf 80%
$self->_progress_update('Deleting tags',80);
##And don't forget to rebuild the Tagman-Object too
$self->tagman(undef);
$self->tagman(TagMan->new($self));
$self->tagman()->refresh_doc_args();

#Fortschrittsbalken auf 90%
$self->_progress_update('Deleting outline',90);
$self->refresh_outline($self); #Outline neu aufbauen (dabei wird auch durchnummeriert)

$self->_progress_update('Updating parts',95);
#Update the parts tree
$self->{'doc_parts'} = {};
$self->update_parts_all();

#Clear histories
my @search_history = ();
$self->ref_search_history ( \@search_history );
my @replace_history = ();
$self->ref_replace_history ( \@replace_history );

#Fortschrittsbalken auf 99%
$self->_progress_update('Ready.',99);
#Kein Dateinamen vergeben
$self->filename('* scratch *');
$self->set_content_unchanged();

#Update the window title
$self->update_window_caption();

#Fortschrittsbalken ausschalten
EXIT:
$self->_progress_finish;
}

=head3 file_open_ascii

$object->file_open_ascii([$filename]); 

Calls InOut::file_open_ascii($self, [$filename]), which opens a normal unformatted text file in the DocForm object.

Parameters:

(+) $filename: OPTIONAL, if this is set to undef or not set, the filename is queried by a FileOpen dialog

=cut

###########################
### Datei oeffnen ASCII ###
###########################
sub file_open_ascii 
{
my $self = shift;
my $filename = shift; #undef if the Filename should be queried by the FileOpen dialog
#Methode im Modul InOut aufrufen
InOut::file_open_ascii($self,$filename);
}

=head3 file_open

$object->file_open([$filename]); 

Calls InOut::file_open($self, [$filename]), which opens an e:doc document in the DocForm object.

Parameters:

(+) $filename: OPTIONAL, if this is set to undef or not set, the filename is queried by a FileOpen dialog

=cut

#####################
### Datei oeffnen ###
#####################
sub file_open 
{
my $self = shift;
my $filename = shift; #undef if the Filename should be queried by the FileOpen dialog
#Call method in package InOut
InOut::file_open($self,$filename);
}

=head3 file_save_as

$object->file_save_as([$filename]); 

Calls InOut::file_save_as($self [, $filename]), which queries for a filename (if not passed here) and saves an e:doc document.

Parameters:

(+) $filename: The filename to be saved (OPTIONAL)

=cut

#############################
### Datei speichern unter ###
#############################
sub file_save_as 
{
my $self = shift;
my $filename = shift;

#Methode im Modul InOut aufrufen
InOut::file_save_as($self, $filename);
}

=head3 file_save_template

$object->file_save_as_template([$filename]); 

Save the actual document into a new template stored in the template-directory
defined in 'edoc.cfg'.

Queries for a filename (if not passed here) and saves an e:doc document by
calling InOut::file_save($self, $filename);

Parameters:

(+) $filename: The filename to be saved (OPTIONAL)

=cut

##################################
### Save document as atemplate ###
##################################
sub file_save_as_template
{
my $self = shift;
my $filename = shift;
$self->top()->Busy();

#Query filename if not given
if (not defined $filename)
	{
	if (-d $main::config->{'GENERAL'}->{'templates_dir'})
		{
		my $dir = $main::config->{'GENERAL'}->{'templates_dir'};
		$dir =~ s/\//\\/g if $main::os_type eq 'win32'; #Convert / into \ on Win32 because of inconsistant behavior of getSaveFile's 'initialdir' option
		$filename = $self->top()->getSaveFile(-filetypes=>[['e:doc document templates', ['.edc', '.edoc']]], -initialfile => 'newtemplate', -defaultextension => '.edc', -initialdir=>$dir);
		$filename =~ s/\\/\//g if $main::os_type eq 'win32'; #Reconvert the slashes for further use on Win32
		}
	else
		{
		$self->message("Templates directory defined in 'edoc.cfg' does not exist.",'ERROR');
		goto EXIT;
		}
	}

if (($filename eq '')||((defined $filename)==0)||($filename eq '* scratch *')) 
	{
	goto EXIT #Sorry for goto 
	}; 

#Save the thing
$self->file_save($filename);

EXIT:
$self->top()->Unbusy();
return $filename;
}


=head3 file_save

$object->file_save([$filename]); 

Calls InOut::file_save($self, [$filename]), which saves an e:doc document .

Parameters:

(+) $filename: OPTIONAL, if this is set to undef or not set, the filename is queried by a FileSave dialog

=cut

#######################
### Datei speichern ###
#######################
sub file_save
{
my $self = shift;
my $filename = shift;
if (not defined $filename)
	{
   $filename = $self->filename();
	}
if (not defined $filename or $filename eq '* scratch *') #If its still not defined or '* scratch *'
	{
	#Neuen Dateinamen erfragen
	$self->file_save_as();
	}
else
	{
	#Methode im Modul InOut aufrufen
	InOut::file_save($self, $filename);
	}
}


=head3 export_to_backend

$object->export_to_backend( [$edc_filename] ); 

Calls InOut::export_to_backend($self, $edc_filename), which converts the e:doc document (edc file) into the backend format

Parameters:

(+) $edc_filename: The name of the e:doc documents file (if not given (undef), the current filename of the document is taken)

=cut

######################
### Backend Export ###
######################
sub export_to_backend
{
my $self = shift;
my $edc_filename = shift; #Name der Edoc-Datei
if ((defined $edc_filename)==0) {$edc_filename = $self->filename()};
#Methode im Modul InOut aufrufen
if (defined $self->path() and not($self->path() eq '') and -d $self->path())
	{
	chdir $self->path();
	}
my $export_filename = InOut::export_to_backend($self, $edc_filename, $self->backend()->{'GLOBAL'}->{'extension'});
# Verwendeten Latex-Dateinamen zurueckliefern
return $export_filename;
}

=head3 par_style_changed

$object->par_style_changed($style, $index1, $index2, $renumber, $do_not_sort, $allow_add_tagdoc, $do_check_line); 

This method is called whenever a part of the document has to be formatted as a different paragraph style (PAR_STYLE).
NOTE: a paragraph style can only be applied to one whole line, so if only a fraction of a line is selected, the formatting
still covers the whole line. There could only be one paragraph style in one line, so an existing paragraph style is
automatically removed.

This method is called as an callback from the OptionMenu widget that sets the paragraph styles and from the
InOut.pm package for opening a document from the disk.

This method calls the corresponding "TagMan" objects "add_par_tag" method.

The method returns the name of the newly created paragraph style.

Parameters:

(+) $style: The name of the style to be applied to the selected line.

(+) $index1, $index2: The index in the Text widget of the line that should be formatted. If the current
    cursor selection should be used, the values of these two parameters should be -1.

(+) $renumber: If this flag is set, the whole NumList this style belongs to gets renumbered. This is
    necessary if the paragraph style is set by user interaction to keep the paragraph numbering right, but if you open a whole
    document it is enough to renumber after all paragraph styles are set (TIME).

(+) $do_not_sort: To allow a correct paragraph numbering the automatically created NumLabels are kept in a sorted list
    in the NumList object. This sorting takes time and again its not necessary to do this after each paragraph style when
    you open a document from disk. Also here it is enough to sort after all paragraphs are read in.

(+) $allow_add_tagdoc: If set to 0 or undef, do not add TagDoc-objects (disturbing at loading/paste-time)

(+) $do_check_line: If set to 0 or undef, do not check the line for previous PAR_STYLES tags (File_open and Paste).

=cut

##########################
### Absatzstil aendern ###
##########################
sub par_style_changed
{
my ($self, $style, $index1, $index2, $renumber, $do_not_sort, $allow_add_tagdoc, $do_check_line) = @_; 
#index1,index2 are -1 if they should not be used
# $renumber = 1 if it should be automatically renumbered, else 0 (loading-time, pasting-time)
# $do_not_sort = 0 if the NumLabel eventually created should be sorted in the NumList (default) and 1 if this should not be done (Loading-time)

$do_check_line = 0 if not defined $do_check_line;

my $tagname; #zum Zurueckliefern
#nderungsflag setzen
$self->set_content_changed();
if ((exists $main::internal->{'PAR_STYLES'}->{$style})||($style eq $self->no_style_text()))
	{
	# Anfangs- und Endindizes errechnen
	my $anf;
	my $ende;
	my $cur_pos = $self->doctext()->index("insert");
	if ($index1== -1) {$anf = $cur_pos} else {$anf=$index1} ;
	if ($index2== -1) {$ende = $cur_pos} else {$ende=$index2} ;
	$ende =~ s/.\d*$/.end/o ;
	$anf =~ s/.\d*$/.0/o ;

	#Falls es existiert ein bereits bestehendes <PAR_STYLES>-Tag loeschen
	$self->tagman()->delete_par_tags_in_area($anf,$ende) if $do_check_line == 1;

	# Wenn es nicht das Basis-Absatzformat ist (welches ja kein Tag bekommt)
	if (not ($style eq $self->no_style_text())) #Wenn Stil nicht '-none-' ist wird ein neues <PAR_STYLES>-Tag erzeugt
		{
		#Neuse Absatzformat mittels Tagmanager erzeugen
		$tagname = $self->tagman()->add_par_tag($style, $anf, $ende, $do_not_sort, $allow_add_tagdoc);
		}
	}
else 
	{
	print "PAR_STYLE: $style is missing in \"edoc.int\".\n"
	}

#Wenn neu durchnummeriert werden soll 
if ($renumber == 1) 
	{
	my $numlist=$self->tagman()->get_num_list('PAR_STYLES', $style);
	if (defined $numlist)
		{
		$numlist->renumber();
		}
	}

return $tagname;
}

=head3 env_style_changed

$object->env_style_changed($style, $index1, $index2, $allow_add_tagdoc, $do_check_area); 

This method is called whenever a part of the document has to be formatted as a different environment style (ENV_STYLE).
NOTE: a environment style can only be applied to whole lines, so if only a fraction of a line is selected, the formatting
still covers the whole lines. 

This method is called as an callback from the OptionMenu widget that sets the environment styles and from the
InOut.pm package for opening a document from the disk.

This method calls the corresponding "TagMan" objects "add_env_tag" method.

The method returns the name of the newly created environment style.

Parameters:

(+) $style: The name of the style to be applied to the selected line(s).

(+) $index1, $index2: The index in the Text widget of the line that should be formatted. If the current
    cursor selection should be used, the values of these two parameters should be -1.

(+) $allow_add_tagdoc: If set to 0 or undef, do not add TagDoc-objects (disturbing at loading/paste-time)

(+) $do_check_area: If set to 0  then do not check area for previous ENV_STYLES tags (File_open and Paste)

=cut

#############################
### Umgebungsstil aendern ###
#############################
sub env_style_changed
{
my ($self, $style, $index1, $index2, $allow_add_tagdoc, $do_check_area) = @_; 
#index1,index2 sind "-1" wenn sie nicht uebergeben werden (beim Oeffnen) sondern aus der SELECTION errechnet werden sollen, 

$do_check_area = 0 if not defined $do_check_area;

my $tagname; #zum Zurueckliefern
#nderungsflag setzen
$self->set_content_changed();

if ((exists $main::internal->{"ENV_STYLES"}->{$style})||($style eq $self->no_style_text()))
	{
	# Anfangs- und Endindizes errechnen
	my $anf;
	my $ende;
	if (not $index1 == -1 and not $index2 == -1)
		{
		$anf = $index1;
		$ende = $index2;
		$ende =~ s/.\d*$/.end/g ;
		$anf =~ s/.\d*$/.0/g ;
		}
	else
		{
		print "2 \n"; #debug1
		my $sel=$self->get_selection();
		if (not $sel == 0) #If there is a selection
			{
			$anf = $sel->{'start_index'};
			$ende = $sel->{'end_index'};
			$anf =~ s/.\d*$/.0/g ;
			$ende =~ s/.\d*$/.end/g ;
			}
	 	else
	 		{
			print "4 \n"; #debug1
			my $cur_pos = $self->doctext()->index("insert");
			if ($cur_pos =~ /(\d+).(\d+)/)
				{
				my $line = $1;
				$anf = $line.".0";
				$ende = $line .".end";
				};
			}
		}
	if ($style eq $self->no_style_text()) #Wenn Stil  '-none-' ist werden alle ENV_STYLES geloesch
		{
		#Bestehende ENV_STYLES loeschen
		$tagname = $self->tagman()->delete_env_tags_in_area($anf,$ende) if $do_check_area == 1;
		}
	else #Wenn Stil nicht '-none-' ist wird ein neues <ENV_STYLES>-Tag erzeugt
		{
		#Neues Absatzformat mittels Tagmanager erzeugen
		$tagname = $self->tagman()->add_env_tag($style,$anf,$ende, $allow_add_tagdoc);
		}
	}
else 
	{
	print "Umgebungsstil: $style existiert nicht in \"edoc.int\".\n"
	}

return $tagname;
}

=head3 char_style_changed

$object->char_style_changed($style, $index1, $index2, $allow_add_tagdoc); 

This method is called whenever a part of the document has to be formatted as a different character style (CHAR_STYLE).

This method is called as an callback from the OptionMenu widget that sets the character styles and from the
InOut.pm package for opening a document from the disk.

This method calls the corresponding "TagMan" objects "add_char_tag" method.

The method returns the name of the newly created character style.

Parameters:

(+) $style: The name of the style to be applied to the selected text.

(+) $index1, $index2: The index in the Text widget of the line that should be formatted. If the current
    cursor selection should be used, the values of these two parameters should be -1.

(+) $allow_add_tagdoc: If set to 0 or undef, do not add TagDoc-objects (disturbing at loading/paste-time)

=cut

###########################
### Zeichenstil aendern ###
###########################
sub char_style_changed
{
my ($self, $style, $index1, $index2, $allow_add_tagdoc) = @_; 
#index1,index2 are -1 if they are not known like at Loading/Pasting-time, they are calculated from the selection then.

my $tagname; #zum Zurueckliefern
#nderungsflag setzen
$self->set_content_changed(); # Set the flag to 'changed'

if ((exists $main::internal->{'CHAR_STYLES'}->{$style})||($style eq $self->no_style_text()))
	{
	# Anfangs- und Endindizes errechnen
	my $anf;
	my $ende;
	my $sel_exists;
	if (not $index1 == -1 and not $index2 == -1)
		{
		$anf = $index1;
		$ende = $index2;
		}
	else
		{
		my $sel = $self->get_selection();
		if (not $sel == 0) #If there is a selection
			{
			$anf=$sel->{'start_index'};
			$ende=$sel->{'end_index'};
			}
	 	else
	 		{
			# If nothing is selected
			goto EXIT;
			}
		}

	if ($style eq $self->no_style_text()) #Wenn Stil  '-none-' ist werden alle CHAR_STYLES geloesch
		{
		#Bestehende CHAR_STYLES loeschen
		$self->tagman()->delete_char_tags_in_area($anf, $ende);
		}
	else #Wenn Stil nicht '-none-' ist wird ein neues <CHAR_STYLES>-Tag erzeugt
		{
		#Neues Absatzformat mittels Tagmanager erzeugen
		$tagname = $self->tagman()->add_char_tag($style, $anf, $ende, $allow_add_tagdoc);
		}
	}
else 
	{
	print "CHAR_STYLES: $style does not exist in \"edoc.int\".\n"
	}

return $tagname;
EXIT:
}

=head3 commands_changed

$object->commands_changed($command, $index1); 

This method is called whenever a new command should be inserted into the document (COMMANDS).
NOTE: a command is represented by the embeding of a "ComObj" object which uses always the index range of one character.

This method is called as an callback from the OptionMenu widget that inserts the commands and from the
InOut.pm package for opening a document from the disk.

This method calls the corresponding "TagMan" objects "add_commad_tag" method.

The method returns the name of the newly created command tag.

Parameters:

(+) $command: The name of the command to be inserted

(+) $index1: The index in the Text widget where the command should be inserted. If the current
    cursor position should be used, the values of the parameter should be -1.

=cut

###############################
### Kommandocombo geaendert ###
###############################
sub commands_changed
{
my ($self, $command, $index) = @_; 
#index ist "-1" wenn sie nicht verwendet werden sollen, 

my $tagname; #zum Zurueckliefern
#nderungsflag setzen
$self->set_content_changed(); # Set the flag to 'changed'

#Kommando-Einfuegecode kommt hier
if ((exists $main::internal->{'COMMANDS'}->{$command})||($command eq $self->no_style_text()))
	{
	# Anfangs- und Endindizes errechnen
	my $position;
	my $cur_pos = $self->doctext()->index("insert");
	if ($index== -1) {$position = $cur_pos} else {$position=$index} ;

		if (($command eq $self->no_style_text())==0) #Wenn $command nicht '-none-' ist wird ein neues <COMMANDS>-Tag erzeugt
		{
		#Neues Command mittels Tagmanager erzeugen
		$tagname = $self->tagman()->add_command_tag($command,$position);
		$self->current_command($self->no_style_text());#Combo zuruecksetzen
		}
	}
else 
	{
	print "Kommando: $command existiert nicht in \"edoc.int\".\n"
	}
	return $tagname;

EXIT:
}

=head3 doc_style_changed

$object->doc_style_changed($style); 

This method is called whenever the currents document DOC_STYLE gets changed.
This changes the way the numbering of paragraphs, images, tables and other objects is done in the document
and also changes the number available styles. Several paragraph, environment, character styles and commands
are only available in some document styles.

This method is called as an callback from the OptionMenu widget that sets the document style and from the
InOut.pm package for opening a document from the disk.


Parameters:

(+) $style: The name of the document style to be applied to the document.

=cut

##############################
### Dokumentformat aendern ###
##############################
sub doc_style_changed
{
my $self = shift;
my $style = shift;
my $ref_internal = $main::internal;

#nderungsflag setzen
$self->set_content_changed(); # Set the flag to 'changed'

#Textfenster auffrischen
if (defined ($self->doctext())) {$self->doctext()->update()};

#current_doc_style setzen
$self->current_doc_style($style); 

#Absatzformat-Liste auf verfuegbare Absatzformate einschraenken
my @a_par_array = Config::build_style_list_from_docstyle($style,'PAR_STYLES',$self->no_style_text());
$self->par_styles(\@a_par_array);
if (defined ($self->{'ui'}->{'par_style_combo'})) 
	{
	$self->{'ui'}->{'par_style_combo'}->configure(-choices => $self->par_styles());
	}

#Umgebungsformat-Liste auf verfuegbare Umgebungsformate einschraenken
my @an_env_array = Config::build_style_list_from_docstyle($style,'ENV_STYLES',$self->no_style_text());
$self->env_styles(\@an_env_array);
if (defined ($self->{'ui'}->{'env_style_combo'})) 
	{
	$self->{'ui'}->{'env_style_combo'}->configure(-choices => $self->env_styles());
	}

#Zeichenformat-Liste auf verfuegbare Zeichenformate einschraenken
my @a_char_array = Config::build_style_list_from_docstyle($style,'CHAR_STYLES',$self->no_style_text());
$self->char_styles(\@a_char_array);
if (defined ($self->{'ui'}->{'char_style_combo'})) 
	{
	$self->{'ui'}->{'char_style_combo'}->configure(-choices => $self->char_styles());
	}

#Commands-Liste auf verfuegbare Commands einschraenken
my @a_commands_array = Config::build_style_list_from_docstyle($style,'COMMANDS',$self->no_style_text());
$self->commands(\@a_commands_array);
if (defined ($self->{'ui'}->{'commands_combo'})) 
	{
	$self->{'ui'}->{'commands_combo'}->configure(-choices => $self->commands());
	}

#Reinitialise the NumLists in the TagMan object
#Befor the window itself istnt created, the TagMan objects does not exist, so we have to check for it
if (defined $self->tagman())
	{
	$self->tagman()->init_num_lists();
	$self->tagman()->refresh_doc_args();
	}
}

=head3 backend_changed

$object->backend_changed($backend_name); 

This method is called whenever the currents backend gets changed.

It sets the current $self->out()-hash to undef and rebuilds it afterwards.


Parameters:

(+) $backend_name: The name of the new backend

=cut

####################################
### Another backend was selected ###
####################################
sub backend_changed
{
my $self = shift;
my $backend_name = shift;

#Set the curren_backend_name-value to the given backend_name, if this method is called from external (f. ex. Macro)
$self->current_backend_name($backend_name);

print "Changed to backend: $backend_name\n"; #debug
my $ref_backend = $self->backend();

#Delete backend info in backend-hash
my $new_hash = {};
$self->backend({});

#and put new info there from the (now corresponding) *.out-file
Config::load_backend($self, $backend_name);

#If the GUI is already built up (not during execution of the first part of the constructor)
#then build (rebuild) the actual 'Backend'-menu
if ((exists $self->{'ui'}->{'menu_left'})&&(defined $self->{'ui'}->{'menu_left'}))
	{
	$self->_build_backend_menu();
	}

#Notify all embeded objects about the changed backend
if (defined $self->tagman() )
	{
	foreach my $object_name (keys %{$self->tagman()->{'objects'}})
		{
		my $ref_object = $self->tagman()->{'objects'}->{$object_name};
		#Get the classname
		my $classname = $ref_object->classname();
		#Get the backend definition
		my $ref_backend = $self->backend()->{'EMBED_OBJECTS'}->{$classname};
		#Pass the output definition to the object
		$ref_object->set_backend($ref_backend);
		}
	}

}


=head3 get_number_of_lines

$object->get_number_of_lines(); 

This method returns the number of lines that exist in the current document.

=cut

###############################
### Zeilenanzahl ermitteln  ###
###############################
sub get_number_of_lines
{
my $self = shift;
my $lastline = $self->doctext()->index("end"); 
$lastline =~ s/.\d*$//g;
return $lastline;
}

=head3 refresh_outline

$object->refresh_outline(); 

This method refreshes the outline view in the tree widget. It uses the information of the NumList objects
that are used for numbering the paragraph styles and the images and tables.

=cut

############################
### Outline neu aufbauen ###
############################
sub refresh_outline
{
my $self = shift;
$self->top()->Busy();
my $outline = $self->outline();
my $tagman = $self->tagman();
my $ui_base = $self->{'ui'};
my $textwidget = $self->doctext();

$outline->deleteAll(); #Alle Eintraege in Outline loeschen

$tagman->read_from_widget($self->doctext()); #Tagliste in TagMan-Objekt auffrischen

#CREATE THE TREE'S ROOT ELEMENT
$outline->add('ROOT',-itemtype=>'imagetext', -text=>'THE DOCUMENT', -image=>$ui_base->{'document_icon'});
my %hash; #This hash is used for seperatly number the NumList-branches

#ORDERING THE NUMLISTS FOR THE OUTLINE-DISPLAY
my @ordered_names; #An ordered list of NumList-names, orderd by the 'outline_order' property in the file 'edoc.int'
my %ordered_num_lists; #for ordering them
my $max_number=0;  #highest used order-index
foreach my $num_list_name (keys %{$self->tagman()->{'doc_numlists'}})
	{
	#Renumber the NumList
	$self->tagman()->{'doc_numlists'}->{$num_list_name}->renumber();
	#If the Numlist is in the outline
	if ((exists $main::internal->{'DOC_NUMLISTS'}->{$num_list_name}->{'in_outline'}) and 
		(defined $main::internal->{'DOC_NUMLISTS'}->{$num_list_name}->{'in_outline'}) and
		($main::internal->{'DOC_NUMLISTS'}->{$num_list_name}->{'in_outline'} == 1))
		{
		#If the 'outline_order' property is set
		if ((exists $main::internal->{'DOC_NUMLISTS'}->{$num_list_name}->{'outline_order'}) and 
			(defined $main::internal->{'DOC_NUMLISTS'}->{$num_list_name}->{'outline_order'}))
			{
			my $order_index = $main::internal->{'DOC_NUMLISTS'}->{$num_list_name}->{'outline_order'};
			$ordered_num_lists{$order_index}=$num_list_name; 
			if ($order_index > $max_number)
				{
				$max_number = $order_index;
				}
			}
		}
	}

for (my $i = 0 ; $i<=$max_number; $i++)
	{
	if (defined $ordered_num_lists{$i})
		{
		push (@ordered_names, $ordered_num_lists{$i});
		}
	}


# RENUMBERING THE NUMLISTS AND CREATE BRANCHES IN THE OUTLINE
my @used_names; #Storing the names of the used NUMLISTS there
foreach my $num_list_name (@ordered_names)
	{
	if ((exists $main::internal->{'DOC_NUMLISTS'}->{$num_list_name}->{'in_outline'}) and 
		(defined $main::internal->{'DOC_NUMLISTS'}->{$num_list_name}->{'in_outline'}) and
		($main::internal->{'DOC_NUMLISTS'}->{$num_list_name}->{'in_outline'} == 1))
		{
		my $text = $num_list_name; #the text for display in the numlist
		if (defined $main::internal->{'DOC_NUMLISTS'}->{$num_list_name}->{'outline_text'})
			{
			$text = $main::internal->{'DOC_NUMLISTS'}->{$num_list_name}->{'outline_text'};
			}
		$outline->add("ROOT~".$num_list_name, -itemtype=>'imagetext', -text=>$text, -image=>$ui_base->{'outline_icons'}->{$num_list_name});
		$hash{"ROOT~".$num_list_name} = $self->tagman()->{'doc_numlists'}->{$num_list_name};
		push (@used_names, $num_list_name);
		}
	}

foreach my $num_list_name (@used_names)
	{
	my $key = "ROOT~".$num_list_name;
	my $value = $hash{$key};
	my %last_items;
	my $root_path = $key;
	my $num_list = $value;
	my @list = @{$num_list->list()}; #NumLabel Objekte holen
	foreach my $item (@list)
		{
		my $ass_tag = $item->associated_tag();
		my $stylegroup = '';
		my $style = '';
		#Extract stylegroup ans style from the tag
		if ($ass_tag =~ /^<(.+?)><(.+?)><(.*?)>:(\d+?)$/o)
			{
			$stylegroup = $1;
			$style = $2;
			}
		#only PAR_STYLES and EMBOBJ can appear in the outline
		if (($stylegroup eq 'PAR_STYLES') or ($stylegroup eq 'EMBOBJ'))
			{
			my $index1 = $tagman->{'tags'}->{$ass_tag}->{0}->{'start_index'}; #Get the start of the first (and only) range of the tag
			my $index2 = $tagman->{'tags'}->{$ass_tag}->{0}->{'end_index'}; #Get the end of the first (and only) range of the tag
			my $text ='';
			if ($stylegroup eq 'PAR_STYLES')
				{
				$text = $item->text().' '.$textwidget->get($index1,$index2);
				}
			if ($stylegroup eq 'EMBOBJ')
				{
				$text = $self->tagman()->{'objects'}->{$ass_tag}->numlabel()->text().' '.$self->tagman()->{'objects'}->{$ass_tag}->caption();
				}
			my $level = $item->level();
			my $parent_path = $last_items{$level-1};
			if ($level>0)
				{
				if (((defined $parent_path)==0)||($level==1))
					{
					$parent_path = $root_path;
					}
				$outline->add($parent_path.'~'.$ass_tag, -itemtype=>'text', -text=>$text, -data=>$ass_tag);
				$last_items{$level}=$parent_path.'~'.$ass_tag;
				}
			}
		}
	}
$outline->autosetmode;
$self->top()->Unbusy();
}

=head3 goto_outline

$object->goto_outline(); 

Jumps to the top of the fraction in the document that is selected in the outline view.

=cut

##################################################
### Im Outline zu einer Zeile im Text springen ###
##################################################
sub goto_outline
{
my $self = shift;
$self->top()->Busy();
my $outline = $self->outline();
my $tagman = $self->tagman();
my $textwidget = $self->doctext();
my $path = $outline->infoAnchor;
$self->{'ui'}->{'big_notebook'}->raise('Document');
if ($path eq 'ROOT')
	{
	$textwidget->see('1.0');
	}
else
	{
	#Get the tagname out of the '-data'-option
	if (defined $outline->infoData($path))
		{
		my $tag = $outline->infoData($path);
		#Get the start-index of the tag
		my $index1=$tagman->{'tags'}->{$tag}->{0}->{'start_index'};
		#jump there
		$textwidget->see($index1);
		}
	}

$self->top()->Unbusy();
}

=head3 refresh_backend

$object->refresh_backend();

Refreshes the backend view.

=cut

#############################
# Latex_Ansicht auffrischen #
#############################
sub refresh_backend
{
my $self = shift;  
$self->top()->Busy();
my @backend_document = Backend::export_to_backend($self) ;#Dokument im backend format aufbauen
$self->{'ui'}->{'backend_text'}->delete('1.0','end');#Textwidget loeschen
for (my $e=0;$e<=$#backend_document;$e++) 
	{
	$self->{'ui'}->{'backend_text'}->insert("end", $backend_document[$e]."\n"); #Text zeilenweise einlesen
	}
$self->top()->Unbusy();
}

=head3 add_part

$object->add_part( $parts_group [, $part_name, $ref_part_content] ); 

This method adds the selected text of the parts-archive.

It asks for a name for that part, if it already exists, it overwrites the old one.

Parameters:

(+) $parts_group: The subbranch the part should be added (Values are: 'GLOBAL', 'PROJECT' or 'DOCUMENT') 

(+) $part_name: The name for the part. If not given or undef, it will be queried. [Optional argument !]

(+) $ref_part_content: A reference to an array holding the content of the part to be added. If this argument is not undef, this
    content will be used instead of the currently selected text in the document ! [Optional argument !]

=cut

###################################################
### Add a new part or overwrite an existing one ###
###################################################
sub add_part
{
my $self = shift;
my $parts_group = shift;
my $part_name = shift;
my $ref_part_content = shift;

$self->top()->Busy();
my $ui_base = $self->{'ui'};
my $parts_tree = $ui_base->{'parts_tree'};
my $textwidget = $self->doctext();

#If not called with content
if (not defined $ref_part_content)
	{
	#Retrive selection info
	my $sel = $self->get_selection();
	if (not $sel==0)
		{
		#Get selection's content
		my @part_content = InOut::create_document_array($self, $sel->{'start_index'}, $sel->{'end_index'});
		$ref_part_content = \@part_content;
		}
	else
		{
		#If no text is selected
		goto EXIT;
		}
	}

#Query for a name
if (not defined $part_name or $part_name eq '')
	{
	$part_name = $self->dialog('Enter the name of the part:', 'Name:', 'new part') ;
	}


#Store the part 
my $ref_parts_root = $self->_get_parts_root($parts_group);
$ref_parts_root->{$part_name} = $ref_part_content;

#Refresh the treeview
$self->_update_parts_branch($parts_group);

if ($parts_group eq 'DOCUMENT')
	{
	$self->set_content_changed();
	}
elsif ($parts_group eq 'PROJECT')
	{
	$self->prjform()->set_content_changed();
	}
elsif ($parts_group eq 'GLOBAL')
	{
	$main::global_parts_changed = 1;
	}

EXIT:
$parts_tree->autosetmode();
$self->top()->Unbusy();
}

=head3 use_part

$object->use_part( [$parts_group, $part_name, $cursor_index] ); 

This method Inserts the content of a part into the document at the current cursor position.
If BOTH of the first 2 optional arguments are given and the specified part exists, this part is inserted,
otherwise the currently selected part (if any) is used for insertion.

If the third optional argument is given, this position will be used in the text for insertion
and not the current cursor position.

Parameters:

(+) $parts_group: In which subbranch of the parts-archive the part is located. Could be 'DOCUMENT', 'PROJECT' or 'GLOBAL'. [Optional argument !]

(+) $part_name: The name for the part. [Optional argument !]

(+) $cursor_index: The insertion point as an valid Textwidget index. [Optional argument !]

=cut

###############################################
### Insert a part into the current document ###
###############################################
sub use_part
{
my $self = shift;
my $parts_group = shift;
my $part_name = shift;
my $cursor_position = shift;

$self->top()->Busy();
my $ui_base = $self->{'ui'};
my $parts_tree = $ui_base->{'parts_tree'};
my $textwidget = $self->doctext();

my $ref_part_content;

#If not called with content
if (not defined $parts_group or not defined $part_name)
	{
	#Retrive the part from the data in the tree
	my $path = $parts_tree->infoAnchor;
	my $data = $parts_tree->infoData($path);
	#Split the data of the selected item into subbranch and entry-name
	if ($data =~ /(DOCUMENT|PROJECT|GLOBAL)~(.*)/)
		{
		$parts_group = $1;
		$part_name = $2;
		}
	else #If its not coded like this (Root items)
		{
		#jump out
		goto EXIT; 
		}
	}

#Get the part 
my $ref_parts_root = $self->_get_parts_root($parts_group);
if (exists $ref_parts_root->{$part_name} and defined $ref_parts_root->{$part_name})
	{
	$ref_part_content = $ref_parts_root->{$part_name};
	}
else
	{
	goto EXIT;
	}

#Retrive cursor info, if not given by the argument
if (not defined $cursor_position)
	{
	$cursor_position = $textwidget->index('insert');
	}

#Insert the part into the document
InOut::insert_array_into_document($self, $cursor_position, $ref_part_content ,0);

$self->set_content_changed();

#Set the focus
$textwidget->focus();

EXIT:
$self->top()->Unbusy();
}

=head3 rename_part

$object->rename_part( [$parts_group, $part_name, $new_name] ); 

This method renames a part. 
If BOTH of the first 2 optional arguments are given and the specified part exists, this part is renamed
otherwise the currently selected part (if any) is used for renaming.

If the third optional argument is given, this new name will be used, otherwise it will be queried.

Parameters:

(+) $parts_group: In which subbranch of the parts-archive the part is located. Could be 'DOCUMENT', 'PROJECT' or 'GLOBAL'. [Optional argument !]

(+) $part_name: The name for the part. [Optional argument !]

(+) $new_name: The new name for the part. [Optional argument !]

=cut

#####################
### Rename a part ###
#####################
sub rename_part
{
my $self = shift;
my $parts_group = shift;
my $part_name = shift;
my $new_name = shift;

$self->top()->Busy();
my $ui_base = $self->{'ui'};
my $parts_tree = $ui_base->{'parts_tree'};


#If not called with content
if (not defined $parts_group or not defined $part_name)
	{
	#Retrive the part from the data in the tree
	my $path = $parts_tree->infoAnchor;
	my $data = $parts_tree->infoData($path);
	#Split the data of the selected item into subbranch and entry-name
	if ($data =~ /(DOCUMENT|PROJECT|GLOBAL)~(.*)/)
		{
		$parts_group = $1;
		$part_name = $2;
		}
	else #If its not coded like this (Root items)
		{
		#jump out
		goto EXIT; 
		}
	}

#Query for a new name
if (not defined $new_name or $new_name eq '')
	{
	$new_name = $self->dialog('Enter the new name of the part:', 'Name:', 'new part') ;
	}

#Copy the part to a new name
$self->copy_part($parts_group, $part_name, $new_name);

#Delete the old one
$self->delete_part($parts_group, $part_name);

if ($parts_group eq 'DOCUMENT')
	{
	$self->set_content_changed();
	}
elsif ($parts_group eq 'PROJECT')
	{
	$self->prjform()->set_content_changed();
	}
elsif ($parts_group eq 'GLOBAL')
	{
	$main::global_parts_changed = 1;
	}


EXIT:
$self->top()->Unbusy();
}

=head3 copy_part

$object->copy_part( [$parts_group, $part_name, $new_name] ); 

This method copies a part. 
If BOTH of the first 2 optional arguments are given and the specified part exists, this part is copied
otherwise the currently selected part (if any) is used for copying.

If the third optional argument is given, this new name will be used, otherwise it will be queried.

Parameters:

(+) $parts_group: In which subbranch of the parts-archive the part is located. Could be 'DOCUMENT', 'PROJECT' or 'GLOBAL'. [Optional argument !]

(+) $part_name: The name for the part. [Optional argument !]

(+) $new_name: The new name for the part. [Optional argument !]

=cut

###################
### Copy a part ###
###################
sub copy_part
{
my $self = shift;
my $parts_group = shift;
my $part_name = shift;
my $new_name = shift;

$self->top()->Busy();
my $ui_base = $self->{'ui'};
my $parts_tree = $ui_base->{'parts_tree'};

my @part_content;

#If not called with content
if (not defined $parts_group or not defined $part_name)
	{
	#Retrive the part from the data in the tree
	my $path = $parts_tree->infoAnchor;
	my $data = $parts_tree->infoData($path);
	#Split the data of the selected item into subbranch and entry-name
	if ($data =~ /(DOCUMENT|PROJECT|GLOBAL)~(.*)/)
		{
		$parts_group = $1;
		$part_name = $2;
		}
	else #If its not coded like this (Root items)
		{
		#jump out
		goto EXIT; 
		}
	}

#Query for a new name
if (not defined $new_name or $new_name eq '')
	{
	$new_name = $self->dialog('Enter the new name of the part:', 'Name:', 'new part') ;
	}

#Get the part and copy it to a new hash entry
my $ref_parts_root = $self->_get_parts_root($parts_group);
if (exists $ref_parts_root->{$part_name} and defined $ref_parts_root->{$part_name})
	{
	#Copy the thing
	@part_content = @{$ref_parts_root->{$part_name}};
	$ref_parts_root->{$new_name} = \@part_content;
	$self->_update_parts_branch($parts_group);
	}
else
	{
	goto EXIT;
	}

if ($parts_group eq 'DOCUMENT')
	{
	$self->set_content_changed();
	}
elsif ($parts_group eq 'PROJECT')
	{
	$self->prjform()->set_content_changed();
	}
elsif ($parts_group eq 'GLOBAL')
	{
	$main::global_parts_changed = 1;
	}


EXIT:
$self->top()->Unbusy();
}

=head3 delete_part

$object->delete_part( [$parts_group, $part_name] ); 

This method deletes a part. 
If BOTH of the first 2 optional arguments are given and the specified part exists, this part is deleted
otherwise the currently selected part (if any) is used for deletion.

Parameters:

(+) $parts_group: In which subbranch of the parts-archive the part is located. Could be 'DOCUMENT', 'PROJECT' or 'GLOBAL'. [Optional argument !]

(+) $part_name: The name for the part. [Optional argument !]

=cut

#####################
### Delete a part ###
#####################
sub delete_part
{
my $self = shift;
my $parts_group = shift;
my $part_name = shift;

$self->top()->Busy();
my $ui_base = $self->{'ui'};
my $parts_tree = $ui_base->{'parts_tree'};

#If not called with content
if (not defined $parts_group or not defined $part_name)
	{
	#Retrive the part from the data in the tree
	my $path = $parts_tree->infoAnchor;
	my $data = $parts_tree->infoData($path);
	#Split the data of the selected item into subbranch and entry-name
	if ($data =~ /(DOCUMENT|PROJECT|GLOBAL)~(.*)/)
		{
		$parts_group = $1;
		$part_name = $2;
		}
	else #If its not coded like this (Root items)
		{
		#jump out
		goto EXIT; 
		}
	}

#Get the part and delete it
my $ref_parts_root = $self->_get_parts_root($parts_group);
if (exists $ref_parts_root->{$part_name} and defined $ref_parts_root->{$part_name})
	{
	#Delete the thing
	delete $ref_parts_root->{$part_name};
	$self->_update_parts_branch($parts_group);
	}
else
	{
	goto EXIT;
	}

if ($parts_group eq 'DOCUMENT')
	{
	$self->set_content_changed();
	}
elsif ($parts_group eq 'PROJECT')
	{
	$self->prjform()->set_content_changed();
	}
elsif ($parts_group eq 'GLOBAL')
	{
	$main::global_parts_changed = 1;
	}


EXIT:
$self->top()->Unbusy();
}


=head3 update_parts_all

$object->update_parts_all(); 

Updates the all subbranches of the parts-treeview.

=cut

##############################################
### Refresh all branches of the PARTS tree ###
##############################################
sub update_parts_all
{
my $self = shift;

$self->top()->Busy();
my $ui_base = $self->{'ui'};
my $parts_tree = $ui_base->{'parts_tree'};

#Refresh them
$self->update_parts_document();
$self->update_parts_project();
$self->update_parts_global();

$self->top()->Unbusy();
}


=head3 update_parts_document

$object->update_parts_document(); 

Updates the DOCUMENT-subbranch of the parts-treeview.

=cut

#####################################################
### Refresh the DOCUMENT branch of the PARTS tree ###
#####################################################
sub update_parts_document
{
my $self = shift;

$self->top()->Busy();
my $ui_base = $self->{'ui'};
my $parts_tree = $ui_base->{'parts_tree'};

#Delete all entries in the DOCUMENT branch
$parts_tree->deleteOffsprings('ROOT~DOCUMENT');

#Add the current ones
foreach my $part_name ( sort $self->doc_parts_names() )
	{
	$parts_tree->addchild('ROOT~DOCUMENT', -data=>"DOCUMENT~$part_name", -itemtype=>'imagetext', -text=>$part_name, -image=>$ui_base->{'part_doc_icon'});
	}

#Refresh the indicators in the tree
$parts_tree->autosetmode();

$self->top()->Unbusy();
}

=head3 update_parts_project

$object->update_parts_project(); 

Updates the PROJECT-subbranch of the parts-treeview.

=cut

####################################################
### Refresh the PROJECT branch of the PARTS tree ###
####################################################
sub update_parts_project
{
my $self = shift;

$self->top()->Busy();
my $ui_base = $self->{'ui'};
my $parts_tree = $ui_base->{'parts_tree'};

#Delete all entries in the DOCUMENT branch
$parts_tree->deleteOffsprings('ROOT~PROJECT');

#Add the current ones
foreach my $part_name ( sort $self->prjform()->prj_parts_names() )
	{
	$parts_tree->addchild('ROOT~PROJECT', -data=>"PROJECT~$part_name", -itemtype=>'imagetext', -text=>$part_name, -image=>$ui_base->{'part_prj_icon'});
	}


#Refresh the indicators in the tree
$parts_tree->autosetmode();

$self->top()->Unbusy();
}

=head3 update_parts_global

$object->update_parts_global(); 

Updates the GLOBAL-subbranch of the parts-treeview.

=cut

###################################################
### Refresh the GLOBAL branch of the PARTS tree ###
###################################################
sub update_parts_global
{
my $self = shift;

$self->top()->Busy();
my $ui_base = $self->{'ui'};
my $parts_tree = $ui_base->{'parts_tree'};

#Delete all entries in the DOCUMENT branch
$parts_tree->deleteOffsprings('ROOT~GLOBAL');

#Add the current ones
foreach my $part_name ( sort keys %{$main::global_parts} )
	{
	$parts_tree->addchild('ROOT~GLOBAL', -data=>"GLOBAL~$part_name", -itemtype=>'imagetext', -text=>$part_name, -image=>$ui_base->{'part_global_icon'});
	}

#Refresh the indicators in the tree
$parts_tree->autosetmode();

$self->top()->Unbusy();
}


=head3 insert_embobj

$tagname = $object->insert_embobj($classname, $data, $index1, $renumber, $do_not_sort);

Inserts an EmbObj-derived widget into the document.

The method returns the tag name of the new created "EMBOBJ"-tag.

Parameters:

(+) $classname: The classname of the EmbObj derived widget to insert

(+) $data: All configuaration data the object needs

(+) $index1: The place in the text widget where the object should be embeded

(+) $renumber: Automatically the associated "NumList" object. [Here: $tagman->img_num_list(), later
    it will be a "NumList" object defined in the configuration file "edoc.obj" and there associated with
    this "EmbObj"-derived class. These "NumList" objects will be stored in a seperate hash-property in the
    corresponding "TagMan" object called "$tagman->numlists($obj_class_name)"]

(+) $do_not_sort: If set to 1, the automatically created "NumList" will not be automatically sorted in the
    corresponding "NumList" object. This is for document loading from disk.

=cut

####################
# EmbObj einfuegen #
####################
sub insert_embobj
{
my ($self, $classname, $data, $index1, $renumber, $do_not_sort) = @_; 
#$index1 is "-1" if it should be calculated from the current selection

# Anfangs- und Endindizes errechnen
my $position;
if ($index1== -1) 
	{
	$position = $self->doctext()->index("insert");
	}
else 
	{
	$position=$index1;
	}

#nderungsflag setzen
$self->set_content_changed(); # Set the flag to 'changed'

#Ueberpruefen ob es die Datei gibt
my $tagname = $self->tagman()->add_embobj_tag($classname, $data, $position,  $do_not_sort);


#Wenn neu durchnummeriert werden soll 
if ($renumber == 1) 
	{
	my $numlist=$self->tagman()->get_num_list('EMBOBJ', $classname);
	if (defined $numlist)
		{
		$numlist->renumber();
		}
	}

#Tag-Namen zurueckliefern
return $tagname;
}

=head3 edit_doc_args

$object->edit_doc_args();

Shows a TagForm window that allows editing of the document arguments

=cut

######################
### Edit doc_args  ###
######################
sub edit_doc_args
{
my $self = shift;

#Create the window
my $dlg=TagForm->new($self, '' , 0, undef, 1);
}

=head3 copy_to_edoc_clipboard

$object->copy_to_edoc_clipboard();

Copies the selected text to the e:dco internal clipboard.

=cut

####################################################
### Copy a part of the document to the clipboard ###
####################################################
sub copy_to_edoc_clipboard
{
my $self = shift;
$self->top()->Busy();
my $sel = $self->get_selection();
if (not $sel==0)
	{
	@main::clipboard = InOut::create_document_array($self, $sel->{'start_index'}, $sel->{'end_index'});
	#Enable the 'PASTE' menu
	#FIXME: (user defined menus now): $self->{'ui'}->{'menu_edit'}->entryconfigure(3, -state=>'normal');

	}

EXIT:
$self->top()->Unbusy();
}

=head3 cut_to_edoc_clipboard

$object->cut_to_edoc_clipboard();

Cuts the selected text to the e:dco internal clipboard.

=cut

####################################################
### Cuts a part of the document to the clipboard ###
####################################################
sub cut_to_edoc_clipboard
{
my $self = shift;
$self->top()->Busy();
my $sel = $self->get_selection();
if (not $sel == 0)
	{
	$self->copy_to_edoc_clipboard();
	$self->doctext()->delete($sel->{'start_index'},$sel->{'end_index'});
	$self->refresh_outline($self); #Outline neu aufbauen (dabei wird auch durchnummeriert Abstze, Bilder,Tabellen etc.)
	#Enable the 'PASTE' menu
	#FIXME: (user defined menus now): $self->{'ui'}->{'menu_edit'}->menu()->entryconfigure(3,-state=>'normal');

	}
EXIT:
$self->top()->Unbusy();
}

=head3 paste_from_edoc_clipboard

$object->paste_from_edoc_clipboard();

Pastes the e:dco internal clipboard into the current document at the cursor position.

=cut

#######################################################
### Paste a part of the document from the clipboard ###
#######################################################
sub paste_from_edoc_clipboard
{
my $self = shift;
$self->top()->Busy();

if (defined $#main::clipboard and $#main::clipboard>=0) #If there is something in the clipboard
	{
	my $position = $self->doctext()->index('insert');
	InOut::insert_array_into_document($self, $position, \@main::clipboard,0);
	}

EXIT:
$self->top()->Unbusy();
}

=head3 get_selection

$object->get_selection();

Returns 0 if there is no selection otherwise it returns a reference to a hash
where the following indizes are stored:

$result->{'start_index'}: The start index of the selection

$result->{'end_index'}: The end index of the selection

=cut

#####################################
### Check for the selection range ###
#####################################
sub get_selection
{
my $self = shift;

my $result;

my $sel_exists = Utils::textwidget_sel_exists($self->doctext() );

if ($sel_exists == 1) #If there is a selection
	{
	my @range = $self->doctext()->tagRanges("sel"); #Get range
	$result->{'start_index'} = shift(@range); #Start_index
	$result->{'end_index'} = shift(@range); #End_index
	}
else
	{
	$result = 0;
	}

return $result;
}

=head3 tag_doctor_pressed

$object->tag_doctor_pressed();

Toggles ON/OFF the Tag-Doctor feature

=cut

#############################################
### Switches ON/OFF the Tagdoctor-feature ###
#############################################
sub tag_doctor_pressed
{
my $self = shift;
$self->tagman()->toggle_tagdoctor_tags();
}

=head3 show_search_form

$object->show_search_form();

Shows the "search and replace" dialog

=cut

#############################################
### Shows the "search and replace" dialog ###
#############################################
sub show_search_form
{
my $self = shift;
#Create the Dialog and store it in the OpenDialogs array
require class::SearchForm; #Klasse fuer das Dokumentfenster
my $dlg = SearchForm->new($self);
$self->search_dialog($dlg); 
}

=head3 update_window_caption

$object->update_window_caption();

Updates the text for the window caption

=cut

#############################
# Update the window caption #
#############################
sub update_window_caption
{
my $self = shift;

$self->top()->configure(-title => "Document: [".$self->filename()."]");

}


=head2 Private methods:

=head3 _get_parts_root

my $ref_hash = $object->_get_parts_root($parts_group);

Get the root of the hash containing the infos of this parts-group.

Parameters:

(+) $parts_group: The name of the 'parts group'. Can be: 'DOCUMENT', 'PROJECT' or 'GLOBAL'.

=cut

########################################
# Get the root of the right parts hash #
########################################
sub _get_parts_root
{
my $self = shift;
my $parts_group = shift;

my $ref_hash = undef;

#If its in the 'DOCUMENT'-branch
if ($parts_group eq 'DOCUMENT')
	{
	$ref_hash = $self->doc_parts_root();
	}
#If its in the 'PROJECT'-branch
elsif ($parts_group eq 'PROJECT') 
	{
	$ref_hash = $self->prjform()->prj_parts_root();

	}
#If its in the 'GLOBAL'-branch
elsif ($parts_group eq 'GLOBAL')
	{
	$ref_hash = $main::global_parts;
	}

return $ref_hash;

}

=head3 _update_parts_branch

$object->_update_parts_branch($parts_group);

Updates the subbranch of the parts tree that holds this parts_group

Parameters:

(+) $parts_group: The name of the 'parts group'. Can be: 'DOCUMENT', 'PROJECT' or 'GLOBAL'.

=cut

#####################################
# Update a branch of the parts tree #
#####################################
sub _update_parts_branch
{
my $self = shift;
my $parts_group = shift;

#If its in the 'DOCUMENT'-branch
if ($parts_group eq 'DOCUMENT')
	{
	$self->update_parts_document();
	}
#If its in the 'PROJECT'-branch
elsif ($parts_group eq 'PROJECT') 
	{
	$self->update_parts_project();
	}
#If its in the 'GLOBAL'-branch
elsif ($parts_group eq 'GLOBAL')
	{
	$self->update_parts_global();
	}
}

=head3 _key_pressed

$object->_key_pressed();

Callback for any KeyPress event. This method sets the content_changed() flag.

=cut

############################################################
### Callback um auf nderungen im Textfeld zu reagieren  ###
############################################################
sub _key_pressed
{
my $self = shift;
#nderungsflag setzen
$self->set_content_changed(); # Set the flag to 'changed'
}

=head3 _cursor_moved

$object->_cursor_moved();

Callback for any cursor movements or mouseclicks in the text widget. This method ensures that
there are always the right paragraph, environment, character styles and command names displayed in
the OptionMenu widgets.

NOTE: This method should be disabled when a document is loaded from disk, because I think it is
often called then and this just costs time for nothing.

=cut

##################################################################
### Callback um auf Cursorbewegungen im Textfeld zu reagieren  ###
##################################################################
sub _cursor_moved
{
my $self = shift;

my $sender = $self->doctext();

my $par_format_gefunden=0; #Flag
my $env_format_gefunden=0; #Flag
my $char_format_gefunden=0; #Flag
my $command_gefunden=0; #Flag

# Cursorposition ermitteln
my $cur_pos = $sender->index("insert");

#Hier auf Tags pruefen und im Optionsfeld anzeigen
my @tagnames = $sender->tagNames($cur_pos);
foreach my $tagname (@tagnames)
	{
	# Wenn ein Absatzformat gefunden wird
	if ( $tagname =~ /^<PAR_STYLES><(.*)><(.*?)>:(\d+)$/ )
		{
		$self->current_par_style($1); #Set current paragraph style
		$par_format_gefunden=1; #Flag setzen
		}
	if ( $tagname =~ /^<ENV_STYLES><(.+)><(.*?)>:(\d+)$/ )
		{
		$self->current_env_style($1); #Set current environment style
		$env_format_gefunden=1; #Flag setzen
		}
	if ( $tagname =~ /^<CHAR_STYLES><(.*)><(.*?)>:(\d+)$/ )
		{
		$self->current_char_style($1); #Set current character style
		$char_format_gefunden=1; #Flag setzen
		}
	if ( $tagname =~ /^<COMMANDS><(.*)><(.*?)>:(\d+)$/ )
		{
		$self->current_command($1); #Set current command
		$command_gefunden=1; #Flag setzen
		}
	}

#Wenn kein Tag gefunden wurde->par_base_format einstellen
if ($par_format_gefunden==0) 
	{
	$self->current_par_style($self->no_style_text()); #Set current paragraph style to none
	}
if ($env_format_gefunden==0) 
	{
	$self->current_env_style($self->no_style_text()); #Set current environment style to none
	}
if ($char_format_gefunden==0) 
	{
	$self->current_char_style($self->no_style_text()); #Set current character style to none
	}
if ($command_gefunden==0) 
	{
	$self->current_command($self->no_style_text()); #Set current command to none
	}
}


=head3 _build_backend_menu

$object->_build_backend_menu();

(Re)builds the 'Backend' menu.

=cut

############################
# Rebuild the backend menu #
############################
sub _build_backend_menu
{
my $self = shift;
#Get menu configuration from $backend-hash and build up the new 'Backend' menu
uiBaseForm::build_menu('DOCUMENT_FORM', $self, 'Backend', $self->backend()->{'BACKEND_MENU'}, $self->backend()->{'BACKEND_MENU_NAMES'});
}

=head3 _show_full_screen_mode

$object->_show_full_screen_mode();

Turns on the full_screen mode.

=cut

#################################
# Turns on the full_screen mode #
#################################
sub _show_full_screen_mode
{
my $self = shift;
my $top = $self->top();
my $ui_base = $self->{'ui'};
my $wnd_info = $self->{'_window_info'} = {};
$wnd_info->{'_geometry'} = $top->geometry();
my $screen_width = $top->screenwidth();
my $screen_height = $top->screenheight();
$top->withdraw();
$ui_base->{'menu'}->packForget();
$ui_base->{'bottomframe'}->packForget();
$ui_base->{'speedbar'}->packForget();
$ui_base->{'toolbox_frame'}->packForget();
$ui_base->{'adj'}->packForget();

$top->overrideredirect(1);
$top->geometry($screen_width."x".$screen_height."+0+0");
$top->deiconify();

my $dlg = $self->{'_ful_screen_mode_wnd'}=$self->top()->Toplevel();
$dlg->configure(-title=>' ');
my $btn = $dlg->Button(-text=>'Close', -command=>sub{$self->full_screen_mode(0)})->pack(-fill=>'both', -expand=>1);
Utils::create_balloonhelp($btn, 'Press here to end full screen mode', 'balloon', undef);
$dlg->transient($self->top());
$dlg->protocol('WM_DELETE_WINDOW',sub{$self->{'_ful_screen_mode_wnd'} = undef; $dlg->destroy()}); #Binding the cleanupfunction to the closing of the window BEFORE the subwidgets are destroyed
my $x = ($self->top()->Width() - 50);
$x = 0 if $x < 0;
$dlg->geometry("+".$x."+1");

$self->top()->idletasks();
$self->top()->raise();
$ui_base->{'doctext'}->focus();
$self->top()->idletasks();


}

=head3 _hide_full_screen_mode

$object->_hide_full_screen_mode();

Turns off the full_screen mode.

=cut

##################################
# Turns off the full_screen mode #
##################################
sub _hide_full_screen_mode
{
my $self = shift;
my $top = $self->top();
my $ui_base = $self->{'ui'};
my $wnd_info = $self->{'_window_info'};

if ($self->full_screen_mode() == 1)
	{
	if (defined $self->{'_ful_screen_mode_wnd'})
		{
		$self->{'_ful_screen_mode_wnd'}->destroy();
		$self->{'_ful_screen_mode_wnd'} = undef;
		}
	$top->withdraw();
	$top->geometry($wnd_info->{'_geometry'});
	$ui_base->{'big_notebook'}->packForget();
	$ui_base->{'menu'}->pack(-side =>'top', -fill=>'x');
	$ui_base->{'bottomframe'}->pack(-side=>'bottom', -expand=>0, -fill=>'x');
	$ui_base->{'speedbar'}->pack(-side=>'top',-fill=>'x', -expand=>0);
	$ui_base->{'big_notebook'}->pack(-fill=>'both', -expand=>1);
	$ui_base->{'text_frame'}->packForget();
	$ui_base->{'toolbox_frame'}->pack(-side=>'left', -fill=>'both', -expand=>1);
	$ui_base->{'adj'}->pack(-side=>'left', -fill=>'y');
	$ui_base->{'text_frame'}->pack(-side=>'left', -fill=>'both', -expand=>1);
	$top->overrideredirect(0);
	$top->deiconify();
	}
}

1; #So muss package enden


=head1 KNOWN BUGS:

=head1 STILL MISSING:


=cut
