#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package EmbObj;

use Tk qw(Ev);
require Tk::Frame;
@ISA = qw(Tk::Derived Tk::Frame);
Tk::Widget->Construct('EmbObj');

require lib::Backend;

=head1 NAME:

Widget: EmbObj

=head1 SYNOPSIS:

use class::EmbObj;

=head1 DIRECTORY/FILE:

The file EmbObj.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

This is a widget derived from Tk::Frame that gets embeded into the documents text widget in the DocForm object (=document window).

It is the base class for all embeded widgets which are defined in the configuration file "edoc.obj". This class contains
the functionality of beeing embeded into the text widget (doctext) of a document window (DocForm object), to be expanded or
collapsed and to be numbered by one of the numbering engines (NumList object) also defined in "edoc.obj" and to be shown
in the outline view as an item in the objects branch.

This class EmbObj is never used, it just builds the base class for all embeded objects like images (ImgObj) or tables (TblObj).

This object has 2 different states.

(+) In the collapsed state, it displays 3 Labels, the right one contains the numbering, the middle one the caption and
    the right one contains a sort of button to open (if exists) the property editor of the (derived) class.

(+) In the expanded state, it shows the content of the object too.

This object is coded as an Perl/Tk widget, so it has no Constructor new, it has the standard Tk functions to initialise itself (Populate,ClassInit,...)
The function Populate() is used like an constructor here.

=head1 STATUS:

Coding: 20% done

Documentation: up-to-date

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

$self->{'foreground'} #The foreground color definition

$self->{'background'} #The background color definition

$self->{'font'} #The font definition

$self->{'relief'} #The relief definition

$self->{'docform'} #A reference to the associated DocForm object

$self->{'tagman'} #A reference to the associated TagMan object

$self->{'window_tag'} #The name of the tag the EmbObj is tagged with in the text widget

$self->{'content_changed'} #Flag, if the content changed (unsaved)

$self->{'numlist'} #A referenve of the Numlist that should hold the internal NumLabel

$self->{'pre_number_text'} #The text that should be shown before the number

$self->{'post_number_text'} #The text that should be shown after the number

$self->{'do_number'} #If the object is numbered or not

$self->{'propedit_classname'} #The classname of the objects property editor is stored here.
The default class of propertyeditor is 'TagForm', the same property editor that is used for
editing Tag-Properties. This property editor can only edit a flat one-dimensional set of arguments(properties).
If the derived widget is more complicated, you have to design your own property editor by deriving from
'ObjPropForm.pm' (STILL MISSING). You will have to call $self->propedit_classname($propedit_classname) in the '_build_ui'
method of the derived widget.

$self->{'classname'} #The name of the EmbObj derived class (later this will be got by perl-methods)

$self->{'_obj_config'} #The configuration data for this object (coming from edoc.int)

$self->{'_expanded'} #Flag wether the object is in expanded state ore not

$self->{'_data'} #The save data of the object (needed for saving it on disk and restoring it again)
This attribute consists of a ONE-LEVEL hash holding the attributes of the object.

$self->{'_data'}->{'caption'} #The caption to be displayed

$self->{'_backend'} #Export information for the current backend (Normaly a sub-hash with the depth of one level)

NOTE: The following attributes are used for the Tk widgets building the GUI and are accessed directly (no get/set methods):

$self->{'_top_base'}

$self->{'_bottom_base'}

$self->{'_resize_handle'}

$self->{'_img_prop'}

$self->{'_btn_prop'}

$self->{'_balloon_prop'}

$self->{'numlabel'} #The internal NumLabel

$self->{'embed_base'} #PUBLIC:A reference to the Frame where the derived class should pack its content into

=head1 METHODS:

=head2 Class Methods:

=head3 ClassInit($class, $parentwindow)

! Not called directly !

This "constructor" initialises a new object of the class EmbObj and calls ClassInit() of the Superclass. 

=cut

##########################
# Klassen-Initalisierung #
##########################
sub ClassInit
{
# Tk-spezifischer Code
my ($class,$mainwindow)=@_; # Klassenname und Mainwindow
$class->SUPER::ClassInit($mainwindow); # Aufruf des Vorfahren
print "Object $class created.\n"; #debug
return $class;
}

=head2 Constructors:

=head3 Populate

$object = EmbObj(%options); 

This "constructor" sets the options of the new object of the class EmbObj. 

The binding of events and the creation of the associated Baloonhelp is done here too.

The keys/values in %options: 

(+) '-foreground' =>  The foreground color 

(+) '-background' =>  The background color 

(+) '-relief' =>  The relief style ('flat', 'raised', 'sunken', 'ridged')

(+) '-font' =>  The font definition

(+) '-docform' =>  A reference to the "DocForm" object to which this "ComObj" widget belongs (!Circular references!)

(+) '-window_tag' =>  The name of the tag that surrounds the embeded "ComObj" in the text widget

(+) '-numlist' => A reference of the associated NumLabel widget

(+) '-classname' => The name of the EmbObj derived class (later this will be got by perl-methods)

(+) '-do_not_sort_numlabel' => If 1 then this numlabel will not be autoamtically sorted (loading time)

(+) '-pre_number_text' => The text that should be shown before the number

(+) '-post_number_text' => The text that should be shown after the number

(+) '-data' => The saved data to be stored into $self->data()

(+) '-obj_config' => The configuration data for this object, stored in $self->obj_config().
This is a reference to a hash that holds the additional configuration data stored in 'edoc.int'.

(+) '-do_number' => If the object is numbered or not

=cut

######################################
# Evaluating the -option/value pairs #
######################################
sub Populate
{
my ($self,$args)=@_; 

$self->{'_data'} = {}; #An empty  hash for holding the attributes of the object

$self->foreground( delete $args->{'-foreground'} );
$self->background( delete $args->{'-background'} );
$self->relief( delete $args->{'-relief'} );
$self->font( delete $args->{'-font'} );
$self->docform( delete $args->{'-docform'} );
$self->tagman( $self->{'docform'}->tagman() );
$self->window_tag( delete $args->{'-window_tag'} );
$self->caption( delete $args->{'-caption'} );
$self->numlist( delete $args->{'-numlist'} );
$self->classname( delete $args->{'-classname'} );
$self->do_not_sort_numlabel( delete $args->{'-do_not_sort_numlabel'} );
$self->pre_number_text( delete $args->{'-pre_number_text'} ); #Text before the number
$self->post_number_text( delete $args->{'-post_number_text'} ); #Text after the number
$self->do_number( delete $args->{'-do_number'} );
$self->obj_config( delete $args->{'-obj_config'} );
$self->propedit_classname('TagForm'); #setting the default property editor class

my $data = delete $args->{'-data'}; #Read this stored data from the options
$self->_set_save_data($data); #Build up the $self->{'_data'} subbranches from this data

$self->{'_backend'} = {};

$self->content_changed(0);

$self->SUPER::Populate($args); #Calling the SUPER class

#Changing the cursor
$self->configure(-cursor=>'arrow');

#CREATE THE WIDGETS OF THE TOP-PANEL
$self->configure(-takefocus=>1, -borderwidth=>2, -relief=>'ridge');
$self->{'_top_base'} = $self->Frame(-takefocus=>1, -relief=>'flat', -borderwidth=>0)->pack(-side=>'top', -fill=>'x');
$self->{'_bottom_base'} = $self->Frame(-takefocus=>1, -relief=>'flat', -borderwidth=>0)->pack(-side=>'bottom', -fill=>'x', -expand=>1);
$self->{'_resize_img'} = $self->Photo(-file=>Utils::process_filename('%EDOC_PATH%/graphics/vh_handle.xpm'));
my $grip = $self->{'_resize_handle'} = $self->{'_bottom_base'}->Label(-image=>$self->{'_resize_img'}, -cursor=>'hand2')->pack(-side=>'right', -fill=>'y', -anchor=>'e');
Utils::create_balloonhelp($grip, "Double-Click: Restore original size\nDrag&Drop:Resize the object", 'balloon', undef);

$self->{'_img_prop'} = $self->Photo(-file=>Utils::process_filename('%EDOC_PATH%/graphics/prop.xpm') );
$self->{'_btn_prop'} = $self->{'_top_base'}->Label(-takefocus=>1, -image=>$self->{'_img_prop'}, -borderwidth=>2, -relief=>'raised')->pack(-side=>'right', -fill=>'y', -expand=>0);
$self->{'_balloon_prop'} = $self->docform()->top()->Balloon(-state=>'balloon');
$self->{'_balloon_prop'}->attach($self->{'_btn_prop'}, -msg=>'Click here to show the property form');

#Creating th frame where there will later be the playground for the derived widgets
$self->embed_base( $self->Frame(-takefocus=>1, -borderwidth=>0, -relief=>'flat')->pack(-side=>'bottom', -fill=>'both', -expand=>1));

#Binding events to the grip
$grip->bind('<Double-Button-1>', sub{
												$self->embed_base->packPropagate(1);
												$self->embed_base()->idletasks;
												$self->embed_base()->packPropagate(0);
												$self->embed_base()->idletasks;
												Tk::break();
												});

$grip->bind('<ButtonPress-1>', [\&_grip_mouse_button1_pressed, $self, Ev('X'), Ev('Y')]);
$grip->bind('<ButtonRelease-1>', [\&_grip_mouse_button1_released, $self, Ev('X'), Ev('Y')]);

$self->embed_base()->idletasks;
$self->embed_base()->packPropagate(0);

#restoring the size of the object (if saved)
if (exists $self->{'_data'}->{'_EMBOBJ_width'} and
	defined $self->{'_data'}->{'_EMBOBJ_width'} and
	exists $self->{'_data'}->{'_EMBOBJ_height'} and
	defined $self->{'_data'}->{'_EMBOBJ_width'} )
	{
	my $width = $self->{'_data'}->{'_EMBOBJ_width'};
	my $height = $self->{'_data'}->{'_EMBOBJ_height'};
	$self->embed_base()->GeometryRequest($width, $height);
	$self->embed_base()->idletasks;
	delete ($self->{'_data'}->{'_EMBOBJ_width'} );
	delete ($self->{'_data'}->{'_EMBOBJ_height'} );
	$self->embed_base()->idletasks;
	}

#Calling the build up of the GUI in the derived classes
$self->build_ui();

return $self;
}

=head3 build_ui

This message is called ONLY in the EmbObj-derived classes to fill the
frame that is thought for embeding the new widgets.

This is an abstract base function, it HAS TO BE overwritten by the derived class.

=cut

################
# Fill the GUI #
################
sub build_ui
{
my $self = shift;
my $embed_base = $self->embed_base();

#NO CODE HERE, JUST AN ABSTRACT FUNCTION

}


=head3 on_create

$object->on_create(); 

This method has to be called after the object has been correctly created, embeded into the textwidget and tagged with his own tag.
Afterwards the 'window_tag' attribute should be set to this tagname by using the method '$obj->window_tag($tagname)'. 

This is IMPORTANT because this 'on_create' method performs the creation of the contained widgets,
binding of events and some more important stuff.

=cut

####################################################################
# Call this AFTER the window_tag was set around the embeded EmbObj #
####################################################################
sub on_create
{
my $self= shift;
if (defined $self->window_tag() and defined $self->classname() )
	{
	if (defined $self->numlist())
		{
		if (($self->numlist()->do_number() == 1 ) and ($self->do_number() == 1)) #If numbering is performed
			{
			#Create the NumLabel widget
			$self->numlabel( 
									$self->{'_top_base'}->NumLabel(-takefocus=>1,
																			-numlist=>$self->numlist(), 
																			-level=>1, 
																			-pre_number_text=>$self->pre_number_text(), 
																			-post_number_text=>$self->post_number_text(), 
																			-text_widget=>$self->docform()->doctext(), 
																			-associated_tag=>$self->window_tag(), 
																			-relief=>$self->relief(), 
																			-font=>$self->font(), 
																			-borderwidth=>1, 
																			-show_number=>1, 
																			-show_bullet=>0
																			)
								);
			$self->numlabel()->pack(-side=>'left', -fill=>'both', -expand=>0);
			#Set the tagname for the NumLabel to the own tagname
			$self->numlabel()->window_tag( $self->window_tag() );
			#Initialise it
			$self->numlabel()->on_create( $self->do_not_sort_numlabel() );
			}
		}
	#Debug message
	print "EmbObj of class ".$self->classname()." with tag ".$self->window_tag()." was created \n"; #debug

	#Create the caption label
	$self->{'_label'}= $self->{'_top_base'}->Label(-takefocus=>1, -textvariable=>\$self->{'_data'}->{'caption'}, -borderwidth=>2, -relief=>'raised')->pack(-side=>'left', -fill=>'both', -expand=>1);
	#And the baloon help
	$self->{'_balloon_label'} = $self->docform()->top()->Balloon(-state=>'balloon');
	$self->{'_balloon_label'}->attach($self->{'_label'}, -msg=>'Click here to expand or collapse the object');
	
	#TK-specific code
	$self->Advertise('label'=>$self->{'_label'});
	$self->ConfigSpecs(DEFAULT=>[$self->{'_label'}]);
	$self->Delegates(DEFAULT => $self->{'_label'});

	if (defined $self->foreground()) 
		{
		$self->{'_label'}->configure(-foreground=>$self->foreground());
		$self->{'_btn_prop'}->configure(-foreground=>$self->foreground());
		$self->{'numlabel'}->configure(-foreground=>$self->foreground()) if defined $self->{'numlabel'};
		}
	if (defined $self->background()) 
		{
		$self->{'_label'}->configure(-background=>$self->background());
		$self->{'_btn_prop'}->configure(-background=>$self->background());
		$self->{'numlabel'}->configure(-background=>$self->background()) if defined $self->{'numlabel'};
		}
	if (defined $self->relief()) 
		{
		$self->{'_label'}->configure(-relief=>$self->relief());
		$self->{'_btn_prop'}->configure(-relief=>$self->relief());
		$self->{'numlabel'}->configure(-relief=>$self->relief()) if defined $self->{'numlabel'};
		}
	if (defined $self->font()) 
		{
		$self->{'_label'}->configure(-font=>$self->font());
		$self->{'_btn_prop'}->configure(-font=>$self->font());
		$self->{'numlabel'}->configure(-font=>$self->font()) if defined $self->{'numlabel'};
		}
	
	#START INITIALISATION #
	$self->expanded(1); #Flag
	$self->content_changed( $self->docform()->content_changed()); #Flag ob Inhalt geaendert wurde
	$self->bind('<Destroy>' => sub{ $self->do_destroy()}); #Aufraeumroutine binden
	$self->{'_label'}->bind('<Button-1>' => sub{ $self->_do_click($self)});
	$self->{'_btn_prop'}->bind('<Button-1>' => sub{ $self->_on_btn_prop_click($self)});
	
	#CONFIGURE THE CURSOR
	$self->{'_label'}->configure(-cursor=>'based_arrow_up');
	$self->{'_btn_prop'}->configure(-cursor=>'hand2');

	}


}


=head2 Destructors:

=head3 DESTROY; 

The standard automatically called PERL destructor for objects.

=cut

#############################
# Automatic Perl destructor #
#############################
sub DESTROY
{
my $self = shift;
print "Object $self destroyed.\n"; #debug
}

=head3 do_detstroy

$object->do_detsroy(); 

This "destructor" method is binded to the "<DESTROY>" event and should be performed whenever
the element is destroyed. It should not be called directly.

=cut

###################################################
# Destruktor zum sauberen aufraeumen des Objektes #
###################################################
sub do_destroy
{
my $self =shift;
#Remove the numlabel
if (defined $self->numlabel())
	{
	$self->numlist()->delete_NumLabel($self->numlabel()); #Remove the Numlabel from the list
	$self->numlabel( undef );
	}
$self = undef;
}

=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 propedit_classname

$classname = $object->propedit_classname(); #gets the classname of the property editor for that opject-class

$object->propedit_classname($classname); #seets the classname of the property editor for that opject-class

=cut

### propedit_classname ###
sub propedit_classname
{
my $self=shift;
@_ ? $self->{'propedit_classname'} = shift : return $self->{'propedit_classname'};
}

=head3 docform

$ref_docform = $object->docform(); #gets a reference to the DocForm object the ComObj belongs to

$object->docform($ref_docform); #sets a reference to the DocForm object the ComObj belongs to

=cut

### docform ###
sub docform
{
my $self=shift;
@_ ? $self->{'docform'} = shift : return $self->{'docform'};
}

=head3 tagman

$ref_tagman = $object->tagman(); #gets a reference to the TagMan object of the DocForm object the ComObj belongs to

$object->tagman($ref_tagman); #sets a reference to the Tagman object of the DocForm object the ComObj belongs to

=cut

### tagman ###
sub tagman
{
my $self=shift;
@_ ? $self->{'tagman'} = shift : return $self->{'tagman'};
}

=head3 content_changed

$flag = $object->content_changed(); #gets the changed-flag of the object

$object->content_changed($flag); #sets the changed-flag of the object

=cut

### content_changed ###
sub content_changed
{
my $self=shift;
@_ ? $self->{'content_changed'} = shift : return $self->{'content_changed'};
}

=head3 embed_base

$ref_emb_base = $object->embed_base(); #gets a reference to the Frame where the content of the derived widget should be packed into

$object->embed_base($ref_emb_base); #sets a reference to the Frame where the content of the derived widget should be packed into

=cut

### embed_base ###
sub embed_base
{
my $self=shift;
@_ ? $self->{'embed_base'} = shift : return $self->{'embed_base'};
}

=head3 caption

$text = $object->caption(); #gets the caption text for the object

$object->caption($text); #sets the caption text for the object

=cut

### caption ###
sub caption
{
my $self=shift;
@_ ? $self->{'_data'}->{'caption'} = shift : return $self->{'_data'}->{'caption'};
}

=head3 do_not_sort_numlabel

$do_not_sort_numlabel = $object->do_not_sort_numlabel(); #gets the 'do_not_sort_numlabel'-property

$object->do_not_sort_numlabel($do_not_sort_numlabel); #sets the 'do_not_sort_numlabel'-property

=cut

### do_not_sort_numlabel ###
sub do_not_sort_numlabel
{
my $self=shift;
@_ ? $self->{'do_not_sort_numlabel'} = shift : return $self->{'do_not_sort_numlabel'};
}


=head3 numlist

$ref_numlist = $object->numlist(); #gets a reference of the NumList that manages this EmbObj-derived widgets

$object->numlist($ref_numlist); #sets a reference of the NumList that manages this EmbObj-derived widgets

=cut

### numlist ###
sub numlist
{
my $self=shift;
@_ ? $self->{'numlist'} = shift : return $self->{'numlist'};
}


=head3 numlabel

$ref_numlabel = $object->numlabel(); #gets a reference of the internal NumLabel object

$object->numlabel($ref_numlabel); #sets a reference of the internal NumLabel object

=cut

### numlabel ###
sub numlabel
{
my $self=shift;
@_ ? $self->{'numlabel'} = shift : return $self->{'numlabel'};
}

=head3 pre_number_text

$pre_number_text = $object->pre_number_text(); #gets the value of the 'pre_number_text' attribute

$object->pre_number_text($pre_number_text); #sets the value of the 'pre_number_text' attribute

=cut

### pre_number_text ###
sub pre_number_text
{
my $self=shift;
@_ ? $self->{'pre_number_text'} = shift : return $self->{'pre_number_text'};
}

=head3 post_number_text

$post_number_text = $object->post_number_text(); #gets the value of the 'post_number_text' attribute

$object->post_number_text($post_number_text); #sets the value of the 'post_number_text' attribute

=cut

### post_number_text ###
sub post_number_text
{
my $self=shift;
@_ ? $self->{'post_number_text'} = shift : return $self->{'post_number_text'};
}

=head3 do_number

$do_number = $object->do_number(); #gets the value of the 'do_number' attribute

$object->do_number($do_number); #sets the value of the 'do_number' attribute

This tells the EmbObj if it has to register itself in the Numlist and be numbered.

=cut

### do_number ###
sub do_number
{
my $self=shift;
@_ ? $self->{'do_number'} = shift : return $self->{'do_number'};
}

=head3 classname

$classname = $object->classname(); #gets the name of the EmbObj-derived class

$object->classname($classname); #sets the name of the EmbObj-derived class

Stupid to pass that to the EmbObj-class, but this will be later automatically dedected by perl-methods.

=cut

### classname ###
sub classname
{
my $self=shift;
@_ ? $self->{'classname'} = shift : return $self->{'classname'};
}

=head3 obj_config

$ref_obj_config = $object->obj_config(); #gets a reference to an hash that stores the objects configuration data 

$object->obj_config($ref_obj_config); #sets a reference to an hash that stores the objects configuration data

=cut

### obj_config ###
sub obj_config
{
my $self=shift;
@_ ? $self->{'_obj_config'} = shift : return $self->{'_obj_config'};
}


=head3 foreground

$color_def = $object->foreground(); #gets the foreground color definition

$object->foreground($color_def); #sets the foreground color definition

=cut

### foreground ###
sub foreground
{
my $self=shift;
@_ ? $self->{'foreground'} = shift : return $self->{'foreground'};
}

=head3 background

$color_def = $object->background(); #gets the background color definition

$object->background($color_def); #sets the background color definition

=cut

### background ###
sub background
{
my $self=shift;
@_ ? $self->{'background'} = shift : return $self->{'background'};
}

=head3 font

$font_def = $object->font(); #gets the font definition

$object->font($font_def); #sets the font definition

=cut

### font ###
sub font
{
my $self=shift;
@_ ? $self->{'font'} = shift : return $self->{'font'};
}

=head3 relief

$relief_def = $object->relief(); #gets the relief definition

$object->relief($relief_def); #sets the relief definition

=cut

### relief ###
sub relief
{
my $self=shift;
@_ ? $self->{'relief'} = shift : return $self->{'relief'};
}

=head3 window_tag

$tag_name = $object->window_tag(); #gets the tag name of the ComObj

$object->window_tag($tag_name); #sets the tag name of the ComObj

=cut

### window_tag ###
sub window_tag
{
my $self=shift;
@_ ? $self->{'window_tag'} = shift : return $self->{'window_tag'};
}

=head3 expanded

$extpanded_flag = $object->expanded(); #gets the state of the ComObj

$object->expanded($expanded_flag); #sets the state of the ComObj

=cut

### expanded ###
sub expanded
{
my $self=shift;
@_ ? $self->{'_expanded'} = shift : return $self->{'_expanded'};
}

=head3 backend

$ref_backend = $object->backend(); #gets a reference to the objects backend hash

$object->backend($ref_backend); #sets a reference to the objects backend hash

=cut

### backend ###
sub backend
{
my $self=shift;
@_ ? $self->{'_backend'} = shift : return $self->{'_backend'};
}


=head2 Other public methods:

=head3 set_backend

$object->set_backend($ref_hash); #Builds up the $self->{'_backend'}-subhash.

This method is called at export-time to feed the EmbObj-derived object with the
export-definition of the actual backend. This is done before calling the method 'export_to_backend()'.

=cut
###################################################
# Initialise the export-definition for the EmbObj #
###################################################
sub set_backend
{
my $self = shift;
my $ref_backend = shift;

#Empty the old backend data
$self->backend( {} );

#Dereference the hash
my %backend = %{$ref_backend};

#Iterate through the keys
foreach my $key (keys %{$ref_backend})
	{
	#Get the associated values
	my $value = $ref_backend->{$key};
	#Process possible e:doc commands
	$value = Language::process_edoc_commands_export($value, $self->docform(), $self->window_tag());
	#store these values in the {'_backend'}-subhash
	$self->backend()->{$key} = $ref_backend->{$key};
	}

#This has to be the last command in the function
$self->_after_backend_changed();
}

=head3 process_backend_field

$result_string = $object->process_arguments($source_string); 

This function parses for special commands (Arguments, other backend definition commands) in the given string and replaces them with the right values.

The function returns the parsed result.

Parameters:

(+) $source_string: The string to be paresed

=cut

#################################################################
# Parses for arguments and backend-definition-language commands #
#################################################################
sub process_backend_field
{
my $self = shift;
my $source_string = shift;

#Parsing arguments
$source_string = Backend::process_arguments($source_string, $self->docform(), $self->window_tag(), 'EMBOBJ', $self->classname(), $self->backend() );

#Parsing other edoc-commands
#$source_string = Language::process_edoc_commands_export($source_string, $self->docform(), $self->window_tag());

return $source_string;
}


=head3 get_current_line

$line = $object->get_current_line(); 

This returns the line number where the EmbObj widget is situated inside of the text widget it belongs to.

=cut

###############################################
# Liefert Zeilennummer im Text-Widget zurueck #
###############################################
sub get_current_line{
my $self = shift;
my @range = $self->docform()->doctext()->tagRanges($self->window_tag()); # Numlabel ist mit Tag markiert um Zeile eruieren zu koennen
my $anfang = shift(@range); #Anfangsindex auslesen
$anfang =~ s/.\d*$//g; #Zeile herausholen
return $anfang; #Wert zurueckliefern
}

=head3 get_save_data

my $save_data = $object->get_save_data(); 

This function builds a string out of the whole $self->{'_data'} structure and returns it.

=cut

############################################################################
# Flatten the {_data} attributes subbranches into a string for saving them #
############################################################################
sub get_save_data
{
my $self = shift;

#This has to be the first command in this function
$self->_before_save();

#In this variable the save data is built up
my $save_data = '';

#Save the actual size of the Object
$self->idletasks;
my $width = $self->embed_base()->Width();
my $height = $self->embed_base()->Height();
$self->{'_data'}->{'_EMBOBJ_width'} = $width;
$self->{'_data'}->{'_EMBOBJ_height'} = $height;

#Iterate over the subbranches and store them into a flat string
foreach my $key (keys %{$self->{'_data'}})
	{
	# get the pairs
	my $save_key = $key;
	my $save_value = $self->{'_data'}->{$key};
	# mask ()= to \( \) and \=
	$save_key =~ s/[()=]/\\$1/g;
	$save_value =~ s/[()=]/\\$1/g;
	#Append them
	$save_data .= "$save_key||$save_value||";
	}


#This has to be the last command in this function
$self->_after_save();

#Return the data to save
return $save_data;
}

=head3 export_to_backend

my @export_text = $object->export_to_backend(); 

This function exports the content of the object to the current backend.

The function is called in the document-exporting process, whenever an embeded object
is found. 

It returns the text for exporting the object.

The function HAS TO BE overwritten in the derived objects, otherwise this dummy function wil be called.

=cut

##################################
# Export to the current backend. #
##################################
sub export_to_backend
{
my $self = shift;

my $result = '';

$result = "ERROR: The programmer of this embeded object seems to have forgotten to write an export function\n"; 

return $result;
}

=head2 Private methods:

=head3 _set_save_data

$object->_set_save_data($data); 

This function builds the whole $self->{'_data'} structure out of the string $data.

Parameters:

(+) $data : The saved data from file

=cut

###############################################################
# Build up the subhash stored in $self->{_data} from a string #
###############################################################
sub _set_save_data
{
my $self = shift;
my $data = shift;

#This has to be the first command in this function
$self->_before_load();

my @items = split(/\|\|/, $data);
while (@items)
	{
	my $key = shift(@items);
	my $value = shift(@items);
	if (defined $value and (not($value eq '')))
		{
		$self->{'_data'}->{$key} = $value; 
		}
	}

#This has to be the last command in this function
$self->_after_load($data);
}


=head3 _do_click

$object->_do_click(); 

This is the callback function for the "<Button-1>" event (Click).

It causes the EmbObj widget to collapse or expand itself.

=cut

###########################
# Callback fuer Mausklick #
###########################
sub _do_click
{
my $sender=shift;
my $self = shift;
if ($self->expanded()==1)
	{
	$self->expanded(0);
	_collapse($self);
	$self->{'_label'}->configure(-cursor=>'based_arrow_down');
	}
else
	{
	$self->expanded(1);
	_expand($self);
	$self->{'_label'}->configure(-cursor=>'based_arrow_up');
	}
}

=head3 _expand

$object->_expand(); 

Expands the object. (Creates and packs the '$self->embed_base()' widget.)

=cut

#####################
# Objekt ausklappen #
#####################
sub _expand
{
my $self = shift;
#Has to be first command in this function
$self->_before_expand();

#Expand the thing
$self->{'_bottom_base'}->pack(-side=>'bottom', -fill=>'x', -expand=>1);
$self->embed_base()->pack(-side=>'bottom', -fill=>'both', -expand=>1);

#Has to be last command in this function
$self->_after_expand();
}

=head3 _collapse

$object->_collapse(); 

Collapses the object. (Destroys the '$self->embed_base()' widget.) ! a nice place for a memory leak -> test!

=cut

#####################
# Objekt einklappen #
#####################
sub _collapse
{
my $self = shift;

#Has to be first command in this function
$self->_before_collapse();

#Collapse the thing
$self->embed_base()->packForget();
$self->{'_bottom_base'}->packForget();

#Has to be last command in this function
$self->_after_collapse();
}

=head3 _on_btn_prop_click

$object->_on_btn_prop_click(); 

Callback for clicking on the 'Properties'-button.


=cut


###########################################
# Callback fuer Klick auf Property-Button #
###########################################
sub _on_btn_prop_click
{
my $self= shift;
if (not ($self->propedit_classname() eq 'TagForm')) #Thats the already required standard PropertyEditor
	{
	require $self->propedit_classname().".pm";
	}
my $propedit = $self->propedit_classname()->new($self->docform(),$self->window_tag(), 1, $self, 0);
}

=head3 _grip_mouse_button1_pressed

$object->_grip_mouse_button1_pressed(); 

This is the callback for pressing the left mousebutton on the resizeing grip.

=cut

##############################
# MousButton pressed on grip #
##############################
sub _grip_mouse_button1_pressed
{
my $sender = shift;
my $self = shift;
my $X = shift;
my $Y = shift;

$self->{'_resizing_drag'} = 1;
$self->{'_resizing_drag_X'} = $X;
$self->{'_resizing_drag_Y'} = $Y;
$sender->configure(-cursor=>'bottom_right_corner');
}

=head3 _grip_mouse_button1_released

$object->_grip_mouse_button1_released(); 

This is the callback for releasing the left mousebutton on the resizing grip.

=cut

###############################
# MousButton released on grip #
###############################
sub _grip_mouse_button1_released
{
my $sender = shift;
my $self = shift;
my $X = shift;
my $Y = shift;

if (exists $self->{'_resizing_drag'} and defined $self->{'_resizing_drag'} and $self->{'_resizing_drag'}==1)
	{
	$self->{'_resizing_drag'} = 0;
	$self->idletasks;
	my $incr_x = $X - $self->{'_resizing_drag_X'};
	my $incr_y = $Y - $self->{'_resizing_drag_Y'};
	my $width = $self->embed_base()->Width();
	my $height = $self->embed_base()->Height();
	print "Width: $width    Height: $height \n"; #debug1
	$width += $incr_x;
	$height += $incr_y;
	$self->embed_base()->GeometryRequest($width,$height);
	print "NewWidth: $width    New Height: $height \n"; #debug1
	$sender->configure(-cursor=>'hand2');
	}
}


=head3 _before_save

$object->_before_save(); 

This function is called before the $self->{'_data'}-subbranch gets saved.

Use this for flatten complicated data structures into {'_data'}.

This Function is just the abstract base function, it has to be overwritten by the classes that derive from EmbObj

=cut

################
# _before_save #
################
sub _before_save
{
my $self= shift;
#No CODE In THIS ABSTRACT BASE CLASS
}

=head3 _after_save

$object->_after_save(); 

This function is called after the $self->{'_data'}-subbranch got saved.


This Function is just the abstract base function, it has to be overwritten by the classes that derive from EmbObj

=cut

################
# _after_save #
################
sub _after_save
{
my $self= shift;
#No CODE In THIS ABSTRACT BASE CLASS
}

=head3 _before_load

$object->_before_load(); 

This function is called before the $self->{'_data'}-subbranch is loaded from file.

This Function is just the abstract base function, it has to be overwritten by the classes that derive from EmbObj

=cut

################
# _before_load #
################
sub _before_load
{
my $self= shift;
#No CODE In THIS ABSTRACT BASE CLASS
}

=head3 _after_load

$object->_after_load(); 

This function is called after the $self->{'_data'}-subbranch is loaded from file.

Use this function to build up complicated data structures from the flat {'_data'}-subbranch

This Function is just the abstract base function, it has to be overwritten by the classes that derive from EmbObj

=cut

################
# _after_load #
################
sub _after_load
{
my $self = shift;
#No CODE In THIS ABSTRACT BASE CLASS
}

=head3 _before_expand

$object->_before_expand(); 

This function is called before the object gets expanded.

This Function is just the abstract base function, it has to be overwritten by the classes that derive from EmbObj

=cut

##################
# _before_expand #
##################
sub _before_expand
{
my $self = shift;
#No CODE In THIS ABSTRACT BASE CLASS
}

=head3 _after_expand

$object->_after_expand(); 

This function is called after the object gets expanded.

This Function is just the abstract base function, it has to be overwritten by the classes that derive from EmbObj

=cut

#################
# _after_expand #
#################
sub _after_expand
{
my $self = shift;
#No CODE In THIS ABSTRACT BASE CLASS
}

=head3 _before_collapse

$object->_before_collapse(); 

This function is called before the object gets collapsed.

This Function is just the abstract base function, it has to be overwritten by the classes that derive from EmbObj

=cut

####################
# _before_collapse #
####################
sub _before_collapse
{
my $self = shift;
#No CODE In THIS ABSTRACT BASE CLASS
}

=head3 _after_collapse

$object->_after_collapse(); 

This function is called after the object gets collapsed.

This Function is just the abstract base function, it has to be overwritten by the classes that derive from EmbObj

=cut

###################
# _after_collapse #
###################
sub _after_collapse
{
my $self = shift;
#No CODE In THIS ABSTRACT BASE CLASS
}

=head3 _after_backend_changed

$object->_after_backend_changed(); 

This function is called after the current backend changed

This Function is just the abstract base function, it has to be overwritten by the classes that derive from EmbObj

=cut

##########################
# _after_backend_changed #
##########################
sub _after_backend_changed
{
my $self = shift;
#No CODE In THIS ABSTRACT BASE CLASS
}

1; #So muss ein Package enden


=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
