#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package ImgObj;

require class::EmbObj; #It seems to be important that you write "class::EmbObj" with "use" or "require"

@ISA = qw(EmbObj); #It seems to be impotant that you write "EmbObj" in @ISA, but you have to write "Tk::Frame" or "Tk::Derived". ???
Tk::Widget->Construct('ImgObj');

=head1 NAME:

Widget: ImgObj

=head1 SYNOPSIS:

use class::ImgObj;

=head1 DIRECTORY/FILE:

The file ImgObj.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

This is a widget derived from class::EmbObj that gets embeded into the documents text widget in the DocForm object (=document window).

This class is for embeding images into the document.

This class is in a very basic stage, it's just for demonstarting that it's quiet easy to add new embeded objects to e:doc.

=head1 STATUS:

Coding: 10% done

Documentation: up-to-date

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

All properties derived from the class EmbObj.

$self->{'_data'}->{'img_name'} #The filepath/name of the image

$self->{'_data'}->{'img_type'} #The type of the image (GIF,JPG....)

NOTE: The following attributes are used for the Tk widgets building the GUI and are accessed directly (no get/set methods):

All GUI attributes derived from the class EmbObj.

=head1 METHODS:

=head2 Constructors:

=head3 build_ui

Here your widget could build up all its own widgets and gets initialised

=cut

################
# Fill the GUI #
################
sub build_ui
{
my $self = shift;
my $embed_base = $self->embed_base();

#If there is no stored data (user-creation)
if (not defined $self->img_name() or $self->img_name() eq '')
	{
	my $filename = $self->docform()->top()->getOpenFile();
	if (Utils::extract_path($filename) eq $self->docform()->prjform()->path() or
		Utils::extract_path($filename) eq $self->docform()->path())
		{
		$filename = Utils::extract_filename($filename);
		}
	$self->img_name ( $filename );
	}
else
	{
	#Getting the filename out of the data (if it is defined there) or show a dialog (If it isnt defined there)
	my $filename = $self->img_name();
	if (Utils::extract_path($filename) eq $self->docform()->prjform()->path() or 
		Utils::extract_path($filename) eq $self->docform()->path())
		{
		$filename = Utils::extract_filename($filename);
		$self->img_name( $filename );
		}
	elsif (not -f $self->img_name())
		{
		$self->img_name(Utils::process_filename_in_document($self->img_name(), $self->docform()));
		}
	}

#Setting the caption
if (defined $self->img_name() and (not defined $self->caption() or $self->caption() eq ''))
	{
	my $caption = 'New image';
	if ($self->img_name() =~ /^(.*)[\\\/](.*?)$/ )
		{
		$caption = $2;
		}
	$self->caption($caption );
	}

#Check for the image type
my $extension2type = {
							'GIF'=>'gif',
							'BMP'=>'(bmp|xbm|xpm)',
							'JPG'=>'(jpg|jpeg)'
							};

my $found_type = 0;
foreach my $type (keys %{$extension2type})
	{
	my $ext = $extension2type->{$type};
	if ($self->img_name() =~ /\.$ext$/i)
		{
		$self->img_type($type);
		$found_type = 1;
		last;
		}
	}

#If type was known -> create the widget that displays it
if (-f $self->img_name() and $found_type == 1)
	{
	#Create the image
	if (($self->img_type() eq 'BMP') or ($self->img_type() eq 'GIF') or ($self->img_type() eq 'JPG')) 
		{
		require Tk::Photo;
		$self->{'_photo'} = $embed_base->Photo(-file=>Utils::process_filename_in_document($self->img_name(), $self->docform() ) );
		$self->{'_img_base'} = $embed_base->Label(-image=>$self->{'_photo'})->pack(-expand=>1, -fill=>'both', -anchor=>'nw');
		}
	}
else
	{
	$self->{'_img_base'} = $embed_base->Label(-text=>"ERROR: Could not create image \"".$self->img_name()."\"" )->pack();
	}
}

=head2 Destructors:

=head3 do_detstroy

$object->do_detsroy(); 

This "destructor" method is binded to the "<DESTROY>" event and should be performed whenever
the element is destroyed. It should not be called directly.

=cut

###################################################
# Destruktor zum sauberen aufraeumen des Objektes #
###################################################
sub do_destroy
{
my $self =shift;
print 'Image '.$self->img_name()." was destroyed \n"; #Debug
$self->SUPER::do_destroy();
}

=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 img_name

$filename = $object->img_name(); #gets the filename of the image to be displayed

$object->img_name($filename); #sets the filename of the image to be displayed

=cut

### img_name ###
sub img_name
{
my $self=shift;
@_ ? $self->{'_data'}->{'img_name'} = shift : return $self->{'_data'}->{'img_name'};
}

=head3 img_type

$filetype = $object->img_type(); #gets the filetype of the image to be displayed

$object->img_type($filetype); #sets the filetype of the image to be displayed

=cut

### img_type ###
sub img_type
{
my $self=shift;
@_ ? $self->{'_data'}->{'img_type'} = shift : return $self->{'_data'}->{'img_type'};
}

=head2 Other public methods:

=head3 export_to_backend

my @export_text = $object->export_to_backend(); 

This function exports the content of the object to the current backend.

The function is called in the document-exporting process, whenever an embeded object
is found. 

It returns the text for exporting the object.

=cut

##################################
# Export to the current backend. #
##################################
sub export_to_backend
{
my $self = shift;

my $result = '';

#Get the export definition

my $backend_field = $self->process_backend_field($self->{'_backend'}->{'main'});

if ($backend_field =~ /%FILENAME%/)
	{
	my $filename = $self->img_name();
	$backend_field =~ s/%FILENAME%/$filename/g ;
	}

$result = $backend_field;

return $result;
}


=head2 Private methods:

=head3 _before_save

$object->_before_save(); 

This function is called before the $self->{'_data'}-subbranch gets saved.

Processing absolute filenames into relative ones if possible

=cut

################
# _before_save #
################
sub _before_save
{
my $self = shift;
#CALLING THE SUPER-CLASS (DO NOT CHANGE THIS LINE)
$self->SUPER::_before_save();

my $filename = $self->{'_data'}->{'img_name'};
my $filepath = Utils::extract_path($filename);
if (defined $filepath and not ($filepath eq ''))
	{
	if ( ($filepath eq $self->docform()->prjform()->path()) or ($filepath eq $self->docform()->path() ) )
		{
		$filename = Utils::extract_filename($filename);
		if (defined $filename and not ($filename eq ''))
			{
			$self->{'_data'}->{'img_name'} = $filename;
			}
		} 
	}

}



1; #So muss ein Package enden


=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
