#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package NumLabel;


require class::NumList;
require Tk::Label;

@ISA = qw(Tk::Derived Tk::Label);
Tk::Widget->Construct('NumLabel');

use strict;

=head1 NAME:

Widget: NumLabel

=head1 SYNOPSIS:

use class::NumLabel;

=head1 DIRECTORY/FILE:

The file NumLabel.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

This is a widget derived from Tk::Label that gets embeded into the documents text widget in the DocForm object (=document window).

This NumLabel object is one item of the numbering engine (NumList.pm). Each NumLabel is added to ONE (never more) NumList and (in most cases)
embeded into the text widget to show the numbering text (f. ex. "3.1.1") there. The NumLabels are menaged by their NumList and kept in a sorted list
(sorted by the place they are embeded in the text widget).

If the NumLabel is assocaited with a EmbObj-derived object, it is not embeded into the text widget, so the position
of the assocated EmbObj-derived object is taken instead of the NumLabels position.

Each numlabe has its "level"-attribute which distincts f. ex. "1" from "1.1".

The NumLabel/NumList - engines are used to work for:

(+) Numbering paragraphs

(+) Numbering several kinds of EmbObj-derived objects (Tables, Images...)

(+) Numbering of "Numbered lists" (environments)

(+) Numbering of "Bulletlists" (environments) [Here the NumLabels do not show numberings, they just display the bullets].

This object is coded as an Perl/Tk widget, so it has no Constructor new, it has the standard Tk functions to initialise itself (Populate,ClassInit,...)
The function Populate() is used like an constructor here.

=head1 STATUS:

Coding: 99% done

Documentation: up-to-date

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

$self->{'show_bullet'} #If set to 1 the NumLabel displays the bullet defined in $self->numlist()->bullet_img()

$self->{'show_number'} #If set to 1 the NumLabel shows the numbering

$self->{'text'} #The text to be shown (f. ex. "Chapter 1:")

$self->{'text_widget'} #Reference to the textwidget where the NumLabel is embeded

$self->{'numlist'} #A reference to the NumList the NumLabel belongs to

$self->{'pre_number_text'} #The text that should be shown before the number

$self->{'post_number_text'} #The text that should be shown after the number

$self->{'associated_tag'} #The name of the the NumLabel is associated with (PAR_STYLES or EMBOBJ or ENV_STYLES)

$self->{'window_tag'} #The name of the tag the NumLabel itself is tagged with in the text widget

=head1 METHODS:

=head2 Class Methods:

=head3 ClassInit($class, $parentwindow)

! Not called directly !

This "constructor" initialises a new object of the class NumLabel and calls ClassInit() of the Superclass. 

=cut

##########################
# Klassen-Initalisierung #
##########################
sub ClassInit
{
# Tk-spezifischer Code
my ($class,$mainwindow)=@_; # Klassenname und Mainwindow
$class->SUPER::ClassInit($mainwindow); # Aufruf des Vorfahren
print "Object $class created.\n"; #debug
return $class;
}


=head2 Constructors:

=head3 Populate

$object = NumLabel(%options); 

This "constructor" sets the options of the new object of the class NumLabel. 

The binding of events and the creation of the associated Baloonhelp is done here too.

The keys/values in %options: 

(+) '-show_bullet' => If set to 1 the NumLabel displays the bullet defined in $self->numlist()->bullet_img()

(+) '-show_number' => If set to 1 the NumLabel shows the numbering

(+) '-text_widget' => Reference to the textwidget where the NumLabel is embeded

(+) '-numlist' => A reference to the NumList the NumLabel belongs to

(+) '-pre_number_text' => The text that should be shown before the number

(+) '-post_number_text' => The text that should be shown after the number

(+) '-associated_tag' => The name of the the NumLabel is associated with (PAR_STYLES or EMBOBJ or ENV_STYLES)

(+) '-window_tag' => The name of the tag the NumLabel itself is tagged with in the text widget

=cut

###########################################################################
# Uebergabe, ev. Filterung und Weiterleitung der "-option=>'value'"-Paare #
###########################################################################
sub Populate
{
my ($self,$args)=@_; #Auslesen der $self-Referenz und der option/value-paare
$self->SUPER::Populate($args); #Weiterleitung an Vorfahr (Basisklasse)

#Initialiserung beginnt #
#Eigenschaften auslesen
$self->show_number( 1 ); #sets the default
$self->show_bullet( 0 ); #sets the default
$self->text( '?' ); #Default
$self->text_widget( delete $args->{'-text_widget'} ); #Zeiger auf Text-Widget
$self->numlist( delete $args->{'-numlist'} ); #Zeiger auf Verwaltungsliste NumLabelList
$self->level( delete $args->{'-level'} ); #Gliederungsebene
$self->pre_number_text( delete $args->{'-pre_number_text'} ); #Text vor Nummer
$self->post_number_text( delete $args->{'-post_number_text'} ); #Text nach Nummer
$self->associated_tag( delete $args->{'-associated_tag'} ); #Name des PAR_STYLES-Tag, zu dem das Numlabel gehrt
$self->window_tag( undef ); #Name des umgebenden Tags des Numlabels
$self->show_number( delete $args->{'-show_number'} ); #Ob eine Nummer angezeigt werden soll
$self->show_bullet( delete $args->{'-show_bullet'} ); #Ob ein Bild statt der Nummer angezeigt werden soll
if ($self->show_number()==1)
	{
	$self->configure(-textvariable=>$self->ref_text());
	}
if ($self->show_bullet()==1)
	{
	$self->configure(-image=>$self->numlist()->bullet_img());
	}
#Destroy binden
$self->bind('<Destroy>' => sub{ $self->do_destroy()}); #Aufraeumroutine binden
#Initialiserung endet #
return $self;
}

=head3 on_create

$object->on_create($do_not_sort); 

This method has to be called after the NumLabel has been correctly created, embeded into the textwidget and tagged with his own tag.
Afterwards the 'window_tag' attribute should be set to this tagname by using the method '$obj->window_tag($tagname)'. After that is done,
you have to call this '$obj->on_create()' method to fully initialise the NumLabel.

This is VERY IMPORTANT because this 'on_create' method performs the registration of the NumLabel to its NumList and
can only be done AFTER the surrounding tag of the NumLabel is set correctly.

Parameters:

(+) $do_not_sort: If set to 1, the NumLabel is not automatically sorted into the NumList (for document opening where many NumLabels are created. 
    Its enough to sort the list once when all NumLabels are created).

=cut

################################################################################
# Wird aufgerufen nachdem Tag rund um NumLabel erzeugt und weitergegeben wurde #
################################################################################
sub on_create
{
my $self= shift;
my $do_not_sort= shift;
#Objekt in Nummernliste eintragen (Kann erst gemacht werden nachdem $self->window_tag() gesetzt wurde
if (defined $self->window_tag())
	{
	$self->numlist()->add_numlabel($self,$do_not_sort); #Traegt sich selbst in NumLabelList ein
	}
else
	{
	print "ERROR: No tag was set around the NumLabel object !\n";
	}
}

=head2 Destructors:

=head3 DESTROY; 

The standard automatically called PERL destructor for objects.

=cut

#############################
# Automatic Perl destructor #
#############################
sub DESTROY
{
my $self = shift;
print "Object $self destroyed.\n"; #debug
}

=head3 do_detstroy

$object->do_detsroy(); 

This "destructor" method is binded to the "<DESTROY>" event and should be performed whenever
the element is destroyed. It should not be called directly.

=cut

###################################################
# Destruktor zum sauberen aufraeumen des Objektes #
###################################################
sub do_destroy
{
my $self = shift; 
print 'Item at Level '.$self->level()." was destroyed \n"; #Debug
$self->numlist()->delete_NumLabel($self); #Entfernt sich selbst aus NumLabelList 
$self = undef;
}

=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 text

$text = $object->text(); #gets the text that should be displayed on the NumLabel

$object->text($text); #sets the text that should be displayed on the NumLabel

=cut

### text ###
sub text
{
my $self=shift;
@_ ? $self->{'text'} = shift : return $self->{'text'};
}

=head3 ref_text

$ref_text = $object->ref_text(); #gets a reference to the  text that should be displayed on the NumLabel (!READ-ONLY!)

=cut

### ref_text ###
sub ref_text
{
my $self=shift;
return \$self->{'text'};
}

=head3 show_number

$show_number = $object->show_number(); #gets the value of the 'show_number' attribute

$object->show_number($show_number); #sets the value of the 'show_number' attribute

=cut

### show_number ###
sub show_number
{
my $self=shift;
@_ ? $self->{'show_number'} = shift : return $self->{'show_number'};
}

=head3 show_bullet

$show_bullet = $object->show_bullet(); #gets the value of the 'show_bullet' attribute

$object->show_bullet($show_bullet); #sets the value of the 'show_bullet' attribute

=cut

### show_bullet ###
sub show_bullet
{
my $self=shift;
@_ ? $self->{'show_bullet'} = shift : return $self->{'show_bullet'};
}

=head3 text_widget

$ref_text_widget = $object->text_widget(); #gets a referenece to the Text widget the NumLabel is embeded in

$object->text_widget($ref_text_widget); #sets a referenece to the Text widget the NumLabel is embeded in

=cut

### text_widget ###
sub text_widget
{
my $self=shift;
@_ ? $self->{'text_widget'} = shift : return $self->{'text_widget'};
}

=head3 window_tag

$tag_name = $object->window_tag(); #gets the name of the tag the NumLabel is tagged with

$object->window_tag($tag_name); #sets the name of the tag the NumLabel is tagged with

=cut

### window_tag ###
sub window_tag
{
my $self=shift;
@_ ? $self->{'window_tag'} = shift : return $self->{'window_tag'};
}

=head3 associated_tag

$tag_name = $object->associated_tag(); #gets the name of the tag the NumLabel is associated with

$object->associated_tag($tag_name); #sets the name of the tag the NumLabel is associated  with

=cut

### associated_tag ###
sub associated_tag
{
my $self=shift;
@_ ? $self->{'associated_tag'} = shift : return $self->{'associated_tag'};
}

=head3 pre_number_text

$pre_number_text = $object->pre_number_text(); #gets the value of the 'pre_number_text' attribute

$object->pre_number_text($pre_number_text); #sets the value of the 'pre_number_text' attribute

=cut

### pre_number_text ###
sub pre_number_text
{
my $self=shift;
@_ ? $self->{'pre_number_text'} = shift : return $self->{'pre_number_text'};
}

=head3 post_number_text

$post_number_text = $object->post_number_text(); #gets the value of the 'post_number_text' attribute

$object->post_number_text($post_number_text); #sets the value of the 'post_number_text' attribute

=cut

### post_number_text ###
sub post_number_text
{
my $self=shift;
@_ ? $self->{'post_number_text'} = shift : return $self->{'post_number_text'};
}


=head3 level

$level = $object->level(); #gets the value of the 'level' attribute

$object->level($level); #sets the value of the 'level' attribute

=cut

### level ###
sub level
{
my $self=shift;
@_ ? $self->{'level'} = shift : return $self->{'level'};
}

=head3 numlist

$ref_numlist = $object->numlist(); #gets a reference to the associated NumList object

$object->numlist($ref_numlist); #sets a reference to the associated NumList object

=cut

### numlist ###
sub numlist
{
my $self=shift;
@_ ? $self->{'numlist'} = shift : return $self->{'numlist'};
}


=head2 Other public methods:

=head3 get_current_line

$line = $object->get_current_line(); 

This returns the line number where the NumLabel widget is situated inside of the text widget it belongs to.

=cut

###############################################
# Liefert Zeilennummer im Text-Widget zurueck #
###############################################
sub get_current_line
{
my $self = shift;
my @range = $self->text_widget()->tagRanges($self->window_tag()); # Numlabel ist mit Tag markiert um Zeile eruieren zu koennen
my $anfang = shift(@range); #Anfangsindex auslesen
$anfang =~ s/.\d*$//go; #Zeile herausholen
return $anfang; #Wert zurueckliefern
}

=head3 renumber

$object->renumber(); 

Cause all NumLabels in the associated NumList to be renumbered.

=cut

########################
# Neu durchnummerieren #
########################
sub renumber
{
my $self=shift;
$self->numlist()->renumber();
}


=head2 Private methods:

nothing here yet.

=cut

1; #So muss ein Package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
