#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package PrjForm;

@ISA = qw(BaseForm);

=head1 NAME:

Object: PrjForm

=head1 SYNOPSIS:

use class::PrjForm;

=head1 DIRECTORY/FILE:

The file PrjForm.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

The object PrjForm encapsulates the project window. It is now coded as a normal non-visual Perl class
that creates an user interface in the $self->{'ui'} branch. It might be more clever to (later) transform it
into a Toplevel-derived widget.

This form class is derived form the class 'BaseForm' that builds up the basic GUI elements
and does form-registrations and deregistartaions in the global @main::forms array.
For viewing this inherited functionality see the files '/class/BaseForm.pm' and '/class/ui/uiBaseForm.pm'.

Now the GUI buildup functions are stored in the package "./ui/uiPrjForm.pm".

=head1 STATUS:

Coding: 10% done

Documentation: up-do-date

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

$self->{'authors'} = [...] #A Reference to an array that holds the authors (Will later become a reference to an hash, when more info per author is stored)

$self ->{'file_ext_array'} #An Array of arrays of arrays that is used for the getOpenFile dialog .

$self->{'filetypes'}->{$name} #A hash that holds info apout the filetypes

$self->{'filetypes'}->{$name}->{'extensions'} = [...] # A reference to an array of extensions of this filetype

$self->{'filetypes'}->{$name}->{'caption'} # The caption (full name) that is used in the tree for that filetype

$self->{'files'}->{$file_name}; #Reference to a hash that holds the additional info to an registered file

$self->{'files'}->{$file_name}->{'type'} #Could be 'LOCAL' for autoread files in the project directory or 'LINK' if a file from somewhere else on the disk was linked into the project

$self->{'files'}->{$file_name}->{'size'} #The filesize on the last refreshing

$self->{'files'}->{$file_name}->{'changed'} #The date/time of the last filechange

$self->{'files'}->{$file_name}->{'file_type'} #The filetype that file belongs to

$self->{'files'}->{$file_name}->{'args'}->{$i}->{'value'} #The value of the argument

$self->{'files'}->{$file_name}->{'args'}->{$i}->{'type'};  #The type of the argument

$self->{'files'}->{$file_name}->{'args'}->{$i}->{'default'};  #The default value of the argument

$self->{'files'}->{$file_name}->{'args'}->{$i}->{'settings'};  #Additional info for the argument

$self->{'files'}->{$file_name}->{'args'}->{$i}->{'hint'};  #The hint-text of the argument

$self->{'current_file'}->{'args'}->{'arg_number'} #The number of arguments of the selected file

$self->{'current_file'}->{'args'}->{$i}->{'value'} #The value of this argument of the selected file

$self->{'current_file'}->{'args'}->{$i}->{'type'};  #The type of the argument of the selected file

$self->{'current_file'}->{'args'}->{$i}->{'default'};  #The default value of the argument of the selected file

$self->{'current_file'}->{'args'}->{$i}->{'settings'};  #Additional info for the argument of the selected file

$self->{'current_file'}->{'args'}->{$i}->{'hint'};  #The hint-text of the argument of the selected file

$self->{'prj_parts'};  #A reference to an hash of arrays. Each hash-key represents the name of a project-part, each array contains the parts content in the clipboard format

$self->{'prj_args'};  #A hash that holds the project arguments

$self->{'prj_args'}->{'arg_number'};  #The number of arguments

$self->{'prj_args'}->{$i}->{'name'};  #The name of the argument

$self->{'prj_args'}->{$i}->{'type'};  #The type of the argument

$self->{'prj_args'}->{$i}->{'default'};  #The default value of the argument

$self->{'prj_args'}->{$i}->{'settings'};  #Additional info for the argument

$self->{'prj_args'}->{$i}->{'hint'};  #The hint-text of the argument

$self->{'prj_args'}->{$i}->{'value'};  #The value of the argument

$self->{'ui'}->{'app_window'} #A referenece to the MainWindow

$self->{'ui'}->{'content'} #A referenece to the content outline (Tree widget)

$self->{'ui'}->{...the rest of this branch...} #References to the widgets that build the user interface

=head1 METHODS:

=head2 Class Methods:

=head2 Constructors:

=head3 new

$object = PrjForm->new(); 

This constructor creates a new object of the class PrjForm. 

This calls class::ui::uiPrjForm::build_ui($self) which builds the window and its containing widgets.

The window is a MainWindow() which is the applications main_window (later refered as $app_window)

=cut

###############
# Konstruktor #
###############
sub new
{
my ($class) = shift;

#Call the contructor of the base class
my $self = $class->SUPER::new('PROJECT_FORM');

#Some initialisations
$self->{'authors'} = [];
$self->{'prj_parts'} = {};
$self->{'prj_args'} = {};
$self->{'current_file'}->{'args'} = {};
$self->{'files'} = {};

#Set the project unchanges
$self->set_content_unchanged();

#Initialise the project arguments hash
$self->_init_args('PROJECT_ARGS', undef );

#Initialise the file arguments hash
$self->_init_args('CURRENT_FILE', undef );

#Build up the user interface
uiPrjForm::build_ui($self);

$self->content()->focus();

$self->app_window()->deiconify();

return $self;
}

=head2 Destructors:

=head3 DESTROY; 

The standard automatically called PERL destructor for objects.
Unfortionatly it is called very late (when the whole application terminates.)

=cut

#############################
# Automatic Perl destructor #
#############################
sub DESTROY
{
my $self = shift;
print "Object $self destroyed.\n"; #debug
}

=head3 do_destroy

$object ->do_destroy(); 

This destructor is binded to the protocol-message WM_DELETE_WINDOW
and performs the 'unsaved changes'-check, destroys the MainWindow manually
and then destroys the PrjForm object manually too.

=cut

##############
# Destruktor #
##############
sub do_destroy
{
my $self = shift;

$self->{'ui'}->{'content'}->configure(-command=>undef, -browsecmd=>undef);

my $ok = Parts::save_global_parts(); 

#Destroy all opened docform objects
foreach my $docform (@main::docforms)
	{
	$docform->top()->deiconify();
	$docform->top()->raise();
	$docform->do_destroy();
	}

#If the document has unsaved changes
if ($self->is_content_changed())
	{
	my $btn = $self->app_window()->Dialog(-title=>'Project not saved yet !', -text=>"There are unsaved changes in the current document.\nDo you want to save them to disk ?", -buttons=>['Yes','No'], -bitmap=>'question')->Show();
	if ($btn eq 'Yes') 
		{
		#save it
		$self->project_save();
		} 
	}

$main::prjform = undef;

#CALL THE DESTRUCTOR OF THE SUPERCLASS  AT THE END
$self->SUPER::do_destroy();

#Finnish the application
exit();
}

=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 app_window

$ref_app_window = $object->app_window(); #gets a reference to the main application window (READ-ONLY)

=cut

### app_window ###
sub app_window
{
my $self=shift;
return $self->{'ui'}->{'app_window'};
}

=head3 content

$ref_content_outline = $object->content(); #gets a reference to the projects content outline (=tree widget) (READ-ONLY)

=cut

### content ###
sub content
{
my $self=shift;
return $self->{'ui'}->{'content'};
}


=head3 prj_parts_root

$ref_root = $object->prj_parts_root(); #gets a reference to the root of the hash of arrays that holds the parts

=cut

### prj_parts_root ###
sub prj_parts_root
{
my $self=shift;
return $self->{'prj_parts'};
}

=head3 prj_parts_names

@prj_parts_names = $object->prj_parts_names(); #gets an array containing the names of all parts in this project (READONLY)

=cut

### prj_parts_names ###
sub prj_parts_names
{
my $self=shift;
return keys %{$self->{'prj_parts'}};
}

=head3 ref_prj_part

$ref_content = $object->ref_prj_part( $name ); #gets a reference to an array containing the part in clipboard-format

$object->ref_prj_part( $name, $ref_content ); #stores a reference to an array containing the part in clipboard-format

Returns undef, if the given part doesn't exist.

Parameters:

(+) $name: The name of the part (has to be an unique string)

(+) $ref_content: A reference to an array holding the content of a part in clipboard format

=cut

### ref_prj_part ###
sub ref_prj_part
{
my $self=shift;
my $name = shift;
my $ref_content = shift;

#Default error-return value
my $return_value = undef;

#If a reference to a part-content was given (SET a value)
if (defined $ref_content and defined $name)
	{
	#Store it
	$self->{'prj_parts'}->{$name} = $ref_content;
	}
else #No reference (GET a value)
	{
	#If a part with this name exists
	if (exists $self->{'prj_parts'}->{$name})
		{
		#Return the reference to the content
		$return_value = $self->{'prj_parts'}->{$name};
		}
	}

return $return_value;
}

=head3 prj_args_number

$prj_args_number = $object->prj_args_number(); #gets the number of project arguments

$object->prj_args_number($prj_args_number); #sets the number of project arguments

=cut

### prj_args_number ###
sub prj_args_number
{
my $self=shift;
@_ ? $self->{'prj_args'}->{'arg_number'} = shift : return $self->{'prj_args'}->{'arg_number'};
}


=head3 file_args_number

$file_args_number = $object->file_args_number(); #gets the number of file arguments

$object->file_args_number($file_args_number); #sets the number of file arguments

=cut

### file_args_number ###
sub file_args_number
{
my $self=shift;
@_ ? $self->{'current_file'}->{'args'}->{'arg_number'} = shift : return $self->{'current_file'}->{'args'}->{'arg_number'};
}


=head3 file_arg_value

$object->file_arg_value($filename, $index, $new_value); #Set a file args value

$value = $object->file_arg_value($filename, $index); #Get a file args value

This method gets or sets the value of a file argument.

If the filename isn't in the project or does not have that many arguments, it returns undef.

Parameter:

(+) $filename : The filename the argument belongs to

(+) $index : The index of the argument

(+) $value : The value to be set

=cut

### file_arg_value ###
sub file_arg_value
{
my $self=shift;
my $filename = shift;
my $index = shift;
my $value = shift;

$return_value = undef;

if ($main::project->{'FILE_ARGS'}->{'arg_number'} >= $index)
	{
	if ( not exists $self->{'files'}->{$filename}->{'args'})
		{
		$self->_init_args('FILE_ARGS', $filename);
		}
	#SET A VALUE
	if (defined $value)
		{
		$self->{'files'}->{$filename}->{'args'}->{$index}->{'value'} = $value;
		if ($self->{'current_file'}->{'args'}->{$index}->{'type'} eq 'MULTILINE')
			{
			$self->{'current_file'}->{'args'}->{$index}->{'_entry'}->delete('1.0', 'end');
			$self->{'current_file'}->{'args'}->{$index}->{'_entry'}->insert('1.0', $value);
			}
		$return_value = $value;
		}
	else #GET THE VALUE
		{
		$return_value = $self->{'files'}->{$filename}->{'args'}->{$index}->{'value'};
		}
	}
else
	{
	print "ERROR: Tried to set value of project-argument number $index, \nbut the current project has not so many arguments defined.\n";
	}

return $return_value;
}

=head3 authors

$object->authors($ref_authors_list); #Set the reference to the authors array

$ref_authors_list = $object->authors(); #Get the reference to the authors array

=cut

### authors ###
sub authors
{
my $self=shift;
my $ref_authors_list = shift;

$return_value = undef;

if (defined $ref_authors_list) #SET THE NEW VALUE
		{
		$self->{'authors'} = $ref_authors_list;
		#Update the widget from the new list
		$self->{'ui'}->{'authors_edit'}->delete('1.0','end');
		my @list = @{$ref_authors_list};
		foreach $author (@list)
			{
			$self->{'ui'}->{'authors_edit'}->insert('end',$author."\n");
			}
		#Return the reference again
		$return_value = $ref_authors_list;
		}
	else #GET THE VALUE
		{
		#Update the list from the widget
		uiPrjForm::authors_changed(undef, $self); 
		#Return the reference
		$return_value = $self->{'authors'};
		}
return $return_value;
}

=head2 Other public methods:

=head3 project_exit

$object->project_exit();

Terminate the applictaion. 

=cut

############
### EXIT ###
############
sub project_exit
{
my $self=shift;

#Calling the Destructor-method of the PrjForm object
$self->do_destroy();
}

=head3 set_prj_arg_value

$object->set_prj_arg_value($index, $value); 

Parameter:

(+) $index : The index of the argument

(+) $value : The value to be set

=cut

#########################
### set_prj_arg_value ###
#########################
sub set_prj_arg_value
{
my $self=shift;
my $index = shift;
my $value = shift;
if ($main::project->{'PROJECT_ARGS'}->{'arg_number'} >= $index )
	{
	$self->{'prj_args'}->{$index}->{'value'} = $value;
	if ($self->{'prj_args'}->{$index}->{'type'} eq 'MULTILINE')
		{
		$self->{'prj_args'}->{$index}->{'_entry'}->delete('1.0', 'end');
		$self->{'prj_args'}->{$index}->{'_entry'}->insert('1.0', $value);
		}
	}
else
	{
	print "ERROR: Tried to set value of project-argument number $index, \nbut the current project has not so many arguments defined.\n";
	}

}

=head3 project_new

$object->project_new(); 

Resets the PrjForm object for starting a new project. 

=cut

###################
### New project ###
###################
sub project_new
{
my $self = shift;

my $filename;
my $path;
my $short_filename;

#If the document has unsaved changes
if ($self->is_content_changed())
	{
	my $btn = $self->app_window()->Dialog(-title=>'Project not saved yet !', -text=>"There are unsaved changes in the current document.\nDo you want to save them to disk ?", -buttons=>['Yes','No'], -bitmap=>'question')->Show();
	if ($btn eq 'Yes') 
		{
		#save it
		$self->project_save();
		}
   else
		{
		goto EXIT;
		}
	}

while (not defined $filename)
	{
	$filename = $self->app_window()->getSaveFile(-filetypes=>[['e:doc projects', ['.edp']]], -initialfile => 'newproject', -defaultextension => '.edp');
	$path = Utils::extract_path($filename);
	$short_filename = Utils::extract_filename($filename);
	opendir (DIR, $path) || die "Cannot open $path \n";
	my $empty = 1;
	#Retrieving file information
	foreach my $file (readdir(DIR))
		{
		if (not ($file eq '.') and not($file eq '..'))
			{
			$empty = 0;
			last;
			}
		}

	if ($empty == 0)
		{
		closedir (DIR);
		$self->message('This directory is not empty!', 'Error:');
		goto EXIT;
		}
	closedir (DIR);
	}

chdir $path;

#Some initialisations
$self->{'authors'} = [];
$self->{'prj_parts'} = {};
$self->{'prj_args'} = {};
$self->{'current_file'}->{'args'} = {};
$self->{'files'} = {};

$self->filename($filename); #Filename

#Initialise the project arguments hash
$self->_init_args('PROJECT_ARGS', undef );

#Initialise the file arguments hash
$self->_init_args('CURRENT_FILE', undef );

#Refresh the projects content (should be empty)
$self->project_refresh();

#Save the thing
$self->project_save();

#Refresh the window caption
$self->update_window_caption();

$self->content()->focus();

EXIT:
$self->app_window()->Unbusy();

return $self->filename();
}

=head3 project_open

$object->project_open([$filename]); 

Calls InOut::project_open($self, [$filename]), which opens a new project into this PrjForm.

Parameters:

(+) $filename: OPTIONAL, if this is set to undef or not set, the filename is queried by a FileOpen dialog

=cut

####################
### Open project ###
####################
sub project_open 
{
my $self = shift;
my $filename = shift;

$self->app_window()->Busy();

#If the document has unsaved changes
if ($self->is_content_changed())
	{
	my $btn = $self->app_window()->Dialog(-title=>'Project not saved yet !', -text=>"There are unsaved changes in the current document.\nDo you want to save them to disk ?", -buttons=>['Yes','No'], -bitmap=>'question')->Show();
	if ($btn eq 'Yes') 
		{
		#save it
		$self->project_save();
		}
   else
		{
		goto EXIT;
		}
	}

#Read the stored info from disk first
InOut::project_open($self, $filename);

#Refresh the content now !!!VERY IMPORTANT BECAUSE THIS READS THE DIRECTORY AND INITIALISES EVERYTHING !!!
$self->project_refresh();

#Select the root element
$self->_select_in_content('<ROOT>');

chdir $self->path();

EXIT:
$self->app_window()->Unbusy();
}

=head3 project_save

$object->project_save(); 

Calls InOut::project_save($self), which saves the PrjForm into the ALREADY EXISTING file

=cut

####################
### Save project ###
####################
sub project_save 
{
my $self = shift;
$self->app_window()->Busy();

#Write back the currently in {'current_file'}->{'args'} stored adata
$self->_write_back_current_file_args();

#Refresh some things
$self->project_refresh();

#Save the thing
InOut::project_save($self);

$self->app_window()->Unbusy();
}


=head3 open_empty_docform

$ref_docform = $object->open_empty_docform(); 

Opens a new document window (DocForm) and returns a reference to it.

=cut

###############################
# Ein Dokumentfenster oeffnen #
###############################
sub open_empty_docform
{
my $self=shift;
$self->app_window()->Busy();

#Change to the projects path (for safety)
chdir $self->path();

#Create the DocForm object
my $docform = DocForm->new($self);

$self->app_window()->Unbusy();

return $docform;
}

=head3 open_docform_file

$ref_docform = $object->open_docform_file($filename); 

Opens a new document window (DocForm), opens a document into it and returns a reference to it.

Parameters:

(+) $filename: The filename of the document to open (incl. path)

=cut

###############################################
# Ein Dokumentfenster oeffnen und Datei laden #
###############################################
sub open_docform_file
{
my $self=shift;

my $filename = shift;

$self->app_window()->Busy();

my $docform = $self->open_empty_docform();

$self->app_window()->Busy();

if ($self->{'files'}->{$filename}->{'type'} eq 'LOCAL')
	{
	$filename = $self->path().$main::path_separator.$filename;
	}

$docform->file_open($filename);

$self->app_window()->Unbusy();

return $docform;
}

=head3 new_file_from_template

$new_filename = $object->new_file_from_template(); 

This method queries first for a template, than queries the filename
the new file should have.

Next it copies the template to the new filename and initialise the
file argument and refreshes the projects content.

After all that is executed the newly created document is opened.

The method returns the filename of the created file if all went ok
and undef if there was an error or it was canceled.

=cut

#########################################
# Create a new document from a template #
#########################################
sub new_file_from_template
{
my $self=shift;
my $filename = undef;

$self->app_window()->Busy();

#Query a template
my $template_filename = TemplateChooser->new($self);

#If the Template-dialog was not canceled
if (defined $template_filename and not ($template_filename eq ''))
	{
	#Query a new and non-existing filename
	my $new_filename;
	my $ext;
	if ( $template_filename =~ /\.([^.]+)$/ )
		{
		$ext = ".".$1;
		}
	while(not defined $new_filename or $new_filename eq '' or (-f $self->path()."/".$new_filename.$ext) )
			{
			$new_filename = $self->dialog('Please enter new filename without extension:', 'New filename') ;
			}

	#Copy the template file to the new filename (INSIDE THE CURRENT PROJECT)
	$from = $main::config->{'GENERAL'}->{'templates_dir'}."/".$template_filename;
	$to = $self->path()."/".$new_filename.$ext;
	use File::Copy;
	copy ($from, $to);
   if (not -f $to)
		{
		$self->message('File could not be created !', 'Error:');
		goto EXIT;
		}
	
	#Refresh the projects view
	$self->project_refresh();

	#Set the 'changed' flag
	$self->set_content_changed();

	#Select the newly added item in the treeview
	$filename = $new_filename.$ext;
	my $filetype = $self->{'files'}->{$filename}->{'filetype'};
	$self->_select_in_content("<ROOT>~<$filetype>~<$filename>");

	#Change to the projects directory (for safety)
	chdir $self->path();
	#Open the file
	$self->_content_dbl_click();
	}

EXIT:
$self->app_window()->Unbusy();
return $filename;
}

=head3 new_document_from_scratch

$new_filename = $object->new_document_from_scratch(); 

This method queries the new filename. Afterwards it creates an empty docform
and saves it with this filename.

Next it initialises the file argument and refreshes the projects content.

The method returns the filename of the created file if all went ok
and undef if there was an error or it was canceled.

=cut

######################################
# Create a new document from scratch #
######################################
sub new_document_from_scratch
{
my $self=shift;
my $filename = undef;
$self->app_window()->Busy();

#Query the new filename
while(not defined $filename or $filename eq '' or (-f $self->path()."/".$filename.".edc"))
	{
	$filename = $self->dialog('Please enter new filename without extension:', 'New filename') ;
	}
#Append the default document extension 'FIXME: Take later the first entry in the filetypes -extensions array
$filename .= ".edc";

chdir $self->path();
my $ref_docform = $self->open_empty_docform();
$ref_docform->file_save_as($filename);
$self->app_window()->idletasks();

#Refresh the projects view
$self->project_refresh();

#Set the 'changed' flag
$self->set_content_changed();

#Select the newly added item in the treeview
my $filetype = $self->{'files'}->{$filename}->{'filetype'};
$self->_select_in_content("<ROOT>~<$filetype>~<$filename>");
$self->app_window()->idletasks();

EXIT:
$self->app_window()->Unbusy();
return $filename;
}


=head3 project_refresh

$object->project_refresh(); 

Refreshes the displayed content of the project (now the content of the directory)

=cut

###########################
### Refresh the project ###
###########################
sub project_refresh
{
my $self = shift;
$self->app_window()->Busy();
my $outline = $self->content();
my $config = $main::config;
my $project = $main::project;

#Empty the thing for the filetypes and their exentsions that si used for the Open and Save dialogs
$self->{'file_ext_array'} = [];
$self->{'extensions_by_filetypes'} = [];

#Delete all entries
$outline->deleteAll(); 
#Create the root entry
$outline->add('<ROOT>', -itemtype=>'imagetext', -text=>'THE PROJECT', -image=>$self->{'ui'}->{'project_icon'});

$self->{'ui'}->{'item_icons'}->{'_LINK_'} = $self->app_window()->Photo(-file=>Utils::process_filename('\graphics\file_link.xpm'));

my $filetypes_by_extension = {};

my @filetype_names = @{$project->{'FILETYPE_NAMES'}};
my @file_ext_array = ();
$self->{'filetypes'} = {};
foreach my $filetype (@filetype_names)
	{
	$self->{'filetypes'}->{$filetype} = {};
	my $filetypes_root = $project->{'FILETYPES'}->{$filetype}; #just to be shorter
	my $caption = $filetypes_root->{'caption'};
	$self->{'filetypes'}->{$filetype}->{'caption'} = $caption;
	if (exists $filetypes_root->{'caption_img'} && defined $filetypes_root->{'caption_img'})
		{
		my $caption_img = $filetypes_root->{'caption_img'};
		#Icons einlesen
		$self->{'ui'}->{'caption_icons'}->{$filetype} = $self->app_window()->Photo(-file=>Utils::process_filename($caption_img));
		$outline->add("<ROOT>~<$filetype>", -itemtype=>'imagetext', -text=>$caption, -image=>$self->{'ui'}->{'caption_icons'}->{$filetype});
		}
	else
		{
		$outline->add("<ROOT>~<$filetype>", -itemtype=>'text', -text=>$caption);
		}
	if (exists $filetypes_root->{'item_img'} && defined $filetypes_root->{'item_img'})
		{
		my $item_img = $filetypes_root->{'item_img'};
		$self->{'ui'}->{'item_icons'}->{$filetype} = $self->app_window()->Photo(-file=>Utils::process_filename($item_img));
		}
	my @ext_list = split(/\|\|/,$filetypes_root->{'extensions'});
	foreach my $extension (@ext_list)
		{
		$filetypes_by_extension->{uc($extension)} = $filetype;
		}
	
	#The stuff for the Open/Save dialogs
	my @file_ext_entry = ();
	my $filetype_name = $filetype;
	push (@file_ext_entry, $filetype_name, \@ext_list);
	push (@file_ext_array, \@file_ext_entry);
	$self->{'filetypes'}->{$filetype}->{'extensions'} = \@ext_list;
	}
#Fill the stuff for the Open/Save dialogs
$self ->{'file_ext_array'} = \@file_ext_array;

#Read all files in the directory
chdir($self->path());
opendir (DIR, '.')||die "Cannot open . \n";

#Retrieving file information
foreach my $file (readdir(DIR))
	{
	if (-f $file && $file=~ /\.([^.]*)$/)
		{
		my @stats = stat($file);
		my $size = $stats[7];
		if ($size > 1024)
			{
			$size = int($size / 1024)." KB";
			}
		else
			{
			$size .= " Bytes";
			}
		$self->{'files'}->{$file}->{'size'} = $size;
		my $mod = $stats[9];
		$self->{'files'}->{$file}->{'changed'} = localtime($mod);
		$self->{'files'}->{$file}->{'type'} = 'LOCAL';
		}
	}
closedir (DIR);

#Delete entries that are now obsolete
foreach my $file (sort keys (%{$self->{'files'}}))
	{
	if (not -f $file )
		{
		$self->{'files'}->{$file} = undef;
		delete $self->{'files'}->{$file};
		}
	}

#Getting extension and filetype from all stored files
my @now_stored_files = sort keys (%{$self->{'files'}});
foreach my $file (@now_stored_files)
	{
	#Init the arguments (only done once)
	$self->_init_args('FILE_ARGS', $file);
	my $extension;
	my $filetype;
	if ($file=~ /\.([^.]*)$/)
		{
		$extension = $self->{'files'}->{$file}->{'extension'} = uc($1);
		}
	else
		{
		$extension = $self->{'files'}->{$file}->{'extension'} = '';
		}

	if (exists $filetypes_by_extension->{$extension} and defined $filetypes_by_extension->{$extension} )
		{
		$filetype = $self->{'files'}->{$file}->{'filetype'} = $filetypes_by_extension->{$extension};
		}
	else
		{
		$filetype = $self->{'files'}->{$file}->{'filetype'} = '_NONE_';
		}

	if (not ($filetype eq '_NONE_'))
		{
		if ($self->{'files'}->{$file}->{'type'} eq 'LINK')
			{
			$outline->add("<ROOT>~<$filetype>~<$file>", -itemtype=>'imagetext', -text=>$file, -data=>$filetype, -image=>$self->{'ui'}->{'item_icons'}->{'_LINK_'} );
			}
		elsif (exists $self->{'ui'}->{'item_icons'}->{$filetype} && defined $self->{'ui'}->{'item_icons'}->{$filetype})
			{
			$outline->add("<ROOT>~<$filetype>~<$file>", -itemtype=>'imagetext', -text=>$file, -data=>$filetype, -image=>$self->{'ui'}->{'item_icons'}->{$filetype} );
			}
		else
			{
			$outline->add("<ROOT>~<$filetype>~<$file>", -itemtype=>'text', -text=>$file, -data=>$filetype);
			}
		$outline->itemCreate("<ROOT>~<$filetype>~<$file>", 1, -itemtype=>'text', -text=>$self->{'files'}->{$file}->{'size'});
		$outline->itemCreate("<ROOT>~<$filetype>~<$file>", 2, -itemtype=>'text', -text=>$self->{'files'}->{$file}->{'changed'});
		}
	}

#Expand branches
$outline->autosetmode;
$self->app_window()->Unbusy();
}


=head3 update_window_caption

$object->update_window_caption();

Updates the text for the window caption

=cut

#############################
# Update the window caption #
#############################
sub update_window_caption
{
my $self = shift;

$self->app_window()->configure(-title => "Project: [".$self->filename()."]");

}

=head3 open_tutorial

$object->open_tutorial();

Opens the file "/samples/tutorial.edp". 

=cut

###############################
# Open the 'Tutorial'-Project #
###############################
sub open_tutorial
{
my $self=shift;
if (-f $main::edoc_path."/samples/tutorial.edp")
	{
	$self->project_open($main::edoc_path."/samples/tutorial.edp");
	}
else
	{
	$self->message('Somebody removed the tutorial project', 'Note');
	}
}


=head3 file_add

$filename = $object->file_add();

Adds a file to the project. If all went ok, it returns the filename, else it returns undef.

The added file will be copied into the project directory and added into the project structure.

If a file already exists in the project with this filename, a number is added to the filename.

=cut

#################################
### Add a file to the project ###
#################################
sub file_add
{
my $self = shift;
my $filename = undef;

#Retrieving the file to add
$filename = $self->app_window()->getOpenFile(-filetypes=>\@{$self->{'file_ext_array'}});
if ( not defined $filename or $filename eq '' or not (-f $filename) ) # If dilog was canceled.
	{
	goto Exit;
	}

# process the filename so that it fit's into the projects directory
my $filename_base = Utils::extract_filename($filename);
my $front = '';
my $back = '';
if ($filename_base =~ /(.*)\.(.*)/)
	{
	$front = $1;
	$back = $2;
	}
else
	{
	$front = $filename_base;
	}
#Generating anew filename if it already exists there
my $counter = 1;
while (-f $self->path()."/".$filename_base)
		{
		if ($back eq '')
			{
			$filename_base = $front."_$counter";
			}
		else
			{
			$filename_base = $front."_$counter.".$back;
			}
		$counter++;
		}

# Copy the file
my $from = $filename;
my $to = $self->path()."/".$filename_base;
use File::Copy;
copy($from, $to);

#Refresh the projects view
$self->project_refresh();

#Set the 'changed' flag
$self->set_content_changed();

#Select the newly added item
my $filetype = $self->{'files'}->{$filename_base}->{'filetype'};
$self->_select_in_content("<ROOT>~<$filetype>~<$filename_base>");

EXIT:
return $filename_base;
}

=head3 file_link

$filename = $object->file_link();

Links a file to the project. If all went ok, it returns the filename, else it returns undef.

The added file will NOT be copied into the project directory. Ther si just a shortcut stored in the project structure.

In this case not only the filename is stored, it gets storet together with the full path.

If a file already exists in the project with this filename, a number is added to the filename. 

=cut

##################################
### Link a file to the project ###
##################################
sub file_link
{
my $self = shift;
my $filename = undef;

#Retrieving the file to add
$filename = $self->app_window()->getOpenFile(-filetypes=>\@{$self->{'file_ext_array'}});
if ( not defined $filename or $filename eq '') # If dilog was canceled.
	{
	goto Exit;
	}

#Test if the filename really exists (The user could have typed maually in the dialog)
if (-f $filename)
	{
	#Generate an entry for that file in the structure
	$self->{'files'}->{$filename}->{'type'} = 'LINK';


	#Refresh the projects view
	$self->project_refresh();

	#Set the 'changed' flag
	$self->set_content_changed();

	#Select the newly added item
	my $filetype = $self->{'files'}->{$filename}->{'filetype'};
	$self->_select_in_content("<ROOT>~<$filetype>~<$filename>");
	}

EXIT:
return $filename;

}


=head3 file_remove

$ok = $object->file_remove();

Removes a file from the project. If all went ok, it returns 1, else it returns 0.

The file will be deleted (physically) from the projects directory.
Linked files cannot be deleted.

=cut

######################################
### remove a file from the project ###
######################################
sub file_remove
{
my $self = shift;
my $ok = 0;
my $path = $self->content()->infoAnchor;
my $filetype = $self->content()->infoData($path);

if (not defined $filetype) {goto EXIT}; #If a caption was selected

my $filename;

# Test if a file was selected
if ($path =~ /<ROOT>~<$filetype>~<(.*)>/)
	{
	$filename = $1;
	}
else
	{
	goto EXIT;
	}

# if it's a LOCAL file and really exists
if ( ($self->{'files'}->{$filename}->{'type'} eq 'LOCAL') and (-f $self->path()."/".$filename) )
	{
	#Delete the file from harddisk
	if (unlink ($self->path()."/".$filename) )
	 	{
		#Set the flag
		$ok = 1;
		
		#Refresh the project
		$self->project_refresh();

		#Set the 'changed' flag
		$self->set_content_changed();

		}
	else
		{
		$self->message("File $filename could not be deleted !",'Error:'); 
		}
	}
else
	{
	$self->message("File is a linked one or does not exist anymore !",'Error:'); 
	}

EXIT:
return $ok;
}

=head3 file_rename

$ok = $object->file_rename();

Renamesa file from the project. If all went ok, it returns the new name, else undef.

The file will be renamed (physically) in the projects directory.

Linked files cannot be renamed.

=cut

####################################
### rename a file in the project ###
####################################
sub file_rename
{
my $self = shift;

my $return_value = undef;
my $path = $self->content()->infoAnchor;
my $filetype = $self->content()->infoData($path);

if (not defined $filetype) {goto EXIT}; #If a caption was selected

my $filename;

# Test if a file was selected
if ($path =~ /<ROOT>~<$filetype>~<(.*)>/)
	{
	$filename = $1;
	}
else
	{
	goto EXIT;
	}

# if it's a LOCAL file and really exists
if ( ($self->{'files'}->{$filename}->{'type'} eq 'LOCAL') and (-f $self->path()."/".$filename) )
	{
	#Cut away the extension, if there is one
	my $name = $filename;
	my $ext = '';
	if ($name =~ /\.(.*)/)
		{
		my $a = $1;
		$name =~ s/\.$a//;
		$ext = ".".$a;
		}
	
	while (-f $self->path()."/".$name.$ext)
		{
		#Query the new name
		$name = $self->dialog('Please enter the new name without extension.', 'New name:', $name) ;
		}

	#Rename the file
	if (rename $self->path()."/".$filename , $self->path()."/".$name.$ext  )
	 	{
		#Set the flag
		$retuen_value = $name.$ext;
		
		#Refresh the project
		$self->project_refresh();

		#Set the 'changed' flag
		$self->set_content_changed();

		#Select the newly added item
		$filename = $name.$ext;
		my $filetype = $self->{'files'}->{$filename}->{'filetype'};
		$self->_select_in_content("<ROOT>~<$filetype>~<$filename>");
		}
	else
		{
		$self->message("File $filename could not be renamed !",'Error:'); 
		}
	}
else
	{
	$self->message("File is a linked one or does not exist anymore !",'Error:'); 
	}

EXIT:
return $return_value;
}

=head3 file_duplicate

$new_name = $object->file_duplicate();

Duplicate a file in the project. If all went ok, it returns the new name, esle undef.

Linked files cannot be duplicated

=cut

#######################################
### duplicate a file in the project ###
#######################################
sub file_duplicate
{
my $self = shift;

my $return_value = undef;
my $path = $self->content()->infoAnchor;
my $filetype = $self->content()->infoData($path);

if (not defined $filetype) {goto EXIT}; #If a caption was selected

my $filename;

# Test if a file was selected
if ($path =~ /<ROOT>~<$filetype>~<(.*)>/)
	{
	$filename = $1;
	}
else
	{
	goto EXIT;
	}

# if it's a LOCAL file and really exists
if ( ($self->{'files'}->{$filename}->{'type'} eq 'LOCAL') and (-f $self->path()."/".$filename) )
	{

	# process the filename so that it fit's into the projects directory
	my $filename_base = $filename;
	my $front = '';
	my $back = '';
	if ($filename_base =~ /(.*)\.(.*)/)
		{
		$front = $1;
		$back = $2;
		}
	else
		{
		$front = $filename_base;
		}
	#Generating anew filename if it already exists there
	my $counter = 1;
	while (-f $self->path()."/".$filename_base)
		{
		if ($back eq '')
			{
			$filename_base = $front."_$counter";
			}
		else
			{
			$filename_base = $front."_$counter.".$back;
			}
		$counter++;
		}

	# Copy the file
	my $from = $self->path()."/".$filename;
	my $to = $self->path()."/".$filename_base;
	use File::Copy;
	if (copy($from, $to) )
		{

		$return_value = $filename_base;

		#Refresh the project
		$self->project_refresh();
	
		#Set the 'changed' flag
		$self->set_content_changed();

		#Select the newly added item
		$filename = $filename_base;
		my $filetype = $self->{'files'}->{$filename}->{'filetype'};
		$self->_select_in_content("<ROOT>~<$filetype>~<$filename>");
		}
	else
		{
		$self->message("File $filename could not be renamed !",'Error:'); 
		}
	}
else
	{
	$self->message("File is a linked one or does not exist anymore !",'Error:'); 
	}

EXIT:
return $return_vaue;
}

=head3 file_unlink

$ok = $object->file_unlink();

Unlinks the selected file from the project. If all went ok, it returns 1 esle 0.

=cut

######################################
### Unlink a file from the project ###
######################################
sub file_unlink
{
my $self = shift;
my $ok = 0;
my $path = $self->content()->infoAnchor;
my $filetype = $self->content()->infoData($path);

if (not defined $filetype) {goto EXIT}; #If a caption was selected

my $filename;

# Test if a file was selected
if ($path =~ /<ROOT>~<$filetype>~<(.*)>/)
	{
	$filename = $1;
	}
else
	{
	goto EXIT;
	}

#Delete the shortcut from the hash if it exists there and is a linked file
if (exists $self->{'files'}->{$filename} and $self->{'files'}->{$filename}->{'type'} eq 'LINK' )
	{
	delete $self->{'files'}->{$filename} ;
	
	#Refresh the project
	$self->project_refresh();

	#Set the 'changed' flag
	$self->set_content_changed();
	
	$ok = 1;
	}
else
	{
	$self->message("This file is not a linked one or info about it was already deleted!",'Error:'); 
	}

EXIT:
return $ok;
}

=head2 Private methods:

=head3 _content_dbl_click

$ref_docform = $object->_content_dbl_click(); 

Callback for the doubleclick event on one content entry. Opens a document window (DocForm) or starts an
external application to open the file.

=cut

#################################################
# Callback fuer Doppelklick auf Content-Outline #
#################################################
sub _content_dbl_click
{
my $self = shift;

my $project = $main::project;

my $path = $self->content()->infoAnchor;
my $filetype = $self->content()->infoData($path);
if (not defined $filetype) {goto EXIT}; #If a caption was selected
my $filename;
my $command = $project->{'FILETYPES'}->{$filetype}->{'command'};
if ($path =~ /<ROOT>~<$filetype>~<(.*)>/)
	{
	$filename = $1;
	}
else
	{
	goto EXIT;
	}

if ($command =~ /INTERN::(.+)/)
	{
	my $line = $1;
	$line =~ s/%FILENAME%/\"$filename\"/g ;
	eval '$self->'.$line;
	}
if ($command =~ /EXTERN::(.+)/)
	{
	my $line = $1;
	$line =~ s/%FILENAME%/$filename/g  ;
	system $line;
	}
if ($command =~ /MACRO::(.+)/)
	{
	my $macro_name = $1;
	$self->execute_macro($macro_name);
	}

EXIT:
}

=head3 _init_args

my $arg_number = $object->_init_args($type [,$filename] );

Initialises the argument-storing branches.

This is only performed if there are not already arguments stored for that file to preven overwriting of
already existing values.

The method returns the number of initialised arguments.

Parameters:

(+) $type: What arguments should be initialised, could be 'PROJECT_ARGS', 'FILE_ARGS', 'CURRENT_FILE'

(+) $filename: The name of the file  (Only to be set when $type is 'FILE_ARGS')

=cut

######################################
# init the argument-storing branches #
######################################
sub _init_args
{
my $self = shift;
my $type = shift;
my $filename = shift;

my $arg_number = 0;
my $project_branch;
my $target_branch;

if ($type eq 'FILE_ARGS')
	{
	if (not exists $self->{'files'}->{$filename}->{'args'})
		{
		$self->{'files'}->{$filename}->{'args'} = {};
		}
	$target_branch = $self->{'files'}->{$filename}->{'args'};
	$project_branch = $main::project->{'FILE_ARGS'};
	}
elsif ($type eq 'CURRENT_FILE')
	{
	if (not exists $self->{'current_file'}->{'args'})
		{
		$self->{'current_file'}->{'args'} = {};
		}
	$target_branch = $self->{'current_file'}->{'args'};
	$project_branch = $main::project->{'FILE_ARGS'};
	}
elsif ($type eq 'PROJECT_ARGS')
	{
	$target_branch = $self->{'prj_args'};
	$project_branch = $main::project->{'PROJECT_ARGS'};
	}
else
	{
	goto EXIT;
	}

if (not exists $target_branch->{'arg_number'} 
	or not defined $target_branch->{'arg_number'})
	{
	
	$arg_number = $project_branch->{'arg_number'};
	
	$target_branch->{'arg_number'} = $arg_number;

	if ($arg_number > 0)
		{
		for (my $i=1 ; $i<=$arg_number; $i++)
			{
			my $name = $project_branch->{"arg_".$i."_name"};
			my $arg_type = $project_branch->{"arg_".$i."_type"};
			my $settings = $project_branch->{"arg_".$i."_settings"};
			my $hint = $project_branch->{"arg_".$i."_hint"};
			my $default = $project_branch->{"arg_".$i."_default"};
			$default = '' if not defined $default;
			$target_branch->{$i}->{'name'} = $name;
			$target_branch->{$i}->{'type'} = $arg_type;
			$target_branch->{$i}->{'settings'} = $settings;
			$target_branch->{$i}->{'value'} = $default;
			$target_branch->{$i}->{'hint'} = $hint;
			}
		}
	}

EXIT:
#Return the number of initialised arguments
return $arg_number;
}

=head3 _on_browse_content

$object->_on_browse_content();

Callback for browsing in the content tree.

=cut

#################################################
### Callback for browsing in the content tree ###
#################################################
sub _on_browse_content
{
my $self = shift;

$self->app_window()->Busy();

my $project = $main::project;
my $path = $self->content()->infoAnchor;
my $filetype = $self->content()->infoData($path);

# exit If there are no arguments defined 
goto EXIT if not defined $self->{'current_file'}->{'args'}->{'arg_number'};

#store the number of arguments
my $arg_number = $self->{'current_file'}->{'args'}->{'arg_number'};

my $filename;

#Get the filename out of the entry
if (defined $filetype and $path =~ /<ROOT>~<$filetype>~<(.*)>/)
	{
	$filename = $1;
	}

#If the callback was called twice with the same entry
if (defined $filename and defined $self->{'current_file'}->{'filename'} and ($self->{'current_file'}->{'filename'} eq $filename) )
	{
	goto EXIT;
	}

#Let the Argument editor appear or disappear
if (defined $filename)
	{
	if (not defined $self->{'current_file'}->{'filename'})
		{
		$self->{'ui'}->{'file_args_base'}->grid(-column=>0, -row=>1, -sticky=>'nsew');
		$self->{'ui'}->{'file_args_frame'}->gridRowconfigure(1, -weight=>1);
		$self->{'ui'}->{'file_args_caption'}->configure(-text=>'File arguments');
		}
	}
else
	{
	if (defined $self->{'current_file'}->{'filename'})
		{
		$self->{'ui'}->{'file_args_base'}->gridForget();
		$self->{'ui'}->{'file_args_caption'}->configure(-text=>'No file arguments');
		}
	}

#Write back the currently in {'current_file'}->{'args'} stored adata
$self->_write_back_current_file_args();

#Exit if there is now no filename (A filetype-caption entry of the was selected or the root entry)
goto EXIT if not defined $filename;

#Store the filename of the now selected entry
$self->{'current_file'}->{'filename'} = $filename;

#Copy the arguments to the 'current_file' hash from the argument-storing hash of the file
for (my $i=1; $i <= $arg_number; $i++)
	{
	if (exists $self->{'files'}->{$filename}->{'args'}->{$i}->{'value'})
		{
		if (defined $self->{'files'}->{$filename}->{'args'}->{$i}->{'value'})
			{
			if ($self->{'current_file'}->{'args'}->{$i}->{'type'} eq 'MULTILINE')
				{
				$self->{'current_file'}->{'args'}->{$i}->{'_entry'}->delete('1.0', 'end' );
				$self->{'current_file'}->{'args'}->{$i}->{'_entry'}->insert('1.0', $self->{'files'}->{$filename}->{'args'}->{$i}->{'value'} );
				}
			else
				{
				$self->{'current_file'}->{'args'}->{$i}->{'value'} = $self->{'files'}->{$filename}->{'args'}->{$i}->{'value'};
				}
			}
		}
	}
 
EXIT:
$self->app_window()->Unbusy();
}

=head3 _write_back_current_file_args

$object->_write_back_current_file_args();

Writes back the data stored for editing in the {'current_file'}->{'args'}-branch.

=cut

###############################################
### Store the currently edited argumet data ###
###############################################
sub _write_back_current_file_args
{
my $self = shift;

#Copy the arguments from the current_file-hash back to the arguments of the previous selected file
if (defined $self->{'current_file'}->{'filename'})
	{
	my $filename = $self->{'current_file'}->{'filename'};
	my $arg_number = $self->{'current_file'}->{'args'}->{'arg_number'};
	for (my $i = 1; $i<=$arg_number; $i++)
		{
		if ($self->{'current_file'}->{'args'}->{$i}->{'type'} eq 'MULTILINE')
				{
				my $text = $self->{'current_file'}->{'args'}->{$i}->{'_entry'}->get('1.0', 'end' );
				$text =~ s/\n$//g ;
				$self->{'files'}->{$filename}->{'args'}->{$i}->{'value'} = $text;
				$self->{'current_file'}->{'args'}->{$i}->{'_entry'}->delete('1.0', 'end' );
				}
			else
				{
				$self->{'files'}->{$filename}->{'args'}->{$i}->{'value'} = $self->{'current_file'}->{'args'}->{$i}->{'value'};
				}
		$self->{'current_file'}->{'args'}->{$i}->{'value'} = undef;
		}
	$self->{'current_file'}->{'filename'} = undef;
	}
}

=head3 _select_in_content

$object->_select_in_content($entry_path);

Selects the specified entry in the content-tree and performs 'browsecmd'.

Parameters:

(+) $entry_path: The path of the entry

=cut

######################################################
### Select the specified entry in the content tree ###
######################################################
sub _select_in_content
{
my $self = shift;
my $path = shift;
$self->content()->selectionSet($path);
$self->content()->anchorSet($path);
$self->content()->see($path);
$self->_on_browse_content();
}


1; #So muss ein Package enden

=head1 KNOWN BUGS:

Properties (hash values) are often directly accessed now, this should be
done with more Get/Set methods.

=head1 STILL MISSING:

(+) More PIM and project management functionality, backup, zip and deploy mechanisms.

(+) Drag&Drop inside of the tree and between the project form and the document form.


=cut
