#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SearchForm;

=head1 NAME:

Object: SearchForm

=head1 SYNOPSIS:

use class::SearchForm;

=head1 DIRECTORY/FILE:

The file SearchForm.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

The object SearchForm encapsulates the search and replace window. It is now coded as a normal non-visual Perl class
that creates an user interface in the $self->{'ui'} branch. It might be more clever to (later) transform it
into a Toplevel-derived widget.

=head1 STATUS:

Coding: 60% done

Documentation: up-do-date

NOTE: Here all objects get correctly destroyed after closing the window --> !NO CIRCULAR REFERENCES!

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

$self->{'docform'} #A reference to the DocForm object this window is associated with

$self->{'ui'}->{'_top'} #A referenece to the Toplevel

$self->{'ui'}->{...the rest of this branch...} #References to the widgets that build the user interface

=head1 Class variables:

=cut

=head1 METHODS:

=head2 Class Methods:

=head2 Constructors:

=head3 new

$object = SearchForm->new($docform); 

This constructor creates a new object of the class SearchForm. 

Parameters: 

(+) $docform: A reference to the DocForm object the SearchForm was created from

=cut

###############
# Konstruktor #
###############
sub new
{
my ($class,$docform)=@_;
my $self = {};
bless($self, $class);
#Initialise
$self->docform( $docform );
my $config = $main::config;
my $activebackground = $config->{'UI'}->{'activebackground'};
my $buttons_style = $config->{'UI'}->{'buttons_style'};
my $buttons_borderwidth = $config->{'UI'}->{'buttons_borderwidth'};
my $speedbarfont = $config->{'UI'}->{'speedbarfont'};
$self->{'search_for'} = '';
$self->{'replace_with'} = '';
$self->{'case'} = 1;
$self->{'regexp'} = 0;
$self->{'backwards'} = 0;
$self->{'cur_pos'} = undef;

#CREATE THE GUI

require Tk::LabEntry;
require Tk::Checkbutton;

#The Toplevel window
$self->{'ui'}->{'_top'}=$self->docform()->top()->Toplevel();
my $ui_base=$self->{'ui'};
$self->top()->configure(-title=>'Search & Replace:');
$self->top()->transient($self->docform()->top());

#2 Panels
$ui_base->{'_top_panel'}=$self->top()->Frame(-borderwidth=>4)->pack(-side=>'top',-expand=>1, -fill=>'both');
$ui_base->{'_bottom_panel'}=$self->top()->Frame()->pack(-side=>'bottom',-expand=>1, -fill=>'x');

#The diaolg widgets
$ui_base->{'_separator1'}=$ui_base->{'_top_panel'}->Label(-background=>'black', -foreground=>'white', -borderwidth=>2, -relief=>'flat', -text=>'Search options:', -justify=>'center')->pack(-side=>'top', -expand=>1, -fill=>'x');

$ui_base->{'_search_field'}=$ui_base->{'_top_panel'}->BrowseEntry(-label=>'Search for:', -browsecmd=>sub{$self->_text_entered($self)}, -label=>'Search for:', -choices=>$self->docform()->ref_search_history(), -variable=>\$self->{'search_for'})->pack(-side=>'top',  -anchor=>'w', -expand=>0, -fill=>'x', -pady=>3);
Utils::set_browseentry_color($ui_base->{'_search_field'}, 'white');
Utils::beautify_browseentry($ui_base->{'_search_field'});
$ui_base->{'_search_field'}->bind('<KeyRelease>'=>sub{$self->_text_entered($self)});
$ui_base->{'_search_field'}->bind('<Button>'=>sub{$self->_text_entered($self)});
Utils::create_balloonhelp($ui_base->{'_search_field'}, 'Enter the text to search for', 'balloon', undef);

$ui_base->{'_case_sensitive'}=$ui_base->{'_top_panel'}->Checkbutton( -justify=>'left', -text=>'case sensitive', -variable=>\$self->{'case'},-anchor=>'w')->pack(-side=>'top', -anchor=>'w', -expand=>1, -fill=>'x');
Utils::create_balloonhelp($ui_base->{'_case_sensitive'}, "Select this if the search\nshould be case sensitive", 'balloon', undef);

$ui_base->{'_backwards'}=$ui_base->{'_top_panel'}->Checkbutton( -justify=>'left', -text=>'search backwards', -variable=>\$self->{'backwards'},-anchor=>'w')->pack(-side=>'top', -anchor=>'w', -expand=>1, -fill=>'x');
Utils::create_balloonhelp($ui_base->{'_backwards'}, "Select this if the search\nshould be done backwards", 'balloon', undef);

$ui_base->{'_is_regex'}=$ui_base->{'_top_panel'}->Checkbutton( -justify=>'left', -text=>'is regular expression', -variable=>\$self->{'regexp'},-anchor=>'w')->pack(-side=>'top',  -anchor=>'w', -expand=>1, -fill=>'x');
Utils::create_balloonhelp($ui_base->{'_is_regex'}, "Select this if the search\nshould use regular expressions", 'balloon', undef);

$ui_base->{'_separator2'}=$ui_base->{'_top_panel'}->Label(-background=>'black', -foreground=>'white', -borderwidth=>2, -relief=>'flat', -text=>'Replace options:', -justify=>'center' )->pack(-side=>'top',  -anchor=>'w', -expand=>0, -fill=>'x');

$ui_base->{'_replace_field'}=$ui_base->{'_top_panel'}->BrowseEntry(-label=>'Replace with:', -browsecmd=>sub{$self->_text_entered($self)}, -label=>'Replace with:', -choices=>$self->docform()->ref_replace_history(), -variable=>\$self->{'replace_with'})->pack(-side=>'top',  -anchor=>'w', -expand=>0, -fill=>'x', -pady=>3);
Utils::set_browseentry_color($ui_base->{'_replace_field'}, 'white');
Utils::beautify_browseentry($ui_base->{'_replace_field'});

$ui_base->{'_replace_field'}->bind('<KeyRelease>'=>sub{$self->_text_entered($self)});
$ui_base->{'_replace_field'}->bind('<Button>'=>sub{$self->_text_entered($self)});
Utils::create_balloonhelp($ui_base->{'_replace_field'}, "Enter the text that should\nbe used for replacing", 'balloon', undef);

#Buttons in the Bottom Panel
$ui_base->{'_close_btn'} = $ui_base->{'_bottom_panel'}->Button(-command=>sub{$self->do_destroy()}, -text=>'Close', -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$speedbarfont)->pack(-padx=>4, -pady=>4,-expand=>1, -fill=>'y', -side=>'left');
Utils::create_balloonhelp($ui_base->{'_close_btn'}, "Close the dialog", 'balloon', undef);

$ui_base->{'_search_btn'} = $ui_base->{'_bottom_panel'}->Button(-command=>sub{$self->search()}, -text=>'Search', -state=>'disabled', -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$speedbarfont)->pack(-padx=>4, -pady=>4,-expand=>1, -fill=>'y', -side=>'left');
Utils::create_balloonhelp($ui_base->{'_search_btn'}, "Execute the search", 'balloon', undef);

$ui_base->{'_replace_btn'} = $ui_base->{'_bottom_panel'}->Button(-command=>sub{$self->replace()}, -text=>'Replace', -state=>'disabled', -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$speedbarfont)->pack(-padx=>4, -pady=>4,-expand=>1, -fill=>'y', -side=>'left');
Utils::create_balloonhelp($ui_base->{'_replace_btn'}, "Replace next hit", 'balloon', undef);

$ui_base->{'_replace_all_btn'} = $ui_base->{'_bottom_panel'}->Button(-command=>sub{$self->replace_all()}, -text=>'Replace All', -state=>'disabled', -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$speedbarfont)->pack(-padx=>4, -pady=>4,-expand=>1, -fill=>'y', -side=>'left');
Utils::create_balloonhelp($ui_base->{'_replace_all_btn'}, "Replace all hits", 'balloon', undef);

#The Cleanup-Routine has to be done with the "protocol-Function", because:
#1.)When using the do_destroy-Method, the callback is only invoked AFTER all other Widgets in the Toplevel are destroyed,
#so the text in the Textwidget can not be saved anymore
#2.)When binding the '<DESTROY>'-Event to the method, it is called whenever one widget
#in the MainWindow is deleted (Even the embeded ones in the Textwidget
#The Cleanup-Function &do_destroy  is needed for:
#1.) Asking the user to save unsaved changes in the document
#2.) To remove not only the MainWindow but also the object "PrjForm" itself
$self->top()->protocol('WM_DELETE_WINDOW',sub{$self->do_destroy()}); #Binding the cleanupfunction to the closing of the window BEFORE the subwidgets are destroyed

$ui_base->{'_search_field'}->focus();

print "Object $self created.\n"; #debug

return $self;
}

=head2 Destructors:

=head3 DESTROY 

The standard automatically called PERL destructor for objects.
Unfortionatly it is called very late (when the whole application terminates.)

=cut

#############################
# Automatic Perl destructor #
#############################
sub DESTROY
{
my $self = shift;
print "Object $self destroyed.\n"; #debug
}

=head3 do_detstroy

$object->do_detsroy(); 

This destructor is binded to the protocol-message WM_DELETE_WINDOW
and performs the cleanup stuff, destroys the Toplevel manually
and then destroys the SearchForm object manually too.

=cut

##############
# Destruktor #
##############
sub do_destroy
{
my $self=shift;
#Find myself in the "$docform->search_dialogs()" property hash
if (defined $self->docform()->search_dialog())
	{
	$self->docform()->search_dialog(undef);
	}

#The default callback for WM_DELETE_WINDOW is not used, because we replaced it with this do_destroy-Function, so we have to destroy the MainWindow manually
$self->{'ui'}->{'_top'}->destroy();
#remove the SearchForm object itself, after the user closed the window
$self = undef;
}

=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 docform

$ref_docform = $object->docform(); #gets a reference to the associated DocForm object

$object ->docform($ref_docform); #sets a reference to the associated DocForm object

=cut

### docform ###
sub docform
{
my $self=shift;
@_ ? $self->{'docform'} = shift : return $self->{'docform'};
}

=head3 top

$ref_top = $object->top(); #gets a reference to the Tolevel window (READ-ONLY)

=cut

### top ###
sub top
{
my $self=shift;
return $self->{'ui'}->{'_top'};
}

=head2 Other public methods:

=head3 search

$object->search(); 

Starts the search.

If something is found, the function returns an reference to an hash that is constructed like this:

$ref_hash->{'start_index'} contains the startindex of the hit

$ref_hash->{'end_index'} contains the endindex of the hit

If nothing was found, the function returns 0;

=cut

#####################
# Starts the serach #
#####################
sub search
{
my $self=shift;

#Update the search history
$self->_update_history();

#Get the searched expression (pattern)
my $search_for = $self->{'search_for'};

my $return_value;

if (not $search_for eq '')
	{
	my $text_widget = $self->docform()->doctext();

	#build the search option flags
	my @options;
	my $char_count = 0; #For counting the characters that matched the search
	if ($self->{'regexp'}==1)
		{
		push (@options, '-regexp');
		}
	else
		{
		push (@options, '-exact');
		}
	if ($self->{'backwards'}==1)
		{
		push (@options, '-backwards');
		}
	else
		{
		push (@options, '-forwards');
		}
	if ($self->{'case'}==0)
		{
		push (@options, '-nocase');
		}

	#Search in the textwidget and get the start index
	my $index1 = $text_widget->search(@options , '-count'=>\$char_count, '--', $search_for, $text_widget->index('insert'));

	#Security check for the 'replace_all()' method
	if (defined $self->{'cur_pos'} and defined $index1 and (not $index1 eq ''))
		{
		if ($self->{'backwards'} == 1)
			{
			if ($text_widget->compare($index1,'>',$self->{'cur_pos'}))
				{
				$return_value = 0;
				goto EXIT;
				}
			}
			else
			{
			if ($text_widget->compare($index1,'<',$self->{'cur_pos'}))
				{
				$return_value = 0;
				goto EXIT;
				}
			}
		}
	#if something has been found
	if ((defined $index1) and (not $index1 eq ''))
		{
		#claculate the end index
		my $index2 = $text_widget->index("$index1 + $char_count chars");
		#clear all selections in the text widget
		Utils::textwidget_unselect($text_widget);
		#set the new selection
		Utils::textwidget_select($text_widget, $index1, $index2);
		#place the cursor in the right place
		if ($self->{'backwards'} == 1)
			{
			$text_widget->markSet('insert', $index2);
			}
			else
			{
			$text_widget->markSet('insert', $index1);
			}
		#scroll to the position
		$text_widget->see($index1);
		#focus into the text widget
		$self->docform()->doctext()->focus();
		#Build up the returned hash
		$return_value->{'start_index'} = $index1;
		$return_value->{'end_index'} = $index2;
		}
	else
		{
		#If nothing was found, return 0
		$return_value = 0;
		}
	}
EXIT:
return $return_value;
}

=head3 replace

$object->replace(); 

Makes the replacement for one hit.

If something is replaced, the function returns 1, if it just selected something it returns -1,  if there are no hits anymore it returns 0.

=cut

#####################
# Replaces one time #
#####################
sub replace
{
my $self = shift;

#Gets a reference to the text widget
my $text_widget = $self->docform()->doctext();
my $retun_value = 0;

#Test if there was already an hit that lead to a selection
if (Utils::textwidget_sel_exists($text_widget) == 0)
	{
	#If there is no selection yet, ->search
	my $result = $self->search();
	if ($result == 0)
		{
		$retun_value = 0;
		}
	else
		{
		$retun_value = -1;
		}
	goto EXIT;
	}
else
	{
	#Get the startindex of the selection
	my @selranges = $text_widget->tagRanges('sel');
	my $index1 = shift(@selranges);
	my $index2 = shift(@selranges);
	#place the cursor in the right palce of the selection, so that the selection will be found again for the first hit
	if ($self->{'backwards'} == 1)
		{
		$text_widget->markSet('insert', $index2);
		}
		else
		{
		$text_widget->markSet('insert', $index1);
		}

	#Perform one search-run
	my $result = $self->search();

	#If there was a hit
	if (defined $result and (not $result == 0))
		{
		$self->_replace($result);
		$self->search();
		$retun_value = 1;
		}
	else
		{
		#If nothing was found, nothing was replaced, return 0
	 	$retun_value = 0;
		}
	}
EXIT:
return $retun_value;

}

=head3 replace_all

$object->replace_all(); 

Makes the replacement for all hits.

If something is replaced, the function returns 1, else it returns 0

=cut

#####################
# Replaces all hits #
#####################
sub replace_all
{
my $self = shift;

my $result = 1;
my $text_widget = $self->docform()->doctext();

#Security-Check for replace_all:
#Necessary if the relace-term could be found again by the search term
#so we have to avoid multiple loops in the text.
#This way it just gets searched up to the end or up to the beginning of the document.
$self->{'cur_pos'} = $text_widget->index('insert');

#Loop through all hits
while (defined $result and (not ($result == 0)) )
	{
	$result = $self->replace();
	$self->{'cur_pos'} = $text_widget->index('insert');
	}

$self->{'cur_pos'} = undef;

}

=head2 Private methods:

=head3 _text_entered

$object->_text_entered()

A callback to switch the state of the buttons on and of depending on wether
there is some text in the LabEntries or not.

=cut

######################################################################
# Text entered in one of the LabEntries /Toggle state of the buttons #
######################################################################
sub _text_entered
{
my $self = shift;

if ($self->{'search_for'} eq '')
	{
	$self->{'ui'}->{'_search_btn'}->configure(-state=>'disabled') if defined $self->{'ui'}->{'_search_btn'};
	}
else
	{
	$self->{'ui'}->{'_search_btn'}->configure(-state=>'normal') if defined $self->{'ui'}->{'_search_btn'};
	}

if ($self->{'replace_with'} eq '')
	{
	$self->{'ui'}->{'_replace_btn'}->configure(-state=>'disabled') if defined $self->{'ui'}->{'_replace_btn'};
	$self->{'ui'}->{'_replace_all_btn'}->configure(-state=>'disabled') if defined $self->{'ui'}->{'_replace_all_btn'};
	}
else
	{
	$self->{'ui'}->{'_replace_btn'}->configure(-state=>'normal') if defined $self->{'ui'}->{'_replace_btn'};
	$self->{'ui'}->{'_replace_all_btn'}->configure(-state=>'normal') if defined $self->{'ui'}->{'_replace_all_btn'};
	}
}

=head3 _replace

$object->_replace($result)

Performs the replacement for the replace() and the replace_all() methods.

Parameters

(+) $result: The result of the actual search-run = method search() 

=cut

############################################################################
# Performs the replacement for the replace() and the replace_all() methods #
############################################################################
sub _replace
{
my $self = shift;
my $result = shift;

my $text_widget = $self->docform()->doctext();

#Gets the indexes of the hit-area
my $start_index = $result->{'start_index'};
my $end_index = $result->{'end_index'};

#Get the replace text
my $replace_text = $self->{'replace_with'};
#If its a rexep text, process the $1, $2 etc. in the repalce text
if ($self->{'regexp'} == 1)
	{
	my $hit = $text_widget->get($start_index, $end_index);
	my $pattern = $self->{'search_for'};
	# If the pattern fits to the hit
	if (
		($hit =~ /$pattern/i and not $self->{'case'} == 1) 
		or 
		($hit =~ /$pattern/ and $self->{'case'} == 1) 
		)
		{
		#Now we have $1, $2 etc... using eval to get their values into $replace text
		my $buffer = eval "return \"".$replace_text."\";";
		$replace_text = $buffer;
		}
	}
#Insert the replace text behind the first character to keep the tags of the 
#first character if the tag is streched over at least 2 characters
$text_widget->insert("$start_index + 1 chars", $replace_text );
#Delete the hits first character
$text_widget->delete($start_index);
#Calcualte the length of the inserted text
my $len = length($replace_text);
#Delete the one character that is still here from the hit area
$text_widget->delete("$start_index + ".($len)." chars", "$end_index + ".($len-1)." chars" );
#Place the cursor in the right place
if ($self->{'backwards'} == 1)
	{
	$text_widget->markSet('insert', $start_index);
	}
	else
	{
	$text_widget->markSet('insert', "$start_index + $len chars");
	}

}

=head3 _update_history

$object->_update_history()

Updates the search and replace history.

=cut

##########################################
# Updates the search and replace history #
##########################################
sub _update_history
{
my $self = shift;

#SEARCH HISTORY
my $search_for = $self->{'search_for'}; #actual search string

my @search_history = @{$self->docform()->ref_search_history()};
my @replace_history = @{$self->docform()->ref_replace_history()};

#test with cached last value, otherwise replace_all()-method will slow down
$SearchForm::last_search_for = '' if not defined $SearchForm::last_search_for;
if (not $SearchForm::last_search_for eq $search_for)
	{
	my $found = 0; #flag
	#Iterate through the list item
	foreach my $item (@search_history)
		{
		$found = 1 if $item eq $search_for;
		}
	#If the string wasn't found 
	if ($found == 0)
		{
		push (@search_history, $search_for);
		@search_history = sort (@search_history);
		$self->{'ui'}->{'_search_field'}->configure(-choices=>\@search_history);
		$self->docform()->ref_search_history(\@search_history);
		}
	#cache this value
	$SearchForm::last_search_for = $search_for;
	}

#REPLACE HISTORY
my $replace_with = $self->{'replace_with'}; #actual replace string

#test with cached last value, otherwise replace_all()-method will slow down
$SearchForm::last_replace_with = '' if not defined $SearchForm::last_replace_with;
if (not $SearchForm::last_replace_with eq $replace_with)
	{
	my $found = 0; #flag
	#Iterate through the list item
	foreach my $item (@replace_history)
		{
		$found = 1 if $item eq $replace_with;
		}
	#If the string wasn't found 
	if ($found == 0)
		{
		push (@replace_history, $replace_with);
		@replace_history = sort (@replace_history);
		$self->{'ui'}->{'_replace_field'}->configure(-choices=>\@replace_history);
		$self->docform()->ref_replace_history(\@replace_history);
		}
	#cache this value
	$SearchForm::last_replace_with = $replace_with;
	}

}

1; #So muss ein Package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
