#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package TagDoc;

require Tk::Label;

@ISA = qw(Tk::Derived Tk::Label);
Tk::Widget->Construct('TagDoc');

use strict;

=head1 NAME:

Widget: TagDoc

=head1 SYNOPSIS:

use class::TagDoc;

=head1 DIRECTORY/FILE:

The file TagDoc.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

This is a widget derived from Tk::Label that gets embeded into the documents text widget in the DocForm object (=document window).

This widget is thought to be used for the TagDoctor-Feature, that displays the start and the end of all tags in the document.

This object is coded as an Perl/Tk widget, so it has no Constructor new, it has the standard Tk functions to initialise itself (Populate,ClassInit,...)
The function Populate() is used like an constructor here.

=head1 STATUS:

Coding: 99% done

Documentation: up-to-date

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

$self->{'text_widget'} #Reference to the textwidget where the TagDoc object is embeded

$self->{'associated_tag'} ##Name of the tag the TagDoc object belongs to

$self->{'type'} #The type of the Dagdoc-object could be 'start' or 'end'

$self->{'tagman'} #A reference to the associated TagMan object

=head1 METHODS:

=head2 Class Methods:

=head3 ClassInit($class, $parentwindow)

! Not called directly !

This "constructor" initialises a new object of the class TagDoc and calls ClassInit() of the Superclass. 

=cut

##########################
# Klassen-Initalisierung #
##########################
sub ClassInit
{
# Tk-spezifischer Code
my ($class,$mainwindow)=@_; # Klassenname und Mainwindow
$class->SUPER::ClassInit($mainwindow); # Aufruf des Vorfahren
print "Object $class created.\n"; #debug
return $class;
}


=head2 Constructors:

=head3 Populate

$object = TagDoc(%options); 

This "constructor" sets the options of the new object of the class TagDoc. 

The binding of events and the creation of the associated Baloonhelp is done here too.

The keys/values in %options: 

(+) '-text_widget' => Reference to the textwidget where the NumLabel is embeded

(+) '-associated_tag' => The name of the the NumLabel is associated with (PAR_STYLES or EMBOBJ or ENV_STYLES)

(+) '-type' => The type of the Dagdoc-object could be 'start' or 'end'

(+) '-tagman' => A reference to the associated TagMan object

=cut

###########################################################################
# Uebergabe, ev. Filterung und Weiterleitung der "-option=>'value'"-Paare #
###########################################################################
sub Populate
{
my ($self,$args)=@_; #Auslesen der $self-Referenz und der option/value-paare
$self->SUPER::Populate($args); #Weiterleitung an Vorfahr (Basisklasse)

$self->text_widget( delete $args->{'-text_widget'} ); #Reference to the text widget
$self->associated_tag( delete $args->{'-associated_tag'} ); #Name of the tag the TagDoc object belongs to
$self->type( delete $args->{'-type'} ); #Type of the TagDoc-Object
$self->tagman( delete $args->{'-tagman'} ); #Reference to the TagMan object
# bindDestroy
$self->bind('<Destroy>' => sub{ $self->do_destroy()});
$self->bind('<Button-3>' => sub{ $self->_popup()});
return $self;
}

=head2 Destructors:

=head3 DESTROY; 

The standard automatically called PERL destructor for objects.

=cut

#############################
# Automatic Perl destructor #
#############################
sub DESTROY
{
my $self = shift;
print "Object $self destroyed.\n"; #debug
}

=head3 do_detstroy

$object->do_detsroy(); 

This "destructor" method is binded to the "<DESTROY>" event and should be performed whenever
the element is destroyed. It should not be called directly.

=cut

###################################################
# Destruktor zum sauberen aufraeumen des Objektes #
###################################################
sub do_destroy
{
my $self = shift; 
#break up circular refereneces;
$self->tagman(undef);
$self->text_widget(undef);
}


=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 type

$type = $object->type(); #gets the value of the 'type' attribute

$object->type($type); #sets the value of the 'type' attribute

=cut

### type ###
sub type
{
my $self=shift;
@_ ? $self->{'type'} = shift : return $self->{'type'};
}

=head3 text_widget

$ref_text_widget = $object->text_widget(); #gets a referenece to the Text widget the NumLabel is embeded in

$object->text_widget($ref_text_widget); #sets a referenece to the Text widget the NumLabel is embeded in

=cut

### text_widget ###
sub text_widget
{
my $self=shift;
@_ ? $self->{'text_widget'} = shift : return $self->{'text_widget'};
}

=head3 associated_tag

$tag_name = $object->associated_tag(); #gets the name of the tag the NumLabel is associated with

$object->associated_tag($tag_name); #sets the name of the tag the NumLabel is associated  with

=cut

### associated_tag ###
sub associated_tag
{
my $self=shift;
@_ ? $self->{'associated_tag'} = shift : return $self->{'associated_tag'};
}

=head3 tagman

$tagman = $object->tagman(); #gets a reference to the associated TagMan object

$object->tagman($tagman); #sets a reference to the associated TagMan object

=cut

### tagman ###
sub tagman
{
my $self=shift;
@_ ? $self->{'tagman'} = shift : return $self->{'tagman'};
}

=head2 Other public methods:

=cut

=head2 Private methods:

=head3 _popup

$object->_popup(); 

Create the popupmenu and display it on the screen

=cut

#######################
# Show the popup menu #
#######################
sub _popup
{
my $self = shift; 

my $activebackground = $main::config->{'UI'}->{'activebackground'};
my $buttons_style = $main::config->{'UI'}->{'buttons_style'};
my $buttons_borderwidth = $main::config->{'UI'}->{'buttons_borderwidth'};

my $label = $self->associated_tag();
#$label =~ s/:(.*)//go ;
#$label =~ s/></::/o ;
#$label =~ s/^<//o ;
#$label =~ s/>$//o ;

#POPUP-Menu creation
my $menu= $self->text_widget()->Menu(-activebackground=>$activebackground,-borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -tearoff=>0);
$menu->add ('command', -label=>$label, -background => 'black', -foreground=>'white');
$menu->add ('command', -label=>'Select this tag', -command=>sub{$self->_select_tag()});
$menu->add ('command', -label=>'Remove this tag', -command=>sub{$self->_remove_tag()});
$menu->add ('command', -label=>'Raise this tag', -command=>sub{$self->_raise_tag()});
$menu->add ('command', -label=>'Lower this tag', -command=>sub{$self->_lower_tag()});
$menu->add ('separator');
$menu->add ('command', -label=>'Empty content of this tag', -command=>sub{$self->_empty_content_of_tag()});
$menu->add ('command', -label=>'Delete content of this tag', -command=>sub{$self->_delete_content_of_tag()});


#Showing it
$menu->Popup(-popover=>'cursor', -popanchor=>'se');
}

=head3 _select_tag

$object->_select_tag(); 

Selecting the associated tag in the text widget

=cut

#########################
# Select associated tag #
#########################
sub _select_tag
{
my $self = shift; 
my $doctext = $self->text_widget();
Utils::textwidget_unselect($doctext);
my @ranges=$doctext->tagRanges($self->associated_tag());
my $start_index = shift (@ranges);
my $end_index = shift (@ranges);
$doctext->tagAdd('sel', $start_index, $end_index);

}

=head3 _remove_tag

$object->_remove_tag(); 

Removing the associated tag in the text widget

=cut

#########################
# Remove associated tag #
#########################
sub _remove_tag
{
my $self = shift; 
my $doctext = $self->text_widget();
my @ranges=$doctext->tagRanges($self->associated_tag()); #Get the tag ranges
while ($#ranges >=0) #While there is a range
	{
	#Calculate them
	my $start_index = shift(@ranges);
	my $start_line = Utils::get_index_info($start_index);
	my $end_index = shift(@ranges);
	my $end_line = Utils::get_index_info($end_index);
	#Remove them
	$self->tagman()->_delete_tag($self->associated_tag(),$start_index,$end_index);
	
	}
}

=head3 _raise_tag

$object->_raise_tag(); 

Raise the associated tag in the text widget

=cut

#########################
# raise associated tag #
#########################
sub _raise_tag
{
my $self = shift; 
my $doctext = $self->text_widget();
my @ranges=$doctext->tagRanges($self->associated_tag()); #Get the tag ranges
while ($#ranges >=0) #While there is a range
	{
	#Calculate them
	my $start_index = shift(@ranges);
	my $start_line = Utils::get_index_info($start_index);
	my $end_index = shift(@ranges);
	my $end_line = Utils::get_index_info($end_index);
	#Raise the tag
	$doctext->tagRaise($self->associated_tag());
	
	#FIXME:
	#Refresh the TagDoctor
	#for (my $line = $start_line; $line<=$end_line; $line++)
	#	{
	#	$self->tagman()->tag_doctor_refresh_line($line);
	#	}
	}
}

=head3 _lower_tag

$object->_lower_tag(); 

Lower the associated tag in the text widget

=cut

#########################
# Lower associated tag #
#########################
sub _lower_tag
{
my $self = shift; 
my $doctext = $self->text_widget();
my @ranges=$doctext->tagRanges($self->associated_tag()); #Get the tag ranges
while ($#ranges >=0) #While there is a range
	{
	#Calculate them
	my $start_index = shift(@ranges);
	my $start_line = Utils::get_index_info($start_index);
	my $end_index = shift(@ranges);
	my $end_line = Utils::get_index_info($end_index);
	#Lower the tag
	$doctext->tagLower($self->associated_tag());

	#FIXME:
	#Refresh the TagDoctor there
	#for (my $line = $start_line; $line<=$end_line; $line++)
	#	{
	#	$self->tagman()->tag_doctor_refresh_line($line);
	#	}
	}
}

=head3 _delete_content_of_tag

$object->_delete_content_of_tag(); 

Delete the text content of the associated tag in the text widget

=cut

#########################################
# Delete content of the  associated tag #
#########################################
sub _delete_content_of_tag
{
my $self = shift; 
my $doctext = $self->text_widget();
my @ranges=$doctext->tagRanges($self->associated_tag()); #Get the tag ranges
while ($#ranges >=0) #While there is a range
	{
	#Calculate them
	my $start_index = shift(@ranges);
	my $start_line = Utils::get_index_info($start_index);
	my $end_index = shift(@ranges);
	my $end_line = Utils::get_index_info($end_index);
	#Delete text there (including TagDoc-Tags)
	$doctext->delete($start_index, $end_index);
	#Get tag ranges that are still there
	@ranges=$doctext->tagRanges($self->associated_tag());
	}
}

=head3 _empty_content_of_tag

$object->_empty_content_of_tag(); 

Empty the text content of the associated tag in the text widget

=cut

#########################################
# Empty content of the  associated tag #
#########################################
sub _empty_content_of_tag
{
my $self = shift; 
my $doctext = $self->text_widget();
my @ranges=$doctext->tagRanges($self->associated_tag()); #Get the tag ranges
while ($#ranges >=0) #While there is a range
	{
	#Calculate them
	my $start_index = shift(@ranges);
	my $start_line = Utils::get_index_info($start_index);
	my $end_index = shift(@ranges);
	my $end_line = Utils::get_index_info($end_index);
	#Delete text there (excluding TagDoc-Tags)
	$doctext->delete($start_index."+ 1 chars", $end_index."- 1 chars");
	#Positioning the cursor between the 2 TagDoc-objects
	$doctext->markSet('insert', $start_index."+ 1 chars");
	}
}

1; #So muss ein Package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
