#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package TagForm;

=head1 NAME:

Object: TagForm

=head1 SYNOPSIS:

use class::TagForm;

=head1 DIRECTORY/FILE:

The file TagForm.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

The object TagForm encapsulates the tag properties (=arguments) editor window. It is now coded as a normal non-visual Perl class
that creates an user interface in the $self->{'ui'} branch. It might be more clever to (later) transform it
into a Toplevel-derived widget.

This window is used to allow to edit the arguments of a STYLE like you can do it with the embeded ArgEdit object in an embeded COMMAND object.
It uses the same ArgEdit class, it just embeds it into this window instead of the documents text widget.

=head1 STATUS:

Coding: 60% done

Documentation: up-do-date

NOTE: Here all objects get correctly destroyed after closing the window --> !NO CIRCULAR REFERENCES!

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

$self->{'tag_style'} #The style name of the tag which arguments should be browsed in this TagForm window

$self->{'tag_name'} #The tag name which arguments should be edited

$self->{'docform'} #A reference to the DocForm object that created the TagForm window

$self->{'ui'}->{'_top'} #A referenece to the Toplevel

$self->{'ui'}->{...the rest of this branch...} #References to the widgets that build the user interface

=head1 METHODS:

=head2 Class Methods:

=head2 Constructors:

=head3 new

$object = TagForm->new($docform, $tag_name, $is_object, $ref_object, $is_doc_style); 

This constructor creates a new object of the class TagForm. 

Parameters: 

(+) $docform: A reference to the DocForm object the TagForm was created from

(+) $tag_name: The name of the tag whichs arguments should be edited

(+) $is_object: Flag is set to 1, if arguments are stored in an object, else it is 0

(+) $ref_object:  Reference to the associated object (if any) or undef

(+) $is_doc_style: Flag is set to 1, if arguments are stored for the whole document itself, else it is 0

=cut

###############
# Konstruktor #
###############
sub new
{
my ($class, $docform, $tag_name, $is_object, $ref_object, $is_doc_style)=@_;
my $self = {};
bless($self, $class);
my $style;
if ($tag_name =~ /<(.+?)><(.+?)><(.*?)>:\d+/) 
	{
	$style = $2;
	}

$self->tag_style( $style );
$self->tag_name( $tag_name );
$self->docform( $docform );
$self->tagman( $docform->tagman() );

my $config = $main::config;
my $speedbarfont = $config->{'UI'}->{'speedbarfont'};
my $activebackground = $config->{'UI'}->{'activebackground'};
my $buttons_style = $config->{'UI'}->{'buttons_style'};
my $buttons_borderwidth = $config->{'UI'}->{'buttons_borderwidth'};

$self->{'_top'}=$self->docform()->top()->Toplevel();
$self->top()->minsize(300,100);
$self->top()->transient($self->docform()->top());
if ($is_doc_style)
	{
	$self->top()->configure(-title=>"Document arguments");
	}
else
	{
	$self->top()->configure(-title=>"Arguments of ".$self->tag_style());
	}
$self->{'_top_panel'}=$self->top()->Frame(-borderwidth=>4)->pack(-side=>'top',-expand=>1, -fill=>'both');
$self->{'_bottom_panel'}=$self->top()->Frame()->pack(-side=>'bottom',-expand=>1, -fill=>'x');
$self->{'_close_btn'}=$self->{'_bottom_panel'}->Button(-command=>sub{$self->do_destroy()}, -text=>'Close', -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$speedbarfont)->pack(-padx=>4, -pady=>4,-expand=>1, -fill=>'both');
$self->{'_arg_edit'} = $self->{'_top_panel'}->ArgEdit(-tagman=>$self->tagman(), -tag_name=>$self->tag_name(), -is_object=>$is_object, -ref_object=>$ref_object, -is_doc_style=>$is_doc_style)->pack(-expand=>1, -fill=>'both');

#The Cleanup-Routine has to be done with the "protocol-Function", because:
#1.)When using the do_destroy-Method, the callback is only invoked AFTER all other Widgets in the Toplevel are destroyed,
#so the text in the Textwidget can not be saved anymore
#2.)When binding the '<DESTROY>'-Event to the method, it is called whenever one widget
#in the MainWindow is deleted (Even the embeded ones in the Textwidget
#The Cleanup-Function &do_destroy  is needed for:
#1.) Asking the user to save unsaved changes in the document
#2.) To remove not only the MainWindow but also the object "PrjForm" itself
$self->top()->protocol('WM_DELETE_WINDOW',sub{$self->do_destroy()}); #Binding the cleanupfunction to the closing of the window BEFORE the subwidgets are destroyed

#Registering the TagForm in the @dialogs array of the document form
push (@{$self->docform()->{'dialogs'}},$self);

print "Object $self created.\n"; #debug

return $self;
}

=head2 Destructors:

=head3 DESTROY 

The standard automatically called PERL destructor for objects.
Unfortionatly it is called very late (when the whole application terminates.)

=cut

#############################
# Automatic Perl destructor #
#############################
sub DESTROY
{
my $self = shift;
print "Object $self destroyed.\n"; #debug
}

=head3 do_detstroy

$object->do_detsroy(); 

This destructor is binded to the protocol-message WM_DELETE_WINDOW
and performs the 'unsaved changes'-check, destroys the Toplevel manually
and then destroys the TblForm object manually too.

=cut

##############
# Destruktor #
##############
sub do_destroy
{
my $self=shift;
#Find myself in the $docform->{'dialogs'} hash
if (defined $self->docform()->dialogs())
	{
	#Iterate over the list items
	for (my $i=0; $i<=$#{$self->docform()->dialogs()}; $i++)
		{
		#Get the stored dialog-reference there
		my $dlg = $self->docform()->dialogs()->[$i];
		#If its this dialog
		if ($dlg == $self)
			{
			#remove it from the array
			splice (@{$self->docform()->dialogs()},$i,1);
			last;
			}
		}
	}

#The default callback for WM_DELETE_WINDOW is not used, because we replaced it with this do_destroy-Function, so we have to destroy the MainWindow manually
$self->top()->destroy();
#remove the TagForm object itself, after the user closed the window
$self = undef;
}

=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 docform

$ref_docform = $object->docform(); #gets a reference to the associated DocForm object

$object ->docform($ref_docform); #sets a reference to the associated DocForm object

=cut

### docform ###
sub docform
{
my $self=shift;
@_ ? $self->{'docform'} = shift : return $self->{'docform'};
}

=head3 tagman

$ref_tagman = $object->tagman(); #gets a reference to the associated TagMan object

$object ->tagman($ref_tagman); #sets a reference to the associated TagMan object

=cut

### tagman ###
sub tagman
{
my $self=shift;
@_ ? $self->{'tagman'} = shift : return $self->{'tagman'};
}

=head3 tag_name

$tag_name = $object->tag_name(); #gets the tags name whichs arguments should be edited

$object ->tagname($tag_name); #sets the tags name whichs arguments should be edited

=cut

### tag_name ###
sub tag_name
{
my $self=shift;
@_ ? $self->{'tag_name'} = shift : return $self->{'tag_name'};
}

=head3 tag_style

$style = $object->tag_style(); #gets the tags style name whichs arguments should be edited

$object ->tag_style($style); #sets the tags style name whichs arguments should be edited

=cut

### tag_style ###
sub tag_style
{
my $self=shift;
@_ ? $self->{'tag_style'} = shift : return $self->{'tag_style'};
}

=head3 top

$style = $object->top(); #gets a reference to the Tolevel window (READ-ONLY)

=cut

### top ###
sub top
{
my $self=shift;
return $self->{'_top'};
}

=head2 Other public methods:

Nothing here yet.

=cut

=head2 Private methods:

Nothing here yet.

=cut

1; #So muss ein Package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
