#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package TblForm;
require class::ui::uiTblForm; #beinhaltet die GUI-Erstellung

package TblForm;

=head1 NAME:

Object: TblForm

=head1 SYNOPSIS:

use class::TblForm;

=head1 DIRECTORY/FILE:

The file TblForm.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

The object TblForm encapsulates the TblObj property-editor window. It is now coded as a normal non-visual Perl class
that creates an user interface in the $self->{'ui'} branch. It might be more clever to (later) transform it
into a Toplevel-derived widget.

This object is in a _very_basic_ stage, it should just demonstrate how it could work.

Now the GUI buildup functions are stored in the package "./ui/uiTblForm.pm".

=head1 STATUS:

Coding: 5% done

Documentation: up-do-date

NOTE: Here all objects get correctly destroyed after closing the window --> !NO CIRCULAR REFERENCES!

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

$self->{'embobj'} #A reference to the TblObj the TblForm belongs to

$self->{'docform'} #A reference to the DocForm object that created the TagForm window

$self->{'tagman'} #A reference to the TagMan object of the DocForm object that created the TagForm window

$self->{'ui'}->{'_top'} #A referenece to the Toplevel

$self->{'ui'}->{...the rest of this branch...} #References to the widgets that build the user interface

=head1 METHODS:

=head2 Class Methods:

=head2 Constructors:

=head3 new

$object = tblForm->new($docform,$embobj); 

This constructor creates a new object of the class TblForm.

This constructor calls 'class::ui::uiTblForm::build_ui($self)' to build up the GUI.

Parameters: 

(+) $docform: A reference to the DocForm object the TblForm was created from

(+) $embobj: A reference to the TblObj derived widget the TblForm belongs to

=cut

###############
# Konstruktor #
###############
sub new
{
my ($class, $docform, $window_tag, $is_object, $embobj)=@_;
# $window_tag and $is_object are not used in this form, they are just passed
# to all property editors (maybe they might be usefull somewhere.
# Note: also tags (not only objects) have property editors.
my $self = {};
bless($self, $class);
$self->embobj( $embobj );
$self->docform( $docform );
$self->tagman( $self->docform()->tagman() );
uiTblForm::build_ui($self);
#The Cleanup-Routine has to be done with the "protocol-Function", because:
#1.)When using the do_destroy-Method, the callback is only invoked AFTER all other Widgets in the Toplevel are destroyed,
#so the text in the Textwidget can not be saved anymore
#2.)When binding the '<DESTROY>'-Event to the method, it is called whenever one widget
#in the MainWindow is deleted (Even the embeded ones in the Textwidget
#The Cleanup-Function &do_destroy  is needed for:
#1.) Asking the user to save unsaved changes in the document
#2.) To remove not only the MainWindow but also the object "PrjForm" itself
$self->top()->protocol('WM_DELETE_WINDOW',sub{$self->do_destroy()}); #Binding the cleanupfunction to the closing of the window BEFORE the subwidgets are destroyed

print "Object $self created.\n"; #debug
return $self;
}

=head2 Destructors:

=head3 DESTROY; 

The standard automatically called PERL destructor for objects.
Unfortionatly it is called very late (when the whole application terminates.)

=cut

#############################
# Automatic Perl destructor #
#############################
sub DESTROY
{
my $self = shift;
print "Object $self destroyed.\n"; #debug
}

=head3 do_destroy

$object ->do_destroy(); 

This destructor is binded to the protocol-message WM_DELETE_WINDOW
and performs the 'unsaved changes'-check, destroys the Toplevel manually
and then destroys the TblForm object manually too.

=cut

##############
# Destruktor #
##############
sub do_destroy
{
my $self = shift;
#The default callback for WM_DELETE_WINDOW is not used, because we replaced it with this do_destroy-Function, so we have to destroy the MainWindow manually
$self->top()->destroy();
#remove the TblForm object itself, after the user closed the window
$self = undef;
}

=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 docform

$ref_docform = $object->docform(); #gets a reference to the associated DocForm object

$object ->docform($ref_docform); #sets a reference to the associated DocForm object

=cut

### docform ###
sub docform
{
my $self=shift;
@_ ? $self->{'docform'} = shift : return $self->{'docform'};
}

=head3 tagman

$ref_tagman = $object->tagman(); #gets a reference to the associated TagMan object

$object ->tagman($ref_tagman); #sets a reference to the associated TagMan object

=cut

### tagman ###
sub tagman
{
my $self=shift;
@_ ? $self->{'tagman'} = shift : return $self->{'tagman'};
}

=head3 embobj

$ref_tblobj = $object->embobj(); #gets a reference to the associated TblObj object

$object ->embobj($ref_tblobj); #sets a reference to the associated TblObj object

=cut

### embobj ###
sub embobj
{
my $self=shift;
@_ ? $self->{'embobj'} = shift : return $self->{'embobj'};
}


=head3 top

$style = $object->top(); #gets a reference to the Tolevel window (READ-ONLY) of this object

=cut

### top ###
sub top
{
my $self=shift;
return $self->{'ui'}->{'_top'};
}

=head2 Other public methods:

Nothing here yet.

=cut

=head2 Private methods:

Nothing here yet.

=cut

1; #So muss ein Package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
