#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package TblObj;

use Tk qw(Ev);
require lib::Utils;
require class::EmbObj; #It seems to be important that you write "class::TblObj" with "use" or "require"
@ISA = qw(Tk::Derived EmbObj); #It seems to be impotant that you write "TblObj" in @ISA, but you have to write "Tk::Frame" or "Tk::Derived". ???
Tk::Widget->Construct('TblObj');

require widgets::XGrid;
use strict;

=head1 NAME:

Widget: TblObj

=head1 SYNOPSIS:

use class::TblObj;

=head1 DIRECTORY/FILE:

The file TblObj.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

This is a widget derived from class::EmbObj that gets embeded into the documents text widget in the DocForm object (=document window).

This class is for embeding tables into the document.

This class is in a very basic stage, it's just for demonstarting that it's quiet easy to add new embeded objects to e:doc.

=head1 STATUS:

Coding: 70% done

Documentation: up-to-date

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

[... List of attributes is missin here ... ]

All attributes derived from the class EmbObj.

NOTE: The following attributes are used for the Tk widgets building the GUI and are accessed directly (no get/set methods):

All GUI attributes derived from the class EmbObj.

=head1 METHODS:

=head2 Constructors:

=head3 build_ui

Here your widget could build up all its own widgets and gets initialised

=cut

################
# Fill the GUI #
################
sub build_ui
{
my $self = shift;
my $embed_base = $self->embed_base();
my $obj_config = $self->obj_config();

#Here the values form the user-entries or from the opened files are stored
my $cols;
my $rows;
$cols = $self->{'_data'}->{'cols'} if defined $self->{'_data'}->{'cols'};
$rows = $self->{'_data'}->{'rows'} if defined $self->{'_data'}->{'rows'};

#Changing the used property editor
$self->propedit_classname('TblForm');

#Query column count from the user if not in the saved data
if (not defined $cols)
	{
	$cols = $self->docform()->dialog('Please enter the number of columns.', 'Columns:', 1 );
	}

#Query row count from the user if not in the saved data
if (not defined $rows)
	{
	$rows = $self->docform()->dialog('Please enter the number of rows.', 'Rows:', 1) ;
	}

#Query table caption from the user if not in the saved data
if (not defined $self->caption() or $self->caption() eq '')
	{
	$self->caption($self->docform()->dialog('Please enter the caption.', 'Caption:', 'new table') );
	}

#Building up the table itself
$self->table( $embed_base->XGrid(-initialrows=>$rows, -initialcols=>$cols, -width=>300, -height=>200)->pack(-expand=>1, -fill=>'both')  );

#Building up configuration data for the cells
my @style_options = ('font','underline', 'justify','background','foreground','anchor','relief','borderwidth'); 
$self->{'style_options'} = {};
my @style_names ;
my $style_number = $self->obj_config()->{'_style_number'};
for (my $i = 1; $i <= $style_number; $i++)
	{
	my $style_name = $self->obj_config()->{"_style_".$i."_name"};
	push (@style_names, $style_name);
	foreach my $option ( @style_options)
		{
		if (exists $self->obj_config()->{"_style_".$i."_".$option} and defined $self->obj_config()->{"_style_".$i."_".$option} )
			{
			$self->{'style_options'}->{$style_name}->{"-".$option} = $self->obj_config()->{"_style_".$i."_".$option};
			}
		}
	$self->table()->define_style($style_name, $self->{'style_options'}->{$style_name});
	}

for (my $x = 1 ; $x <= $self->table()->cols(); $x++)
	{
	if (exists $self->{'_data'}->{"col_width_".$x} and defined $self->{'_data'}->{"col_width_".$x})
		{
		$self->table()->set_col_width($x,$self->{'_data'}->{"col_width_".$x});
		}
	for (my $y = 1 ; $y <= $self->table()->rows(); $y++)
		{
		if (exists $self->{'_data'}->{"row_height_".$y} and defined $self->{'_data'}->{"row_height_".$y})
			{
			$self->table()->set_row_height($y,$self->{'_data'}->{"row_height_".$y});
			}
		}
	}

#Building up the content
for (my $x = 1 ; $x <= $self->table()->cols(); $x++)
	{
	for (my $y = 1 ; $y <= $self->table()->rows(); $y++)
		{
		if (exists $self->{'_data'}->{"cell_content_".$x."_".$y} and defined $self->{'_data'}->{"cell_content_".$x."_".$y})
			{
			my $content = $self->{'_data'}->{"cell_content_".$x."_".$y};
			$content =~ s/%NL%/\n/g;
			$self->table()->put($x, $y, $content);
			my $style_name = $self->{'_data'}->{"cell_style_".$x."_".$y};
		 	$self->table()->set_cell_style( $x, $y, $style_name );
			}
		}
	}
}

=head2 Destructors:

=head3 do_detstroy

$object->do_detsroy(); 

This "destructor" method is binded to the "<DESTROY>" event and should be performed whenever
the element is destroyed. It should not be called directly.

=cut

###################################################
# Destruktor zum sauberen aufraeumen des Objektes #
###################################################
sub do_destroy
{
my $self = shift;
print $self->caption()." was destroyed \n"; #Debug
$self->SUPER::do_destroy();
}

=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 table

$ref_table = $object->table(); #Gets a reference to the XGrid widget that displays the table

$object->table($ref_table); #sets a reference to the XGrid widget that displays the table

=cut

### table ###
sub table
{
my $self=shift;
@_ ? $self->{'_table'} = shift : return $self->{'_table'};
}

=head2 Other public methods:

=head3 export_to_backend

my @export_text = $object->export_to_backend(); 

This function exports the content of the object to the current backend.

The function is called in the document-exporting process, whenever an embeded object
is found. 

It returns the text for exporting the object.

=cut

##################################
# Export to the current backend. #
##################################
sub export_to_backend
{
my $self = shift;

my $result = '';

my $backend = $self->backend();
my $col_hdir_left = $backend->{'col_hdir_left'};
my $col_hdir_center = $backend->{'col_hdir_center'};
my $col_hdir_right = $backend->{'col_hdir_right'};
my $colspec_item = $backend->{'colspec_item'};
my $colspec_divider = $backend->{'colspec_divider'};

#Process COLSPECS definition in all other backend fields
if (defined $colspec_item and defined $colspec_divider)
	{
	my $colspecs = '';
	for (my $x = 0; $x<=($self->table()->cols()-1); $x++)
		{
		my $col_item = $colspec_item;
		my $col_hdir = $col_hdir_left;
		my $col_width = '';
		my $col_width_percent = 100 / $self->table()->cols();
		#FIXME:
		#$col_hdir = $self->{'_data'}->{"col_".$x."_hdir"} if defined $self->{'_data'}->{"col_".$x."_hdir"};
		#$col_width = $self->{'_data'}->{"col_".$x."_width"} if defined $self->{'_data'}->{"col_".$x."_width"};
		$col_item =~ s/%COL_WIDTH%/$col_width/g ;
		$col_item =~ s/%COL_WIDTH_PERCENT%/$col_width_percent/g ;
		$col_item =~ s/%COL_HDIR%/$col_hdir/g ;
		$colspecs .= $col_item;
 	   $colspecs .= $colspec_divider if $x < ($self->table()->cols()-1);
		}
	foreach my $field_name (keys %{$self->backend()})
		{
		$self->backend()->{$field_name} =~ s/%COLSPECS%/$colspecs/g ;
		}
	}

my $table_front = $backend->{'table_front'};
my $table_end = $backend->{'table_end'};
my $row_front = $backend->{'row_front'};
my $row_end = $backend->{'row_end'};
my $row_divider = $backend->{'row_divider'};
my $style_number = $self->obj_config()->{'_style_number'};
my %style_front;
my %style_end;
my %style_divider;
for (my $i = 1; $i <= $style_number; $i++)
	{
	my $style_name = $self->obj_config()->{"_style_".$i."_name"};
	$style_front{$style_name} = $self->process_backend_field($backend->{"style_".$i."_front"});
	$style_end{$style_name} = $self->process_backend_field($backend->{"style_".$i."_end"});
	$style_divider{$style_name} = $self->process_backend_field($backend->{"style_".$i."_divider"});
	}

#Export the start-tag of the table
$result .= $table_front;

#Iterate through the rows
for (my $y = 1; $y <= $self->table()->rows() ; $y++)
	{

	#Export the start-tag of the row
	$result .= $row_front if not ($row_front eq '');

	#Iterate through the cells
	for (my $x = 1; $x <= $self->table()->cols(); $x++)
		{
		#Get the style of the table
		my $style_name = $self->table()->get_cell_style($x, $y);
		#Export the start-tag of the cell
		$result .= $style_front{$style_name} if ( exists $style_front{$style_name} and defined $style_front{$style_name} and not ($style_front{$style_name} eq '') ); 
		
		#Export the start-tag of the cell
		$result .= $self->table()->get($x, $y);

		#Export the end-tag of the cell
		$result .= $style_end{$style_name} if ( exists $style_end{$style_name} and defined $style_end{$style_name} and not ($style_end{$style_name} eq '') );
		$result .= $style_divider{$style_name}  if ( exists $style_divider{$style_name} and defined $style_divider{$style_name} and not ($style_divider{$style_name} eq '')  and $x < $self->table()->cols() );
		}

	#Export the end-tag of the row
	$result .= $row_end if not ($row_end eq '');
	$result .= $row_divider  if not ($row_divider eq '') and ($y < $self->table()->rows());
	}


#Export the end-tag of the table
$result .= "$table_end";

return $result;
}

=head2 Private methods:

=head3 _before_save

$object->_before_save(); 

This function is called before the $self->{'_data'}-subbranch gets saved.

It's used here to build up the flat {'_data'} subbranch from the more complicated datastructures.

=cut

################
# _before_save #
################
sub _before_save
{
my $self = shift;
#CALLING THE SUPER-CLASS (DO NOT CHANGE THIS LINE)
$self->SUPER::_before_save();

$self->{'_data'}->{'cols'} = $self->table()->cols();
$self->{'_data'}->{'rows'} = $self->table()->rows();

for (my $y = 1 ; $y <= $self->table()->rows(); $y++)
	{
	my $row_height = $self->table()->get_row_height($y);;
	$self->{'_data'}->{"row_height_".$y} = $row_height;
	}

for (my $x = 1 ; $x <= $self->table()->cols(); $x++)
	{
	my $col_width = $self->table()->get_col_width($x);
	$self->{'_data'}->{"col_width_".$x} = $col_width;
	for (my $y = 1 ; $y <= $self->table()->rows(); $y++)
		{
		my $content = $self->table()->get($x, $y);
		my $style = $self->table()->get_cell_style($x, $y);
		if (defined $content)
			{
			$content =~ s/\n$//g;
			$content =~ s/\n/%NL%/g;
			$self->{'_data'}->{"cell_content_".$x."_".$y} = $content;
			}
		if (defined $style)
			{
			$self->{'_data'}->{"cell_style_".$x."_".$y} = $style;
			}
		}
	}
}

=head3 _after_load

$object->_after_load(); 

This function is called after the $self->{'_data'}-subbranch is loaded from file.

It's used here to read information the flat {'_data'} subbranch into the more complicated datastructures.

=cut

###############
# _after_load #
###############
sub _after_load
{
my $self = shift;
#CALLING THE SUPER-CLASS (DO NOT CHANGE THIS LINE)
$self->SUPER::_after_load();

#YOUR CODE COMES HERE...

}

1; #So muss ein Package enden


=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
