#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package TemplateChooser;

use Tk::TList;
use Tk::DialogBox;

=head1 NAME:

Object: TemplateChooser

=head1 SYNOPSIS:

use class::TemplateChooser;

=head1 DIRECTORY/FILE:

The file TemplateChooser.pm is located in the subdirectory './class'

=head1 DESCRIPTION:

The object TemplateChooser encapsulates the dialog used for choosing a template.

=head1 STATUS:

Coding: 99% done

Documentation: up-do-date

NOTE: Here all objects get correctly destroyed after closing the window --> !NO CIRCULAR REFERENCES!

=head1 ATTRIBUTES:

Do not set/read them directly, use the Get/Set methods instead.

$self->{'prjform'} #A reference to the PrjForm object this window is associated with

$self->{'ui'}->{'_top'} #A referenece to the Toplevel

$self->{'ui'}->{...the rest of this branch...} #References to the widgets that build the user interface

=head1 Class variables:

=cut

=head1 METHODS:

=head2 Class Methods:

=head2 Constructors:

=head3 new

$object = TemplateChooser->new($ref_prjform); 

This constructor creates a new object of the class TemplateChooser.

it returns (fter the dialog was executed) the filename of the choosen template.

Parameters: 

(+) $ref_prjform: A reference to the PrjForm object the TemplateChooser was created from

=cut

###############
# Konstruktor #
###############
sub new
{
my ($class, $ref_prjform, $ref_templatename )=@_;
my $self = {};
bless($self, $class);

#Initialise some properties
$self->prjform( $ref_prjform );

my $return_value = undef;
my $config = $main::config;
my $activebackground = $config->{'UI'}->{'activebackground'};
my $buttons_style = $config->{'UI'}->{'buttons_style'};
my $buttons_borderwidth = $config->{'UI'}->{'buttons_borderwidth'};
my $speedbarfont = $config->{'UI'}->{'speedbarfont'};

my $templates_dir = $config->{'GENERAL'}->{'templates_dir'};

if (not -d $templates_dir)
	{
	$ref_prjform->message("The template directory '$template_dir' defined in edoc.cfg does not exist !",'ERROR:');
	$self->do_destroy();
	}


#Get a list with all names (short names) of the filetypes
my @alist = sort keys %{$self->prjform()->{'filetypes'}};
$self->{'filetypes_short'} = \@alist;

#Build up a list with the long names in the same sort order
$self->{'filetypes_long'} = [];
foreach $type ( @{$self->{'filetypes_short'}} )
	{
	push(@{$self->{'filetypes_long'}}, $self->prjform()->{'filetypes'}->{$type}->{'caption'});
	}

foreach $filetype (@{$self->{'filetypes_short'}})
	{
	$self->{'files_by_filetype'}->{$filetype} = [];
	}

#Read all files in the template directory
chdir $templates_dir;
opendir (DIR, '.')||die "Cannot open $templates_dir \n";
#Retrieving file information
foreach my $file (readdir(DIR))
	{
	if ($file=~ /\.([^.]*)$/)
		{
		foreach $filetype (@{$self->{'filetypes_short'}})
			{
			foreach my $ext (@{$self->prjform()->{'filetypes'}->{$filetype}->{'extensions'}})
				{
				if (uc($ext) eq uc($1))
					{
					push (@{$self->{'files_by_filetype'}->{$filetype}}, $file);
					}
				}
			}
		}
	}
closedir (DIR);

chdir $self->prjform()->{'project'}->{'path'};

#The Toplevel window

my $dlg = $self->{'ui'}->{'_dlg'} = $self->prjform()->app_window()->DialogBox(-title=>'Choose a template',  -buttons=>['Ok', 'Cancel'], -default_button=>'Ok');
my $ui_base = $self->{'ui'};

#Variables to store the current selected filetype and the currently selected file
$self->{'current_filetype_long'} = 'DOCUMENTS';
$self->{'current_file'} = '';

my $panel = $dlg->Frame(-relief=>'flat')->pack(-side=>'top', -fill=>'x', -expand=>1, -padx=>2, -pady=>2);
my $label = $panel->Label(-text=>'Filetypes:', -anchor=>'e')->pack(-side=>'left', -fill=>'y');
$ui_base->{'_filetypes'} = $panel->BrowseEntry(
										-width=>30,
										-variable=>\$self->{'current_filetype_long'}, 
										-browsecmd=>[\&_filetypes_changed, $self],
										-font=>$speedbarfont,
										-choices=>$self->{'filetypes_long'}
										)->pack(-side=>'left', -fill=>'both', -expand=>1);
Utils::set_browseentry_color($ui_base->{'_filetypes'}, 'white');
Utils::beautify_browseentry($ui_base->{'_filetypes'});
Utils::create_balloonhelp($ui_base->{'_filetypes'}, 'Chosse a filetype here', 'balloon', undef);
$ui_base->{'_files'} = $dlg->TList(
												-browsecmd=>[\&_files_changed, $self], 
												-background=>'white', 
												-font=>$speedbarfont,
												-orient=>'vertical',
												-selectmode=>'single'
												)->pack(-side=>'top', -fill=>'both', -expand=>1);

$ui_base->{'_files'}->focus();
$dlg->idletasks();
$self->_filetypes_changed();
$dlg->idletasks();
$self->_files_changed();
$dlg->idletasks();

#Buttons in the Bottom Panel
my $btn = $dlg->Show(-global=>1);

if ($btn eq 'Cancel') 
	{
	$self->{'current_file'} = undef;
	}

return $self->{'current_file'};
}

=head2 Destructors:

=head3 DESTROY 

The standard automatically called PERL destructor for objects.
Unfortionatly it is called very late (when the whole application terminates.)

=cut

#############################
# Automatic Perl destructor #
#############################
sub DESTROY
{
my $self = shift;
print "Object $self destroyed.\n"; #debug
}

=head3 do_detstroy

$object->do_detsroy(); 

=cut

##############
# Destruktor #
##############
sub do_destroy
{
my $self=shift;

#remove the TemplateChooser object itself, after the user closed the window
$self = undef;
}

=head2 Public data methods (=Get/Set methods):

=cut

##############################
### PUBLIC GET/SET METHODS ###
##############################

=head3 prjform

$ref_prjform = $object->prjform(); #gets a reference to the associated prjform object

$object ->prjform($ref_prjform); #sets a reference to the associated prjform object

=cut

### prjform ###
sub prjform
{
my $self=shift;
@_ ? $self->{'prjform'} = shift : return $self->{'prjform'};
}

=head2 Other public methods:

=head3 _filetypes_changed

$object->_filetypes_changed(); 

This is the callback for a change in the BrowseEntry that displays the filetypes.

=cut 

###########################
# Filetypes combo changed #
###########################
sub _filetypes_changed
{
my $self = shift;

#Shortcut to the TList
my $list = $self->{'ui'}->{'_files'};
	
#Delete all entries
$list->delete(0,'end'); 

#Get the short filetype code
my $filetype_long = $self->{'current_filetype_long'};
my $filetype_short = undef;
for (my $i = 0; $i<= $#{$self->{'filetypes_long'}}; $i++)
	{
	if ($self->{'filetypes_long'}->[$i] eq $filetype_long)
		{
		$filetype_short = $self->{'filetypes_short'}->[$i];
		}
	}

if (defined $filetype_short)
	{
	my $icon = $self->prjform()->{'ui'}->{'item_icons'}->{$filetype_short};
	my @fileslist = @{$self->{'files_by_filetype'}->{$filetype_short}};
	
	foreach my $file (sort @fileslist)
		{
		$list->insert('end', -itemtype=>'imagetext', -text=>$file, -image=>$icon);
		}
	
	$list->selectionClear();
	$list->selectionSet(0) if ($#fileslist >= 0);
	}
}

=head3 _files_changed

$object->_files_changed(); 

This is the callback for a change in the TList that displays the template files.

=cut 

######################
# Files list changed #
######################
sub _files_changed
{
my $self = shift;
my $templatename = '';

my $list = $self->{'ui'}->{'_files'};

my $path = $list->infoSelection;
if (not defined $path or $path eq '')
	{
	goto EXIT;
	}

my $file = $list->entrycget($path, '-text');

if (not defined $file or $file eq '')
	{
	goto EXIT;
	}

$templatename = $file;

EXIT:
$self->{'current_file'} = $templatename;
return $templatename;
}


1; #So muss ein Package enden



=head1 KNOWN BUGS:

There is too much direct access to hash-values in the PrjForm class in this class.

We will have to create more Get/Set methods in PrjForm.pm and use them here later.

=head1 STILL MISSING:

=cut
