#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package uiBaseForm;

=head1 NAME:

Package: uiBaseForm

=head1 SYNOPSIS:

use class::ui::uiBaseForm;

=head1 DIRECTORY/FILE:

The file uiBaseForm.pm is located in the subdirectory './class/ui'

=head1 DESCRIPTION:

This packages contains all functions that are needed to build and modify the user interface of a BaseForm object (BaseForm.pm).

Because BaseForm is only a never-directly used base class, it only conatins the build up of
the very basic widgets: Toplevel, Menubar, Speedbar and Statusbar.

This is the stuff I don't like on the Tk toolkit: Large spaghetti-code just to produce the GUI of one Window.
So I put all these functions in a separate file in ./class/ui, where every window has one file
that produces the GUI. I try to prevent to work in these files often, because I hate them.

This package file is not designed to function as an object, so there are no attributes, no methods and no Get/Set functions,
it is only a package of functions that belong thematically together (They are working with the same window object).

The windows object is placed in ./class, there is all of the windows functionality 

=head1 STATUS:

Coding: 90% done

Documentation: up-do-date


=head1 FUNCTIONS:

=cut

use strict;

=head2 build_ui

uiDocForm::build_ui($ref_form, $config_name); 

Builds up the basic user interface of a Form object and stores the references to all created
widgets in the $ref_docform->{'ui'} hash.

The attributes are accessed directly ($ref_docform->{'ui'}->{...}) because there is no Get/Set-method for each widget.

F.ex. the Toplevel is created in $ref_docform->{'ui'}->{'top'}.

Parameters:

(+) $ref_form: A reference to the BaseForm derived form object

(+) $config_name : The name of the section in edoc.cfg that describes the GUI

=cut

################################
# Build up the base UI-Widgets #
################################
sub build_ui
{
my $ref_form = shift;
my $config_name = shift;

my $config = $main::config;

### Hauptfenster mu zuerst aufgebaut werden ###
if ($config_name eq 'PROJECT_FORM')
	{
	$ref_form->{'ui'}->{'app_window'} = MainWindow->new();
	$ref_form->{'ui'}->{'top'} = $ref_form->{'ui'}->{'app_window'};
	}
else
	{
	$ref_form->{'ui'}->{'top'}= $main::prjform->app_window()->Toplevel();
	}

$ref_form->{'ui'}->{'top'}->withdraw();

# Creating the application icon
$main::application_icon = $ref_form->{'ui'}->{'top'}->Pixmap(-file => Utils::process_filename($config->{'UI'}->{'application_icon'}) ); 
$ref_form->{'ui'}->{'top'}->Icon(-image => $main::application_icon );

### read some configurations into simple scalars, so that the code is shorter ###
my $speedbarfont = $config->{'UI'}->{'speedbarfont'};
my $toolboxfont = $config->{'UI'}->{'toolboxfont'}; 
my $activebackground = $config->{'UI'}->{'activebackground'};
my $buttons_style = $config->{'UI'}->{'buttons_style'};
my $buttons_borderwidth = $config->{'UI'}->{'buttons_borderwidth'};
my $ui_base = $ref_form->{'ui'};
my $top = $ui_base->{'top'};

### Menu ###
$ui_base->{'menu'} = $top->Frame(-relief=>'raised', -borderwidth=>2)->pack(-side=>'top', -fill=>'x', -expand=>0);

### Right menu entries ###
$ui_base->{'menu_right'} = $ui_base->{'menu'}->Frame()->pack(-side =>'right', -fill=>'y', -expand=>1, -anchor=>'e');

### Left menu entries ###
$ui_base->{'menu_left'} = $ui_base->{'menu'}->Frame()->pack(-side =>'left', -fill=>'y', -expand=>1, -anchor=>'w');


### User defined menus ###
my @menulist = @{$config->{$config_name}->{'MENU'}->{'_MENULIST'}};
foreach my $menu (@menulist)
	{
	build_menu($config_name, $ref_form, $menu, $config->{$config_name}->{'MENU'}->{$menu}, $config->{$config_name}->{'MENU'}->{$menu}->{'_ENTRYLIST'});
	}


### Basiszeile mit Dateiname ###
$ui_base->{'bottomframe'} = $top->Frame(-borderwidth=>2,-height=>28, -relief=>'raised')->pack(-side=>'bottom', -expand=>0, -fill=>'x');
$ui_base->{'basisfile'} = $ui_base->{'bottomframe'}->Frame(-relief=>'sunken')->pack(-side=>'left', -expand=>0);
$ui_base->{'file_label'} = $ui_base->{'basisfile'}->Label(-textvariable=>$ref_form->ref_filename())->pack(-side=>'left', -expand=>0);
$ui_base->{'file_changed_label'} = $ui_base->{'basisfile'}->Label(-textvariable=>$ref_form->ref_content_changed())->pack(-side=>'right', -fill=>'both', -expand=>0);
$ui_base->{'progress_bar'} = $ui_base->{'bottomframe'}->Canvas(-width=>202, -height=>16)->pack(-side=>'right',-anchor=>'center', -expand=>0);

### Speedbar  ###
build_speedbar($config_name, $ref_form);
}

=head2 build_speedbar

uiBaseForm::build_speedbar($config_name, $ref_form); 

Builds the speedbar and sets the commands to the right macros defined in 'edoc.cfg'

This is done in a seperate function, because its better to have an overview like this.

Parameters:

(+) $config_name: The name of the section in edoc.cfg that holds the info

(+) $ref_form: Reference to the object which user interface has to be created

=cut

sub build_speedbar
{
my $config_name = shift;
my $ref_form = shift;
my $ref_config = $main::config;
### read some configurations into simple scalars, so that the code is shorter ###
my $speedbarfont = $ref_config->{'UI'}->{'speedbarfont'};
my $toolboxfont = $ref_config->{'UI'}->{'toolboxfont'}; 
my $activebackground = $ref_config->{'UI'}->{'activebackground'};
my $buttons_style = $ref_config->{'UI'}->{'buttons_style'};
my $buttons_borderwidth = $ref_config->{'UI'}->{'buttons_borderwidth'};
my $ui_base = $ref_form->{'ui'};
my $form_wnd;
if ($config_name eq 'DOCUMENT_FORM')
	{
	$form_wnd = $ui_base->{'top'};
	}
elsif ($config_name eq 'PROJECT_FORM')
	{
	$form_wnd = $ui_base->{'app_window'};
	}
else
	{
	goto EXIT;
	}
#Now lets do the job and build a speedbar
$ui_base->{'speedbar'} = $form_wnd->Frame(-borderwidth=>2,-height=>28, -relief=>'raised')->pack(-side=>'top',-fill=>'x', -expand=>0);
my %speedbar_cfg = %{$ref_config->{$config_name}->{'SPEEDBAR'}};
#Possible options for a separator and a button
my $button_layout_options = '%%ipadx%%ipady%%activebackground%%highlightbackground%%activeforeground%%highlightforeground%%color%%activecolor%%highlightcolor%%text%%font%%background%%borderwidth%%foreground%%justify%%relief%%width%%height%%padx%%pady%%align%%underline%%cursor%%';
my $separator_layout_options = '%%label%%takefocus%%background%%borderwidth%%relief%%cursor%%width%%height';
#Get the ordered button and separator list
my @buttonlist = @{$ref_config->{$config_name}->{'SPEEDBAR_NAMES'}};
#Its shorter with this ref
my $speedbar = $ui_base->{'speedbar'};
#Iterate through it
foreach my $btn_name (@buttonlist)
	{
	#Get the config of the actual item (button or separator)
	my %btn_cfg = %{$speedbar_cfg{$btn_name}};
	#If its a separator
	if ($btn_cfg{'type'} eq 'separator')
		{
		#Create it
		my $separator = $speedbar->Frame(-borderwidth=>$buttons_borderwidth, -relief=>$buttons_style);
		#Iterate through its options
		while ((my $btn_key,my $btn_value) = each %btn_cfg)
			{
			# test if it's a separators layout-option
			if ($separator_layout_options =~ /%%$btn_key%%/ )
				{
				$separator->configure("-".$btn_key=>$btn_value);
				}
			}
		#pack'em
		$separator->pack(-side=>'left', -fill=>'y', -expand=>0);
		}
	#If its a button
	if ($btn_cfg{'type'} eq 'button')
		{
		#Create it
		my $button = $speedbar->Button(-activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style);
		$ui_base->{'speedbar'}->{$btn_name} = $button;
		#Iterate through its options
		while ((my $btn_key,my $btn_value) = each %btn_cfg)
			{
			# test if it's a buttons layout-option
			if ($button_layout_options =~ /%%$btn_key%%/)
				{
				$button->configure("-".$btn_key=>$btn_value);
				}
			else
				{
				# The 'hint' argument
				if ($btn_key eq 'hint')
					{
					#Help him/her
					my $baloon = $form_wnd->Balloon(-state=>'balloon');
					$baloon->attach($button, -msg=>$btn_value);
					}
				# The 'image' argument
				if ($btn_key eq 'image')
					{
					#Paint him/her
					my $img = $form_wnd->Photo(-file=>Utils::process_filename($btn_value));
					$button->configure(-image=>$img);
					}
				# The 'command' argument
				if ($btn_key eq 'command')
					{
					#If its a macro
					if ($btn_value =~ /^MACRO::(.+)$/)
						{
						my $macro_name = $1;
						#And here the command is set
						$button->configure(-command=>sub{$ref_form->execute_macro($macro_name)});
						}
					}
				}
			}
		#Pack it
		$button->pack(-side=>'left', -fill=>'y', -expand=>0);
		}
	}
EXIT:
}

=head2 build_menu

uiBaseForm::build_menu($config_name, $ref_form, $menu_name, $ref_menu_cfg, $ref_entrylist); 

Builds a menu with the data from some configuration files:

The TOOLS menu is stored in the 'edoc.cfg'-file (can be in $form_class = 'PrjForm' and 'DocForm')

the BACKEND menu is stored in the '*.out' files (Only available in $form_class = 'DocForm').

Parameters:

(+) $config_name: The name of the section in edoc.cfg that holds the info

(+) $ref_form: Reference to the object which user interface has to be created

(+) $menu_name: The name of the menu in the main menu

(+) $ref_menu_cfg : A reference to a hash that contains the data for the menu

(+) $ref_entrylist : A referenece to an array that holds the names of the submenuitems sorted the way they were written in the configuration fil

=cut

################################
# Build up a user defined menu #
################################
sub build_menu
{
my $config_name = shift;
my $ref_form = shift;
my $menu_name = shift; #The name of the entry in the mainmenu
my $ref_menu_cfg = shift; #A hash that contains the data for the menu
my $ref_entrylist = shift; #An array that holds the names of the submenuitems sorted the way they were written in the configuration fil
my $ref_config = $main::config;
my %menu_cfg = %$ref_menu_cfg;
my @entrylist = @$ref_entrylist;

### read some configurations into simple scalars, so that the code is shorter ###
my $speedbarfont = $ref_config->{'UI'}->{'speedbarfont'};
my $toolboxfont = $ref_config->{'UI'}->{'toolboxfont'}; 
my $activebackground = $ref_config->{'UI'}->{'activebackground'};
my $optionmenu_background = $ref_config->{'UI'}->{'optionmenu_background'};

#Jump out if the Backendmenu is tried to be created for another form as the DocForm
goto Exit if ($menu_name eq 'Backend' and (not $config_name eq 'DOCUMENT_FORM'));

#Get the right places in the objects hash trees
my $ui_base = $ref_form->{'ui'};
my $form_wnd;
if ($config_name eq 'DOCUMENT_FORM')
	{
	$form_wnd = $ui_base->{'top'};
	}
elsif ($config_name eq 'PROJECT_FORM')
	{
	$form_wnd = $ui_base->{'app_window'};
	}
else
	{
	goto EXIT;
	}


#If this menu already exists->destroy it
if ((exists $ui_base->{'menus'}->{$menu_name})&&(defined $ui_base->{'menus'}->{$menu_name}))
	{
	$ui_base->{'menus'}->{$menu_name}->destroy();
	}

#Now lets do the job and build a menu
$ui_base->{'menus'}->{$menu_name} = $ui_base->{'menu_left'}->Menubutton(-tearoff=>0, -text =>$menu_name, -activebackground=>$activebackground );
#Iterate through the entries
foreach my $entry_name (@entrylist)
	{
	#Get the config of the actual entry
	my %entry_cfg = %{$menu_cfg{$entry_name}};
	#If it's an separator
	if ($entry_cfg{'type'} eq 'separator')
		{
		#create it
		$ui_base->{'menus'}->{$menu_name}->separator(-activebackground=>$activebackground);
		}
	#If its a command
	if ($entry_cfg{'type'} eq 'menu')
		{
		# Get the 'command' argument
		my $menu_command = $entry_cfg{'command'};
		#Get the text of the entry
		my $text = $entry_cfg{'text'};
		if ($menu_command =~ /^MACRO::(.+)$/)
			{
			#Get the macros name
			my $macro_name = $1;
			#Add the entry to the menu
			$ui_base->{'menus'}->{$menu_name}->command(-label =>$text,-command=>sub{$ref_form->execute_macro($macro_name)}, -activebackground=>$activebackground) ;
				}
		else
			{
			print "There was no macro defined for menu entry \"$text\"\n";
			}
		}
	}

$ui_base->{'menus'}->{$menu_name}->pack(-side=>'left');

EXIT:

}

1; #So mu package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
