#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package uiDocForm;

=head1 NAME:

Package: uiDocForm

=head1 SYNOPSIS:

use class::ui::uiDocForm;

=head1 DIRECTORY/FILE:

The file uiDocForm.pm is located in the subdirectory './class/ui'

=head1 DESCRIPTION:

This packages contains all functions that are needed to build and modify the user interface of a DocForm object (DocForm.pm).

This is the stuff I don't like on the Tk toolkit: Large spaghetti-code just to produce the GUI of one Window.
So I put all these functions in a separate file in ./class/ui, where every window has one file
that produces the GUI. I try to prevent to work in these files often, because I hate them.

This package file is not designed to function as an object, so there are no attributes, no methods and no Get/Set functions,
it is only a package of functions that belong thematically together (They are working with the same window object).

The windows object is placed in ./class, there is all of the windows functionality 

=head1 STATUS:

Coding: 90% done

Documentation: up-do-date


=head1 FUNCTIONS:

=cut

use strict;

=head2 build_ui_first_step

uiDocForm::build_ui_first_step($ref_docform); 

Builds up the user interface of a DocForm object and stores the references to all created
widgets in the $ref_docform->{'ui'} hash.

This function builds up everything except the OptionMenus, because these need to be uild up later
in the processing of the constructor of the DocForm object (DocForm.pm). This is done in the 
other functions called 'build_ui_second_step(..)' and  'build_ui_third_step(..)'in this package.


The attributes are accessed directly ($ref_docform->{'ui'}->{...}) because there is no Get/Set-method for each widget.

F.ex. the Toplevel is created in $ref_docform->{'ui'}->{'top'}.

Parameters:

(+) $ref_docform: Reference to the DocForm object which user interface has to be created

=cut

##########################################
# Aufbau des UI des Hauptfensters 1.Teil #
##########################################
sub build_ui_first_step
{
my $ref_docform = shift;
my $ref_config = $main::config;

### Hauptfenster mu zuerst aufgebaut werden ###
$ref_docform->update_window_caption();
$ref_docform->{'ui'}->{'top'}->geometry('+0+0'); #put it in the upper left corner
$ref_docform->{'ui'}->{'top'}->Icon(-image => $main::application_icon );

### read some configurations into simple scalars, so that the code is shorter ###
my $speedbarfont = $ref_config->{'UI'}->{'speedbarfont'};
my $toolboxfont = $ref_config->{'UI'}->{'toolboxfont'}; 
my $activebackground = $ref_config->{'UI'}->{'activebackground'};
my $buttons_style = $ref_config->{'UI'}->{'buttons_style'};
my $buttons_borderwidth = $ref_config->{'UI'}->{'buttons_borderwidth'};
my $ui_base = $ref_docform->{'ui'};
my $doc_form_wnd = $ui_base->{'top'};

### Menu: Info ###
$ui_base->{'menu_info'} = $ui_base->{'menu_right'}->Menubutton (-tearoff=>0, -text =>"Info", -activebackground=>$activebackground );
$ui_base->{'menu_info'}->command (-label =>'About e:doc', -activebackground=>$activebackground, -command=>sub{$ref_docform->show_aboutbox()});
$ui_base->{'menu_info'}->pack(-side=>'left');

### Big Notebook ###
$ui_base->{'big_notebook'} = $doc_form_wnd->NoteBook(-dynamicgeometry=>1, borderwidth=>2, -relief=>'ridge')->pack(-fill=>'both', -expand=>1);

### Documents main text view  and toolbox ###
$ui_base->{'doc_page'}=$ui_base->{'big_notebook'}->add('Document', label=>'Document');
$ui_base->{'toolbox_frame'} = $ui_base->{'doc_page'}->Frame(-borderwidth=>0, -relief=>'flat', -width=>240);
$ui_base->{'adj'} = $ui_base->{'doc_page'}->Adjuster(-widget=>$ui_base->{'toolbox_frame'}, -side=>'left');
$ui_base->{'text_frame'} = $ui_base->{'doc_page'}->Frame(-borderwidth=>0, -relief=>'flat');

### Outline view###
$ui_base->{'outline_page'}=$ui_base->{'big_notebook'}->add('Outline', label=>'Outline');
$ui_base->{'outline_base_frame'} = $ui_base->{'outline_page'}->Frame(-borderwidth=>2, -relief=>'sunken')->pack(-expand=>0, -side=>'bottom', fill=>'x', anchor=>'s');
$ui_base->{'outline_frame'} = $ui_base->{'outline_page'}->Frame(-borderwidth=>2, -relief=>'sunken')->pack(fill=>'both', expand=>1);
$ui_base->{'outline'} = $ui_base->{'outline_frame'}->Scrolled('Tree',-font=>$speedbarfont, -separator=>'~', -indicator=>1, -drawbranch=>1, -background=>'white',-scrollbars=>'osoe')->pack( -side=>'left', -fill=>'both', -expand=>'yes');;
$ui_base->{'outline'}->bind('<Double-Button-1>'=>sub{$ref_docform->goto_outline});
$ui_base->{'outline_refresh'} = $ui_base->{'outline_base_frame'}->Button(-activebackground=>$activebackground, -text=>'Refresh outline view',-command=>sub{$ref_docform->refresh_outline},-borderwidth=>$buttons_borderwidth,-relief=>$buttons_style)->pack(-expand=>0, -side=>'left');
$ui_base->{'outline_goto'} = $ui_base->{'outline_base_frame'}->Button(-activebackground=>$activebackground, -text=>'Jump to this item in the document',-command=>sub{$ref_docform->goto_outline},-borderwidth=>$buttons_borderwidth,-relief=>$buttons_style)->pack(-expand=>0, -side=>'left');
$ui_base->{'outline_comboballoon'} = $doc_form_wnd->Balloon(-state=>'balloon');
$ui_base->{'outline_comboballoon'}->attach($ui_base->{'outline'}, -msg=>'Doubleclick on an entry entry to jump to it in the document.');
$ui_base->{'document_icon'} = $doc_form_wnd->Photo(-file=>Utils::process_filename('graphics/docs.xpm'));
#Load the images for the NumLists from disk and create Photo-widgets with them
foreach my $list_name (keys %{$main::internal->{'DOC_NUMLISTS'}})
	{
	if (exists $main::internal->{'DOC_NUMLISTS'}->{$list_name}->{'outline_img'} and defined $main::internal->{'DOC_NUMLISTS'}->{$list_name}->{'outline_img'})
		{
		$ui_base->{'outline_icons'}->{$list_name} = $doc_form_wnd->Photo(-file=>Utils::process_filename($main::internal->{'DOC_NUMLISTS'}->{$list_name}->{'outline_img'}));
		}
	else
		{
		$ui_base->{'outline_icons'}->{$list_name} = undef;
		}
	}
### Backend view ###
$ui_base->{'backend_page'}=$ui_base->{'big_notebook'}->add('Backend', label=>'Backend');
$ui_base->{'backend_base_frame'} = $ui_base->{'backend_page'}->Frame(-borderwidth=>2, -relief=>'sunken')->pack(-expand=>0, -side=>'bottom', fill=>'x', anchor=>'s');
$ui_base->{'backend_text_frame'} = $ui_base->{'backend_page'}->Frame(-borderwidth=>2, -relief=>'sunken')->pack(fill=>'both', expand=>1);
$ui_base->{'backend_text'} = $ui_base->{'backend_text_frame'}->Scrolled('Text',-background=>'white', -font=>'-*-courier-*-*-*--*-160-*-*-*-*-*-*', -borderwidth =>2,  -scrollbars=>'se' )->pack( -side=>'left', -fill=>'both', -expand=>1);
$ui_base->{'backend_refresh'} = $ui_base->{'backend_base_frame'}->Button(-activebackground=>$activebackground, -text=>'Refresh backend view',-command=>sub{$ref_docform->refresh_backend()},-borderwidth=>$buttons_borderwidth,-relief=>$buttons_style)->pack(-expand=>0, -side=>'left');
$ui_base->{'plaintext_refresh'} = $ui_base->{'backend_base_frame'}->Button(-activebackground=>$activebackground, -text=>'Refresh plain text',-command=>sub{$ui_base->{'backend_text'}->delete("1.0","end"); my @inhalt=$ui_base->{'doctext'}->get("1.0", "end"); foreach my $line (@inhalt) {$ui_base->{'backend_text'}->insert("end", $line)}; }, -borderwidth=>$buttons_borderwidth,-relief=>$buttons_style)->pack(-expand=>0, -side=>'left');
}

=head2 build_ui_second_step

uiDocForm::build_ui_second_step($ref_docform); 

Builds up the rest of the user interface of a DocForm object. Everything that is not done by the
function 'build_ui_first_step'. 

uiDocForm::build_ui_second_step() is called very late in the DocForm-constructor, when almost everything of the DocForm
object is initialised already. This is necessery for the style-lists and the backend.
They are filled, but the Frame that holds them is still not packed.


The attributes are accessed directly ($ref_docform->{'ui'}->{...}) because there is no Get/Set-method for each widget.

Parameters:

(+) $ref_docform: Reference to the DocForm object which user interface has to be created

=cut

##########################################
# Aufbau des UI des Hauptfensters 2.Teil #
##########################################
sub build_ui_second_step
{
my $ref_docform = shift;
my $ref_config = $main::config;

### read some configurations into simple scalars, so that the code is shorter ###
my $speedbarfont = $ref_config->{'UI'}->{'speedbarfont'};
my $toolboxfont = $ref_config->{'UI'}->{'toolboxfont'}; 
my $activebackground = $ref_config->{'UI'}->{'activebackground'};
my $buttons_style = $ref_config->{'UI'}->{'buttons_style'};
my $buttons_borderwidth = $ref_config->{'UI'}->{'buttons_borderwidth'};
my $ui_base = $ref_docform->{'ui'};
my $doc_form_wnd = $ui_base->{'top'};

#Upper-part of the toolbox
$ui_base->{'top_notebook'} = $ui_base->{'toolbox_frame'}->NoteBook(-font=>$toolboxfont, -borderwidth=>2, -relief=>'ridge')->pack(-side=>'top', -fill=>'x', -expand=>0);

#Styles page
$ui_base->{'top_styles_page'}=$ui_base->{'top_notebook'}->add('Styles', label=>'Styles');
	#The backend frame
	$ui_base->{'backend_frame'} = $ui_base->{'top_styles_page'}->Frame()->pack(-side=>'top',-fill=>'x', -expand=>0);
		#BACKEND-COMBO
		$ui_base->{'backend_label'} = $ui_base->{'backend_frame'}->Label(-font=>$toolboxfont, -justify=>'left', -text=>'Backends:')->pack(-side=>'top', -fill=>'x', -padx=>2, -pady=>4); 
		$ui_base->{'backend_combo'} = $ui_base->{'backend_frame'}->BrowseEntry(-font=>$toolboxfont, -variable => $ref_docform->ref_current_backend_name(), -choices => $ref_docform->backend_names(), -browsecmd => sub {$ref_docform->backend_changed($ref_docform->current_backend_name())} )->pack(-side=>'top', -fill=>'x', -padx=>2); 
		$ui_base->{'backend_combo'}->Subwidget('entry')->Subwidget('entry')->configure(-state=>'disabled');
		Utils::set_browseentry_color($ui_base->{'backend_combo'}, $ref_config->{'UI'}->{'backends_background'});
		Utils::beautify_browseentry($ui_base->{'backend_combo'});
		Utils::create_balloonhelp($ui_base->{'backend_combo'}, 'Choose a backend', 'balloon', undef);
	#The styles frame
	$ui_base->{'style_frame'} = $ui_base->{'top_styles_page'}->Frame()->pack(-side=>'top',-fill=>'x', -expand=>0);
		#DOC_STYLE_COMBO
		$ui_base->{'doc_style_label'} = $ui_base->{'style_frame'}->Label(-font=>$toolboxfont, -justify=>'left', -text=>'Document styles:')->pack(-side=>'top', -fill=>'x', -padx=>2, -pady=>4); 
		$ui_base->{'doc_style_combo'} = $ui_base->{'style_frame'}->BrowseEntry(-font=>$toolboxfont, -variable=>$ref_docform->ref_current_doc_style(), -choices => $ref_docform->doc_styles(), -browsecmd=>sub {$ref_docform->doc_style_changed($ref_docform->current_doc_style())} )->pack(-side=>'top', -fill=>'x', -padx=>2); 
		$ui_base->{'doc_style_combo'}->Subwidget('entry')->Subwidget('entry')->configure(-state=>'disabled');
		Utils::set_browseentry_color($ui_base->{'doc_style_combo'}, $ref_config->{'UI'}->{'doc_styles_background'});
		Utils::beautify_browseentry($ui_base->{'doc_style_combo'});
		Utils::create_balloonhelp($ui_base->{'doc_style_combo'}, 'Choose a document class', 'balloon', undef);
		#PAR_STYLE_COMBO
		$ui_base->{'par_style_label'} = $ui_base->{'style_frame'}->Label(-font=>$toolboxfont, -justify => 'left', -text=>'Paragraph styles:')->pack(-side=>'top', -fill=>'x', -padx=>2, -pady=>4); 
		$ui_base->{'par_style_combo'} = $ui_base->{'style_frame'}->BrowseEntry(-font=>$toolboxfont, -variable => $ref_docform->ref_current_par_style(), -browsecmd => sub {$ref_docform->par_style_changed($ref_docform->current_par_style(), -1, -1, 1, 0, 1, 1)})->pack(-side=>'top', -fill=>'x', -padx=>2); 
		$ui_base->{'par_style_combo'}->Subwidget('entry')->Subwidget('entry')->configure(-state=>'disabled');
		Utils::set_browseentry_color($ui_base->{'par_style_combo'}, $ref_config->{'UI'}->{'par_styles_background'});
		Utils::beautify_browseentry($ui_base->{'par_style_combo'});
		Utils::create_balloonhelp($ui_base->{'par_style_combo'}, 'Choose a paragraph style', 'balloon', undef);
		#ENV_STYLE_COMBO
		$ui_base->{'env_style_label'} = $ui_base->{'style_frame'}->Label(-font=>$toolboxfont, -justify => 'left', -text=>'Environment styles:')->pack(-side=>'top', -fill=>'x', -padx=>2, -pady=>4); 
		$ui_base->{'env_style_combo'} = $ui_base->{'style_frame'}->BrowseEntry(-font=>$toolboxfont, -variable => $ref_docform->ref_current_env_style(), -browsecmd => sub {$ref_docform->env_style_changed($ref_docform->current_env_style(), -1, -1, 1, 1)})->pack(-side=>'top', -fill=>'x', -padx=>2); 
		$ui_base->{'env_style_combo'}->Subwidget('entry')->Subwidget('entry')->configure(-state=>'disabled');
		Utils::set_browseentry_color($ui_base->{'env_style_combo'}, $ref_config->{'UI'}->{'env_styles_background'});
		Utils::beautify_browseentry($ui_base->{'env_style_combo'});
		Utils::create_balloonhelp($ui_base->{'env_style_combo'}, 'Choose an environment style', 'balloon', undef);
		#CHAR_STYLE_COMBO
		$ui_base->{'char_style_label'} = $ui_base->{'style_frame'}->Label(-font=>$toolboxfont, -justify => 'left', -text=>'Character styles:')->pack(-side=>'top', -fill=>'x', -padx=>2, -pady=>4); 
		$ui_base->{'char_style_combo'} = $ui_base->{'style_frame'}->BrowseEntry(-font=>$toolboxfont, -variable => $ref_docform->ref_current_char_style(), -browsecmd => sub {$ref_docform->char_style_changed($ref_docform->current_char_style(), -1, -1, 1)})->pack(-side=>'top', -fill=>'x', -padx=>2); 
		$ui_base->{'char_style_combo'}->Subwidget('entry')->Subwidget('entry')->configure(-state=>'disabled');
		Utils::set_browseentry_color($ui_base->{'char_style_combo'}, $ref_config->{'UI'}->{'char_styles_background'});
		Utils::beautify_browseentry($ui_base->{'char_style_combo'});
		Utils::create_balloonhelp($ui_base->{'char_style_combo'}, 'Choose a character style', 'balloon', undef);
		#COMMANDS_COMBO
		$ui_base->{'commands_label'} = $ui_base->{'style_frame'}->Label( -font=>$toolboxfont, -justify => 'left', -text =>'Commands:')->pack(-side=>'top', -fill=>'x', -padx=>2, -pady=>4); 
		$ui_base->{'commands_combo'} = $ui_base->{'style_frame'}->BrowseEntry(-font=>$toolboxfont, -variable =>$ref_docform->ref_current_command(), -browsecmd => sub {$ref_docform->commands_changed($ref_docform->current_command(),-1)})->pack(-side=>'top', -fill=>'x', -padx=>2); 
		$ui_base->{'commands_combo'}->Subwidget('entry')->Subwidget('entry')->configure(-state=>'disabled');
		Utils::set_browseentry_color($ui_base->{'commands_combo'}, $ref_config->{'UI'}->{'commands_background'});
		Utils::beautify_browseentry($ui_base->{'commands_combo'});
		Utils::create_balloonhelp($ui_base->{'commands_combo'}, 'Insert a command', 'balloon', undef);
		#Spacer
		$ui_base->{'spacing_label'} = $ui_base->{'style_frame'}->Label( -font=>$toolboxfont, -justify => 'left', -text =>' ')->pack(-side=>'top', -fill=>'x', -padx=>2); 
#The parts page
$ui_base->{'top_parts_page'}=$ui_base->{'top_notebook'}->add('Parts', label=>'Parts');
	#The frame for the parts
	$ui_base->{'parts_frame'} = $ui_base->{'top_parts_page'}->Frame(-borderwidth=>2, -relief=>'sunken')->pack(-fill=>'both', -expand=>1);
		#A frame for the buttons
		$ui_base->{'parts_button_frame'} = $ui_base->{'parts_frame'}->Frame(-borderwidth=>2, -relief=>'flat');
			#The Label
			$ui_base->{'parts_label_add'} = $ui_base->{'parts_button_frame'}->Label(-text=>'Add', -background=>'black', -foreground=>'white', -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont)->grid(-column=>'0', -row=>'0', -columnspan=>3, -sticky=>'nsew');
			#The "Add..."-buttons
			$ui_base->{'parts_btn_add_document'} = $ui_base->{'parts_button_frame'}->Button(-text=>'Document', -command=>sub{$ref_docform->add_part('DOCUMENT')}, -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont)->grid(-column=>'0', -row=>'1', -sticky=>'nsew');
				Utils::create_balloonhelp($ui_base->{'parts_btn_add_document'}, "Add the selected text as a document-wide part.", 'balloon', undef);
			$ui_base->{'parts_btn_add_project'} = $ui_base->{'parts_button_frame'}->Button(-text=>'Project', -command=>sub{$ref_docform->add_part('PROJECT')}, -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont)->grid(-column=>'1', -row=>'1', -sticky=>'nsew');
				Utils::create_balloonhelp($ui_base->{'parts_btn_add_project'}, "Add the selected text as a project-wide part.", 'balloon', undef);
			$ui_base->{'parts_btn_add_global'} = $ui_base->{'parts_button_frame'}->Button(-text=>'Global', -command=>sub{$ref_docform->add_part('GLOBAL')}, -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont)->grid(-column=>'2', -row=>'1', -sticky=>'nsew');
				Utils::create_balloonhelp($ui_base->{'parts_btn_add_global'}, "Add the selected text as a global part.", 'balloon', undef);
			#The Label
			$ui_base->{'parts_label_manage'} = $ui_base->{'parts_button_frame'}->Label(-text=>'Manage', -background=>'black', -foreground=>'white', -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont)->grid(-column=>'0', -row=>'2', -columnspan=>3, -sticky=>'nsew');
			#The Buttons
			$ui_base->{'parts_btn_rename'} = $ui_base->{'parts_button_frame'}->Button(-text=>'Rename', -command=>sub{$ref_docform->rename_part()}, -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont)->grid(-column=>'0', -row=>'3', -sticky=>'nsew');
				Utils::create_balloonhelp($ui_base->{'parts_btn_rename'}, "Rename the selected part.", 'balloon', undef);
			$ui_base->{'parts_btn_move'} = $ui_base->{'parts_button_frame'}->Button(-text=>'Move', -command=>sub{}, -state=>'disabled', -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont)->grid(-column=>'1', -row=>'3', -sticky=>'nsew');
				Utils::create_balloonhelp($ui_base->{'parts_btn_move'}, "Move the selected part into another branch.", 'balloon', undef);
			$ui_base->{'parts_btn_copy'} = $ui_base->{'parts_button_frame'}->Button(-text=>'Copy', -command=>sub{$ref_docform->copy_part()}, -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont)->grid(-column=>'2', -row=>'3', -sticky=>'nsew');
				Utils::create_balloonhelp($ui_base->{'parts_btn_copy'}, "Copy the selected part.", 'balloon', undef);
			$ui_base->{'parts_btn_delete'} = $ui_base->{'parts_button_frame'}->Button(-text=>'Delete', -command=>sub{$ref_docform->delete_part()}, -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont)->grid(-column=>'0', -row=>'4', -sticky=>'nsew');
				Utils::create_balloonhelp($ui_base->{'parts_btn_delete'}, 'Delete the selected part from the parts archive.', 'balloon', undef);
			$ui_base->{'parts_btn_edit'} = $ui_base->{'parts_button_frame'}->Button(-text=>'Edit', -command=>sub{}, -state=>'disabled', -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont)->grid(-column=>'1', -row=>'4', -sticky=>'nsew');
				Utils::create_balloonhelp($ui_base->{'parts_btn_edit'}, 'Edit the selected part in a seperate document window.', 'balloon', undef);
			$ui_base->{'parts_btn_use'} = $ui_base->{'parts_button_frame'}->Button(-text=>'Use', -command=>sub{$ref_docform->use_part()}, -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont)->grid(-column=>'2', -row=>'4', -sticky=>'nsew');
				Utils::create_balloonhelp($ui_base->{'parts_btn_use'}, 'Insert the selected part into the current document.', 'balloon', undef);
		$ui_base->{'parts_button_frame'}->pack(-side=>'top', -fill=>'x', -expand=>1, -pady=>4);
		#The tree
		$ui_base->{'parts_tree'} = $ui_base->{'parts_frame'}->Scrolled('Tree', -separator=>'~', -indicator=>1, -drawbranch=>1, -scrollbars=>'osoe', -background=>'white', -font=>$speedbarfont )->pack(-fill=>'both', -expand=>1);
			#The balloon-help
			Utils::create_balloonhelp($ui_base->{'parts_tree'}, "Choose here a part to be inserted\n into the current document", 'balloon', undef);
			#The icon
			$ui_base->{'parts_root_icon'} = $ui_base->{'top'}->Photo(-file=>Utils::process_filename('graphics/parts.xpm'));
			$ui_base->{'parts_doc_icon'} = $ui_base->{'top'}->Photo(-file=>Utils::process_filename('graphics/parts1.xpm'));
			$ui_base->{'parts_prj_icon'} = $ui_base->{'top'}->Photo(-file=>Utils::process_filename('graphics/parts2.xpm'));
			$ui_base->{'parts_global_icon'} = $ui_base->{'top'}->Photo(-file=>Utils::process_filename('graphics/parts3.xpm'));
			$ui_base->{'part_doc_icon'} = $ui_base->{'top'}->Photo(-file=>Utils::process_filename('graphics/part1.xpm'));
			$ui_base->{'part_prj_icon'} = $ui_base->{'top'}->Photo(-file=>Utils::process_filename('graphics/part2.xpm'));
			$ui_base->{'part_global_icon'} = $ui_base->{'top'}->Photo(-file=>Utils::process_filename('graphics/part3.xpm'));
			#The standard enties
			$ui_base->{'parts_tree'}->add('ROOT',  -data=>'ROOT', -itemtype=>'imagetext', -text=>'Parts', -image=>$ui_base->{'parts_root_icon'});
			$ui_base->{'parts_tree'}->add('ROOT~GLOBAL', -data=>'ROOT_GLOBAL', -itemtype=>'imagetext', -text=>'Global', -image=>$ui_base->{'parts_global_icon'});
			$ui_base->{'parts_tree'}->add('ROOT~PROJECT',-data=>'ROOT_PROJECT', -itemtype=>'imagetext', -text=>'Project', -image=>$ui_base->{'parts_prj_icon'});
			$ui_base->{'parts_tree'}->add('ROOT~DOCUMENT',-data=>'ROOT_DOCUMENT', -itemtype=>'imagetext', -text=>'Document', -image=>$ui_base->{'parts_doc_icon'});
			#Expand everything
			$ui_base->{'parts_tree'}->autosetmode();

#The extras page
$ui_base->{'top_extras_page'}=$ui_base->{'top_notebook'}->add('Extras', label=>'Extras');
	#etras frame
	$ui_base->{'extras_frame'} = $ui_base->{'top_extras_page'}->Frame(-borderwidth=>2, -relief=>'sunken')->pack(fill=>'both', expand=>1);
	$ui_base->{'extras_dummy'} = $ui_base->{'extras_frame'}->Label(font=>$speedbarfont, -text=>"Sorry, not \nimplemented yet!")->pack( -side=>'left', -fill=>'both', -expand=>'yes');;

#The Tagdoctor-button
$ui_base->{'tagdoctor_button'} = $ui_base->{'toolbox_frame'}->Button(-activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont, -textvariable=>\$ref_docform->{'_tag_doctor_on'},-command=>sub{$ref_docform->tag_doctor_pressed()})->pack(-side=>'top', -fill=>'x', -expand=>0, -pady=>2, -padx=>10);
$ui_base->{'tagdoctor_comboballoon'} = $doc_form_wnd->Balloon(-state=>'balloon');
$ui_base->{'tagdoctor_comboballoon'}->attach($ui_base->{'tagdoctor_button'}, -msg=>"Click here to show/hide \n the borders of the style tags.");

#The fullscreen button
$ui_base->{'fullscreen_button'} = $ui_base->{'toolbox_frame'}->Button(-activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$toolboxfont, -text=>'FULL SCREEN MODE', -command=>sub{$ref_docform->full_screen_mode(1)})->pack(-side=>'top', -fill=>'x', -expand=>0, -padx=>10);
$ui_base->{'fullscreen_comboballoon'} = $doc_form_wnd->Balloon(-state=>'balloon');
$ui_base->{'fullscreen_comboballoon'}->attach($ui_base->{'fullscreen_button'}, -msg=>"Click here to turn on the full screen mode.\n('ESC' returns to normal mode)");

$ui_base->{'toolbox_frame'}->pack(-side=>'left', -fill=>'both', -expand=>1);
$ui_base->{'adj'}->pack(-side=>'left', -fill=>'y');
$ui_base->{'text_frame'}->pack(-side=>'left', -fill=>'both', -expand=>1);

uiDocForm::build_doctext($ref_docform,1,1);


#END SUB
}

=head2 build_ui_third_step

uiDocForm::build_ui_third_step($ref_docform); 

Builds up the third part of the user interface of a DocForm object. 

uiDocForm::build_ui_third_step() is called very late in the DocForm-constructor, when almost everything of the DocForm
object is initialised already. This is necessery for the style-lists and the backend.
This function performs the packing of the Toolbox that contains the style-selection
drop-down-components.

The attributes are accessed directly ($ref_docform->{'ui'}->{...}) because there is no Get/Set-method for each widget.

Parameters:

(+) $ref_docform: Reference to the DocForm object which user interface has to be created

=cut

###########################################
# Aufbau des UI des Hauptfensters 3. Teil #
###########################################
sub build_ui_third_step
{
my $ref_docform = shift;
my $ref_config = $main::config;
my $ui_base = $ref_docform->{'ui'};

#Now schow the form
$ref_docform->{'ui'}->{'top'}->deiconify();

#END SUB
}



=head2 build_doctext

my $new_doctext = uiDocForm::build_doctext($ref_docform, $pack_it, $bind_events); 

Builds the 'doctext' textwidget and sets all neccessary bindings for key and mouse events.

This is done in a seperate function, because the text widget has to be rebuild several times again.
This is because tests have shown that it is much more faster to destroy and rebuild an exiting text widget
instead of clearing all text and tags in it.

The function return a reference to the new text widget.

Parameters:

(+) $ref_docform: Reference to the DocForm object which user interface has to be created

(+) $pack_it: If set to 1, the textwidget gets packed now, if set to 0, it stays unpacked. 
(This is used for allowing it to build up a loaded document invisible in the background and displaying it at-once. 
This is much fatser and prevents flickering on the screen.

(+) $bind_events: If set to 1, the cursor movements get binded now. (same reason as the above option)

=cut


#######################
# Textwidget aufbauen #
#######################
sub build_doctext
{
#This is in a separate sub, because its needed more tham once in the DocForm-Object
my $ref_docform = shift;
my $pack_it = shift; 
my $bind_events = shift;

my $ui_base = $ref_docform->{'ui'};

#Create the widget
$ui_base->{'doctext'} = $ui_base->{'text_frame'}->Scrolled('Text', -borderwidth =>2, -scrollbars=>'osoe', -insertofftime=>0 );
#Remove default popup menu
$ui_base->{'doctext'}->menu(undef);
#pack the widget, if wanted
if ($pack_it==1) 
	{
	$ui_base->{'doctext'}->pack( -side=>'left', -fill=>'both', -expand=>1);
	}

#bind the events
if ($bind_events == 1)
	{
	uiDocForm::doctext_bind_events($ref_docform, $ui_base->{'doctext'});
	}

#configure the look
my %options; #Hash fr Darstellungsoptionen
my @display_options = ('font','background','borderwidth','foreground','spacing1','spacing2','spacing3','wrap','offset','lmargin1','lmargin2','rmargin','tabs','padx','pady');
for (my $i=0;$i<= $#display_options;$i++) #Durch alle definierten Optionen zur Tagdarstellung gehen
	{
	my $option=$display_options[$i]; #Eine Option auslesen
	if (exists($main::internal->{'BASE_TEXT'}->{$option})) #Wenn diese in INTERNAL-Struktur definiert ist
		{
		$options{"-$option"}=$main::internal->{'BASE_TEXT'}->{$option}; #In Options-Hash eintragen
		}
	}
$ui_base->{'doctext'}->configure(%options);

#Return a reference to the newly created widget
return $ui_base->{'doctext'};
}


=head2 doctext_bind_events

uiDocForm::doctext_bind_events($ref_docform, $ref_doctext); 

Binds the events to the Text widget

This is done seperatly to allow creating of the doctext and binding the events later.
Only used for loading a new document (speed issues).

Parameters:

(+) $ref_docform: A reference to the DocForm object which user interface has to be created

(+) $ref_doctext: A reference to the Text widget

=cut

#####################################
# Binding events of the text widget #
#####################################
sub doctext_bind_events
{
my $ref_docform = shift;
my $ref_doctext = shift; 

#binding the events
$ref_doctext->bind('<Key-Left>'=>sub{$ref_docform->_cursor_moved($ref_docform)});
$ref_doctext->bind('<Key-Right>'=>sub{$ref_docform->_cursor_moved($ref_docform)});
$ref_doctext->bind('<Key-Up>'=>sub{$ref_docform->_cursor_moved($ref_docform)});
$ref_doctext->bind('<Key-Down>'=>sub{$ref_docform->_cursor_moved($ref_docform)});
$ref_doctext->bind('<Key-Return>'=>sub{$ref_docform->_cursor_moved($ref_docform)});
$ref_doctext->bind('<Button-1>'=>sub{$ref_docform->_cursor_moved($ref_docform)});
$ref_doctext->bind('<Key>'=>sub{$ref_docform->_key_pressed($ref_docform)});
$ref_doctext->bind('<Key-Escape>'=>sub{$ref_docform->full_screen_mode(0)});
$ref_doctext->tagBind('sel',"<Any-Enter>"=>sub{shift->configure(-cursor=>'hand2')});
$ref_doctext->tagBind('sel',"<Any-Leave>"=>sub{shift->configure(-cursor=>'xterm')});
}

1; #So mu package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
