package uiPrjForm;

=head1 NAME:

Package: uiPrjForm

=head1 SYNOPSIS:

use class::ui::uiPrjForm;

=head1 DIRECTORY/FILE:

The file uiPrjForm.pm is located in the subdirectory './class/ui'

=head1 DESCRIPTION:

This packages contains all functions that are needed to build and modify the user interface of a PrjForm object (PrjForm.pm).

This is the stuff I don't like on the Tk toolkit: Large spaghetti-code just to produce the GUI of one Window.
So I put all these functions in a separate file in ./class/ui, where every window has one file
that produces the GUI. I try to prevent to work in these files often, because I hate them.

This package file is not designed to function as an object, so there are no attributes, no methods and no Get/Set functions,
it is only a package of functions that belong thematically together (They are working with the same window object).

The windows object is placed in ./class, there is all of the windows functionality 

=head1 STATUS:

Coding: 10% done

Documentation: up-do-date


=head1 FUNCTIONS:

=cut

use strict;

=head2 build_ui

uiPrjForm::build_ui($ref_prjform); 

Builds up the user interface of a PrjForm object and stores the references to all created
widgets in the $ref_prjform->{'ui'} hash.

The attributes are accessed directly ($ref_prjform->{'ui'}->{...}) because there is no Get/Set-method for each widget.

Because the project window is the applications main window, it is created here and stored in $ref_prjform->{'ui'}->{'app_window'}.

The function returns a reference to the applications main window.

Parameters:

(+) $ref_prjform: Reference to the PrjForm object which user interface has to be created

=cut


############################
# Build the user interface #
############################
sub build_ui
{
my $ref_prjform = shift; #Zeiger auf das prjForm-Objekt

#Getting some global settings
my $internal = $main::internal;
my $config = $main::config;
my $project = $main::project;

my $activebackground = $config->{'UI'}->{'activebackground'};
my $buttons_style = $config->{'UI'}->{'buttons_style'};
my $buttons_borderwidth = $config->{'UI'}->{'buttons_borderwidth'};
my $speedbarfont = $config->{'UI'}->{'speedbarfont'};

#Creating the application window

$ref_prjform->update_window_caption();

$ref_prjform->top()->minsize(220,220);
$ref_prjform->top()->geometry('794x580+0+0');
$ref_prjform->top()->withdraw();
my $app_window = $ref_prjform->top();
my $ui_base = $ref_prjform->{'ui'};

$main::combo_pixmap = $app_window->Pixmap(-file=>Utils::process_filename('%EDOC_PATH%/graphics/cbxarrow.xpm'));

$ui_base->{'menu_info'} = $ui_base->{'menu_right'}->Menubutton (-tearoff=>0, -text =>"Info", -activebackground=>$activebackground );
$ui_base->{'menu_info'}->command (-label =>'About e:doc', -activebackground=>$activebackground, -command=>sub{$ref_prjform->show_aboutbox()});
$ui_base->{'menu_info'}->command (-label =>'Tutorial project', -activebackground=>$activebackground, -command=>sub{$ref_prjform->open_tutorial()} );
$ui_base->{'menu_info'}->pack(-side=>'left');

#Pages
$ui_base->{'notebook'} = $app_window->NoteBook(-borderwidth=>2, -relief=>'ridge')->pack(-fill=>'both', -expand=>'both');
$ui_base->{'content_page'}=$ui_base->{'notebook'}->add('Content', -label=>'Content');
$ui_base->{'prj_args_page'}=$ui_base->{'notebook'}->add('Prj_Args', -label=>'Project info');
$ui_base->{'authors_page'}=$ui_base->{'notebook'}->add('Authors', -label=>'Authors');
$ui_base->{'history_page'}=$ui_base->{'notebook'}->add('History', -label=>'History');

#Content-Page
$ui_base->{'content'} = $ui_base->{'content_page'}->Scrolled('Tree', -separator=>'~', -indicator=>1, -drawbranch=>1, -columns=>3, -header=>1, -background=>'white', -scrollbars=>'osoe', -font=>$speedbarfont, -browsecmd=>sub{$ref_prjform->_on_browse_content()}, -command=>sub{$ref_prjform->_content_dbl_click()} );
$ui_base->{'adj'} = $ui_base->{'content_page'}->Adjuster(-widget=>$ui_base->{'content'}, -side=>'left');
$ui_base->{'file_args_frame'} = $ui_base->{'content_page'}->Scrolled('Pane', -relief=>'ridge', -scrollbars=>'osoe', -borderwidth=>2, -width=>200);
$ui_base->{'content'}->pack( -side=>'left', -fill=>'both', -expand=>'yes');
$ui_base->{'adj'}->pack( -side=>'left', -fill=>'y');
$ui_base->{'file_args_frame'}->pack(-side=>'left', -fill=>'both', -expand=>1);
$ui_base->{'file_args_caption'} = $ui_base->{'file_args_frame'}->Label(-text=>'No file arguments', -background=>'black', -foreground=>'white')->grid(-row=>0, -column=>0, -sticky=>'nsew');
$ui_base->{'file_args_frame'}->gridColumnconfigure(0, -weight=>0);
$ui_base->{'file_args_base'} = $ui_base->{'file_args_frame'}->Frame(-borderwidth=>2, -relief=>'ridge');

$ui_base->{'content'}->headerCreate(0, -itemtype=>'text', -relief=>'raised', -text=>'Files and groups' );
$ui_base->{'content'}->headerCreate(1, -itemtype=>'text', -relief=>'raised', -text=>'Size' );
$ui_base->{'content'}->headerCreate(2, -itemtype=>'text', -relief=>'raised', -text=>'Modified' );
Utils::create_balloonhelp($ui_base->{'content'}, 'Doubleclick on a file entry to open it.', 'balloon', undef);

$ui_base->{'project_icon'} = $app_window->Photo(-file=>Utils::process_filename('%EDOC_PATH%/graphics/project.xpm'));

#Info-Page

$ui_base->{'prj_args_frame'} = $ui_base->{'prj_args_page'}->Scrolled('Pane', -scrollbars=>'osoe', -borderwidth=>2, -relief=>'ridge', -width=>400)->pack(-anchor=>'w', -fill=>'both', -expand=>1);
$ui_base->{'prj_args_caption'} = $ui_base->{'prj_args_frame'}->Label(-text=>'The projects arguments:', -background=>'black', -foreground=>'white')->pack(-anchor=>'nw', -fill=>'x', -expand=>0);
$ui_base->{'prj_args_base'} = $ui_base->{'prj_args_frame'}->Frame(-borderwidth=>2, -relief=>'ridge')->pack(-ipadx=>5, -ipady=>5, -anchor=>'nw', -fill=>'y', -expand=>0);

#Authors-Page
$ui_base->{'authors_frame'} = $ui_base->{'authors_page'}->Frame(-borderwidth=>2, -relief=>'sunken')->pack(fill=>'both', expand=>1);
$ui_base->{'authors_edit'} = $ui_base->{'authors_frame'}->Scrolled('Text', -scrollbars=>'osoe')->pack(-side=>'left', -fill=>'both', -expand=>1);
$ui_base->{'authors_edit'}->bind('<KeyRelease>' => [\&authors_changed,$ref_prjform] );
$ui_base->{'authors_dummy'} = $ui_base->{'authors_frame'}->Label(-font=>$speedbarfont, -text=>'Here will be more later')->pack( -side=>'left', -fill=>'y', -expand=>0);;

#History-Page
$ui_base->{'history_frame'} = $ui_base->{'history_page'}->Frame(-borderwidth=>2, -relief=>'sunken')->pack(fill=>'both', expand=>1);
$ui_base->{'history_dummy'} = $ui_base->{'history_frame'}->Label(-font=>$speedbarfont, -text=>'Sorry, not implemented yet!')->pack( -side=>'left', -fill=>'both', -expand=>'yes');;

#Build up the project form argument editor
uiPrjForm::build_ui_arg_editor($ref_prjform, 'PROJECT');

#Build up the project form argument editor for single files
uiPrjForm::build_ui_arg_editor($ref_prjform, 'FILES');

#Returning a reference to the window
return $app_window;
}


=head2 build_ui_arg_editor

uiPrjForm::build_ui_arg_editor($ref_prjform, $type); 

Builds up the part that displays the project argument editor.

Parameters:

(+) $ref_prjform: Reference to the PrjForm object which user interface has to be created

(+) $type: Is 'PROJECT' for the big editor for the project itself, or 'FILES' for the samller one beside the tree

=cut

############################
# Project arguments editor #
############################
sub build_ui_arg_editor
{
my $ref_prjform = shift; #Reference of the PrjForm object
my $type = shift;

#Getting some global settings
my $internal = $main::internal;
my $config = $main::config;
my $project = $main::project;

my $padx=3;
my $pady=3;

my $activebackground = $config->{'UI'}->{'activebackground'};
my $buttons_style = $config->{'UI'}->{'buttons_style'};
my $buttons_borderwidth = $config->{'UI'}->{'buttons_borderwidth'};
my $speedbarfont = $config->{'UI'}->{'speedbarfont'};
my $app_window = $ref_prjform->{'ui'}->{'app_window'};

my $ui_base;
my $base;
my $arg_base_branch;

if ($type eq 'PROJECT')
	{
	$ui_base = $ref_prjform->{'ui'};
	$base = $ui_base->{'prj_args_base'};
	$arg_base_branch = $ref_prjform->{'prj_args'};
	}
elsif ($type eq 'FILES')
	{
	$ui_base = $ref_prjform->{'ui'};
	$base = $ui_base->{'file_args_base'};
	$arg_base_branch = $ref_prjform->{'current_file'}->{'args'};
	}
else
	{
	goto EXIT;
	}

#Build up the GUI part#
my $arg_number;

if (not exists $arg_base_branch->{'arg_number'} or
	not defined $arg_base_branch->{'arg_number'})
	{
	$arg_number = 0;
	}
else
	{
	$arg_number = $arg_base_branch->{'arg_number'};
	}

if ($arg_number > 0)
	{
	for (my $i=1 ; $i<=$arg_number; $i++)
		{
		next if not exists $arg_base_branch->{$i}->{'type'} or not defined $arg_base_branch->{$i}->{'type'};

		#Build  and configure the base widgets
		$arg_base_branch->{$i}->{'_label'} = $base->Label(-text=>$arg_base_branch->{$i}->{'name'}.":", -justify=>'right', -anchor=>'e',  -borderwidth=>2)->grid(-column=>0, -row=>($i-1), -sticky=>'ne', -padx=>$padx, -pady=>$pady);

		#If the type is 'LIST', show a dropdown-list
		if ($arg_base_branch->{$i}->{'type'} =~ /LIST$/)
			{
			if (not exists $arg_base_branch->{$i}->{'settings'} or not defined $arg_base_branch->{$i}->{'settings'})
				{
				$arg_base_branch->{$i}->{'settings'} = '';
				}
			my @list;
			if ($arg_base_branch->{$i}->{'type'} eq 'LIST')
				{
				@list = sort( split(/\|\|/, $arg_base_branch->{$i}->{'settings'}) );
				}
			#If the type is 'AUTHORS_LIST', show a dropdown-list
			elsif ($arg_base_branch->{$i}->{'type'} eq 'AUTHORS_LIST')
				{
				@list = @{$ref_prjform->authors()}; 
				}
			#If the type is 'FILE_LIST', show a dropdown-list
			elsif ($arg_base_branch->{$i}->{'type'} eq 'FILES_LIST') 
				{
				@list = (); 
				}
			#Build up the dropdown list
			my $entry = $arg_base_branch->{$i}->{'_entry'} = $base->BrowseEntry(-choices=>\@list, -textvariable=>\$arg_base_branch->{$i}->{'value'}, -width=>40)->grid(-column=>1, -row=>($i-1), -sticky=>'nsew', -padx=>$padx, -pady=>$pady);
			Utils::set_browseentry_color($entry, 'white');
			Utils::beautify_browseentry($entry);
			}
		#If the type is set to 'MULTILINE'
		elsif ($arg_base_branch->{$i}->{'type'} eq 'MULTILINE')
			{
			#Create a normal Entry widget
			my $frame = $arg_base_branch->{$i}->{'_entry_frame'} = $base->LabFrame(-borderwidth=>1, -labelside=>'left', -relief=>'flat')->grid(-column=>1, -row=>($i-1), -sticky=>'nsew', -padx=>$padx, -pady=>$pady);
			my $entry = $arg_base_branch->{$i}->{'_entry'} = $frame->Scrolled('Text', -scrollbars=>'oe', -wrap=>'word',-background=>'white', -width=>40, -height=>8)->pack(-fill=>'both', -expand=>1, -side=>'left');
			$entry->insert('1.0', $arg_base_branch->{$i}->{'value'});
			$entry->bind('<KeyRelease>' => [\&text_arg_changed,$ref_prjform, $arg_base_branch, $i] );
			}
		#If the type is set to 'TEXT'
		elsif ($arg_base_branch->{$i}->{'type'} eq 'TEXT')
			{
			#Create a normal Entry widget
			my $frame = $arg_base_branch->{$i}->{'_entry_frame'} = $base->LabFrame(-borderwidth=>1, -labelside=>'left', -relief=>'flat')->grid(-column=>1, -row=>($i-1), -sticky=>'nsew', -padx=>$padx, -pady=>$pady);
			my $entry = $arg_base_branch->{$i}->{'_entry'} = $frame->Entry(-textvariable=>\$arg_base_branch->{$i}->{'value'}, -background=>'white', -width=>40)->pack(-fill=>'both', -expand=>1, -side=>'left');
			}
		#Bind the key-events for the document_changed()-flag
		$arg_base_branch->{$i}->{'_entry'}->bind('<Key>'=>sub{$ref_prjform->set_content_changed()});
		if (exists $arg_base_branch->{$i}->{'hint'} and defined $arg_base_branch->{$i}->{'hint'})
			{
			$arg_base_branch->{$i}->{'_balloon'} = $base->Balloon(-state=>'balloon');
			$arg_base_branch->{$i}->{'_balloon'}->attach($arg_base_branch->{$i}->{'_entry'}, -msg=>$arg_base_branch->{$i}->{'hint'});
			}
		}
	}

$base->gridColumnconfigure(1, -weight=>1);

}

=head2 update_arg_editors

uiPrjForm::update_arg_editors($ref_prjform, $type); 

Update the combos in the editors

Parameters:

(+) $ref_prjform: Reference to the PrjForm object which user interface has to be created

(+) $type: Is 'PROJECT' for the big editor for the project itself, or 'FILES' for the samller one beside the tree

=cut

###############################
# Update the arguments editor #
###############################
sub update_arg_editors
{
my $ref_prjform = shift; #Reference of the PrjForm object
my $type = shift;

#Getting some global settings
my $internal = $main::internal;
my $config = $main::config;
my $project = $main::project;

my $app_window = $ref_prjform->{'ui'}->{'app_window'};

my $ui_base;

my $base;
my $arg_base_branch;

if ($type eq 'PROJECT')
	{
	$ui_base = $ref_prjform->{'ui'};
	$base = $ui_base->{'prj_args_base'};
	$arg_base_branch = $ref_prjform->{'prj_args'};
	}
elsif ($type eq 'FILES')
	{
	$ui_base = $ref_prjform->{'ui'};
	$base = $ui_base->{'file_args_base'};
	$arg_base_branch = $ref_prjform->{'current_file'}->{'args'};
	}
else
	{
	goto EXIT;
	}

#Build up the GUI part#
my $arg_number;

if (not exists $arg_base_branch->{'arg_number'} or
	not defined $arg_base_branch->{'arg_number'})
	{
	$arg_number = 0;
	}
else
	{
	$arg_number = $arg_base_branch->{'arg_number'};
	}

if ($arg_number > 0)
	{
	for (my $i=1 ; $i<=$arg_number; $i++)
		{
		next if not exists $arg_base_branch->{$i}->{'type'} or not defined $arg_base_branch->{$i}->{'type'};

		#If the type is 'LIST', show a dropdown-list
		if ($arg_base_branch->{$i}->{'type'} =~ /LIST$/)
			{
			if (not exists $arg_base_branch->{$i}->{'settings'} or not defined $arg_base_branch->{$i}->{'settings'})
				{
				$arg_base_branch->{$i}->{'settings'} = '';
				}
			my @list;
			if ($arg_base_branch->{$i}->{'type'} eq 'LIST')
				{
				@list = sort( split(/\|\|/, $arg_base_branch->{$i}->{'settings'}) );
				}
			#If the type is 'AUTHORS_LIST', show a dropdown-list
			elsif ($arg_base_branch->{$i}->{'type'} eq 'AUTHORS_LIST')
				{
				@list = @{$ref_prjform->authors()}; 
				}
			#If the type is 'FILE_LIST', show a dropdown-list
			elsif ($arg_base_branch->{$i}->{'type'} eq 'FILES_LIST') 
				{
				@list = (); 
				}
			#Build up the dropdown list
			$arg_base_branch->{$i}->{'_entry'}->configure(-choices=>\@list, -textvariable=>\$arg_base_branch->{$i}->{'value'});
			}
		}
	}
}   


=head2 text_arg_changed

uiPrjForm::text_arg_changed($ref_prjform, $arg_base_branch, $arg_index); 

Updates the value of the argument.

Parameters:

(+) $ref_prjform: Reference to the PrjForm object.

(+) $arg_base_branch: The hash where the argument gets stored

(+) $arg_index: The index number of the argument

=cut

###########################
# The text widget changed #
###########################
sub text_arg_changed
{
my $sender = shift;
my $ref_prjform = shift; #Reference of the PrjForm object
my $arg_base_branch = shift; #The hash where the args are stored
my $arg_index = shift; #The index of the argument

my $text = $arg_base_branch->{$arg_index}->{'_entry'}->get('1.0', 'end');
$text =~ s/(\n+)$//g ; #remove Newlines at the end
$arg_base_branch->{$arg_index}->{'value'} = $text;
}

=head2 authors_changed

uiPrjForm::authors_changed($ref_prjform); 

Updates the authors list.

Parameters:

(+) $ref_prjform: Reference to the PrjForm object.

=cut

###################################
# The authors-text widget changed #
###################################
sub authors_changed
{
my $sender = shift;
my $ref_prjform = shift; #Reference of the PrjForm object

my $text = $ref_prjform->{'ui'}->{'authors_edit'}->get('1.0', 'end');
$text =~ s/(\n+)$//g ; #remove Newlines at the end
my @authors_list = split(/\n/, $text);
$ref_prjform->{'authors'} = \@authors_list;
}


1; #So mu package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
