package uiTblForm;

=head1 NAME:

Package: uiTblForm

=head1 SYNOPSIS:

use class::ui::uiTblForm;

=head1 DIRECTORY/FILE:

The file uiTblForm.pm is located in the subdirectory './class/ui'

=head1 DESCRIPTION:

This packages contains all functions that are needed to build and modify the user interface of a TblForm object (TblForm.pm).

This is the stuff I don't like on the Tk toolkit: Large spaghetti-code just to produce the GUI of one Window.
So I put all these functions in a separate file in ./class/ui, where every window has one file
that produces the GUI. I try to prevent to work in these files often, because I hate them.

This package file is not designed to function as an object, so there are no attributes, no methods and no Get/Set functions,
it is only a package of functions that belong thematically together (They are working with the same window object).

The windows object is placed in ./class, there is all of the windows functionality 

=head1 STATUS:

Coding: 1% done

Documentation: up-do-date


=head1 FUNCTIONS:

=cut

use strict;

=head2 build_ui

uiDocForm::build_ui($ref_tblform, ref_parent_window); 

Builds up the user interface of a TblForm object and stores the references to all created
widgets in the $ref_tblform->{'ui'} hash.

The attributes are accessed directly ($ref_tblform->{'ui'}->{...}) because there is no Get/Set-method for each widget.

F.ex. the Toplevel is created in $ref_tblform->{'ui'}->{'_top'}.


Parameters:

(+) $ref_tblform: Reference to the TblForm object which user interface has to be created

=cut


###################################
# Aufbau des UI des Hauptfensters #
###################################
sub build_ui
{
my $ref_tblform = shift; #Zeiger auf das TblForm-Objekt
### Fenster mu zuerst aufgebaut werden ###
$ref_tblform->{'ui'}->{'_top'}= $ref_tblform->docform()->top()->Toplevel(-title=>'Properties of Table');
$ref_tblform->{'ui'}->{'_top'}->minsize(400,300);
my $ui_base = $ref_tblform->{'ui'};
my $config = $main::config;
my $activebackground = $config->{'UI'}->{'activebackground'};
my $buttons_style = $config->{'UI'}->{'buttons_style'};
my $buttons_borderwidth = $config->{'UI'}->{'buttons_borderwidth'};
my $speedbarfont = $config->{'UI'}->{'speedbarfont'};;

#Grundbanels
$ui_base->{'_bottom_frame'} = $ui_base->{'_top'}->Frame()->pack(-fill=>'x',-side=>'bottom', -expand=>0);
$ui_base->{'_top_frame'} = $ui_base->{'_top'}->Frame()->pack(-fill=>'both',-side=>'top', -expand=>1);
#Knpfe
$ui_base->{'_btn_ok'} = $ui_base->{'_bottom_frame'}->Button(-text=>'OK', -pady=>3, -padx=>5, -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$speedbarfont)->pack(-side=>'left', -expand=>1, -fill=>'both');
$ui_base->{'_btn_cancel'} = $ui_base->{'_bottom_frame'}->Button(-text=>'Cancel', -pady=>3, -padx=>5, -activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -font=>$speedbarfont)->pack(-side=>'right', -expand=>1, -fill=>'both');
#Laschen
$ui_base->{'_notebook'} = $ui_base->{'_top_frame'}->NoteBook(-borderwidth=>2, -relief=>'ridge')->pack(-fill=>'both', -expand=>'both');
$ui_base->{'_table_page'}=$ui_base->{'_notebook'}->add('Table', -label=>'Table');
$ui_base->{'_row_page'}=$ui_base->{'_notebook'}->add('Row', -label=>'Row');
$ui_base->{'_col_page'}=$ui_base->{'_notebook'}->add('Column', -label=>'Column');
$ui_base->{'_cell_page'}=$ui_base->{'_notebook'}->add('Cell', -label=>'Cell');
$ui_base->{'_extras_page'}=$ui_base->{'_notebook'}->add('Extras', -label=>'Extras');
#The caption 
my $frame1 = $ui_base->{'_table_page'}->Frame()->pack(-fill=>'x', -side=>'top', -anchor=>'n', -expand=>1);
$ui_base->{'_caption'} = $frame1->Entry(-textvariable=>\$ref_tblform->embobj()->{'args'}->{1}->{'value'})->pack(-side=>'right', -fill=>'y', -expand=>1);
$ui_base->{'_caption'}->bind('<KeyRelease>'=> sub{$ref_tblform->embobj()->caption($ref_tblform->embobj()->{'args'}->{1}->{'value'})} );

my $label1 = $frame1->Label(-text=>'Caption:', -anchor=>'e')->pack(-side=>'left', -fill=>'both', -expand=>1);

}


1; #So mu package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
