#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package Backend;
use strict;

=head1 NAME:

Package: Backend

=head1 SYNOPSIS:

use lib::Backend;

=head1 DIRECTORY/FILE:

The file Backend.pm is located in the subdirectory './lib'

=head1 DESCRIPTION:

This package contains all function that is needed to transfer the document into the backend files.

=head1 STATUS:

Coding: 80%

Documentation: up-do-date

=head1 FUNCTIONS:

=head2 export_to_backend

@latex_file_array = Backend::export_to_backend($ref_docform);

This function transfers the document into the backend format and returns the content of the backend file in an array.

Parameters: 

(+) $ref_docform: Reference of the DocForm object that holds the document to be exported


=cut

#################################
# Transform into backend format #
#################################
sub export_to_backend
{
my $ref_docform = shift;
my $ref_backend = $ref_docform->backend(); #The anonymous configuration hash, that holds the export definition of the current backend
my $ref_internal = $main::internal; #The anonymous configuration hash, that holds the internal style definition of e:doc
my $text_widget = $ref_docform->doctext();
my $tag_manager = $ref_docform->tagman();
my $doc_style = $ref_docform->current_doc_style();
my $doc_window = $ref_docform->top();
 
#EXPORT begins

#Fortschrittsanzeige initialisieren
$ref_docform->_progress_init('Starting export of document.');

#Jene Felder aus edoc.int, die den LATEX-Export von Stilen bestimmen, in einem Hash aus Arrays zusammenbauen
my $backend_fields = 
	{
	'PAR_STYLES'	=>	['front','main','end'],
	'ENV_STYLES'	=>	['front_line','end_line','each_line_front','each_line_end'],
	'CHAR_STYLES'	=>	['front','main','end'],
	'COMMANDS'		=>	['main'],
	'DOC_STYLES'	=> ['paragraph_front','paragraph_end','paragraph_front_line','paragraph_end_line']
	};

#Progress to 10%
$ref_docform->_progress_update('Analyzing document head.',10);

my @backend_document;

#Adding the preamble
my $preamble = Language::process_edoc_commands_export($ref_backend->{'DOC_STYLES'}->{$doc_style}->{'preamble'}."\n", $ref_docform, undef);
$preamble = Backend::process_arguments($preamble, $ref_docform, 'DOC_STYLES', 'DOC_STYLES' , $doc_style , $ref_backend);
push (@backend_document, $preamble );

#Adding the documents start-text
my $start_text = Language::process_edoc_commands_export($ref_backend->{'DOC_STYLES'}->{$doc_style}->{'text_front'}."\n", $ref_docform, undef) ;
$start_text = Backend::process_arguments($start_text, $ref_docform, 'DOC_STYLES', 'DOC_STYLES' , $doc_style , $ref_backend);
push (@backend_document, $start_text);

#Export-Textwidget erzeugen (bleibt unsichtbar)
my $exporttext = $ref_docform->{'puffer_text'};

#Progress to 20%
$ref_docform->_progress_update('Analyzing document content.',20);

#Text + alle Tags + alle Objekte in dieses Textwidget umkopieren
$tag_manager->copy_text_widget($text_widget,$exporttext,-1,-1);

#Progress to 30%
$ref_docform->_progress_update('Masking special characters.',30);

#MASKING FOR BACKEND
#The way this is coded now is very slow, if the backend has many characters to mask and they are
#used often (means in many different paragraphs of the document), it takes up to 80%-90% of total 
#export-time. So there is a lot space for improvements in the next 40 lines. But it works for now.
my $ref_mask = $ref_backend->{'MASK'}; #shortcut

print "MASKING:\n"; #debug1
foreach my $key (keys %{$ref_backend->{'MASK'}}) {print "$key = ".$ref_backend->{'MASK'}->{$key}."\n"}; #debug1

my $characters_to_mask = quotemeta( join('',keys %{$ref_mask}) ); #Get a strig of all characters to mask
my $lastline = $exporttext->index('end'); #calculate the index of the last line
$lastline =~ s/\.(.*)//;
for (my $line=1;$line <= $lastline;$line++) #iterate ovet the lines
	{
	my $line_content = $exporttext->get($line.".0", $line.".end"); #get the full content of the line
	if ($line_content =~ /[$characters_to_mask]/o) #If there is one or mor characters there that has to be masked
		{
		my $end_of_line = $exporttext->index($line.".end"); #calculate the index of the last character in this line
		$end_of_line =~ s/(.*?)\.//;
		for (my $char=0;$char<=$end_of_line;$char++) #iterate over all characters
			{
			#Test if the styles for this character are all set to 'do_mask = 1' in *.out-file
			my $do_mask=1;
			my @tags= $exporttext->tagNames("$line.$char");
			foreach my $tag (@tags)
				{
				if ($tag =~ /^<(.+?)><(.+?)>/o)
					{
					my $stylegroup = $1;
					my $style = $2;
					if (defined $ref_backend->{$stylegroup}->{$style}->{'do_mask'} and ($ref_backend->{$stylegroup}->{$style}->{'do_mask'} == 0))
						{
						$do_mask=0;
						}
					}
				}
			next if $do_mask==0; #If one of the tags of the character is set to 'do_mask =0' then do not mask him -> get the next one
			my $character=$exporttext->get("$line.$char"); #get the actual character
			if ($character =~ s/([$characters_to_mask])/$ref_mask->{$1}/ge ) #mask him
				{
				$exporttext->insert("$line.$char + 1 chars",$character); #insert the new character(s) right of the place
				my @taglist = $exporttext->tagNames("$line.$char"); #Get all tags that the old character has
				foreach my $tagname (@taglist) #iterate over this taglist
					{
					#Applying this tag to the newly inserted characters
					$exporttext->tagAdd($tagname, "$line.$char + 1 chars", "$line.$char + ".(length($character)+1)." chars");
					}
				$exporttext->delete("$line.$char"); #delete the old character
				my $difference = length($character)-1; #calculate length differences
				if (! $difference == 0) #if there is one
					{
					$end_of_line=$end_of_line+$difference; #be aware of changings in linelength
					$char=$char+$difference; #move this distance to the right (for beeing sure to not remask something
					}
				}
			}
		}
	}

#PROCESSING OF TAGS
#Progress to 50%
$ref_docform->_progress_update('Processing tags.',50);

#Processing all lines that are not part of PAR_STYLES and ENV_STYLES
my $last_line = $exporttext->index('end');
$last_line =~ s/\.(.*)//g;
for (my $line=1; $line<=$last_line;$line++)
	{
	my @tagnamelist = my @tagnames = $exporttext->tagNames("$line.0");
	my $found=0; #flag
	foreach my $tagname (@tagnamelist)
		{
		$found=1 if (($tagname =~  /^<PAR_STYLES>.*$/)||($tagname =~  /^<ENV_STYLES>.*$/));
		}
		if ($found==0)
			{
			#The lines that are not part of a paragraph or environment style are temporarely tagged with a DOC_STYLES-tag
			#this will cause the following export routine to apply the settings doen in the DOC_STYLES section
			#for normal 'plain' lines
			$exporttext->tagAdd("<DOC_STYLES><$doc_style><>:$line","$line.0","$line.end");
			}
	}

#Alle Tagnamen holen
my @tagnames = $exporttext->tagNames();
#Durch Tagnamenliste gehen
foreach my $tagname (@tagnames)
	{
	#Tagname wird spter noch gebraucht
	my $tag_merker = $tagname;
	#Tags aufsplitten in Gruppen + Stilnamen
	if ($tagname =~  /^<(.+)><(.+)><(.*?)>:.+$/)
		{
		my $stylegroup = $1;
		my $style = $2;
		#EMBOBJ-derived object are treated differently than the normal tags
		if (($stylegroup eq 'EMBOBJ'))
			{
			#Get a reference to the object.
			my $ref_object = $ref_docform->tagman()->{'objects'}->{$tagname};
			#Get the replacing text
			my $text = $ref_object->export_to_backend();
			#Insert the text into the document
			my @range = $exporttext->tagRanges($tag_merker);
			#Wenn Bereichsliste nicht leer
			while ($#range >0)
				{
				#Anfang und Ende des Tags auslesen
				my $ende = pop (@range);
				my $anf = pop (@range);
				$anf = $exporttext->index($anf);
				$ende = $exporttext->index($ende);
				$exporttext->delete($anf,$ende);
				$exporttext->insert($anf,$text) if exists $ref_object->backend()->{'do_export'} and defined $ref_object->backend()->{'do_export'} and (not $ref_object->backend()->{'do_export'} == 0);
				}
			}
		#Treat the 'normal' tags
		if (($stylegroup eq 'PAR_STYLES')||($stylegroup eq 'ENV_STYLES')||($stylegroup eq 'CHAR_STYLES')||($stylegroup eq 'COMMANDS')||($stylegroup eq 'DOC_STYLES'))
			{
			#Backend-Exportfelder fr diesen Stil aus $backend_fields-Hash holen
			my @backend_field_list = @{$backend_fields->{$stylegroup}};
			#Bereich des Tags auslesen
			my @range = $exporttext->tagRanges($tag_merker);
			#Wenn Bereichsliste nicht leer
			while ($#range >0)
				{
				#Anfang und Ende des Tags auslesen
				my $ende = pop (@range);
				my $anf = pop (@range);
				$anf = $exporttext->index($anf);
				$ende = $exporttext->index($ende);
				#If the property do_export is set to 0 in this style
				if (exists $ref_backend->{$stylegroup}->{$style}->{'do_export'} and
					defined $ref_backend->{$stylegroup}->{$style}->{'do_export'} and
				 	$ref_backend->{$stylegroup}->{$style}->{'do_export'} == 0)
					{
					#Delete the tags content
					$exporttext->delete($anf,$ende);
					#move on to the range of this tag or to the next tag
					next;
			 		}
				#Bereich in Zeilen und Spalten zerlegen
				my $startline = $anf;
				$startline =~ s/\.(.*)//g;
				my $endline = $ende;
				$endline =~ s/\.(.*)//g;
				#strings die vorne ($a) und hinten($b) oder ber alles ($c) eingefgt werden 
				my ($front,$main,$end,$front_line,$end_line,$each_line_front,$each_line_end,$paragraph_end,$paragraph_front,$paragraph_end_line,$paragraph_front_line) = ('','','','','','','','','','','','','','','');
				#Important: Remove the tag from this area 
				$exporttext->tagRemove($tag_merker,$anf,$ende);
				#Durch diese Liste mit Backend-Exportfeldern gehen
				foreach my $backend_field (@backend_field_list)
					{
					#Wenn es dieses Backend-Feld fr diesen Stil in Datei "edoc.int" gibt
					if (exists $ref_backend->{$stylegroup}->{$style}->{$backend_field})
						{
						#Und wenn es nicht leer ist
						if ((defined $ref_backend->{$stylegroup}->{$style}->{$backend_field} )==1)
							{
							#den Wert des Backend-Feldes auslesen
							my $field_value = $ref_backend->{$stylegroup}->{$style}->{$backend_field};

							#Process special backend-commands
							$field_value = Backend::process_arguments($field_value, $ref_docform, $tagname, $stylegroup, $style, $ref_backend);

							#Je nach Typ des Backend-Feldes vorne oder hinten anhngen und ggf. Newline dazugeben
							if ( not $field_value eq '')
								{
								#If the field is set to '%DELETE%'
								if ( $field_value =~ /^(\w*)%DELETE%(\w*)$/o )
									{
									#Overwrite place with '';
									$field_value = '';
									}
								if ($backend_field eq 'front') {$front = $front.$field_value }
								elsif ($backend_field eq 'main') {$main = $field_value }
								elsif ($backend_field eq 'end') {$end = $end.$field_value }
								elsif ($backend_field eq 'front_line') {$front_line = $field_value."\n".$front_line}
								elsif ($backend_field eq 'end_line') {$end_line = $end_line."\n".$field_value}
								elsif ($backend_field eq 'each_line_front') {$each_line_front = $each_line_front.$field_value} 
								elsif ($backend_field eq 'each_line_end') {$each_line_end = $each_line_end.$field_value}
								elsif ($backend_field eq 'paragraph_front') {$paragraph_front = $paragraph_front.$field_value} 
								elsif ($backend_field eq 'paragraph_end') {$paragraph_end = $paragraph_end.$field_value}
								elsif ($backend_field eq 'paragraph_front_line') {$paragraph_front_line = $field_value."\n".$paragraph_front_line}
								elsif ($backend_field eq 'paragraph_end_line') {$paragraph_end_line = $paragraph_end_line."\n".$field_value};
								}
							}
						}
					else 
						{
						print "ERROR: Backend exportfield $backend_field is not defined in \"".$ref_docform->current_backend_name().".out\" !\n";
						}
					}
				#Sometimes the user applies a character style over some lines, then this should be treated
				#for each line seperatly
				if (($stylegroup eq 'CHAR_STYLES')&&(($startline == $endline)==0))
					{
					$each_line_front = $front;
					$each_line_end = $end;
					$front = '';
					$end = '';
					}
				#if main was used, delete this area first
				if (($main eq '')==0)
					{
					#export main after deleting the tags space
					$exporttext->delete($anf,$ende);
					$exporttext->insert($anf,$main);
					}
				#if each line has to get some things in front or ath the end
				if ((($each_line_front eq '')==0)||(($each_line_end eq '')==0))
					{
					#This stuff has to be done, because if there are newline in the each_line_* properties, the environments lower border and the lines that contain text move.
					my $f_offset = 0; #Newline-counter in the each_line_front field
					my $e_offset = 0; #Newline-counter in the each_line_end field
					while ($each_line_front =~ /\n/g ) {$f_offset++}; #Count them
					while ($each_line_end =~ /\n/g ) {$e_offset++}; #Count them
					for (my $line=$startline; $line<=$endline;$line++)
						{
						$exporttext->insert("$line.0",$each_line_front); #Insert the fieldvalue in front of the line
						$endline = $endline + $f_offset; #Shift by the number of newlines
						$line=$line + $f_offset; #Shift by the number of newlines
						$exporttext->insert("$line.end",$each_line_end); #Insert the fieldvalue at the end of the line
						$endline = $endline + $e_offset; #Shift by the number of newlines
						$line=$line + $e_offset; #Shift by the number of newlines
						}
					}
				#if each paragraph has to get some things
				if ((($paragraph_front eq '')==0)||(($paragraph_end eq '')==0))
					{
					#Here we have only to deal with the content of the front field, because a paragraph does only consist of one line
					my $f_offset = 0; #Newline-counter in the each_line_front field
					while ($paragraph_front =~ /\n/g ) {$f_offset++}; #Count them
					for (my $line=$startline; $line<=$endline;$line++)
						{
						$exporttext->insert("$line.0", $paragraph_front);
						$endline = $endline + $f_offset; #Shift by the number of newlines
						$line=$line + $f_offset; #Shift by the number of newlines
						$exporttext->insert("$line.end", $paragraph_end);
						}
					}

				#export the other backend fields --> export $b(end) BEFORE $a(front)!
				if (($end eq '')==0) {$exporttext->insert($ende,$end)};
				if (($front eq '')==0) {$exporttext->insert($anf,$front)};
				if (($end_line eq '')==0) {$exporttext->insert("$endline.end",$end_line)};
				if (($front_line eq '')==0) {$exporttext->insert("$startline.0",$front_line)};
				if (($paragraph_end_line eq '')==0) {$exporttext->insert("$endline.end",$paragraph_end_line)};
				if (($paragraph_front_line eq '')==0) {$exporttext->insert("$startline.0",$paragraph_front_line)};
				#Important: Get the actual ranges again for the while-loop
				@range = $exporttext->tagRanges($tag_merker);
				}
			}
		}
	}

#Progress to 80%
$ref_docform->_progress_update('Finishing the work.',80);

#BUILDING UP THE ARRAY OF THE EXPORT_TEXT:
#Dokument-Inhalt als reine Textinfo ausgeben und zwischen allen Zeilen eine Leerzeile = Absatz
my $endline = $exporttext->index('end');
$endline =~ s/\.(.*)//g;
for (my $line=1;$line <= $endline;$line++)
	{
	my $line_text = $exporttext->get("$line.0", "$line.end");
	push (@backend_document, $line_text); 
	}

#Progress to 90%
$ref_docform->_progress_update('Processing document footer.',90);

#Text-Startcommand der Dokumentklasse ausgeben
my $end_text = Language::process_edoc_commands_export($ref_backend->{'DOC_STYLES'}->{$doc_style}->{'text_end'}."\n", $ref_docform, undef);
$end_text = Backend::process_arguments($end_text, $ref_docform, 'DOC_STYLES', 'DOC_STYLES' , $doc_style , $ref_backend);
push (@backend_document, $end_text);

#Removing progress bar
$ref_docform->_progress_update('Ready',99);
$ref_docform->_progress_finish;

#EXPORT ends

return @backend_document;
}


=head2 process_arguments

$return_string = Backend::process_arguments($source_string, $ref_docform, $tag_name, $style_group, $style, $ref_backend);

This function parses the given $source_string for arguments and replaces them with the arguments value.

The function returns the parsed result.

Parameters: 

(+) $source_string: The string to mask.

(+) $ref_docform: A reference to the DocForm object that exports the object

(+) $tag_name: The tagname of the tag that gest exportet or the name of the window_tag that covers an embeded object

(+) $style_group: The stylegroup

(+) $style: The style name that gets exported

(+) $ref_backend: A reference to the backend-subhash

=cut

#########################
# Process the arguments #
#########################
sub process_arguments
{
my $source_string = shift;
my $docform = shift;
my $tag_name = shift;
my $style_group = shift;
my $style = shift;
my $backend = shift; #reference to the backend hash

my $arg_number;
my $arg_base;
if ($style_group eq 'EMBOBJ')
	{
	$arg_base = $docform->tagman()->{'objects'}->{$tag_name}->{'args'};
	$arg_number = $arg_base->{'arg_number'};
	}
elsif ($style_group eq 'DOC_STYLES')
	{
	$arg_base = $docform->tagman()->{'doc_args'};
	$arg_number = $arg_base->{'arg_number'};
	}
else
	{
	$arg_base = $docform->tagman()->{'args'}->{$tag_name};
	$arg_number = $main::internal->{$style_group}->{$style}->{'arg_number'};
	}

#Wenn es Argumente fr diesen Stil gibt
if ($arg_number>0)
	{
	#Durch alle Argumente iterieren
	for (my $i=1;$i<=$arg_number;$i++)
		{
		#Wert+Name aus Tagmanager-Objekt holen
		my $arg_name = $arg_base->{$i}->{'name'};
		my $arg_value = $arg_base->{$i}->{'value'};
		if ($style_group eq 'DOC_SYTLES')
			{
			print "$arg_name = $arg_value \n"; #debug1
			}

		#Wenn Argumenten-Wert nicht undef ist
		if (defined $arg_value)
			{
			#Ersetzen des Platzhalters
			my $from = quotemeta("%ARG_".$i."_VALUE%");
			my $to = $arg_value; # !no quotemeta here
			#Test if the style is set to 'do_mask = 1' in *.out-file
			if (defined $backend->{$style_group}->{$style}->{'do_mask'} and ($backend->{$style_group}->{$style}->{'do_mask'} == 1))
				{
				$to = Backend::mask_for_backend( $to, $backend ); #mask him
				}
			# I had to use the # instead of / because "=~ s/\$from/\$to/g" didnt work
			$source_string =~ s#$from#$to#g;
			}
		#If defined arguments name
		if (defined $arg_name)
			{
			#process the placeholder
			my $from = quotemeta("%ARG_".$i."_NAME%");
			my $to = $arg_name; # !no quotemeta here
			#Test if the style is set to 'do_mask = 1' in *.out-file
			if (defined $backend->{$style_group}->{$style}->{'do_mask'} and ($backend->{$style_group}->{$style}->{'do_mask'} == 1))
				{
				$to = Backend::mask_for_backend( $to, $backend ); #mask him
				}
			# I had to use the # instead of / because "=~ s/\$from/\$to/g" didnt work
			$source_string =~ s#$from#$to#g;
			}
		}
	}
#Process special backend-commands
$source_string = Language::process_edoc_commands_export($source_string, $docform, $tag_name);

return $source_string;
}



=head2 mask_for_backend

$string = Backend::mask_for_backend($source_string);

This function masks (=escapes) all special characters definded in the 'MASK'-block of the backends '*.out'-file.

Parameters: 

(+) $source_string: The string to mask.

(+) $ref_backend: A reference to the $backend-hash

=cut

###########################################
# Mask special characters for the backend #
###########################################
sub mask_for_backend
{
my $source_string = shift;
my $ref_backend = shift;
my $ref_mask = $ref_backend->{'MASK'}; #shortcut
my $characters_to_mask = quotemeta( join('',keys %{$ref_mask}) ); #Get a strig of all characters to mask
$source_string =~ s/([$characters_to_mask])/$ref_mask->{$1}/ge ;
return $source_string;
}




1; #So mu package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
