#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package Config;

use strict;
require lib::Filesys;

=head1 NAME:

Package: Config

=head1 SYNOPSIS:

use lib::Config;

=head1 DIRECTORY/FILE:

The file Config.pm is located in the subdirectory './lib'

=head1 DESCRIPTION:

This package contains all functions that transfer the information from the configuration files
into the anonymous hashes that keep this information in memory during runtime ($main::internal, $main::config, $main::obj ...).

The actual reading is done by a simple loop that puts all of the files content into an array.
It uses the functions from the package lib::Filesys to get some blocks out of the filearray.
This data is then transferred into the right positions in the configuration hashes.

=head1 STATUS:

Coding: 90% 

Documentation: up-do-date


=head1 FUNCTIONS:

=head2 load_internal

Config::load_internal();

Read the content of the file '\config\edoc.int' into an array and call the functions that read the
content of that array into the right branches of the anonymous $main::internal-hash.

=cut

sub load_internal
{
my @int_file_array =  InOut::build_array_from_file(Utils::process_filename('config/edoc.int')); #Array fuer originalen Dateiinhalt von "edoc.int"

#Informationen auswerten
Config::load_internal_global(\@int_file_array); 
Config::load_internal_numlists(\@int_file_array); 
my @styles_array =Filesys::cut_data_array(\@int_file_array,'STYLES',0); 
Language::read_stylegroup($main::internal, 'DOC_STYLES', \@styles_array); #Referenz auf Internal-Hash zum Laden der Dokumentformate uebergeben
Language::read_stylegroup($main::internal, 'PAR_STYLES', \@styles_array); #Referenz auf Internal-Hash zum Laden der Dokumentformate uebergeben
Language::read_stylegroup($main::internal, 'ENV_STYLES', \@styles_array); #Referenz auf Internal-Hash zum Laden der Dokumentformate uebergeben
Language::read_stylegroup($main::internal, 'CHAR_STYLES', \@styles_array); #Referenz auf Internal-Hash zum Laden der Dokumentformate uebergeben
Language::read_stylegroup($main::internal, 'COMMANDS', \@styles_array); #Referenz auf Internal-Hash zum Laden der Dokumentformate uebergeben
Language::read_stylegroup($main::internal, 'EMBOBJ', \@styles_array); #Referenz auf Internal-Hash zum Laden der Dokumentformate uebergeben
Language::process_property_value_commands($main::internal); #Process all '%PROPERTY_VALUE%'-commands
}

=head2 load_internal_global

Config::load_internal_global($ref_file_array)

Loads all global definitions form the file '\config\edoc.int'.

It gets the information from the previously created file_array that holds the content of the file 'edoc.int'
and  to read the desired GLOBAL block correctly into the anonymous hash referenced by $ref_internal.

Parameters: 

(+) $ref_file_array: A reference to the array that holds the content of '\config\edoc.int'

=cut

##################################
# Globale Einstellungen einlesen #
##################################
sub load_internal_global
{
my $ref_file_array = shift; #Zeiger auf Array welches Dateiinhalt von "edoc.int" enthlt
my $ref_internal = $main::internal; #Zeiger auf $main::internal-Hash
#Generellen "GLOBAL"-Block der Datei "edoc.int" einlesen
my %global=Filesys::cut_data_hash($ref_file_array,'GLOBAL'); 
while ((my $key,my $value) = each %global)
		{
		$ref_internal->{'GLOBAL'}->{$key}=$value;
		}
#Generellen "TEXT"-Block der Datei "edoc.int" einlesen
my %text=Filesys::cut_data_hash($ref_file_array,'BASE_TEXT'); 
while ((my $key,my $value) = each %text)
		{
		$ref_internal->{'BASE_TEXT'}->{$key}=$value;
		}
}

=head2 load_internal_numlists

Config::load_internal_numlists($ref_file_array)

Read all information about the numbering engines from the DOC_NUMLISTS block

Parameters: 

(+) $ref_file_array: A reference to the array that holds the content of '\config\edoc.int'

=cut

###########################
# Read DOC_NUMLISTS block #
###########################
sub load_internal_numlists
{
my $ref_file_array = shift; #Zeiger auf Array welches Dateiinhalt von "edoc.int" enthlt
my $ref_internal = $main::internal; #Zeiger auf $main::internal-Hash
#Read NUM_LIST block
my @numlist_array = Filesys::cut_data_array($ref_file_array,'DOC_NUMLISTS'); 
#Get the names of all sub-blocks
my @names = Filesys::read_blocklist(\@numlist_array);
#Iterate over the namelist
foreach my $name (@names)
	{
	print "Read NumList: $name \n"; #debug
	#Read each specific sub-block into an hash
	my %options=Filesys::cut_data_hash(\@numlist_array,$name); 
	#Store the values in the $main::internal-hash
	while ((my $key,my $value) = each %options)
		{
		$ref_internal->{'DOC_NUMLISTS'}->{$name}->{$key}=$value;
		}
	}
}

=head2 load_project

Config::load_project()

Read the content of the file '\config\edoc.prj' into an array and call the functions that read the
content of that array into the right branches of the anonymous $main::project-hash.

=cut

sub load_project
{

my @prj_file_array =  InOut::build_array_from_file(Utils::process_filename('config/edoc.prj')); #Array fuer originalen Dateiinhalt von "edoc.prj"

Config::load_project_filetypes(\@prj_file_array);
Config::load_project_args(\@prj_file_array);

}


=head2 load_config

Config::load_config()

Read the content of the file '\config\edoc.cfg' into an array and call the functions that read the
content of that array into the right branches of the anonymous $main::config-hash.

=cut

sub load_config
{

my @cfg_file_array =  InOut::build_array_from_file(Utils::process_filename('config/edoc.cfg')); #Array fuer originalen Dateiinhalt von "edoc.cfg"

#Informationen auswerten
Config::load_config_inline_macros(\@cfg_file_array); 

Config::load_config_ui(\@cfg_file_array);

Config::load_config_general(\@cfg_file_array);

#Getting the data for the DocForm
my @docform_array = Filesys::cut_data_array(\@cfg_file_array,'DOCUMENT_FORM');
Config::load_config_speedbar(\@docform_array, 'DOCUMENT_FORM');
Config::load_config_menu(\@docform_array, 'DOCUMENT_FORM');

#Getting the data for the PrjForm
my @prjform_array = Filesys::cut_data_array(\@cfg_file_array,'PROJECT_FORM');
Config::load_config_speedbar(\@prjform_array, 'PROJECT_FORM');
Config::load_config_menu(\@prjform_array, 'PROJECT_FORM');

}

=head2 load_config_inline_macros

Config::load_config_inline_macros($ref_file_array)

Gets the inline_macros first from the 'INLINE_MACROS' block of the file '\config\edoc.cfg'.

Parameters: 

(+) $ref_file_array: A reference to the array that holds the content of '\config\edoc.cfg'

=cut

######################################################
# Inline-Macros-Einstellungen aus edoc.cfg einlesen #
######################################################
sub load_config_inline_macros
{
my $ref_file_array = shift; #Zeiger auf Array welches Dateiinhalt von "edoc.cfg" enthlt

my $ref_config = $main::config; #Zeiger auf $main::config-Hash

#"INLINE_MACROS"-Block der Datei "edoc.cfg" einlesen
my %ui=Filesys::cut_data_hash($ref_file_array,'INLINE_MACROS'); 
while ((my $key,my $value) = each %ui)
		{
		$ref_config->{'INLINE_MACROS'}->{$key}=$value;
		}
}


=head2 load_config_ui

Config::load_config_ui($ref_file_array)

Gets the user interface definition from the 'UI' block of the file '\config\edoc.cfg'.

Parameters: 

(+) $ref_file_array: A reference to the array that holds the content of '\config\edoc.cfg'

=cut

######################################################
# User-Interface-Einstellungen aus edoc.cfg einlesen #
######################################################
sub load_config_ui
{

my $ref_file_array = shift; #Zeiger auf Array welches Dateiinhalt von "edoc.cfg" enthlt
my $ref_config = $main::config; #Zeiger auf $main::config-Hash

#"USERINTERFACE"-Block der Datei "edoc.cfg" einlesen
my %ui=Filesys::cut_data_hash($ref_file_array,'USERINTERFACE'); 
while ((my $key,my $value) = each %ui)
		{
		$ref_config->{'UI'}->{$key}=$value;
		}
}

=head2 load_config_general

Config::load_config_general($ref_file_array)

Gets the general configurations from the 'GENERAL' block of the file '\config\edoc.cfg'.

Parameters: 

(+) $ref_file_array: A reference to the array that holds the content of '\config\edoc.cfg'

=cut

################################
# General config from edoc.cfg #
################################
sub load_config_general
{

my $ref_file_array = shift; #Reference to the array that holds the content of edoc.cfg
my $ref_config = $main::config; #Reference to the $main::config hash

#Read the GENERAL block
my %general_hash=Filesys::cut_data_hash($ref_file_array,'GENERAL'); 
while ((my $key,my $value) = each %general_hash)
		{
		$ref_config->{'GENERAL'}->{$key}=$value;
		}
}


=head2 load_project_args

Config::load_project_args($ref_prj_array)

Get the definitions of the project arguments from the configuration file

Parameters: 

(+) $ref_prj_array: A reference to the array that holds the content.

=cut

#############################
# Get the project arguments #
#############################
sub load_project_args
{
my $ref_prj_array = shift; 
my $ref_project = $main::project; #Reference to the $main::project-Hash

#Read the "PROJECT_ARGUMENTS" block 
my %project_args=Filesys::cut_data_hash($ref_prj_array,'PROJECT_ARGUMENTS'); 
while ((my $key,my $value) = each %project_args)
		{
		$ref_project->{'PROJECT_ARGS'}->{$key} = Language::process_edoc_commands($value);
		}

#Read the "FILES_ARGUMENTS" block 
my %files_args=Filesys::cut_data_hash($ref_prj_array,'FILES_ARGUMENTS'); 
while ((my $key,my $value) = each %files_args)
		{
		$ref_project->{'FILE_ARGS'}->{$key} = $value;
		}
}

=head2 load_project_filetypes

Config::load_project_filetypes($ref_prj_array)

Gets the filetapes definition from the file '\config\edoc.prj'.

This is where the user can define several filetypes and associated actions (MACROS) for doubleclicking on them.

Parameters: 

(+) $ref_prj_array: A reference to the array that holds the content of '\config\edoc.prj'

=cut


################################
# Get the FILTYPES definitions #
################################
sub load_project_filetypes
{
my $ref_prj_array = shift; 

my $ref_project = $main::project; #Reference to %main::project-Hash

#read the block FILETYPES
my @filetypes_array=Filesys::cut_data_array($ref_prj_array,'FILETYPES',0); 
#Liste der darin enthaltenen Blcke = Filetypes lesen
my @filetype_names=Filesys::read_blocklist(\@filetypes_array); 
#Alle Filetypes einlesen
foreach my $filetype (@filetype_names)
	{
	if ($filetype =~ /~/ )
		{
		die ("\nERROR: The character '~' is not allowed in filetype names,\nplease change that in the file 'edoc.prj'!\n\n\n") ;
		}
	#In Namensliste aufnehmen
	push (@{$ref_project->{'FILETYPE_NAMES'}},$filetype);
	#Fr jedes Format den Daten-Hash einlesen
	my %actual_filetype=Filesys::cut_data_hash(\@filetypes_array,$filetype); 
	#Mit den angegebenen Werten berschreiben
	while ((my $key,my $value) = each %actual_filetype)
		{
		#Interne Edoc-Datenstruktur mit den Daten fllen
		$ref_project->{'FILETYPES'}->{$filetype}->{$key}=$value;
		}
	}
}


=head2 load_config_speedbar

Config::load_config_speedbar($ref_form_array, $form_name);

Gets the speedbar definitions from the 'SPEEDBAR' block of the file '\config\edoc.cfg'.

Parameters: 

(+) $ref_form_array: A reference to the array that holds the content of the forms block in '\config\edoc.cfg'

(+) $form_name: The form name to which the speedbar belongs

=cut

###############################################
# Get the Speedbar settings for a form object #
###############################################
sub load_config_speedbar
{
my $ref_form_array = shift; #The content of the block
my $form_name = shift;

my $ref_config = $main::config; #Zeiger auf $main::config-Hash

#Prototypen einlesen
my %proto=Filesys::cut_data_hash($ref_form_array,'SPEEDBAR_PROTOTYPE'); 
#Ganzen Bereich mit Button-Definitionen einlesen
my @speedbar_array =Filesys::cut_data_array($ref_form_array,'SPEEDBAR',0); 
#Liste der Knopfnamen holen
my @buttonlist=Filesys::read_blocklist(\@speedbar_array); 
$ref_config->{$form_name}->{'SPEEDBAR_NAMES'}=\@buttonlist;
#Alle Buttons der Reihe nach einlesen
foreach my $name (@buttonlist)
	{
	#Fr jeden Button den Daten-Hash einlesen
	my %actual_button=Filesys::cut_data_hash(\@speedbar_array,$name); 
	#Zuerst Daten des Protypen setzen
	while ((my $key,my $value) = each %proto)
		{
		$ref_config->{$form_name}->{'SPEEDBAR'}->{$name}->{$key}=$value;
		}
	#Dann Button-Definitionen setzten
	while ((my $key,my $value) = each %actual_button)
		{
		$ref_config->{$form_name}->{'SPEEDBAR'}->{$name}->{$key}=$value;
		}
	}
}

=head2 load_config_menu

Config::load_config_menu($ref_form_array, $form_name);

Gets the 'MENU' entries definition from the file '\config\edoc.cfg'.

Parameters: 

(+) $ref_form_array: A reference to the array that holds the content of the forms block in '\config\edoc.cfg'

(+) $form_name: The form name to which the menu belongs

=cut

##########################################
# Get the menu entries for a Form object #
##########################################
sub load_config_menu
{
my $ref_form_array = shift; #The content of the block
my $form_name = shift;

my $ref_config = $main::config; #Zeiger auf $main::config-Hash

#Ganzen Bereich mit Toolsmen-Definitionen einlesen
my @menu_array =Filesys::cut_data_array($ref_form_array, 'MENU', 0); 
#Liste der Blcke holen
my @menulist=Filesys::read_blocklist(\@menu_array);

#Create an array in the tree
$ref_config->{$form_name}->{'MENU'}->{'_MENULIST'} = \@menulist;

foreach my $menu (@menulist)
	{
	#Ganzen Bereich mit Toolsmen-Definitionen einlesen
	my @entries_array =Filesys::cut_data_array(\@menu_array, $menu, 0); 
	#Liste der Blcke holen
	my @entrylist=Filesys::read_blocklist(\@entries_array);
	$ref_config->{$form_name}->{'MENU'}->{$menu}->{'_ENTRYLIST'} = \@entrylist;
	foreach my $entry (@entrylist)
		{
		#Fr jeden Menue-Eintrag ein Daten-Array einlesen
		my %entry=Filesys::cut_data_hash(\@entries_array, $entry); 
		$entry{'_NAME'} = $entry;
		$ref_config->{$form_name}->{'MENU'}->{$menu}->{$entry} = \%entry;
		}
	}
}

=head2 load_macros

Config::load_macros()

Read the content of the file '\config\edoc.mac' into an array and call the functions that read the
content of that array into the right branches of the anonymous $main::config-hash.

NOTE: The macros are stored in a subbranch of the $main::config-hash, because they do not belong
to the internal documents definition and do not belong to a backend. They are only kept in a seperate
file, because '\config\edoc.cfg' will get rather long when everything is in it.

=cut

sub load_macros
{
my @mac_file_array =  InOut::build_array_from_file(Utils::process_filename('config/edoc.mac')); #Array fuer originalen Dateiinhalt von "edoc.int"

#Informationen auswerten
Config::load_config_macro_header(\@mac_file_array);
Config::load_config_macros(\@mac_file_array); 
}

=head2 load_config_macro_header

Config::load_config_macro_header($ref_file_array)

Gets the macro header definitions from the 'MACRO_HEADER' block of the file '\config\edoc.cfg'.

The macro header is a macrocode that is inserted automatically at the beginning of each defined macro
in the block 'MACROS' and automatically executed before the macro code. This is used to set definitions
you can use inside of all macros.

The macros execution and parsing functionality will later move into a seperate package called 'Macros', its now kept in
several parts of PrjForm.pm, uiPrjForm.pm, DocForm.pm and uiDocForm.pm .

Parameters: 

(+) $ref_file_array: A reference to the array that holds the content of '\config\edoc.cfg'

=cut

######################################
# Macro-Header aus edoc.cfg einlesen #
######################################
sub load_config_macro_header
{
my $ref_file_array = shift; #Zeiger auf Array welches Dateiinhalt von "edoc.cfg" enthlt
my $ref_config = $main::config; #Zeiger auf $main::config-Hash

#Macroheader einlesen
my @header_array =Filesys::cut_data_array($ref_file_array,'MACRO_HEADER',1); 
$ref_config->{'MACRO_HEADER'}=\@header_array;
}

=head2 load_config_macros

Config::load_config_macros($ref_file_array)

Gets the macros code from the 'MACROS' block of the file '\config\edoc.cfg'.

This is the macros names and code you can assign to menu entries and speedbuttons and for actions in the project form.

The macros execution and parsing functionality will later move into a seperate package called 'Macros', its now kept in
several parts of PrjForm.pm, uiPrjForm.pm, DocForm.pm and uiDocForm.pm .

Parameters: 

(+) $ref_file_array: A reference to the array that holds the content of '\config\edoc.cfg'

=cut

################################
# Macros aus edoc.cfg einlesen #
################################
sub load_config_macros
{
my $ref_file_array = shift; #Zeiger auf Array welches Dateiinhalt von "\config\edoc.cfg" enthlt
my $ref_config = $main::config; #Zeiger auf $main::config-Hash

#Ganzen Bereich mit Macro-Definitionen einlesen
my @macros_array =Filesys::cut_data_array($ref_file_array,'MACROS',0); 
#Liste der Macronamen holen
my @macrolist=Filesys::read_blocklist(\@macros_array); 
#Alle Macros der Reihe nach einlesen
foreach my $name (@macrolist)
	{
	#Fr jedes Macro ein Daten-Array einlesen
	my @macro=Filesys::cut_data_array(\@macros_array,$name,1); 
	$ref_config->{'MACROS'}->{$name}=\@macro;
	}
}


=head2 load_backend

Config::load_backend($ref_docform, $backend_name)

Read the content of the file '{backend_name}.out' into an array and call the functions that read the
content of that array into the right branches of the anonymous $backend-hash.

Parameters: 

(+) $ref_docform: A reference to the docform object, that has changed its current_backend_name

(+) $backend_name: The name of the backend to load

=cut


sub load_backend
{
my $ref_docform = shift;
my $backend_name = shift;

#Get a reference to the anonymous $backend-hash of the DocForm object
my $ref_backend = $ref_docform->backend();

#A flag if the DocForm GUI is already build up, because at the first call this isn't done
my $draw_progress = 0;  #Ok, this looks like a diry hack!
if (defined $ref_docform->progress_bar()) {$draw_progress = 1};

#Informationen auswerten
#Initialising progress bar in DocForm object
$ref_docform->_progress_init('Reading backend') if $draw_progress ==1;

# Put the content of the backends *.out file into an array
$ref_docform->_progress_update('Reading configuration file',10) if $draw_progress ==1;
my @backend_file_array =  InOut::build_array_from_file(Utils::process_filename('%EDOC_PATH%/config/'.$backend_name.'.out')); 

#Read global data
$ref_docform->_progress_update('Reading global data',15) if $draw_progress ==1;
Config::load_backend_global($ref_backend, \@backend_file_array); #Referenz auf Backend-Hash zum Laden der allgem. Einstellungen uebergeben

#Read masking data
$ref_docform->_progress_update('Reading masking data',20) if $draw_progress ==1;
Config::load_backend_mask($ref_backend, \@backend_file_array); #Referenz auf Backend-Hash zum Laden der allgem. Einstellungen uebergeben

#Read Backend menu
$ref_docform->_progress_update('Reading menu data',30) if $draw_progress ==1;
Config::load_backend_backendmenu($ref_backend, \@backend_file_array); #Referenz auf Backend-Hash zum Laden der allgem. Einstellungen uebergeben

#Read the STYLES section
$ref_docform->_progress_update('Reading styles block',40) if $draw_progress ==1;
my @styles_array =Filesys::read_data_array(\@backend_file_array,'STYLES',0); 

#DOC_STYLES
$ref_docform->_progress_update('Reading document sytles',50) if $draw_progress ==1;
Language::read_stylegroup($ref_backend, 'DOC_STYLES', \@styles_array); #Referenz auf Backend-Hash zum Laden der Dokumentformate uebergeben
#PAR_STYLES
$ref_docform->_progress_update('Reading paragraph sytles',60) if $draw_progress ==1;
Language::read_stylegroup($ref_backend, 'PAR_STYLES', \@styles_array); #Referenz auf Backend-Hash zum Laden der Dokumentformate uebergeben
#ENV_STYLES
$ref_docform->_progress_update('Reading environment sytles',70) if $draw_progress ==1;
Language::read_stylegroup($ref_backend, 'ENV_STYLES', \@styles_array); #Referenz auf Backend-Hash zum Laden der Dokumentformate uebergeben
#CHAR_STYLES
$ref_docform->_progress_update('Reading character sytles',80) if $draw_progress ==1;
Language::read_stylegroup($ref_backend, 'CHAR_STYLES', \@styles_array); #Referenz auf Backend-Hash zum Laden der Dokumentformate uebergeben
#COMMANDS
$ref_docform->_progress_update('Reading commands',90) if $draw_progress ==1;
Language::read_stylegroup($ref_backend, 'COMMANDS', \@styles_array); #Referenz auf Backend-Hash zum Laden der Dokumentformate uebergeben
#EMBED_OBJECTS
$ref_docform->_progress_update('Reading objects',99) if $draw_progress ==1;
Language::read_stylegroup($ref_backend, 'EMBED_OBJECTS', \@styles_array); #Referenz auf Backend-Hash zum Laden der Dokumentformate uebergeben

Language::process_property_value_commands($ref_backend); #Process all '%PROPERTY_VALUE%'-commands

#Terminate progrssbar
$ref_docform->_progress_update('Ready',99) if $draw_progress ==1;
$ref_docform->_progress_finish if $draw_progress ==1;

}

=head2 load_backend_global

Config::load_backend_global($ref_backend, $ref_file_array)

Loads all global definitions form the '\config\*.out' file.

It gets the information from the previously created file_array that holds the content of the file '*.out'
and  to read the desired GLOBAL block correctly into the anonymous hash referenced by $ref_backend.

Parameters: 

(+) $ref_backend: A reference to the anonymous $backend hash

(+) $ref_file_array: A reference to the array that holds the content of '\config\*.out'

=cut

##################################
# Globale Einstellungen einlesen #
##################################
sub load_backend_global
{
my $ref_backend = shift; #Zeiger auf Hash
my $ref_file_array = shift; #Zeiger auf Array welches Dateiinhalt enthlt
#Generellen "GLOBAL"-Block der Datei "edoc.int" einlesen
my %global=Filesys::cut_data_hash($ref_file_array,'GLOBAL'); 
while ((my $key,my $value) = each %global)
		{
		$ref_backend->{'GLOBAL'}->{$key}=$value;
		}
}

=head2 load_backend_mask

Config::load_backend_mask($ref_backend, $ref_file_array)

Loads all information for backends character-masking (=escaping) form the '\config\*.out' file.

It gets the information from the previously created file_array that holds the content of the file '*.out'
and  to read the desired MASK block correctly into the anonymous hash referenced by $ref_backend.

Parameters: 

(+) $ref_backend: A reference to the anonymous $backend hash

(+) $ref_file_array: A reference to the array that holds the content of '\config\*.out'

=cut

#################################
# Read masking info for backend #
#################################
sub load_backend_mask
{
my $ref_backend = shift; #Zeiger auf Hash
my $ref_file_array = shift; #Zeiger auf Array welches Dateiinhalt enthlt
#Generellen "GLOBAL"-Block der Datei "edoc.int" einlesen
my %global=Filesys::cut_data_hash($ref_file_array,'MASK'); 
while ((my $key,my $value) = each %global)
		{
		$ref_backend->{'MASK'}->{$key}=$value;
		}
}


###################################################
# Eintrge fr "Tools"-Men aus edoc.cfg einlesen #
###################################################
sub load_backend_backendmenu
{
my $ref_backend = shift; #Zeiger auf %backend-Hash
my $ref_file_array = shift; #Zeiger auf Array welches Dateiinhalt von "*.out" enthlt
#Ganzen Bereich mit Toolsmen-Definitionen einlesen
my @menu_array =Filesys::cut_data_array($ref_file_array,'BACKEND_MENU',0); 
#Liste der Blcke holen
my @menulist=Filesys::read_blocklist(\@menu_array);
$ref_backend->{'BACKEND_MENU_NAMES'}=\@menulist;
#Alle Macros der Reihe nach einlesen
foreach my $name (@menulist)
	{
	#Fr jeden Menue-Eintrag ein Daten-Array einlesen
	my %menu=Filesys::cut_data_hash(\@menu_array,$name); 
	$ref_backend->{'BACKEND_MENU'}->{$name}=\%menu;
	}
}



=head2 build_style_list_from_docstyle

Config::build_style_list_from_docstyle($doc_style,$style_group, $none)

Parses the information of the stylegroups for the 'in_document_style'property, builds and returns a new
stylelist for the given stylegroup.

This is neccesary because some styles are only valid in special document styles and should not appear in
the Optionmenu widgets if another document style is set for the document.

Parameters: 

(+) $doc_style: the new actual document style name

(+) $style_group: The name of the stylegroup for which the new style list should be created

(+) $none: The text for the none-entry (what should be shown in the Optionmenu if no style is set at the cursor position.

=cut

##################################################################################################
# Bildet Unterlisten fr Absatz-, Umgebungs- und Zeichenformate je nach bergebenem Dokumentstil #
##################################################################################################
sub build_style_list_from_docstyle
{
my ($doc_style,$style_group, $none)=@_;
my $ref_internal = $main::internal;
my @list;

foreach my $key (keys %{$ref_internal->{$style_group}}) #Liste mit Formaten aufbauen
	{
	if (not ($key eq $none))
		{
		if 
			(
			#If the style is valid in all document styles or
			(($ref_internal->{$style_group}->{$key}->{'in_document_style'} eq 'ALL' or
			#If the actual docstyle is listed in the "in_document_style" property of the style
			Filesys::is_in_stringlist($doc_style,$ref_internal->{$style_group}->{$key}->{'in_document_style'},'||') == 1)
			and $key !~ /^_/ ) # if the style is not an abstract one
			)
			{
			unshift(@list, $key); #add this style to the list
			}
		}
	}
@list = sort(@list);
unshift(@list, $none);
return @list
}


1; #So mu package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

This package should also read information from the now missing configuration 
files into the now missing hashes:

(+) $obj : holds the content of the configuration file 'edoc.obj'

=cut
