#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package InOut;
use strict;
# use XML::Parser; #not used yet

=head1 NAME:

Package: InOut

=head1 SYNOPSIS:

use lib::InOut;

=head1 DIRECTORY/FILE:

The file InOut.pm is located in the subdirectory './lib'

=head1 DESCRIPTION:

This package contains all function that read/write something directly to/from disk or call
any external commands.

Many things will disapear from this package and move in a more generally form into the package 'Backend.pm' which
has to be created to support different backends.

The things that are done with the DVI and Postscript creating/viewing programs will be defined in the
'*.out'-configuration-files and then performed by method of 'Backend.pm'.

=head1 STATUS:

Coding: 90% but some functions will not survive long here.

Documentation: up-do-date

=head1 FUNCTIONS:

=head2 file_open_ascii

InOut::file_open_ascii($ref_docform [, $filename] );

This function reads a document in pure text format (ASCII) and writes it into the textwidget of the given DocForm object.

Parameters: 

(+) $ref_docform: Reference of the DocForm object

(+) $filename: Filename of the textfile to opne, if missing, a dialogbox will ask for it

=cut

#########################
### Datei ffnen ASCII###
#########################
sub file_open_ascii 
{
my $ref_docform = shift; # Reference to the calling DocForm-Object
my $ filename = shift;
$ref_docform->top()->Busy();
#Bei gendertem Text ums Speichern nachfragen
if ($ref_docform->is_content_changed())
	{
	my $btn=$ref_docform->top()->Dialog(-title=>'Work not saved yet !', -text=>"There are unsaved changes in the current document.\nDo you want to save them to disk ?", -buttons=>['Yes','No'], -bitmap=>'question')->Show();
	if ($btn eq 'Yes') {$ref_docform->file_save($ref_docform->filename())};
	}
#Dateinamen abfragen
if ((defined $filename) == 0)
	{
	$filename = $ref_docform->top()->getOpenFile(-filetypes=>[['Text files', ['.txt', '.text']],['All files', ['*']]]);
	}
if (($filename ne "")&&(defined $filename)) # wenn einer angegeben wurde 
	{
	#Schon jetzt das Flag setzen, da sonst in $ref_docform->file_new()nochmal die Sicherheitsabfrage gefragt wird.
	$ref_docform->set_content_unchanged();
	#Textinfo lschen
	$ref_docform->file_new();
	#Laden + eintragen des Filenamens
   $ref_docform->filename($filename);
	chdir $ref_docform->path();
	open (IN,"<".$ref_docform->filename())|| die ("datei existiert noch nicht ".$ref_docform->filename()."\n");
	while (<IN>)
		{
		$ref_docform->doctext()->insert("end", $_);
		}
	close(IN);
	}
#Titel anpassen
$ref_docform->update_window_caption();
$ref_docform->refresh_outline($ref_docform); #Outline neu aufbauen (dabei wird auch durchnummeriert)
#nderungsflag setzen
$ref_docform->set_content_unchanged();
$ref_docform->top()->Unbusy();
}


=head2 file_open

InOut::file_open($ref_docform [, $file_name] );

This function reads a document and writes it into the textwidget of the given DocForm object.

Parameters: 

(+) $ref_docform: Reference of the DocForm object

(+) $filename: The filename&path of the document to open, if not defined, a dialogbox will ask for it

=cut

####################
### Datei ffnen ###
####################
sub file_open 
{
my $ref_docform = shift; # Reference to the calling DocForm-Object
my $filename = shift; # Undef if called from Menu 'File'->'Open'
#Display hourglass
$ref_docform->top()->Busy();

#Flag for a newly created and empty DocForm object
my $empty_docform = 0;
$empty_docform = 1 if ($ref_docform->filename() eq '* scratch *');

#initialise progress bar
$ref_docform->_progress_init('Preparing for deleting old content');

#Bei geaendertem Text ums Speichern nachfragen

if ($empty_docform == 0 and $ref_docform->is_content_changed())
	{
	my $btn=$ref_docform->top()->Dialog(-title=>'Document not saved yet !', -text=>"There are unsaved changes in the current document.\nDo you want to save them to disk ?", -buttons=>['Yes','No'], -bitmap=>'question')->Show();
	if ($btn eq 'Yes') {$ref_docform->file_save($ref_docform->filename())};
	}

#Fortschrittsanzeige initialisieren
$ref_docform->_progress_update('Opening file',10);

if ((defined $filename) == 0)
	{
	#Dateinamen abfragen
	$filename = $ref_docform->top()->getOpenFile(-filetypes=>[['e:doc documents', ['.edc', '.edoc']]]);
	if (((defined $filename)==0)||($filename eq '')) # wenn keiner angegeben wurde 
		{
		goto Exit;
		}
	}

#Checking if the user gave a filename
$ref_docform->filename($filename);
if (($ref_docform->filename() eq '')||((defined $ref_docform->filename())==0)) {goto EXIT}; #Sorry for goto 

#Progress message...
$ref_docform->_progress_update('Clearing old document',20);

#Reference to the Textwidget to be filled
my $new_doctext;
#Puffers for the old values
my $old_doctext;
my $old_tagman;

#If DocForm is still empty, we do not need to create the TextWidget again in the background
if ($empty_docform == 0)
	{
	#storing old objects
	$old_doctext=$ref_docform->doctext();
	$old_tagman=$ref_docform->tagman();

	#Build a new Textwidget without packing it and without binding any events
	$new_doctext = uiDocForm::build_doctext($ref_docform,0,0); 

	#Progress message...
	$ref_docform->_progress_update('Deleting tags',30);
	$ref_docform->tagman(TagMan->new($ref_docform));

		}
else
	{
	$new_doctext=$ref_docform->doctext();
	}

#Progress message...
$ref_docform->_progress_update('Reading file:'.$ref_docform->filename(),40);

#Read file
my @file_array; #Array fr originalen Dateiinhalt
#Open the file
open (IN,"<".$ref_docform->filename())|| die ("Datei existiert  nicht \"".$ref_docform->filename()."\"\n");
my $line;
while (<IN>)
	{
	my $line = $_;
	chop($line);
	push(@file_array,$line ); #Append line to the array
	}
close(IN);

chdir $ref_docform->path();

#Read new document style
my %doc_info = Filesys::cut_data_hash(\@file_array,'DOC_STYLES');
$ref_docform->current_doc_style($doc_info{'doc_style'});
#Set the new document style in the DocForm object (automatically refreshes the OptionMenus)
$ref_docform->doc_style_changed($ref_docform->current_doc_style());

#Read the document arguments
my %doc_args = Filesys::cut_data_hash(\@file_array,'DOC_ARGS');
for (my $i = 1; $i <= $ref_docform->tagman()->doc_args_number(); $i++)
	{
	if (exists $doc_args{"arg_".$i."_value"} and defined $doc_args{"arg_".$i."_value"})
		{
		my $arg_value = $doc_args{"arg_".$i."_value"};
		$ref_docform->tagman()->set_doc_arg_value($i, $arg_value );
		}
	}

#Read the document PARTS
$ref_docform->_progress_update('Reading parts ...',50);

#Read the parts block from the array and build up the hash with the document parts
Parts::read_parts_array(\@file_array, $ref_docform->doc_parts_root()); 

#Refresh the parts treeview
$ref_docform->update_parts_document();

#Read the search history
my @search_history = Filesys::cut_data_array(\@file_array,'SEARCH_HISTORY');
if (not defined @search_history or not ($#search_history >= 0))
	{
	@search_history = ();
	}
$ref_docform->ref_search_history ( \@search_history );

#Read the replace history
my @replace_history = Filesys::cut_data_array(\@file_array,'REPLACE_HISTORY');
if (not defined @replace_history or not ($#replace_history >= 0))
	{
	@replace_history = ();
	}
$ref_docform->ref_replace_history ( \@replace_history );

#Build up documents text and formatting
InOut::insert_array_into_document($ref_docform, '1.0', \@file_array, 1);

#If DocForm is still empty, we do not need to create the TextWidget again in the background
if ($empty_docform == 0)
	{
	#Deleting the old Textwidget and the old Tagman object
	$old_doctext->destroy();
	$old_doctext = undef;
	$old_tagman = undef;
	#Pack the new Textwidget
	$ref_docform->doctext()->pack( -side=>'left', -fill=>'both', -expand=>'yes');
	#Now bind the events for the new Text widget
	uiDocForm::doctext_bind_events($ref_docform, $ref_docform->doctext());
	}

#Update title
$ref_docform->update_window_caption();

#Set documents flag to unchanged
$ref_docform->set_content_unchanged();

EXIT:
#Leave the function correctly
$ref_docform->_progress_finish;
$ref_docform->top()->Unbusy();
}

=head2 file_save_as

InOut::file_save_as($ref_docform [,$filename]);

This function asks for a filename and then calls the function InOut::file_save for saving the document to disk

Parameters: 

(+) $ref_docform: Reference of the DocForm object

(+) $filename: The filename to be saved (OPTIONAL)

=cut

#############################
### Datei speichern unter ###
#############################
sub file_save_as 
{
my $ref_docform = shift; # Reference to the calling DocForm-Object
my $filename = shift;

$ref_docform->top()->Busy();

#Query filename if not given
if (not defined $filename)
	{
	$filename = $ref_docform->top()->getSaveFile(-filetypes=>[['e:doc documents', ['.edc', '.edoc']]], -initialfile => 'newdocument', -defaultextension => '.edc');
	}

if (($filename eq '')||((defined $filename)==0)||($filename eq '* scratch *')) {goto EXIT}; #Sorry for goto 

#Save the thing
$ref_docform->file_save($filename);

EXIT:
$ref_docform->top()->Unbusy();
}

=head2 file_save

InOut::file_save($ref_docform [, $file_name] );

This function writes the document to disk.

Parameters: 

(+) $ref_docform: Reference of the DocForm object

(+) $filename: The filename&path of the document to save, if not defined, a dialogbox will ask for it

=cut

#######################
### Datei speichern ###
#######################
sub file_save
{
my $ref_docform = shift; # Reference to the calling DocForm-Object
my $filename = shift;
my $doctop =  $ref_docform->top();
my $doctext = $ref_docform->doctext();
$doctop->Busy();
#Fortschrittsanzeige initialisieren
$ref_docform->_progress_init('Preparing file:');
$doctext->update();
if (not defined $filename or $filename eq '' or $filename eq '* scratch *')
	{
	$filename = $doctop->getSaveFile(-filetypes=>[['e:doc documents', ['.edc', '.edoc']]], -initialfile => 'newdocument', -defaultextension => '.edc');
	}

$ref_docform->_progress_init('Processing document...');

#Fortschrittsanzeige auf 20%
$ref_docform->_progress_update('Storing doc headers',10);

my @file_array = ();

#Storing the META-DATA
my %meta_hash;
$meta_hash{'fs_version'}=$main::internal->{'fs_version'};
$meta_hash{'prg_version'}=$main::internal->{'prg_version'};
Filesys::write_data_hash(\@file_array,'META',\%meta_hash,1);

#Storing DOC_STYLE
my %doc_style_hash;
$doc_style_hash{'doc_style'} = $ref_docform->current_doc_style();
Filesys::write_data_hash(\@file_array,'DOC_STYLES',\%doc_style_hash,1);

#Storing the document arguments
my %doc_args = ();
for (my $i = 1; $i <= $ref_docform->tagman()->doc_args_number(); $i++)
	{
	if (exists $ref_docform->tagman()->{'doc_args'}->{$i}->{'value'} and defined $ref_docform->tagman()->{'doc_args'}->{$i}->{'value'})
		{
		my $arg_value = $ref_docform->tagman()->{'doc_args'}->{$i}->{'value'};
		$doc_args{"arg_".$i."_value"} = $arg_value;
		}
	}
Filesys::write_data_hash(\@file_array,'DOC_ARGS',\%doc_args,1);

#Get the content of the document (without the parts stored inside)
my @document_array = InOut::create_document_array($ref_docform, -1, -1);
foreach my $line (@document_array) { push (@file_array, $line) };

#Storing configuration data
my %config_hash;
#Nothing is done here yet
Filesys::write_data_hash(\@file_array,'CONFIG',\%config_hash,1);


#Build up an array of all parts stored in the document and append it to the file using a 'PARTS' block
my @parts_array = Parts::build_parts_array($ref_docform->doc_parts_root()); 
Filesys::write_data_array(\@file_array,'PARTS', \@parts_array, 1);

#Write search history
my @search_array = @{$ref_docform->ref_search_history()}; 
Filesys::write_data_array(\@file_array,'SEARCH_HISTORY', \@search_array, 1);

#Write replace history
my @replace_array = @{$ref_docform->ref_replace_history()}; 
Filesys::write_data_array(\@file_array,'REPLACE_HISTORY', \@replace_array, 1);


#Wenn auf Abbrechen geklickt wurde->Abbrechen
if (not defined $filename or $filename eq '' or $filename eq '* scratch *') {goto Exit};

#Fortschrittsanzeige auf 80%
$ref_docform->_progress_update('Writing file to disk',90);
#Datei speichern
my $HANDLE = IO::File->new(">".$filename) || die ("Fehler beim speichern von ".$filename."\n");
foreach my $line (@file_array) {print $HANDLE $line."\n"};
$HANDLE->close(); #Datei schlieen

#Fortschrittsanzeige auf 1%
$ref_docform->_progress_update('Done.',99);

#Filenamen speichern
$ref_docform->filename($filename);

chdir $ref_docform->path();

#Titel anpassen
$ref_docform->update_window_caption();
#nderungsflag setzen
$ref_docform->set_content_unchanged();

EXIT:
$ref_docform->_progress_finish;
$doctop->Unbusy();
}

=head3 create_document_array

@array = InOut::create_document_array($ref_docform, $start_index, $end_index);

This function generates a textarray that has all information about the text-part
mentioned by $start_index and $end_index. This textarray contains all of the
formating info in the e:doc-file-format.

It is used for preparing an array for the file->save function and for copying
some text (incl. formats) into the e:doc-clipboard.

Parameters: 

(+) $ref_docform: A reference to the DocForm object

(+) $start_index: The start-index of the document-part to save in the textarray

(+) $end_index: The end-index of the document-part to save in the textarray

=cut

###############################################################################
### Create an textarray in the e:doc fileformat from a part of the document ###
###############################################################################
sub create_document_array
{
my $ref_docform = shift;
my $start_index = shift;
my $end_index = shift;

my $doctop =  $ref_docform->top();
my $doctext = $ref_docform->doctext();
my $tagman = $ref_docform->tagman();

my @result_array;
$doctop->Busy();

#Fortschrittsanzeige initialisieren
$ref_docform->_progress_init('Processing document...');

#Fortschrittsanzeige auf 20%
$ref_docform->_progress_update('Storing text info',20);
my $exporttext = $ref_docform->{'puffer_text'};
$tagman->copy_text_widget($doctext, $exporttext, $start_index, $end_index);  #Copy text to Puffer and clip it
my $content=$exporttext->get('1.0', 'end'); #Getting the full text of the allready clipped text
my @puffer=split(/\n/, $content);
my @text;
for (my $e=0;$e<=$#puffer;$e++) 
	{
	my $line = Filesys::mask_string($puffer[$e]);
	push (@text, $line);
	}
Filesys::write_data_array(\@result_array,'TEXT',\@text, 1);
if ($start_index == -1 && $end_index==-1)
	{
	$end_index=$doctext->index('end');
	$start_index='1.0';
	}
else
	{
	$end_index=$doctext->index($end_index);
	$start_index=$doctext->index($start_index);
	}

#Fortschrittsanzeige auf 60%
$ref_docform->_progress_update('Storing tags',40);
### Formate speichern
my @tagnames = $tagman->sort_tags_by_stylegroup($exporttext);
my @tags; #For adding the tags to the array
my %par_tags; #Hash for storing info of PAR_STYLES tags, becuase these are sorted by their linenumbers for faster opening the document (no need to search each NumLabels place)
foreach my $tagname (@tagnames)
	{
	my $savetagname;
	my $save_it=0; #flag, damit z.b. Numerierungstags nicht mitgespeichert werden
	my $ranges = 0; #How many ranges are there to store in @result_array
	my @tag_ranges = $exporttext->tagRanges($tagname);
	$ranges = $#tag_ranges+1;
	if ($tagname =~ /<(.+?)><(.+?)><(.*?)>:(\d+?)/)
		{
		my $group = $1;
		my $style = $2;
		my $data = $3;
		#CLIPPING OF TAGRANGES
		my $arg_branch;
		#Format-Tags speichern
		if (($group =~ /^PAR_STYLES/)||($group =~ /^ENV_STYLES/)||($group =~ /^CHAR_STYLES/)||($group =~ /^COMMANDS/))
			{
			$save_it = 1;
			$savetagname = "<$group><$style><$data>";
			$arg_branch = $tagman->{'args'}->{$tagname};
			}
		#save the derived Objects form EmbObj
		if ($group =~ /^EMBOBJ/)
			{
			my $obj = $tagman->{'objects'}->{$tagname};
			my $obj_data = $obj->get_save_data();
			$save_it = 1;
			$savetagname = "<$group><$style><$obj_data>";
			$arg_branch = $tagman->{'objects'}->{$tagname}->{'args'};
			}
		if ($save_it == 1)
			{
			$savetagname=$savetagname.'<<<';#Indexliste Anfang
			# Argumente mitspeichern, falls vorhanden
			$savetagname=$savetagname.($ranges); #Anzahl der Indizes
			foreach (@tag_ranges){$savetagname=$savetagname."||$_"}; #Indizes anhngen
			$savetagname=$savetagname.'>>>';#Indexliste Ende
			$savetagname=$savetagname.'<<<';#Argumentenliste Anfang
			my $arg_number = $arg_branch->{'arg_number'};
			$arg_number = 0 if not defined $arg_number; #Wenn keine Argumente -> auf 0 setzen 
			$savetagname=$savetagname.$arg_number; #Anzahl der Argumente
			for (my $i = 1; $i <= $arg_number; $i++)
			  	{
				my $argument = $arg_branch->{$i}->{'value'};
				$argument =~ s/\n/%NL%/g ; #convert newlines to %NL%
				$savetagname=$savetagname.'||'.Filesys::mask_string($argument); #Argument speichern
				}
			$savetagname=$savetagname.'>>>';#Argumentenlise Ende
			
			# If the tag has ranges
			if ($ranges>0)
				{
				#If the tag is a PAR_STYLES tag and has a range, -store info in a hash for sorting them by linenumbers
				if ($group eq 'PAR_STYLES')
					{
					my $line = shift(@tag_ranges);
					$line =~ s/\.(\d+)$//go ;
					$par_tags{$line} = $savetagname;
					}
				else #If there is one range->push it directly into tag array (PAR_STYLES are treated seperatly to sort them by their appearance in the text)
					{
					push(@tags,$savetagname);
					}
				}
			}
		}
	}
# Append the Par_STYLES tags in reverse order, sorted by their line numbers to speed up re-opening of the document
foreach my $line (sort {$a<=>$b} (keys %par_tags))
	{
	push(@tags,$par_tags{$line});
	}

#Tags in Dateiarray eintragen
Filesys::write_data_array(\@result_array,'TAGS',\@tags,1);

#Fortschrittsanzeige auf 99%
$ref_docform->_progress_update('Done.',80);

EXIT:

$ref_docform->_progress_finish;
$doctop->Unbusy();

return @result_array;
}

=head3 insert_array_into_document

InOut::insert_array_into_document($ref_docform, $start_index, $ref_array, $is_full_file_open);

This function inserts formated textfrom the given textarray that contains all 
information about the text-part into the actual document at the place mentioned by $start_index.

It is used for building up the document in the file->open function and for pasting
some text (incl. formats) from the e:doc-clipboard.

Parameters: 

(+) $ref_docform: A reference to the DocForm object

(+) $start_index: The start-index of the document-part to save in the textarray

(+) $ref_array: A reference to the array that contains the info

(+) $is_full_file_open: Flag is set to '1' if the whole document is to be opened, what means that also the info
    about document style and other things should be processed and the existing document should be cleared first.
    if set to 0 or undef only the text-portion gets inserted into the existingdocument.

=cut

#########################################################
### Insert into document from an array in file format ###
#########################################################
sub insert_array_into_document
{
my $ref_docform=shift;
my $start_index = shift;
my $ref_array = shift;
my $is_full_file_open = shift;
$is_full_file_open = 0 if not defined $is_full_file_open;

my $doctext = $ref_docform->doctext();
my $tagman = $ref_docform->tagman();

my $do_not_sort = $is_full_file_open; #Wether NumLabls in PAR_STYLES have to be sorted into the numlist or not

$ref_docform->top()->Busy();

$start_index = $doctext->index($start_index);
my $start_line;
my $start_char;
if ($start_index =~ /^(\d+)\.(\d+)$/o)
	{
	$start_line = $1;
	$start_char = $2;
	}
#Fortschrittsanzeige initialisieren
$ref_docform->_progress_init('Processing file');

#Fortschrittsanzeige auf 20%
$ref_docform->_progress_update('Analyzing file',20);

my @inhalt = Filesys::read_data_array($ref_array,'TEXT',0); #Textinfo einlesen
if ($is_full_file_open==1)
	{
	my $ending ="\n";
	for (my $e=0;$e<=$#inhalt;$e++) 
		{
		$ending = '' if $e == $#inhalt;
		my $line = Filesys::unmask_string($inhalt[$e]);
		$doctext->insert("end", $line.$ending); 
		}
	}
else
	{
	for (my $e=0;$e<=$#inhalt;$e++) 
		{
		my $line = Filesys::unmask_string($inhalt[$e]);
		if ($e==0)#Insert the first line at the $start_line.$start_char-position
			{
			$doctext->insert("$start_line.$start_char", $line); 
			}
		else
	 		#the following lines have to be calculated differently.
			{
			$doctext->insert(($start_line+$e).'.0', $line."\n"); 
			}
		}
	}

#Fortschrittsanzeige auf 40%
$ref_docform->_progress_update('Processing tags',40);

#Store the names of the created tags here for inserting the TagDoc objects later
my @tags_for_tagdoc; 
my @stylegroups_for_tagdoc;
my @styles_for_tagdoc;

my $tagdoc_on = $ref_docform->tag_doctor_on();

#Absatzformate auslesen
my @tag_array=Filesys::read_data_array($ref_array,'TAGS',0);

for (my $e=0;$e<=$#tag_array;$e++) 
	{
	my $line = $tag_array[$e];
	if ($line =~ /^<(.+?)><(.+?)><(.*?)><<<(.+?)>>><<<(.*)>>>\s*$/o)
		{
		my $group = $1;
		my $style = $2;
		my $data = $3;
		my $indexlist = $4;
		my $arglist = $5;
		my $tagname; #Hier wird der zurckgelieferte Tagname abgelegt (fr Argumentenspeicherung in Hash)
		my @list = split(/\|\|/o,$indexlist); #The list is split by '||'
		my $number=shift(@list);  
		my @index_list;
		my $arg_branch; #a reference to the base structure where the arguments are stored
		if ($is_full_file_open == 0) #If only a fraction was inserted
			{
			while (@list)
				{
				my $old_index = shift(@list);
				my $new_index;
				if ($old_index =~ /^(\d+)\.(\d+)$/o)
					{
					my $old_line = $1;
					my $old_char = $2;
					$new_index = ($start_line+$old_line-1).'.'.($start_char*($old_line==1)+$old_char);
					push(@index_list, $new_index);
					}
				}
			}
		else
			{
			@index_list = @list;
			}
		#Shift the two indexes if they are the wrong way around
		for (my $i=1;$i<=($number/2);$i++)
			{
			my $index1=shift(@index_list);
			my $index2=shift(@index_list);
			if ($doctext->compare($index1,'>',$index2)) #If $index1 greater then $index2->switch them around
				{
				my $mem = $index2;
				$index2=$index1;
				$index1=$mem;
				}
			#PAR_STYLES aufbauen
			if ($group =~ /^PAR_STYLES/o)
				{
				$tagname = $ref_docform->par_style_changed($style, $index1, $index2, 0, $do_not_sort, 0, 0);
				$arg_branch = $tagman->{'args'}->{$tagname};
				$doctext->tagLower($tagname); #PAR_STYLES-tags should be the lower tags
				if ($tagdoc_on == 1)
					{
					push (@tags_for_tagdoc, $tagname); #Store the created tagname for later
					push (@stylegroups_for_tagdoc, $group); #Store the created stylegroup for later
					push (@styles_for_tagdoc, $style); #Store the created style for later
					}
				}
			#ENV_STYLES aufbauen
			elsif ($group =~ /^ENV_STYLES/o)
				{
				$tagname = $ref_docform->env_style_changed($style, $index1, $index2, 0, 0);
				$arg_branch = $tagman->{'args'}->{$tagname};
				$doctext->tagLower($tagname); #ENV_STYLES-tags should be the lower tags
				if ($tagdoc_on == 1)
					{
					push (@tags_for_tagdoc, $tagname); #Store the created tagname for later
					push (@stylegroups_for_tagdoc, $group); #Store the created stylegroup for later
					push (@styles_for_tagdoc, $style); #Store the created style for later
					}
				}
			#CHAR_STYLES aufbauen
			elsif ($group =~ /^CHAR_STYLES/o)
				{
				$tagname = $ref_docform->char_style_changed($style, $index1, $index2, 0);
				$arg_branch = $tagman->{'args'}->{$tagname};
				$doctext->tagRaise($tagname); #CHAR_STYLES-tags should be the upper tags
				if ($tagdoc_on == 1)
					{
					push (@tags_for_tagdoc, $tagname); #Store the created tagname for later
					push (@stylegroups_for_tagdoc, $group); #Store the created stylegroup for later
					push (@styles_for_tagdoc, $style); #Store the created style for later
					}
				}
			#COMMANDS aufbauen
			elsif ($group =~ /^COMMANDS/o)
				{
				$tagname = $ref_docform->commands_changed($style, $index1);
				$arg_branch = $tagman->{'args'}->{$tagname};
				$doctext->delete("$index1 + 1 chars"); #delete the ' ' (space) that was the placeholder for the embeded widget in the saved text
				}
			#EMBOBJ aufbauen
			elsif ($group =~ /^EMBOBJ/o)
				{
				$tagname = $ref_docform->insert_embobj($style, $data , $index1, 0, 0);
				$arg_branch = $tagman->{'objects'}->{$tagname}->{'args'};
				$doctext->delete("$index1 + 1 chars"); #delete the ' ' (space) that was the placeholder for the embeded widget in the saved text
				}
   		}
		#Argumente aufbauen
		if (defined $tagname) #Wenn neues Tag erfolgreich erzeugt und Name zurckgeliefert wurde
			{
			my @list = split(/\|\|/o,$arglist); #The list is split by '||'
			my $number=shift(@list);
			for (my $i = 1; $i <= $number; $i++)
				{
				my $arg_value=shift(@list);
				$arg_value = '' if not defined $arg_value;
				$arg_value = Filesys::unmask_string($arg_value);
				$arg_value =~ s/%NL%/\n/g; #convert newlines from %NL%
				$arg_branch->{$i}->{'value'} = $arg_value;
				}
			}
		}
	}
if ($tagdoc_on == 1 and $#tags_for_tagdoc >=0)
	{
	while ($#tags_for_tagdoc >= 0)
		{
		my $tag = shift(@tags_for_tagdoc);
		my $stylegroup = shift(@stylegroups_for_tagdoc);
		my $style = shift(@styles_for_tagdoc);
		$tagman->insert_tag_doctor_objects($tag, $stylegroup, $style);
		}
	}

#Fortschrittsanzeige auf 80%
$ref_docform->_progress_update('Renumbering',80);
$ref_docform->refresh_outline($ref_docform); #Outline neu aufbauen (dabei wird auch durchnummeriert Abstze, Bilder,Tabellen etc.)
#Fortschrittsanzeige auf 99%
$ref_docform->_progress_update('Ready',99);
$ref_docform->_progress_finish;

EXIT:

$ref_docform->_progress_finish;
$ref_docform->top()->Unbusy();
}


=head2 build_array_from_file

@filearray = InOut::build_array_from_file($filename)

Reads all lines from a file and builds an array with it. 
This array is first cleared of unnecessary whitespaces and comments,
parsed for <<<INCLUDE_FILE::.....>>> commands, which cause the subroutine to call 
itself recusively and then the whole array checked for the correct opening and closing of
blocks. This parsed and extracted array is returned.

Parameters: 

(+) $filename: The filename&path to read into the array

=cut

sub build_array_from_file
{
my $filename = shift;
my @filearray; #Array fuer originalen Dateiinhalt
#Einlesen des Dateiinhaltes
open (IN,"<$filename")|| die ("File does not exist or is not readable -> \"$filename\"");
my $line;
while (<IN>)
	{
	$line = $_; 
	$line =~ s/^\s*//o; #Remove whitespaces in front
	$line =~ s/[^\\]#.*\n$/\n/o ; #Remove comments that are not masked with a backslash
	$line =~ s/^#.*\n$/\n/o ; #Remove commands at the beginning of lines
	$line =~ s/\s*\n$/\n/o; #Remove whitespaces at the end of the line
	if (not $line eq '') #If Line is not empty
		{
		push(@filearray,$line); #Zeile an Array anhaengen
		}
	}
close(IN);
#Here we should expand all possible INCLUDE-statements
Filesys::expand_include(\@filearray);
#Check if the blocks are set correctly
my $check=Filesys::check(\@filearray);
if ($check>-1) 
	{
	die ("Error in line $check in file \"$filename\" \n");
	}
return @filearray
}

=head2 export_to_backend

$ps_filename = InOut::export_to_backend($ref_docform , $edc_filename, $extension );

Exports the e:doc document in  the DocForm object into the backends format and
writes down the file on disk with the same filename than the e:doc-document but the
extension specified in the corresponding *.out file of the backend.

Parameters: 

(+) $ref_docform: Reference of the DocForm object

(+) $edc_filename: The filename&path of the e:doc document

(+) $extension : The extension the exported file should have

=cut

######################
### Backend Export ###
######################
sub export_to_backend
{
my $ref_docform = shift; # Reference to the calling DocForm-Object
my $edc_filename = shift; #Name der Edoc-Datei
my $extension = shift; #The extension the exported file should have
$ref_docform->top()->Busy();
#Name der Latex-datei aus Edoc-Filename berechnen
my $export_filename = $edc_filename;
###THIS MUST CHANGE (EXTENSION) #######
$export_filename =~ s/\.\w*$/\.$extension/g;
###THIS MUST CHANGE (EXTENSION) #######
#File-Handle anlegen
open (OUT, ">$export_filename") || die ("Fehler beim speichern von $export_filename");
#Latex-Dokument als reine Textinfo aufbauen
my @backend_document = Backend::export_to_backend($ref_docform) ;
#Textinfo in Datei schreiben
foreach my $line (@backend_document)
	{
	print OUT $line."\n";
	}
#Dateinamen retourliefern
close OUT;
print "File closed again ! \n"; #debug
$ref_docform->top()->Unbusy();
return $export_filename;
}

=head2 save_to_XML

$ok_flag = InOut::save_to_XML($ref_docform , $edc_filename );

Saves the document in XML format and returns an flag if the saving was ok.

NOT CODED YET !

Parameters: 

(+) $ref_docform: Reference of the DocForm object

(+) $edc_filename: The filename&path of the e:doc document

=cut


########################################
# Speichert das Dokument im XML-Format #
########################################
sub save_to_XML 
{
my $ref_docform = shift;
my $edc_filename = shift;
my $ok;

#CODE IS MISSING HERE !

return $ok;
} 

=head2 open_as_XML

$ok_flag = InOut::open_as_XML($ref_docform , $edc_filename );

Open the document in XML format and returns an flag if the opening was ok.

NOT CODED YET !

Parameters: 

(+) $ref_docform: Reference of the DocForm object

(+) $edc_filename: The filename&path of the e:doc document

=cut

####################################
# Liest das Dokument im XML-Format #
####################################
sub open_as_XML 
{
my $ref_docform = shift;
my $edc_filename = shift;
my $ok;

#CODE IS MISSING HERE !

return $ok;
} 

=head2 project_open

InOut::project_open($ref_prjform [, $file_name] );

This function reads a project and writes it into the given project form.

Parameters: 

(+) $ref_prjform: Reference of the PrjForm object

(+) $filename: The filename&path of the project to open, if not defined, a dialogbox will ask for it

=cut

####################
### Project open ###
####################
sub project_open 
{
my $ref_prjform = shift; # Reference to the calling PrjForm-Object
my $filename = shift; # Undef if called from Menu 'Project'->'Open'
#Display hourglass
$ref_prjform->app_window()->Busy();

#Flag for a newly created and empty DocForm object
my $empty_prjform = 0;
$empty_prjform = 1 if ($ref_prjform->filename() eq '* scratch *');

#initialise progress bar
$ref_prjform->_progress_init('Preparing for a another project');

#Save if there is already an open project
if ($empty_prjform == 0 and $ref_prjform->is_content_changed())
	{
	my $btn=$ref_prjform->top()->Dialog(-title=>'Project not saved yet !', -text=>"There are unsaved changes in the current document.\nDo you want to save them to disk ?", -buttons=>['Yes','No'], -bitmap=>'question')->Show();
	if ($btn eq 'Yes') {$ref_prjform->project_save($ref_prjform)};
	}

#Progressbar
$ref_prjform->_progress_update('Opening file',10);

if ((defined $filename) == 0)
	{
	#Ask for filename
	$filename = $ref_prjform->app_window()->getOpenFile(-filetypes=>[['e:doc projects', ['.edp', '.EDP', '.Edp']]]);
	if (((defined $filename)==0)||($filename eq '')) #If nothing was entered 
		{
		goto Exit;
		}
	}

if (($ref_prjform->filename() eq '') or (not defined $ref_prjform->filename()) ) {goto EXIT}; #Sorry for goto 

#Checking if the user gave a filename
$ref_prjform->filename($filename);

#Progress message...
$ref_prjform->_progress_update('Clearing old project',20);

#Empty some hashes
$ref_prjform->{'authors'} = [];
$ref_prjform->{'prj_parts'} = {};
$ref_prjform->{'files'} = {};

#Initialise the project arguments hash
$ref_prjform->_init_args('PROJECT_ARGS', undef );

#Initialise the file arguments hash
$ref_prjform->_init_args('CURRENT_FILE', undef );


#Progress message...
$ref_prjform->_progress_update('Reading file:'.$ref_prjform->filename(),40);

#Changing the current path
my $project_path = $ref_prjform->path();
chdir($project_path); #FIXME: EDOC_PATH HAS TO BE EXPANDED FIRST

#Read file
my @file_array; #Array fr originalen Dateiinhalt
#Open the file
open (IN,"<".$ref_prjform->filename())|| die ("Datei existiert  nicht \"".$ref_prjform->filename()."\"\n");
my $line;
while (<IN>)
	{
	my $line = $_;
	chop($line);
	push(@file_array,$line ); #Append line to the array
	}
close(IN);

my $ok = Filesys::check(\@file_array);
if ( not $ok ) {print "Incorrect file-format in edp file \n"}; 

#Read the META block
my %meta_info = Filesys::cut_data_hash(\@file_array,'META');

#Read AUTHORS block
my @authors_array = Filesys::cut_data_array(\@file_array, 'AUTHORS');
$ref_prjform->authors(\@authors_array);

#Read the CONFIG block
my %prj_config = Filesys::cut_data_hash(\@file_array,'CONFIG');

#Read the PROJECT_ARGUMENTS block
my %prj_args = Filesys::cut_data_hash(\@file_array,'PROJECT_ARGUMENTS');
for (my $i=1; $i<= $ref_prjform->prj_args_number(); $i++)
	{
	if (exists $prj_args{"arg_".$i."_value"} and defined $prj_args{"arg_".$i."_value"})
		{
		my $value = $prj_args{"arg_".$i."_value"};
		$ref_prjform->set_prj_arg_value($i, $value )
		}
	}

#Read PROJECT block
my %project_hash = Filesys::cut_data_hash(\@file_array, 'PROJECT');
foreach my $key (keys %project_hash)
	{
	$ref_prjform->{'project'}->{$key} = $project_hash{$key};
	}

#Read FILES block
my @file_info_array = Filesys::cut_data_array(\@file_array, 'FILEINFO');
my @file_names = Filesys::read_blocklist(\@file_info_array);

#Read FILE-Infos
foreach my $file_name (@file_names)
	{
	my %file_info = Filesys::read_data_hash(\@file_info_array, $file_name);
	#Read file argument entries first and delete them afterwards from the hash
	for (my $i=1; $i<= $ref_prjform->file_args_number(); $i++)
		{
		if (exists $file_info{"arg_".$i."_value"} and defined $file_info{"arg_".$i."_value"})
			{
			my $value = $file_info{"arg_".$i."_value"};
			$ref_prjform->file_arg_value($file_name, $i, $value );
			delete $file_info{"arg_".$i."_value"};
			}
		}
	#Read the rest of the file_info entries
	while ((my $key, my $value) = each %file_info)
		{
		$ref_prjform->{'files'}->{$file_name}->{$key} = $value;
		}
	}

#Read PARTS block
$ref_prjform->_progress_update('Reading parts ...',50);
#Read the parts block from the array and build up the hash with the project parts
Parts::read_parts_array(\@file_array, $ref_prjform->prj_parts_root()); 

#Update the content of the combos in the argument editors
uiPrjForm::update_arg_editors($ref_prjform, 'PROJECT');
uiPrjForm::update_arg_editors($ref_prjform, 'FILES');

#Update title
$ref_prjform->update_window_caption();

#Set project flag to unchanged
$ref_prjform->set_content_unchanged();

EXIT:
#Leave the function correctly
$ref_prjform->_progress_finish;
$ref_prjform->app_window()->Unbusy();

}

=head2 project_save

InOut::project_save($ref_prjform);

This function saves the projectform into its file.

NOTE: The file is created already after 'Project'->'New', so there is no need to ask for a filename.

Parameters: 

(+) $ref_prjform: Reference of the PrjForm object

=cut

####################
### Project save ###
####################
sub project_save 
{
my $ref_prjform = shift; # Reference to the calling PrjForm-Object

$ref_prjform->app_window()->Busy();
$ref_prjform->_progress_init('Storing project...');

$ref_prjform->_progress_update('Storing meta data',10);
#Create the array that holds the file
my @file_array = ();

#STORE METADATA
my %meta_hash;
$meta_hash{'fs_version'}=$main::internal->{'fs_version'};
$meta_hash{'prg_version'}=$main::internal->{'prg_version'};
Filesys::write_data_hash(\@file_array,'META',\%meta_hash,1);

$ref_prjform->_progress_update('Storing config',20);

#STORE CONFIG
my %config_hash;
#Nothing is done here yet
Filesys::write_data_hash(\@file_array,'CONFIG',\%config_hash,1);

#Storing the PROJECT_ARGUMENTS
my %prj_args = ();
for (my $i=1; $i<= $ref_prjform->prj_args_number(); $i++)
	{
	if (exists $ref_prjform->{'prj_args'}->{$i}->{'value'} and defined $ref_prjform->{'prj_args'}->{$i}->{'value'})
		{
		my $value = $ref_prjform->{'prj_args'}->{$i}->{'value'};
		$prj_args{"arg_".$i."_value"} = $value;
		}
	}
Filesys::write_data_hash(\@file_array, 'PROJECT_ARGUMENTS', \%prj_args, 1);

#STORE AUTHORS block
my $ref_authors_array = $ref_prjform->authors();
Filesys::write_data_array(\@file_array,'AUTHORS', $ref_authors_array, 1);

#STORE PARTS
$ref_prjform->_progress_update('Storing parts',30);
#Build up an array of all parts stored in the document and append it to the file using a 'PARTS' block
my @parts_array = Parts::build_parts_array($ref_prjform->prj_parts_root()); 
Filesys::write_data_array(\@file_array,'PARTS', \@parts_array, 1);


#STORE FILE_INFOS
$ref_prjform->_progress_update('Storing file info',40);
my @file_info_array;
foreach my $file (keys %{$ref_prjform->{'files'}})
	{
	#Read the whole {'files'}-subbranch into a new hash
	my %file_info = %{$ref_prjform->{'files'}->{$file}};
	#Build up new entries for storing argument values
	for (my $i=1; $i<= $ref_prjform->file_args_number(); $i++)
		{
		if (defined $ref_prjform->file_arg_value($file, $i) )
			{
			my $value = $ref_prjform->file_arg_value($file, $i);
			$file_info{"arg_".$i."_value"} = $value;
			}
		}
	#Delete the {'args'} subbranch BEFORE storing onto disk
	delete $file_info{'args'};
	#Write it down
	Filesys::write_data_hash(\@file_info_array, $file, \%file_info, 1);
	}
Filesys::write_data_array(\@file_array,'FILEINFO', \@file_info_array, 1);

$ref_prjform->_progress_update('Writing to disk',80);
#Datei speichern
my $HANDLE = IO::File->new(">".$ref_prjform->filename()) || die ("Error when writing file ".$ref_prjform->filename()."\n");
foreach my $line (@file_array) {print $HANDLE $line."\n"};
$HANDLE->close(); #Datei schlieen

#Fortschrittsanzeige auf 1%
$ref_prjform->_progress_update('Done.',99);

#nderungsflag setzen
$ref_prjform->set_content_unchanged();

EXIT:
$ref_prjform->_progress_finish;
$ref_prjform->app_window()->Unbusy();

}



=head2 build_filelist_by_extension

@filelist = InOut::build_filelist_by_extension($dir, $ext);

Gets all files with the giveb extension in the directory and returns the filenames as an array.

Parameters: 

(+) $dir: The directory path

(+) $ext: The extension (excl. the point, f. ex. 'txt' finds all '*.txt'-files)

Returned value: the filelist

=cut

sub build_filelist_by_extension
{
my $dir = shift; #the directory
my $ext = shift; #the extension
my @filelist;
opendir(DIR, $dir);
@filelist = grep {/\.$ext$/} readdir(DIR);
closedir(DIR);
return @filelist;
}
1; #So mu package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

(+) A method called 'call_external_program(...)' that represents an call for external programs to be used in macros
you can define in 'edoc.cfg'.

(+) The XML stuff is missing.
=cut
