#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package Language;
use strict;

=head1 NAME:

Package: Language

=head1 SYNOPSIS:

use lib::Language;

=head1 DIRECTORY/FILE:

The file Language.pm is located in the subdirectory './lib'

=head1 DESCRIPTION:

This package contains all function that are needed for tha style definition language and the output definition language in the 'edoc.int' and the '*.out'-files.

=head1 STATUS:

Coding: 80%

Documentation: up-do-date

=head1 FUNCTIONS:

=head2 read_stylegroup

Language::read_stylegroup($ref_hash, $stylegroup_name, $ref_styles_array)

Because all style definitions are coded in the same way in the configuration files, this function retriecves
the information form the $file_array and reads the given ($stylegroup_name) and creates the corresponding branches in
the anonymous configurationa hash.

Parameters: 

(+) $ref_hash: A reference to the anonymous hash

(+) $stylegroup_name: The name of the stylegroup to be read (PAR_STYLES, ENV_STYLES, CHAR_STYLES, DOC_STYLES, COMMANDS, EMBOBJ)

(+) $ref_styles_array: A reference to the array that holds the STYLES block of the configuration file

=cut

##############################################################
# Read a whole stylegroup block and process special commands #
##############################################################
sub read_stylegroup
{
#This function was implemented to read all STYLE-DEFINITIONS (they have all the same structure vor ENV_STYLES, CHAR_STYLES, PAR_STYLES, COMMANDS, TABLES, IMAGES)
my $ref_hash = shift; #Reference to the $main::internal or $backend-Hash
my $stylegroup_name = shift;
my $ref_styles_array = shift; #Zeiger auf Array welches STYLES block enthaelt
#Auf den darin liegenden Stilgruppen-Block einschrnken
my @stylegroup_array=Filesys::cut_data_array($ref_styles_array,$stylegroup_name,0); 
#Liste der darin enthaltenen Blcke = Formate = Stile einlesen
my @style_names=Filesys::read_blocklist(\@stylegroup_array); 
#Alle Formate einlesen
foreach my $name (@style_names)
	{
	#Build one hash for every defined style in this stylegroup
	my %actual_style=Filesys::cut_data_hash(\@stylegroup_array,$name); 
	
	#Work with inheritance:
	#Set all properties that are defined in the SUPER-style (The one that is inherited from)
	if ( exists $actual_style{'_INHERIT'} and defined $actual_style{'_INHERIT'} )
		{
		my $inherited_from = $actual_style{'_INHERIT'};
		if ( exists $ref_hash->{$stylegroup_name}->{$inherited_from} )
			{
			foreach  my $key (keys %{$ref_hash->{$stylegroup_name}->{$inherited_from}})
				{
				if ( not exists  $actual_style{$key} ) 
					{
					$actual_style{$key} = $ref_hash->{$stylegroup_name}->{$inherited_from}->{$key};
					}
				}
			}
		else
			{
			die ("ERROR: stylegroup:$stylegroup_name  style:$name inherits from $inherited_from, but this style is not defined (before) the definition of $name.\n");  
			}
		}

	#Iterate over the hash with the configuration settings of the actual style
	foreach  my $key (keys %actual_style)
		{
		#Process all special e:doc commands
		if (defined $key) #If the style has at least one setting
			{
			#Fill hash with the properties that are defined in the style definition
			$ref_hash->{$stylegroup_name}->{$name}->{$key}=$actual_style{$key};
			}
		}

	# If there are arguments defined -> build up the {'args'}-branch
	if (exists $actual_style{'arg_number'})
		{
		my $arg_number=$actual_style{'arg_number'};
		$ref_hash->{$stylegroup_name}->{$name}->{'arg_number'}=$arg_number;
		for (my $i=1; $i<=$arg_number; $i++)
			{
			if (exists $actual_style{'arg_'.$i.'_name'}) 
				{$ref_hash->{$stylegroup_name}->{$name}->{'args'}->{$i}->{'name'}=$actual_style{'arg_'.$i.'_name'}}
			else
				{$ref_hash->{$stylegroup_name}->{$name}->{'args'}->{$i}->{'name'}="Property".$i};
			if (exists $actual_style{'arg_'.$i.'_default'})
				{$ref_hash->{$stylegroup_name}->{$name}->{'args'}->{$i}->{'default'}=$actual_style{'arg_'.$i.'_default'}}
			else
				{$ref_hash->{$stylegroup_name}->{$name}->{'args'}->{$i}->{'default'}=''};
			if (exists $actual_style{'arg_'.$i.'_type'})
				{$ref_hash->{$stylegroup_name}->{$name}->{'args'}->{$i}->{'type'}=$actual_style{'arg_'.$i.'_type'}}
			else
				{$ref_hash->{$stylegroup_name}->{$name}->{'args'}->{$i}->{'type'}='TEXT'};
			if (exists $actual_style{'arg_'.$i.'_settings'})
				{$ref_hash->{$stylegroup_name}->{$name}->{'args'}->{$i}->{'settings'}=$actual_style{'arg_'.$i.'_settings'}}
			else
				{$ref_hash->{$stylegroup_name}->{$name}->{'args'}->{$i}->{'settings'}=''};
			if (exists $actual_style{'arg_'.$i.'_hint'})
				{$ref_hash->{$stylegroup_name}->{$name}->{'args'}->{$i}->{'hint'}=$actual_style{'arg_'.$i.'_hint'}}
			else
				{$ref_hash->{$stylegroup_name}->{$name}->{'args'}->{$i}->{'hint'}=''};
			}
		}
	else
		{
		#If there was no argument info (wether in the inherited nor in the actual style definition) set arg_number to 0
		if ((exists $ref_hash->{$stylegroup_name}->{$name}->{'arg_number'})==0)
			{
			$ref_hash->{$stylegroup_name}->{$name}->{'arg_number'}=0;
			}
		}
	}
}

=head2 process_property_value_commands

Language::process_property_value_commands($ref_hash)

This function replaces all %PROPERTY_VALUE%-commands with their values. It is called after all
stylegroups have been written and all inheritances are done.

Parameters: 

(+) $ref_hash: A reference to the anonymous hash

=cut

#####################################################
# Process all %PROPERTY_VALUE%-commands in the hash #
#####################################################
sub process_property_value_commands
{

my $ref_hash = shift; #Reference to the $main::internal or $backend-Hash

my @stylegroups = ('DOC_STYLES','PAR_STYLES','CHAR_STYLES','ENV_STYLES','COMMANDS','EMBOBJ');
foreach my $stylegroup ( @stylegroups )
	{
	foreach  my $style (keys %{$ref_hash->{$stylegroup}})
		{
		my $ref_stylehash = $ref_hash->{$stylegroup}->{$style};
		#If there is and %PROPERTY_VALUE%-statement -> get the value of the referenced property there
		while ((my $key, my $value) = each  %{$ref_stylehash})
			{
			if ($value =~ /\%PROPERTY_VALUE::(.+)\%/)
				{
				my $option = $1;
				if (defined $ref_stylehash->{$option})
					{
					$ref_stylehash->{$key}= $ref_stylehash->{$option};
					}
				}
			}
		}
	}
}


=head2 process_edoc_commands

$return_value = Language::process_edoc_commands($string);

This function processes all special commands edoc knows for his configuration files.

This are commands for 'Newline', 'Tabulator', '%INLINE_MACRO::....%' etc.

Parameters: 

(+) $string: The string that contains (maybe) the commands.

=cut

##############################################################################
# Process special commands in the configuration files 'edoc.int' and '*.out' #
##############################################################################
sub process_edoc_commands
{
my $string = shift;
my $config = $main::config;

#For now we are only processing special symbols for newline, tab, carriage-return  here
$string =~ s/%NL%/\n/g ; #Newline
$string =~ s/%TAB%/\t/g ; #Tab
$string =~ s/%CR%/\r/g ; #Carriage Return
$string =~ s/%NOTHING%//g ; #Nothing
if ($string =~ /%TODAY%/ )
	{
	my ($day, $month, $year) = (localtime)[3,4,5];
	$year = 1900+$year;
	$month = $month+1;
	$string =~ s/%TODAY%/$day\.$month\.$year/g
	}
while ($string =~ /%INLINE_MACRO::(.+?)%/)
	{
	my $inline_name = $1;
	my $replace_value;
	if (not exists $config->{'INLINE_MACROS'}->{$inline_name} or  not defined $config->{'INLINE_MACROS'}->{$inline_name})
		{
		print "ERROR: INLINE_MACRO \"$inline_name\" is not defined in file \"edoc.str\"\n";
		$replace_value = '';
		}
	else
		{
		print "Replacing INLINE_MACRO \"$inline_name\"\n"; #debug
		$replace_value = $config->{'INLINE_MACROS'}->{$inline_name};
		}
	$string =~ s/%INLINE_MACRO::$inline_name%/$replace_value/g ;
	}

return $string;
}


=head2 process_edoc_commands_export

$return_value = Language::process_edoc_commands_export($string, $ref_docform, $tagname);

This function processes all special commands edoc knows for his configuration files
and respects actual data of the current active document and it's current backend

This are commands for %IF::.....% etc.

Parameters: 

(+) $string: The string that contains (maybe) the commands.

(+) $ref_docform: A reference to the calling DocForm object

(+) $tagname. The tag that gets processed for export

=cut

############################################################################
# Process special commands an data of the current active documents backend #
############################################################################
sub process_edoc_commands_export
{
my $string = shift;
my $docform = shift;
my $tagname = shift;
my $doctext = $docform->doctext();
my $backend = $docform->backend();
my $config = $main::config;

$string = Language::process_edoc_commands($string);

#Processing MACRO-statements in export definition 
while ($string =~ /%MACRO::(.+?)%/)
	{
	my $macro_name = $1;
	my $result = Macro::execute($macro_name, $docform, undef);
	if ($result)
		{
		$string =~ s/%MACRO::(.+?)%/$result/ ; 
		}
	}

# Parsing for IF-construct 
# Syntax: %IF::bool-term :: true-part :: False-part ::ENDIF%
#Example: %IF::%ARG_2_VALUE% eq 'Page-Reference'::\pageref{%ARG_1_VALUE%}::\ref{%ARG_1_VALUE%}::ENDIF%
while ($string =~ /%IF::(.+?)::(.*?)::(.*?)::ENDIF%/o ) 
	{
	my $orig_clause = $1;
	my $orig_true_part = $2;
	my $orig_false_part = $3;
	my $clause = $orig_clause;
	my $true_part = $orig_true_part;
	my $false_part = $orig_false_part;
	$clause =~ s/^\s*//g;
	$clause =~ s/\s*$//g;
	$true_part =~ s/^\s*//g;
	$true_part =~ s/\s*$//g;
	$false_part =~ s/^\s*//g;
	$false_part =~ s/\s*$//g;
	my $result = eval ($clause);
	my $regex = quotemeta("%IF::".$orig_clause."::".$orig_true_part."::".$orig_false_part."::ENDIF%");
	if ($result)
		{
		$string =~ s/$regex/$true_part/ ; 
		}
	else
		{
		$string =~ s/$regex/$false_part/ ; 
		}
	}
while ($string =~ /%NUMBERING%/o)
	{
	if ($tagname =~ /^<(.+?)><(.+?)>.*/)
		{
		my $stylegroup = $1;
		my $style = $2;
		if (defined $main::internal->{$stylegroup}->{$style}->{'connect_to_doc_numlist'} )
			{
			my $numlist_name = $main::internal->{$stylegroup}->{$style}->{'connect_to_doc_numlist'};
			if (defined $docform->tagman()->{'doc_numlists'}->{$numlist_name})
				{
				my $numlist = $docform->tagman()->{'doc_numlists'}->{$numlist_name};
				my @numlabels = @{$numlist->list()};
				foreach my $numlabel (@numlabels)
					{
					if ($numlabel->associated_tag() eq $tagname)
						{
						my $text = $numlabel->text();
						$string =~ s/%NUMBERING%/$text/geo ;
						last;
						}
					}
				}
			}
		}
	}
# Replace with the content (text) of the given text.
#Example: "%TAG_CONTENT::PAR_STYLES::Title::1%" will be replaced with
#the content of the first tag of the style PAR_STYLES::Title
while ($string =~ /%TAG_CONTENT::(.+?)::(.+?)::(\d+?)%/o)
	{
	my $stylegroup = $1; 
	my $style = $2;
	my $order_index = $3;
	my $replacer='';
	my $mask="<$stylegroup><$style>";
	my @tags = $doctext->tagNames();
	my %tag_places;
	my $result_tag = '';
	foreach my $tag (@tags)
		{
		if ($tag =~ /^$mask/o)
			{
			my @range = $doctext->tagRanges ($tag);
			my $start = shift(@range);
			my $line;
			my $col;
			if ($start =~ /(\d+)\.(\d+)$/o)
				{
				$line = $1;
				$col = $2;
				}
			$tag_places{$line} = $tag;
			}
		}
	my @sorted = sort(keys %tag_places);
	my $counter = 0;
	foreach my $key (@sorted)
		{
		my $tag_name=$tag_places{$key};
		$counter++;
		if ($counter==$order_index)
			{
			my @range = $doctext->tagRanges ($tag_name);
			my $start_index = shift(@range);
			my $end_index = shift(@range);
			$replacer = $doctext->get($start_index, $end_index);
			}
		}
	$string =~ s/%TAG_CONTENT::(.+?)::(.+?)::(\d+?)%/$replacer/ ;
	}

return $string;
}



1; #So mu package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
