#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package Macro;
use strict;

=head1 NAME:

Package: Macro

=head1 SYNOPSIS:

use lib::Macro;

=head1 DIRECTORY/FILE:

The file Macro.pm is located in the subdirectory './lib'

=head1 DESCRIPTION:

This package contains all function that are dealing with macros.

=head1 STATUS:

Coding: "open end"-package

Documentation: up-do-date

=head1 FUNCTIONS:

=head2 execute

$return_value = Macro::execute($macro_name, $ref_form, $ref_args, $oneliner_code);

This function executes the given macro and returns a scalar value. You will have to put a return-statement
into the macros code that returns the desired value. If a macro is used inside of other macros and has to
return an array or a hash, you will have to return these by reference and derefence them in the
calling macro by using '@{$return_value}' or '%{$scalar_value}'.

Parameters: 

(+) $macro_name: The name of the macro to execute

(+) $ref_form: A reference to the calling orm object 

(+) $ref_args: A reference to an hash holding the arguments that are passed to the macro 

(+) $oneliner_code: If maco was a onliner (not stored in an extra block, syntax "%ONELINER::...%", only the
    code between "{" and "}" get's executed. Is undef or '' if a normal macro gets called by name.

=cut

####################
# Macro ausfuehren #
####################
sub execute
{
my $macro_name = shift;
my $ref_form = shift;
my $ref_args = shift;
my $oneliner_code = shift;

my $ref_internal = $main::internal;
my $ref_config = $main::config;

my @macro_array;
my $macro_str;
#If the macro exists
if (defined $oneliner_code and not ($oneliner_code eq ''))
	{
	#Get the macros content
	push (@macro_array, $oneliner_code );
	#Get the header in front of it
	unshift (@macro_array, @{$ref_config->{'MACRO_HEADER'}});
	#Put the array into a string (for eval)
	foreach my $line (@macro_array)
	 	{
		$macro_str .= $line;
		}
	}
elsif ((exists $ref_config->{'MACROS'}->{$macro_name})&&(defined $ref_config->{'MACROS'}->{$macro_name}))
	{
	#Get the macros content
	@macro_array=@{$ref_config->{'MACROS'}->{$macro_name}};
	#Get the header in front of it
	unshift (@macro_array, @{$ref_config->{'MACRO_HEADER'}});
	#Put the array into a string (for eval)
	foreach my $line (@macro_array)
	 	{
		$macro_str .= $line;
		}
	}
#If macro name was wrong ->>errormsg
else
	{
	print "ERROR: Macro ".$macro_name." not found in \"edoc.mac\"\n";
	}
#Set  global references to the calling form and the argument hash
$main::macro_form = $ref_form;
$main::macro_args = $ref_args;

#And here its eval(uat)ed
my $return_value = eval $macro_str;

#Remove the global references
$main::macro_form = undef;
$main::macro_args = undef;

return $return_value;
}



1; #So mu package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
