#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package Parts;
use strict;

=head1 NAME:

Package: Parts

=head1 SYNOPSIS:

use lib::Parts;

=head1 DIRECTORY/FILE:

The file Parts.pm is located in the subdirectory './lib'

=head1 DESCRIPTION:

This package contains all function that are needed for handeling the 'parts'.

=head1 STATUS:

Coding: 50%

Documentation: up-do-date

=head1 FUNCTIONS:

=head2 load_global_parts()

my $ok = Parts::load_global_parts(); 

Reads the global parts from the file 'edoc.pts'.

-> This function might move into the package 'InOut.pm' because it reads from the disk !

=cut

#################################
### Read the file 'edoc.pts'  ###
#################################
sub load_global_parts
{
my @parts_array;

print "Loading the global parts ";

if (defined Utils::process_filename('config/edoc.pts'))
	{
	#Open the file
	open (IN,"<".$main::edoc_path."/config/edoc.pts" )|| die ("File doesn't exist \"".Utils::process_filename('config/edoc.pts')."\"\n");
	my $line;
	while (<IN>)
		{
		#reading it line by line
		my $line = $_;
		#Remove \n at the end
		chop($line);
		push(@parts_array, $line ); #Append line to the array
		print ".";
		}
	close(IN);
	}
else
	{
	die "ERROR: File 'edoc.pts' is missing in application directory !\n";
	}

#Read the parts block from the array and build up the hash with the global parts
Parts::read_parts_array(\@parts_array, $main::global_parts); 

print "Done.\n";

return 1; #return 1 if everything went o.k., no error check is done here yet
}

=head2 save_global_parts()

my $ok = Parts::save_global_parts(); 

Saves the global parts to the file 'edoc.pts'.

-> This function might move into the package 'InOut.pm' because it writes to the disk !

=cut

##################################
### Write the file 'edoc.pts'  ###
##################################
sub save_global_parts
{

if ($main::global_parts_changed == 1)
	{
	print "Saving the global parts ";
	
	#Building up the array
	my @parts_array = ();
	my @an_array = Parts::build_parts_array($main::global_parts); 
	Filesys::write_data_array(\@parts_array,'PARTS', \@an_array, 1);
	
	#Saving file
	my $HANDLE = IO::File->new(">".$main::edoc_path."/config/edoc.pts") || die ("Error while writing file /config/edoc.pts\n");
	foreach my $line (@parts_array) {print $HANDLE $line."\n"; print "." };
	$HANDLE->close(); #Close file
	
	print "Done.\n";
	}

return 1; #return 1 if everything went o.k., no error check is done here yet
}


=head2 read_parts_array()

Parts::read_parts_array($ref_array, $ref_parts_hash_root); 

Reads the array-block called 'PARTS' in the referenced array and builds ub the hash of arrays in the $ref_parts_hash_root.

Parameters:

(+) $ref_array: A reference to an array in the e:doc filesys that has a 'PARTS' block.

(+) $ref_parts_hash_root : A reference to a hash that will hold all parts there (as a hash of arrays)

=cut

#########################
### Read parts array  ###
#########################
sub read_parts_array
{
my $ref_array = shift;
my $ref_parts_hash_root = shift;

#Delete the old parts there
foreach my $old_part_name ( keys %{$ref_parts_hash_root} )
	{
	delete ($ref_parts_hash_root->{$old_part_name});
	}

#Get the new ones from the file_array
my @parts_array = Filesys::cut_data_array($ref_array,'PARTS');

#Read the parts names
my @parts_list = Filesys::read_blocklist(\@parts_array);

#Iterate through them
foreach my $part_name (@parts_list)
	{
	#Get the content
	my @content = Filesys::cut_data_array(\@parts_array,$part_name);
	#Store it in the DocForm attribute
	$ref_parts_hash_root->{$part_name} = \@content;
	}
}


=head2 build_parts_array()

my @array = Parts::build_parts_array($ref_parts_hash_root); 

Builds up an array including all parts in an array.

This array will be usually saved as an array-block called 'PARTS'.

Parameters:

(+) $ref_parts_hash_root : A reference to a hash that holds all parts there (as a hash of arrays)

=cut

#############################
### Build up parts array  ###
#############################
sub build_parts_array
{
my $ref_parts_hash_root = shift;

my @return_array = ();

foreach my $part_name (sort keys %{$ref_parts_hash_root})
	{
	Filesys::write_data_array(\@return_array, $part_name, $ref_parts_hash_root->{$part_name}, 1);
	}

return @return_array;
}


1; #So mu package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
