#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package TkPatch;

use strict;

=head1 NAME:

Package: TkPatch

=head1 SYNOPSIS:

use lib::TkPatch;

=head1 DIRECTORY/FILE:

The file TkPatch.pm is located in the subdirectory './lib'

=head1 DESCRIPTION:

This package contains all function that extend some functionality in the
original Perl/Tk distribution files. They do not create new derived or
congregate widgets, they just override single functions in the exiting 
widgets.

These functions are not called form somewhere inside of e:doc directly,
they override the ones that already exist.

They are placed in this seperate file because -if this functionality gets
added to the original Perl/Tk distribution- they might easily be deleted 
from here.

=head1 STATUS:

Coding: this is an open end package, so no status percentage can be mentioned here

Documentation: up-do-date

=head1 FUNCTIONS:

=head2 Tk::Text::UpDownLine

Until the original Tk::Text widget is able to scroll line-by-line when
the content is wrapped, I add this functionality here.

=cut

sub Tk::Text::UpDownLine
{
 my ($w,$n) = @_;
 my $i = $w->index('insert');
 my ($line,$char) = split(/\./,$i);
 my $string = $w->get($line.'.0', $i);

 ### PATCH STARTS HERE ###
 $string = '' if not defined $string;
 ### PATCH ENDS HERE ###
 
 $string = Tk::Text::expand($string);
 
 $char=length($string);
 $line += $n;

 $string = $w->get($line.'.0', $line.'.0 lineend');
 ### PATCH STARTS HERE ###
 $string = '' if not defined $string;
 ### PATCH ENDS HERE ###

 $string = Tk::Text::expand($string);

 $string = substr($string, 0, $char);

 $string = Tk::Text::unexpand($string);
 $char = length($string);

### PATCH STARTS HERE ###
my $new; #T he new index after movement
my $line_offset = $w->cget('-spacing2'); # storing the space beween the lines
my $coord; # The calculated new coordinates

# The new line index is already calculated, ensure that it is visible, because we are working with screen coordinates
$w->see("$line.0") ; 

# Get the boundig box of the actual character and the actual line  
(my $bx, my $by, my $bw, my $bh) = $w->bbox($i);
(my $lx, my $ly, my $lw, my $lh) = $w->dlineinfo($i);

#Now check where we are in the wrapped line (if it is no wrapped line, don't worry, the first clause will be true
#if direction is up
if ($n == -1)
	{
	#Test if its the first line in a wrapped line block 
	(my $fx, my $fy, my $fw, my $fh) = $w->dlineinfo(($line-$n).".0");
	if ($fy == $ly)
		{
 		# Get the info from the previous index-line boundingbox
		(my $nx, my $ny, my $nw, my $nh) = $w->dlineinfo("$line.end");
		$coord = '@'.$bx.','.($ny+$nh-1);
		}
	#somewhere else in the block
	else
		{
		# just move up into the next screen-line
		$coord = '@'.$bx.','.($ly-$line_offset-1);
		}
	}
else #direction is down
	{
	#Test if its the last line in wrapped line block 
	(my $fx, my $fy, my $fw, my $fh) = $w->dlineinfo(($line-$n).".end");
	if ($fy == $ly)
		{
		# move down as far as the -spacing options tell us that we have to go down
		$coord = '@'.$bx.','.($ly+3+$lh+$w->cget('-spacing1')+$w->cget('-spacing3'));
		}
	else #somewhere else in the block
		{
		# just move down into the next screen-line
		$coord = '@'.$bx.','.($ly+$lh+$line_offset+1);
		}
	}
# Calcualte a new index out of the coordinates
$new = $w->index($coord); 

# Get the bounding box of the character we try to move in
(my $nx, my $ny, my $nw, my $nh) = $w->bbox($new);
 
# If we came in right of the middle of the character
if ($bx > ($nx+($nw/2)))
	{
	# move the cursor one character to the right
	$new = $w->index("$new + 1 chars"); 
	}

# Do not scroll past the ends of the document
if ($w->compare($new,'==','end') || $w->compare($new,'<','1.0'))
	{
	$new = $i
	}

### PATCH ENDS HERE ###
 $Tk::prevPos = $new;
 $Tk::char = $char;
 return $new;
}

1; #So mu package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
