#This source file belongs to e:doc
#Copyright (C) 1999  Thomas M. Schmickl
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License.
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package Utils;
use strict;

=head1 NAME:

Package: Utils

=head1 SYNOPSIS:

use lib::Utils;

=head1 DIRECTORY/FILE:

The file Utils.pm is located in the subdirectory './lib'

=head1 DESCRIPTION:

This package contains all function that did not fit into one of the other lib-packages.

=head1 STATUS:

Coding: this is an open-end package, so no status percentage can be mentioned here

Documentation: up-do-date

=head1 FUNCTIONS:

=head2 get_index_info

$line_number = Utils::get_index_info($index);

or

($line_number, $col_number) = Utils::get_index_info($index);

Extracts the linenumber (and the column number) from the given index and returns it

=cut

########################################
#Get the pieced of a text widget index #
########################################
sub get_index_info
{
my $index = shift;
my $line;
my $col;
if ($index =~ /^(\d+)\.(\d+)$/o )
	{
	$line =  $1;
	$col = $2;
	}
if (wantarray)
	{
	return ($line, $col);
	}
else
	{
	return $line;
	}
}

=head2 test_hash_value

$reult = Utils::test_hash_value($ref_hashbranch, $entryname);

Tests if the hash-value exists, is defined and has a value

Returns 0 if the value doesn't exist, is not defined or ''.

Returns the value if there is a value.

Parameters:

(+) $ref_hashbranch: The branch in the hash that contains the entry

(+) $entryname: The name of the entry there

Example:

If you wan to test $hash->{'xxx'}->{'yyy'}, you have to call:

my $result = Utils::test_hash_value($hash->{'xxx'}, 'yyy');

=cut

sub test_hash_value
{
my $ref_hashbranch;
my $entryname;

if (exists $ref_hashbranch->{$entryname} and defined $ref_hashbranch->{$entryname} and (not($ref_hashbranch->{$entryname} eq '') ) )
	{
	return $ref_hashbranch->{$entryname};
	}
else
	{
	return 0;
	}
}


=head2 build_display_font

$display_font = Utils::build_display_font($base_font, $lower_font, $lower_mask, $higher_font, $higher_mask);

One BIG problem is that in the Perl/Tk text widget one tag has one font-definition and if tags overlap each other only the
uppermost tags font is displayed.

So I have to develop amechanism that collects all informations from all tags that overlap each other, calculate the
appropriate font definition and add a special display-only uppermost tag with the combined font definition.

To decide what part of a styles font definition is important I invented the font_mask property. Look at the following example:

The base font of the textwidget is f. ex.: 

font = "-adobe-times-*-*-*--*-160-*-*-*-*-*-*"

font_mask = "-*-*-*-*-*--*-*-*-*-*-*-*-*"

which means that the base font is 'Adobe Times 16pt' and none of the definitions parts is very important

The font definition for the style 'Bold' is:

font = "-adobe-times-bold-r-normal-*-180-*-*-*-*-*-*-*"

font_mask "-*-*-bold-*-*-*-*-*-*-*-*-*-*-*"

which means that the 'Bold' style applied to an area without any existing tags (formatting) is 'Adobe Times 18pt bold',
but only the '-bold-' attribute is _very_important_ for the style.

The font definition of the  style 'Italic' is:

font = "-adobe-times-medium-o-normal-*-180-*-*-**p-*-*-*"

font_mask "-*-*-*-o-*-*-*-*-*-*-*-*-*-*"

which means that only the '-o-' attributes are _very_important_ for this style.

So if both styles overlap each other, there should be applied another tag with the font definition a follows:

font = "-adobe-times-bold-o-normal-*-180-*-*-*-*-*-*-*" which is a combination of the 3 definitions in respect of the 'font_mask' properties.

This function will return the new font definition of the display-font.

NOTE: THIS FUNCTION IS NOT CODED YET !


Parameters: 

(+) $base_font: The base font of the text widget

(+) $lower_font: The lower fonts definition

(+) $lower_mask: The lower fonts mask

(+) $higher_font: The lower fonts definition

(+) $higher_mask: The lower fonts mask

=cut

sub build_display_font 
{
my ($base_font, $lower_font, $lower_mask, $higher_font, $higher_mask) = @_;
my $display_font;
#CODE MISSING HERE

return $display_font
}


=head2 get_tagnames_from_line

@names = Utils::get_tagnames_from_line($ref_text_widget, $line);

Builds a list of tagnames from that line

It returns the value of the given $optionin the existing data-stringlist ($data).

Utils::get_embobj_data('[[[rows=2||cols=4]]]','rows') would return the value 2.

Parameters: 

(+) $ref_text_widget: A reference to the text widget

(+) $line: The linenumber

=cut

###############################################
# Gets an array of tagnames from a given line #
###############################################
sub get_tagnames_from_line
{
my $text_widget = shift;
my $line = shift;
my @array;
my %tags_unique;
my $startcol = 0;
my $endcol=$text_widget->index("$line.end"); 
# Aktuelle Zeile Zeichenweise durchgehen und ENV_STYLES suchen
for (my $c=$startcol; $c<=$endcol; $c++)
	{
	# Alle Tags an dieser Position auslesen
	my @tags = $text_widget->tagNames("$line.$c");
	# Durch alle gefundenen Tags gehen
	foreach my $tag (@tags)
		{
		$tags_unique{$tag}=1;
		}
	}

# Gefundene <CHAR_STYLES>-Tags loeschen
foreach my $key (keys %tags_unique)
	{
	push(@array,$key);
	}
return @array;
}

=head2 textwidget_sel_exists

Utils::textwidget_sel_exists($ref_text_widget);

Tests for an existing selection in the text_widget and return 1 if the sel exists, 
otherwise the function returns 0.

Parameters: 

(+) $ref_text_widget: A reference to the text widget

=cut

##################################
# Test for an existing selection #
##################################
sub textwidget_sel_exists
{
my $text_widget = shift;

#Get the ranges of the 'sel'-tag
my @ranges = $text_widget->tagRanges('sel');
#If the 'sel' tag exists and has at least one range
if (@ranges)
	{
	#The selection exists
	return 1;
	}
else
	{
	#No selection exists
	return 0;
	}
}


=head2 textwidget_select

Utils::textwidget_select($ref_text_widget, $index1, $index2);

Sets the selection in the text_widget

Parameters: 

(+) $ref_text_widget: A reference to the text widget

(+) $index1, $index2 : The range for the selection

=cut

#####################################
# Selects a range in the textwidget #
#####################################
sub textwidget_select
{
my $text_widget = shift;
my $index1 = shift;
my $index2 = shift;

#Unselect first
Utils::textwidget_unselect($text_widget);

#Set the selection
$text_widget->tagAdd('sel', $index1, $index2);

}


=head2 textwidget_unselect

Utils::textwidget_unselect($ref_text_widget);

Removes all ranges of the 'sel'-tag in the textwidget.

Parameters: 

(+) $ref_text_widget: A reference to the text widget


=cut

#########################################################
# Removes all ranges of the 'sel'-tag in the textwidget #
#########################################################
sub textwidget_unselect
{
my $text_widget = shift;

#If there is a text widget
if (defined $text_widget)
	{
	#get the ranges of the selection
	my @ranges= $text_widget->tagRanges('sel');
	#if there are ranges
	while (@ranges)
		{
		#get the start index
		my $index1 = shift(@ranges);
		#get the end index
		my $index2 = shift(@ranges);
		#remove the 'sel'-tag
		$text_widget->tagRemove('sel', $index1, $index2);
		}
	}
}

=head2 create_balloonhelp

$ref_baloon = Utils::create_balloonhelp($ref_widget, $text, $state, $ref_statusbar);

Creates a balloonhelp and associates it with the widget

The functions returns a reference to the created balloon widget.

Parameters: 

(+) $ref_widget: A reference to the widget

(+) $text: The text do be displayed in the balloon

(+) $state: The state of the balloon ('balloon', 'status', 'both', 'none'). Default is 'balloon'

(+) $ref_statusbar: A reference for the statusbar(Label) that should display the balloonhelp's text, if state is 'status' or 'both', else set it to undef

=cut

######################################
# Creates a balloonhelp for a widget #
######################################
sub create_balloonhelp
{
my $widget = shift;
my $text = shift;
my $state = shift;
my $ref_statusbar = shift;
#Check if all parameters are right
if (defined $widget)
	{
	$state = 'balloon' if not defined $state;
	if (not defined $ref_statusbar)
		{
		$state = 'balloon' if ($state eq 'status' or $state eq 'both');
		}
	#Create ballon
	my $balloon;
	if ($state eq 'balloon' or $state eq 'none')
		{
		$balloon = $widget->Balloon(-state=>$state);
		}
	else
		{
		$balloon = $widget->Balloon(-state=>$state, -statusbar=>$ref_statusbar);
		}
	$balloon->attach($widget, -msg=>$text);
	return $balloon;
	}
else
	{
	return undef;
	}
}

=head2 set_browseentry_color

Utils::set_browseentry_color($ref_browseentry, $color);

Sets the given color in a BrowseEntry widget.

Parameters: 

(+) $ref_browseentry: A reference to the BrowseEntry widget.

(+) $color: The color to be set.

=cut

#######################################
# Set a color in a BrowseEntry widget #
#######################################
sub set_browseentry_color
{
my $ref_browseentry = shift;
my $color = shift;

$ref_browseentry->Subwidget('entry')->Subwidget('entry')->configure(-background=>$color);
$ref_browseentry->Subwidget('slistbox')->configure(-background=>$color);
}

=head2 beautify_browseentry

Utils::beautify_browseentry($ref_browseentry);

Changes the BrowseEntry widget into the right e:doc look.

Parameters: 

(+) $ref_browseentry: A reference to the BrowseEntry widget.

=cut

####################################
# Make the BrowseEntry look better #
####################################
sub beautify_browseentry
{
my $ref_browseentry = shift;
my $speedbarfont = $main::config->{'UI'}->{'speedbarfont'};
my $toolboxfont = $main::config->{'UI'}->{'toolboxfont'}; 
my $activebackground = $main::config->{'UI'}->{'activebackground'};
my $buttons_style = $main::config->{'UI'}->{'buttons_style'};
my $buttons_borderwidth = $main::config->{'UI'}->{'buttons_borderwidth'};

$ref_browseentry->Subwidget('arrow')->configure(-activebackground=>$activebackground, -borderwidth=>$buttons_borderwidth, -relief=>$buttons_style, -image=>$main::combo_pixmap);

}

=head2 process_filename

$new_filename = Utils::process_filename($filename);

This function checks if the given file exists. If not, it tries to
append the e:doc-filepath stored in '$main::edoc_path' in front of
the filename. If this does not exist also, it returns undef, which means
that the file was not found.

It returns the processed filename&path.

Parameters: 

(+) $filename: The filename to be processed

=cut

################################
# Processes the given filename #
################################
sub process_filename
{
my $filename = shift;

my $return_value = undef;

if ($filename =~ /%EDOC_PATH%/)
	{
	$filename =~ s/%EDOC_PATH%/$main::edoc_path/g ;
	}

#Check if the file name is in the currents project directory
if (defined $main::prjform and -f $main::prjform->path().$main::path_separator.$filename)
	{
	$return_value = $main::prjform->path().$main::path_separator.$filename;
	}
#Check if the file name fits like it is (absolute path)
elsif (-f $filename)
	{
	$return_value = $filename;
	}
#Or maybe it's relative to the edoc-path
else 
	{ 
	$filename = $main::edoc_path.$main::path_separator.$filename;
	if (-f $filename)
		{
		$return_value = $filename;
		}
	}


return $return_value;
}

=head2 process_filename_in_document

$new_filename = Utils::process_filename_in_document($filename, $ref_docform);

This function checks if the given file exists. If not, it tries to
append the active document's filepath stored in front of
the filename. If this does not exist also, it returns undef, which means
that the file was not found.

It returns the processed filename&path.

Parameters: 

(+) $filename: The filename to be processed

(+) $ref_docform: A reference to the DocForm object that holds the document

=cut

############################################################
# Processes the given filename for use inside of documents #
############################################################
sub process_filename_in_document
{
my $filename = shift;
my $ref_docform = shift;

my $orig_filename = $filename;
my $return_value = undef;

if ($filename =~ /%EDOC_PATH%/)
	{
	$filename =~ s/%EDOC_PATH%/$main::edoc_path/g ;
	}

#Check if the file name is in the currents project directory
if (defined $ref_docform->prjform() and -f $ref_docform->prjform()->path().$main::path_separator.$filename)
	{
	$return_value = $ref_docform->prjform()->path().$main::path_separator.$filename;
	}
#Check if the file name fits like it is (absolute path)
elsif (-f $filename)
	{
	$return_value = $filename;
	}
else # else check if the path is relative to the documents path  
	{
	my $path = $ref_docform->path();
	if (defined $path and not ($path eq ''))
		{
		$path = $1;
		$filename = $path.$main::path_separator.$orig_filename;
		}
	if (-f $filename)
		{
		$return_value = $filename;
		}
	else # else check if the file is relative to the current path
		{
		$filename = ".".$main::path_separator.$orig_filename;
		if (-f $filename)
			{
			$return_value = $filename;
			}
		}
	}

return $return_value;
}

=head2 extract_path

$path_only = Utils::extract_path($filename_incl_path);

Extracts and returns the filepath of the given filename. If there is no path,
the function returns ''.

Parameters: 

(+) $filename_incl_path: The filename to be processed

=cut

####################################
# Extract the path from a filename #
####################################
sub extract_path
{
my $filename_incl_path = shift;

my $return_value = '';

if ($filename_incl_path =~ /(.*)[\\\/]/)
	{
	$return_value = $1;
	}

return $return_value;
}

=head2 extract_filename

$filename_only = Utils::extract_filename($filename_incl_path);

Extracts and returns the filepath of the given filename. If there is no path,
the function returns ''.

Parameters: 

(+) $filename_incl_path: The filename to be processed

=cut

#######################################################
# Extract the filename only from a filename_incl_path #
#######################################################
sub extract_filename
{
my $filename_incl_path = shift;

my $return_value = '';

if ($filename_incl_path =~ /(.*[\\\/])/)
	{
	my $a = quotemeta($1);
	$filename_incl_path =~ s/$a//g ;
	$return_value = $filename_incl_path;
	}

return $return_value;
}
1; #So mu package enden

=head1 KNOWN BUGS:

=head1 STILL MISSING:

=cut
